@@ -1,7 +1,8 @@
 public class FloatMultidim2 {
   public float[][] f(int y) {
+    float[][] a1 = null;
     if (y > 0 && y < 5) {
-      float[][] a1 = new float[2][y];
+      a1 = new float[2][y];
       int j;
       if (y > 1) {
         j = 1;
@@ -9,9 +10,8 @@ public class FloatMultidim2 {
         j = 0;
       }
       a1[1][j] = 1.0f;
-      return a1;
-    } else {
-      return new float[1][1];
     }
+    assert a1 == null;
+    return a1;
   }
 }
