@@ -35,7 +35,7 @@ codet character_refine_preprocesst::convert_char_function(
 
 /// The returned expression is true when the first argument is in the interval
 /// defined by the lower and upper bounds (included)
-/// \param arg: Expression we want to bound
+/// \param chr: Expression we want to bound
 /// \param lower_bound: Integer lower bound
 /// \param upper_bound: Integer upper bound
 /// \return A Boolean expression
@@ -65,7 +65,7 @@ exprt character_refine_preprocesst::in_list_expr(
 
 /// Determines the number of char values needed to represent the specified
 /// character (Unicode code point).
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A integer expression of the given type
 exprt character_refine_preprocesst::expr_of_char_count(
@@ -125,14 +125,13 @@ codet character_refine_preprocesst::convert_compare(conversion_inputt &target)
   return code_assignt(result, expr);
 }
 
-
 /// Converts function call to an assignment of an expression corresponding to
 /// the java method Character.digit:(CI)I. The function call has one character
 /// argument and an optional integer radix argument. If the radix is not given
 /// it is set to 36 by default.
 /// \param target: a position in a goto program
 /// \return code assigning the result of the Character.digit function to the
-///         left-hand-side of the given target
+///   left-hand-side of the given target
 codet character_refine_preprocesst::convert_digit_char(
   conversion_inputt &target)
 {
@@ -308,7 +307,7 @@ codet character_refine_preprocesst::convert_hash_code(conversion_inputt &target)
 /// Returns the leading surrogate (a high surrogate code unit) of the surrogate
 /// pair representing the specified supplementary character (Unicode code point)
 /// in the UTF-16 encoding.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return An expression of the given type
 exprt character_refine_preprocesst::expr_of_high_surrogate(
@@ -319,16 +318,7 @@ exprt character_refine_preprocesst::expr_of_high_surrogate(
   exprt u400=from_integer(0x0400, type);
 
   plus_exprt high_surrogate(uD800, div_exprt(minus_exprt(chr, u10000), u400));
-  return high_surrogate;
-}
-
-/// Converts function call to an assignment of an expression corresponding to
-/// the java method Character.highSurrogate:(C)Z
-codet character_refine_preprocesst::convert_high_surrogate(
-  conversion_inputt &target)
-{
-  return convert_char_function(
-    &character_refine_preprocesst::expr_of_high_surrogate, target);
+  return std::move(high_surrogate);
 }
 
 /// Determines if the specified character is an ASCII lowercase character.
@@ -338,16 +328,18 @@ codet character_refine_preprocesst::convert_high_surrogate(
 exprt character_refine_preprocesst::expr_of_is_ascii_lower_case(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   return in_interval_expr(chr, 'a', 'z');
 }
 
 /// Determines if the specified character is an ASCII uppercase character.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_ascii_upper_case(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   return in_interval_expr(chr, 'A', 'Z');
 }
 
@@ -356,7 +348,7 @@ exprt character_refine_preprocesst::expr_of_is_ascii_upper_case(
 ///    TODO: for now this is only for ASCII characters, the
 ///          following unicode categories are not yet considered:
 ///          TITLECASE_LETTER MODIFIER_LETTER OTHER_LETTER LETTER_NUMBER
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return An expression of the given type
 exprt character_refine_preprocesst::expr_of_is_letter(
@@ -374,7 +366,7 @@ exprt character_refine_preprocesst::expr_of_is_letter(
 ///          TITLECASE_LETTER MODIFIER_LETTER OTHER_LETTER LETTER_NUMBER
 ///          and contributory property Other_Alphabetic as defined by the
 ///          Unicode Standard.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return An expression of the given type
 exprt character_refine_preprocesst::expr_of_is_alphabetic(
@@ -396,14 +388,16 @@ codet character_refine_preprocesst::convert_is_alphabetic(
 /// Determines whether the specified character (Unicode code point) is in the
 /// Basic Multilingual Plane (BMP). Such code points can be represented using a
 /// single char.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_bmp_code_point(
   const exprt &chr, const typet &type)
 {
-  binary_relation_exprt is_bmp(chr, ID_le, from_integer(0xFFFF, chr.type()));
-  return is_bmp;
+  (void)type; // unused parameter
+  return and_exprt(
+    binary_relation_exprt(chr, ID_le, from_integer(0xFFFF, chr.type())),
+    binary_relation_exprt(chr, ID_ge, from_integer(0, chr.type())));
 }
 
 /// Converts function call to an assignment of an expression corresponding to
@@ -417,12 +411,13 @@ codet character_refine_preprocesst::convert_is_bmp_code_point(
 }
 
 /// Determines if a character is defined in Unicode.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return An expression of the given type
 exprt character_refine_preprocesst::expr_of_is_defined(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   // The following intervals are undefined in unicode, according to
   // the Unicode Character Database: http://www.unicode.org/Public/UCD/latest/
   exprt::operandst intervals;
@@ -475,11 +470,11 @@ codet character_refine_preprocesst::convert_is_defined_int(
 /// DECIMAL_DIGIT_NUMBER.
 ///
 ///   TODO: for now we only support these ranges of digits:
-///         '\u0030' through '\u0039', ISO-LATIN-1 digits ('0' through '9')
-///         '\u0660' through '\u0669', Arabic-Indic digits
-///         '\u06F0' through '\u06F9', Extended Arabic-Indic digits
-///         '\u0966' through '\u096F', Devanagari digits
-///         '\uFF10' through '\uFF19', Fullwidth digits
+///         '\\u0030' through '\\u0039', ISO-LATIN-1 digits ('0' through '9')
+///         '\\u0660' through '\\u0669', Arabic-Indic digits
+///         '\\u06F0' through '\\u06F9', Extended Arabic-Indic digits
+///         '\\u0966' through '\\u096F', Devanagari digits
+///         '\\uFF10' through '\\uFF19', Fullwidth digits
 ///         Many other character ranges contain digits as well.
 /// \param chr: An expression of type character
 /// \param type: A type for the output
@@ -487,6 +482,7 @@ codet character_refine_preprocesst::convert_is_defined_int(
 exprt character_refine_preprocesst::expr_of_is_digit(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   exprt latin_digit=in_interval_expr(chr, '0', '9');
   exprt arabic_indic_digit=in_interval_expr(chr, 0x660, 0x669);
   exprt extended_digit=in_interval_expr(chr, 0x6F0, 0x6F9);
@@ -495,7 +491,7 @@ exprt character_refine_preprocesst::expr_of_is_digit(
   or_exprt digit(
     or_exprt(latin_digit, or_exprt(arabic_indic_digit, extended_digit)),
     or_exprt(devanagari_digit, fullwidth_digit));
-  return digit;
+  return std::move(digit);
 }
 
 /// Converts function call to an assignment of an expression corresponding to
@@ -519,12 +515,13 @@ codet character_refine_preprocesst::convert_is_digit_int(
 
 /// Determines if the given char value is a Unicode high-surrogate code unit
 /// (also known as leading-surrogate code unit).
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_high_surrogate(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   return in_interval_expr(chr, 0xD800, 0xDBFF);
 }
 
@@ -539,22 +536,23 @@ codet character_refine_preprocesst::convert_is_high_surrogate(
 }
 
 /// Determines if the character is one of ignorable in a Java identifier, that
-/// is, it is in one of these ranges: '\u0000' through '\u0008' '\u000E' through
-/// '\u001B' '\u007F' through '\u009F'
+/// is, it is in one of these ranges: '\\u0000' through '\\u0008' '\\u000E'
+/// through '\\u001B' '\\u007F' through '\\u009F'
 ///
 ///    TODO: For now, we ignore the FORMAT general category value
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_identifier_ignorable(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   or_exprt ignorable(
     in_interval_expr(chr, 0x0000, 0x0008),
      or_exprt(
       in_interval_expr(chr, 0x000E, 0x001B),
       in_interval_expr(chr, 0x007F, 0x009F)));
-  return ignorable;
+  return std::move(ignorable);
 }
 
 /// Converts function call to an assignment of an expression corresponding to
@@ -770,12 +768,13 @@ codet character_refine_preprocesst::convert_is_low_surrogate(
 /// specification.
 ///
 ///    TODO: For now only ASCII characters are considered
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return An expression of the given type
 exprt character_refine_preprocesst::expr_of_is_mirrored(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   return in_list_expr(chr, {0x28, 0x29, 0x3C, 0x3E, 0x5B, 0x5D, 0x7B, 0x7D});
 }
 
@@ -812,12 +811,13 @@ codet character_refine_preprocesst::convert_is_space(conversion_inputt &target)
 
 /// Determines if the specified character is white space according to Unicode
 /// (SPACE_SEPARATOR, LINE_SEPARATOR, or PARAGRAPH_SEPARATOR)
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_space_char(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   std::list<mp_integer> space_characters=
     {0x20, 0x00A0, 0x1680, 0x202F, 0x205F, 0x3000, 0x2028, 0x2029};
   exprt condition0=in_list_expr(chr, space_characters);
@@ -846,12 +846,13 @@ codet character_refine_preprocesst::convert_is_space_char_int(
 
 /// Determines whether the specified character (Unicode code point) is in the
 /// supplementary character range.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_supplementary_code_point(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   return binary_relation_exprt(chr, ID_gt, from_integer(0xFFFF, chr.type()));
 }
 
@@ -866,12 +867,13 @@ codet character_refine_preprocesst::convert_is_supplementary_code_point(
 }
 
 /// Determines if the given char value is a Unicode surrogate code unit.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_surrogate(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   return in_interval_expr(chr, 0xD800, 0xDFFF);
 }
 
@@ -902,12 +904,13 @@ codet character_refine_preprocesst::convert_is_surrogate_pair(
 }
 
 /// Determines if the specified character is a titlecase character.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_title_case(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   std::list<mp_integer>title_case_chars=
     {0x01C5, 0x01C8, 0x01CB, 0x01F2, 0x1FBC, 0x1FCC, 0x1FFC};
   exprt::operandst conditions;
@@ -939,12 +942,13 @@ codet character_refine_preprocesst::convert_is_title_case_int(
 
 /// Determines if the specified character is in the LETTER_NUMBER category of
 /// Unicode
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_letter_number(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   // The following set of characters is the general category "Nl" in the
   // Unicode specification.
   exprt cond0=in_interval_expr(chr, 0x16EE, 0x16F0);
@@ -966,7 +970,7 @@ exprt character_refine_preprocesst::expr_of_is_letter_number(
 ///
 ///    TODO: For now we do not allow connecting punctuation, combining mark,
 ///          non-spacing mark
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_unicode_identifier_part(
@@ -1058,6 +1062,7 @@ codet character_refine_preprocesst::convert_is_upper_case_int(
 exprt character_refine_preprocesst::expr_of_is_valid_code_point(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   return binary_relation_exprt(chr, ID_le, from_integer(0x10FFFF, chr.type()));
 }
 
@@ -1074,14 +1079,15 @@ codet character_refine_preprocesst::convert_is_valid_code_point(
 /// Determines if the specified character is white space according to Java. It
 /// is the case when it one of the following: * a Unicode space character
 /// (SPACE_SEPARATOR, LINE_SEPARATOR, or PARAGRAPH_SEPARATOR) but is not also a
-/// non-breaking space ('\u00A0', '\u2007', '\u202F'). * it is one of these:
+/// non-breaking space ('\\u00A0', '\\u2007', '\\u202F'). * it is one of these:
 /// U+0009  U+000A U+000B U+000C U+000D U+001C U+001D U+001E U+001F
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A Boolean expression
 exprt character_refine_preprocesst::expr_of_is_whitespace(
   const exprt &chr, const typet &type)
 {
+  (void)type; // unused parameter
   exprt::operandst conditions;
   std::list<mp_integer> space_characters=
     {0x20, 0x1680, 0x205F, 0x3000, 0x2028, 0x2029};
@@ -1115,7 +1121,7 @@ codet character_refine_preprocesst::convert_is_whitespace_int(
 /// Returns the trailing surrogate (a low surrogate code unit) of the surrogate
 /// pair representing the specified supplementary character (Unicode code point)
 /// in the UTF-16 encoding.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A integer expression of the given type
 exprt character_refine_preprocesst::expr_of_low_surrogate(
@@ -1126,19 +1132,9 @@ exprt character_refine_preprocesst::expr_of_low_surrogate(
   return plus_exprt(uDC00, mod_exprt(chr, u0400));
 }
 
-/// Converts function call to an assignment of an expression corresponding to
-/// the java method Character.lowSurrogate:(I)C
-/// \param target: a position in a goto program
-codet character_refine_preprocesst::convert_low_surrogate(
-  conversion_inputt &target)
-{
-  return convert_char_function(
-    &character_refine_preprocesst::expr_of_low_surrogate, target);
-}
-
 /// Returns the value obtained by reversing the order of the bytes in the
 /// specified char value.
-/// \param expr: An expression of type character
+/// \param chr: An expression of type character
 /// \param type: A type for the output
 /// \return A character expression of the given type
 exprt character_refine_preprocesst::expr_of_reverse_bytes(
@@ -1159,66 +1155,6 @@ codet character_refine_preprocesst::convert_reverse_bytes(
     &character_refine_preprocesst::expr_of_reverse_bytes, target);
 }
 
-/// Converts the specified character (Unicode code point) to its UTF-16
-/// representation stored in a char array. If the specified code point is a BMP
-/// (Basic Multilingual Plane or Plane 0) value, the resulting char array has
-/// the same value as codePoint. If the specified code point is a supplementary
-/// code point, the resulting char array has the corresponding surrogate pair.
-/// \param expr: An expression of type character
-/// \param type: A type for the output
-/// \return A character array expression of the given type
-exprt character_refine_preprocesst::expr_of_to_chars(
-  const exprt &chr, const typet &type)
-{
-  array_typet array_type=to_array_type(type);
-  const typet &char_type=array_type.subtype();
-  array_exprt case1(array_type);
-  array_exprt case2(array_type);
-  exprt low_surrogate=expr_of_low_surrogate(chr, char_type);
-  case1.copy_to_operands(low_surrogate);
-  case2.move_to_operands(low_surrogate);
-  exprt high_surrogate=expr_of_high_surrogate(chr, char_type);
-  case2.move_to_operands(high_surrogate);
-  return if_exprt(expr_of_is_bmp_code_point(chr, type), case1, case2);
-}
-
-/// Converts function call to an assignment of an expression corresponding to
-/// the java method Character.toChars:(I)[C
-/// \param target: a position in a goto program
-codet character_refine_preprocesst::convert_to_chars(conversion_inputt &target)
-{
-  return convert_char_function(
-    &character_refine_preprocesst::expr_of_to_chars, target);
-}
-
-/// Converts function call to an assignment of an expression corresponding to
-/// the java method Character.toCodePoint:(CC)I
-/// \param target: a position in a goto program
-codet character_refine_preprocesst::convert_to_code_point(
-  conversion_inputt &target)
-{
-  const code_function_callt &function_call=target;
-  assert(function_call.arguments().size()==2);
-  const exprt &arg0=function_call.arguments()[0];
-  const exprt &arg1=function_call.arguments()[1];
-  const exprt &result=function_call.lhs();
-  const typet &type=result.type();
-
-  // These operations implement the decoding of a unicode symbol encoded
-  // in UTF16 for the supplementary planes (above U+10000).
-  // The low ten bits of the first character give the bits 10 to 19 of
-  // code point and the low ten bits of the second give the bits 0 to 9,
-  // then 0x10000 is added to the result. For more explenations see:
-  //   https://en.wikipedia.org/wiki/UTF-16
-
-  exprt u010000=from_integer(0x010000, type);
-  exprt mask10bit=from_integer(0x03FF, type);
-  shl_exprt m1(bitand_exprt(arg0, mask10bit), from_integer(10, type));
-  bitand_exprt m2(arg1, mask10bit);
-  bitor_exprt pair_value(u010000, bitor_exprt(m1, m2));
-  return code_assignt(result, pair_value);
-}
-
 /// Converts the character argument to lowercase.
 ///
 ///    TODO: For now we only consider ASCII characters but ultimately
@@ -1233,8 +1169,7 @@ exprt character_refine_preprocesst::expr_of_to_lower_case(
   minus_exprt transformed(
     plus_exprt(chr, from_integer('a', type)), from_integer('A', type));
 
-  if_exprt res(expr_of_is_ascii_upper_case(chr, type), transformed, chr);
-  return res;
+  return if_exprt(expr_of_is_ascii_upper_case(chr, type), transformed, chr);
 }
 
 /// Converts function call to an assignment of an expression corresponding to
@@ -1276,7 +1211,7 @@ exprt character_refine_preprocesst::expr_of_to_title_case(
   or_exprt plus_8_set(
     plus_8_interval1, or_exprt(plus_8_interval2, plus_8_interval3));
 
-  if_exprt res(
+  return if_exprt(
     in_list_expr(chr, increment_list),
     plus_1,
     if_exprt(
@@ -1285,12 +1220,7 @@ exprt character_refine_preprocesst::expr_of_to_title_case(
       if_exprt(
         plus_8_set,
         plus_8,
-        if_exprt(
-          in_list_expr(chr, plus_9_list),
-          plus_9,
-          chr))));
-
-  return res;
+        if_exprt(in_list_expr(chr, plus_9_list), plus_9, chr))));
 }
 
 /// Converts function call to an assignment of an expression corresponding to
@@ -1326,8 +1256,7 @@ exprt character_refine_preprocesst::expr_of_to_upper_case(
   minus_exprt transformed(
     plus_exprt(chr, from_integer('A', type)), from_integer('a', type));
 
-  if_exprt res(expr_of_is_ascii_lower_case(chr, type), transformed, chr);
-  return res;
+  return if_exprt(expr_of_is_ascii_lower_case(chr, type), transformed, chr);
 }
 
 /// Converts function call to an assignment of an expression corresponding to
@@ -1426,8 +1355,6 @@ void character_refine_preprocesst::initialize_conversion_table()
       &character_refine_preprocesst::convert_get_type_int;
   conversion_table["java::java.lang.Character.hashCode:()I"]=
       &character_refine_preprocesst::convert_hash_code;
-  conversion_table["java::java.lang.Character.highSurrogate:(I)C"]=
-      &character_refine_preprocesst::convert_high_surrogate;
   conversion_table["java::java.lang.Character.isAlphabetic:(I)Z"]=
       &character_refine_preprocesst::convert_is_alphabetic;
   conversion_table["java::java.lang.Character.isBmpCodePoint:(I)Z"]=
@@ -1516,8 +1443,6 @@ void character_refine_preprocesst::initialize_conversion_table()
       &character_refine_preprocesst::convert_is_whitespace_char;
   conversion_table["java::java.lang.Character.isWhitespace:(I)Z"]=
       &character_refine_preprocesst::convert_is_whitespace_int;
-  conversion_table["java::java.lang.Character.lowSurrogate:(I)C"]=
-      &character_refine_preprocesst::convert_is_low_surrogate;
 
   // Not supported "java::java.lang.Character.offsetByCodePoints:([CIIII)I"
   // Not supported "java::java.lang.Character.offsetByCodePoints:"
@@ -1525,13 +1450,9 @@ void character_refine_preprocesst::initialize_conversion_table()
 
   conversion_table["java::java.lang.Character.reverseBytes:(C)C"]=
       &character_refine_preprocesst::convert_reverse_bytes;
-  conversion_table["java::java.lang.Character.toChars:(I)[C"]=
-      &character_refine_preprocesst::convert_to_chars;
 
   // Not supported "java::java.lang.Character.toChars:(I[CI)I"
 
-  conversion_table["java::java.lang.Character.toCodePoint:(CC)I"]=
-      &character_refine_preprocesst::convert_to_code_point;
   conversion_table["java::java.lang.Character.toLowerCase:(C)C"]=
       &character_refine_preprocesst::convert_to_lower_case_char;
   conversion_table["java::java.lang.Character.toLowerCase:(I)I"]=
