@@ -22,7 +22,9 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 #include <ansi-c/c_misc.h>
 #include <ansi-c/expr2c_class.h>
 
+#include "java_expr.h"
 #include "java_qualifiers.h"
+#include "java_string_literal_expr.h"
 #include "java_types.h"
 
 std::string expr2javat::convert(const typet &src)
@@ -127,15 +129,9 @@ std::string expr2javat::convert_struct(
   bool first=true;
   size_t last_size=0;
 
-  for(struct_typet::componentst::const_iterator
-      c_it=components.begin();
-      c_it!=components.end();
-      c_it++)
+  for(const auto &c : components)
   {
-    if(c_it->type().id()==ID_code)
-    {
-    }
-    else
+    if(c.type().id() != ID_code)
     {
       std::string tmp=convert(*o_it);
       std::string sep;
@@ -155,7 +151,10 @@ std::string expr2javat::convert_struct(
 
       dest+=sep;
       dest+='.';
-      dest+=c_it->get_string(ID_pretty_name);
+      irep_idt field_name = c.get_pretty_name();
+      if(field_name.empty())
+        field_name = c.get_name();
+      dest += id2string(field_name);
       dest+='=';
       dest+=tmp;
     }
@@ -197,19 +196,17 @@ std::string expr2javat::convert_constant(
     std::string dest;
     dest.reserve(char_representation_length);
 
-    mp_integer int_value;
-    if(to_integer(src, int_value))
-      UNREACHABLE;
+    const char16_t int_value = numeric_cast_v<char16_t>(src);
 
-    dest += "(char)'" + utf16_native_endian_to_java(int_value.to_long()) + '\'';
+    // Character literals in Java have type 'char', thus no cast is needed.
+    // This is different from C, where charater literals have type 'int'.
+    dest += '\'' + utf16_native_endian_to_java(int_value) + '\'';
     return dest;
   }
   else if(src.type()==java_byte_type())
   {
     // No byte-literals in Java, so just cast:
-    mp_integer int_value;
-    if(to_integer(src, int_value))
-      UNREACHABLE;
+    const mp_integer int_value = numeric_cast_v<mp_integer>(src);
     std::string dest="(byte)";
     dest+=integer2string(int_value);
     return dest;
@@ -217,9 +214,7 @@ std::string expr2javat::convert_constant(
   else if(src.type()==java_short_type())
   {
     // No short-literals in Java, so just cast:
-    mp_integer int_value;
-    if(to_integer(src, int_value))
-      UNREACHABLE;
+    const mp_integer int_value = numeric_cast_v<mp_integer>(src);
     std::string dest="(short)";
     dest+=integer2string(int_value);
     return dest;
@@ -227,9 +222,7 @@ std::string expr2javat::convert_constant(
   else if(src.type()==java_long_type())
   {
     // long integer literals must have 'L' at the end
-    mp_integer int_value;
-    if(to_integer(src, int_value))
-      UNREACHABLE;
+    const mp_integer int_value = numeric_cast_v<mp_integer>(src);
     std::string dest=integer2string(int_value);
     dest+='L';
     return dest;
@@ -257,8 +250,7 @@ std::string expr2javat::convert_rec(
   qualifierst &new_qualifiers = *clone;
   new_qualifiers.read(src);
 
-  const std::string d=
-    declarator==""?declarator:(" "+declarator);
+  const std::string d = declarator.empty() ? declarator : (" " + declarator);
 
   const std::string q=
     new_qualifiers.as_string();
@@ -289,7 +281,7 @@ std::string expr2javat::convert_rec(
     // so we make one up, loosely inspired by the syntax
     // of lambda expressions.
 
-    std::string dest="";
+    std::string dest;
 
     dest+='(';
     const java_method_typet::parameterst &parameters = method_type.parameters();
@@ -322,29 +314,25 @@ std::string expr2javat::convert_rec(
     return expr2ct::convert_rec(src, qualifiers, declarator);
 }
 
-std::string expr2javat::convert_java_this(
-  const exprt &src,
-  unsigned precedence)
+std::string expr2javat::convert_java_this()
 {
-  return "this";
+  return id2string(ID_this);
 }
 
-std::string expr2javat::convert_java_instanceof(
-  const exprt &src,
-  unsigned precedence)
+std::string expr2javat::convert_java_instanceof(const exprt &src)
 {
-  if(src.operands().size()!=2)
-  {
-    unsigned precedence;
-    return convert_norep(src, precedence);
-  }
+  const auto &instanceof_expr = to_java_instanceof_expr(src);
 
-  return convert(src.op0())+" instanceof "+convert(src.op1().type());
+  return convert(instanceof_expr.tested_expr()) + " instanceof " +
+         convert(instanceof_expr.target_type());
 }
 
-std::string expr2javat::convert_java_new(
-  const exprt &src,
-  unsigned precedence)
+std::string expr2javat::convert_code_java_new(const exprt &src, unsigned indent)
+{
+  return indent_str(indent) + convert_java_new(src) + ";\n";
+}
+
+std::string expr2javat::convert_java_new(const exprt &src)
 {
   std::string dest;
 
@@ -380,7 +368,7 @@ std::string expr2javat::convert_code_java_delete(
     return convert_norep(src, precedence);
   }
 
-  std::string tmp=convert(src.op0());
+  std::string tmp = convert(to_unary_expr(src).op());
 
   dest+=tmp+";\n";
 
@@ -392,17 +380,29 @@ std::string expr2javat::convert_with_precedence(
   unsigned &precedence)
 {
   if(src.id()=="java-this")
-    return convert_java_this(src, precedence=15);
+  {
+    precedence = 15;
+    return convert_java_this();
+  }
   if(src.id()==ID_java_instanceof)
-    return convert_java_instanceof(src, precedence=15);
+  {
+    precedence = 15;
+    return convert_java_instanceof(src);
+  }
   else if(src.id()==ID_side_effect &&
           (src.get(ID_statement)==ID_java_new ||
            src.get(ID_statement)==ID_java_new_array ||
            src.get(ID_statement)==ID_java_new_array_data))
-    return convert_java_new(src, precedence=15);
+  {
+    precedence = 15;
+    return convert_java_new(src);
+  }
   else if(src.id()==ID_side_effect &&
           src.get(ID_statement)==ID_throw)
-    return convert_function(src, "throw", precedence=16);
+  {
+    precedence = 16;
+    return convert_function(src, "throw");
+  }
   else if(src.id()==ID_code &&
           to_code(src).get_statement()==ID_exception_landingpad)
   {
@@ -420,14 +420,16 @@ std::string expr2javat::convert_with_precedence(
     return "pod_constructor";
   else if(src.id()==ID_virtual_function)
   {
-    return "VIRTUAL_FUNCTION(" +
-      id2string(src.get(ID_C_class)) +
-      "." +
-      id2string(src.get(ID_component_name)) +
-      ")";
+    const class_method_descriptor_exprt &virtual_function =
+      to_class_method_descriptor_expr(src);
+    return "CLASS_METHOD_DESCRIPTOR(" + id2string(virtual_function.class_id()) +
+           "." + id2string(virtual_function.mangled_method_name()) + ")";
+  }
+  else if(
+    const auto &literal = expr_try_dynamic_cast<java_string_literal_exprt>(src))
+  {
+    return '"' + MetaString(id2string(literal->value())) + '"';
   }
-  else if(src.id()==ID_java_string_literal)
-    return '"'+MetaString(src.get_string(ID_value))+'"';
   else if(src.id()==ID_constant)
     return convert_constant(to_constant_expr(src), precedence=16);
   else
@@ -442,7 +444,7 @@ std::string expr2javat::convert_code(
 
   if(statement==ID_java_new ||
      statement==ID_java_new_array)
-    return convert_java_new(src, indent);
+    return convert_code_java_new(src, indent);
 
   if(statement==ID_function_call)
     return convert_code_function_call(to_code_function_call(src), indent);
