@@ -20,10 +20,7 @@ void jar_filet::initialize_file_index()
 {
   const size_t file_count=m_zip_archive.get_num_files();
   for(size_t index=0; index<file_count; index++)
-  {
-    const auto filename=m_zip_archive.get_filename(index);
-    m_name_to_index.emplace(filename, index);
-  }
+    m_name_to_index.emplace(m_zip_archive.get_filename(index), index);
 }
 
 /// This constructor creates a jar_file object whose contents
@@ -77,12 +74,20 @@ optionalt<std::string> jar_filet::get_entry(const std::string &name)
   }
 }
 
+/// Wrapper for `std::isspace` from `cctype`
+/// \param ch: the character to check
+/// \return true if the parameter is considered to be a space in the current
+///   locale, else false
 static bool is_space(const char ch)
 {
   return std::isspace(ch) != 0;
 }
 
 /// Remove leading and trailing whitespace characters from string
+/// \param begin: iterator to start search in string
+/// \param end: iterator to end search in string
+/// \return string truncated from begin to end and all whitespace removed at the
+///   begin and end
 static std::string trim(
   const std::string::const_iterator begin,
   const std::string::const_iterator end)
