@@ -10,6 +10,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <util/config.h>
 #include <util/expr_initializer.h>
+#include <util/journalling_symbol_table.h>
 #include <util/string_constant.h>
 #include <util/suffix.h>
 
@@ -22,10 +23,25 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "java_object_factory.h"
 #include "java_string_literals.h"
 #include "java_utils.h"
+#include <util/fresh_symbol.h>
+#include <util/nondet.h>
 
 #define JAVA_MAIN_METHOD "main:([Ljava/lang/String;)V"
 
-static void create_initialize(symbol_table_baset &symbol_table)
+static optionalt<codet> record_return_value(
+  const symbolt &function,
+  const symbol_table_baset &symbol_table);
+
+static code_blockt record_pointer_parameters(
+  const symbolt &function,
+  const std::vector<exprt> &arguments,
+  const symbol_table_baset &symbol_table);
+
+static codet record_exception(
+  const symbolt &function,
+  const symbol_table_baset &symbol_table);
+
+void create_java_initialize(symbol_table_baset &symbol_table)
 {
   // If __CPROVER_initialize already exists, replace it. It may already exist
   // if a GOTO binary provided it. This behaviour mirrors the ANSI-C frontend.
@@ -36,20 +52,7 @@ static void create_initialize(symbol_table_baset &symbol_table)
   initialize.base_name=INITIALIZE_FUNCTION;
   initialize.mode=ID_java;
 
-  initialize.type = java_method_typet({}, empty_typet());
-
-  code_blockt init_code;
-
-  namespacet ns(symbol_table);
-
-  symbol_exprt rounding_mode=
-    ns.lookup(CPROVER_PREFIX "rounding_mode").symbol_expr();
-
-  init_code.add(
-    code_assignt(rounding_mode, from_integer(0, rounding_mode.type())));
-
-  initialize.value=init_code;
-
+  initialize.type = java_method_typet({}, java_void_type());
   symbol_table.add(initialize);
 }
 
@@ -69,22 +72,6 @@ static bool should_init_symbol(const symbolt &sym)
   return is_java_string_literal_id(sym.name);
 }
 
-/// Get the symbol name of java.lang.Class' initializer method. This method
-/// should initialize a Class instance with known properties of the type it
-/// represents, such as its name, whether it is abstract, or an enumeration,
-/// etc. The method may or may not exist in any particular symbol table; it is
-/// up to the caller to check.
-/// The method's Java signature is:
-/// void cproverInitializeClassLiteral(
-///   String name,
-///   boolean isAnnotation,
-///   boolean isArray,
-///   boolean isInterface,
-///   boolean isSynthetic,
-///   boolean isLocalClass,
-///   boolean isMemberClass,
-///   boolean isEnum);
-/// \return Class initializer method's symbol name.
 irep_idt get_java_class_literal_initializer_signature()
 {
   static irep_idt signature =
@@ -104,7 +91,7 @@ static const symbolt *get_class_literal_initializer(
 {
   if(symbol.value.is_not_nil())
     return nullptr;
-  if(symbol.type != symbol_typet("java::java.lang.Class"))
+  if(symbol.type != struct_tag_typet("java::java.lang.Class"))
     return nullptr;
   if(!has_suffix(id2string(symbol.name), JAVA_CLASS_MODEL_SUFFIX))
     return nullptr;
@@ -116,19 +103,38 @@ static constant_exprt constant_bool(bool val)
   return from_integer(val ? 1 : 0, java_boolean_type());
 }
 
-static void java_static_lifetime_init(
+void java_static_lifetime_init(
   symbol_table_baset &symbol_table,
   const source_locationt &source_location,
   bool assume_init_pointers_not_null,
-  object_factory_parameterst object_factory_parameters,
+  java_object_factory_parameterst object_factory_parameters,
   const select_pointer_typet &pointer_type_selector,
   bool string_refinement_enabled,
   message_handlert &message_handler)
 {
-  symbolt &initialize_symbol=*symbol_table.get_writeable(INITIALIZE_FUNCTION);
-  code_blockt &code_block=to_code_block(to_code(initialize_symbol.value));
+  symbolt &initialize_symbol =
+    symbol_table.get_writeable_ref(INITIALIZE_FUNCTION);
+  PRECONDITION(initialize_symbol.value.is_nil());
+  code_blockt code_block;
+
+  const symbol_exprt rounding_mode =
+    symbol_table.lookup_ref(CPROVER_PREFIX "rounding_mode").symbol_expr();
+  code_block.add(
+    code_assignt{rounding_mode, from_integer(0, rounding_mode.type())});
+
   object_factory_parameters.function_id = initialize_symbol.name;
 
+  // If there are strings given using --string-input-value, we need to add
+  // them to the symbol table now, so that they appear in __CPROVER_initialize
+  // and we can refer to them later when we initialize input values.
+  for(const auto &val : object_factory_parameters.string_input_values)
+  {
+    // We ignore the return value of the following call, we just need to
+    // make sure the string is in the symbol table.
+    get_or_create_string_literal_symbol(
+      val, symbol_table, string_refinement_enabled);
+  }
+
   // We need to zero out all static variables, or nondet-initialize if they're
   // external. Iterate over a copy of the symtab, as its iterators are
   // invalidated by object_factory:
@@ -137,9 +143,15 @@ static void java_static_lifetime_init(
   for(const auto &entry : symbol_table.symbols)
     symbol_names.push_back(entry.first);
 
-  for(const auto &symname : symbol_names)
+  // Don't use a for-each loop here because the loop extends the list, and the
+  // for-each loop may only read `.end()` once.
+  for(
+    auto symbol_it = symbol_names.begin();
+    symbol_it != symbol_names.end();
+    ++symbol_it)
   {
-    const symbolt &sym=*symbol_table.lookup(symname);
+    const auto &symname = *symbol_it;
+    const symbolt &sym = symbol_table.lookup_ref(symname);
     if(should_init_symbol(sym))
     {
       if(const symbolt *class_literal_init_method =
@@ -152,65 +164,71 @@ static void java_static_lifetime_init(
 
         bool class_is_array = is_java_array_tag(sym.name);
 
-        exprt name_literal(ID_java_string_literal);
-        name_literal.set(ID_value, to_class_type(class_symbol.type).get_tag());
+        journalling_symbol_tablet journalling_table =
+          journalling_symbol_tablet::wrap(symbol_table);
 
-        symbol_exprt class_name_literal =
-          get_or_create_string_literal_symbol(
-            name_literal, symbol_table, string_refinement_enabled);
+        symbol_exprt class_name_literal = get_or_create_string_literal_symbol(
+          to_class_type(class_symbol.type).get_tag(),
+          journalling_table,
+          string_refinement_enabled);
+
+        // If that created any new symbols make sure we initialise those too:
+        const auto &new_symbols = journalling_table.get_inserted();
+        symbol_names.insert(
+          symbol_names.end(), new_symbols.begin(), new_symbols.end());
 
         // Call the literal initializer method instead of a nondet initializer:
 
         // For arguments we can't parse yet:
         side_effect_expr_nondett nondet_bool(java_boolean_type(), sym.location);
 
+        const auto &java_class_type = to_java_class_type(class_symbol.type);
+
         // Argument order is: name, isAnnotation, isArray, isInterface,
         // isSynthetic, isLocalClass, isMemberClass, isEnum
 
-        code_function_callt initializer_call;
-        initializer_call.function() = class_literal_init_method->symbol_expr();
-
-        code_function_callt::argumentst &args = initializer_call.arguments();
-
-        // this:
-        args.push_back(address_of_exprt(sym.symbol_expr()));
-        // name:
-        args.push_back(address_of_exprt(class_name_literal));
-        // isAnnotation:
-        args.push_back(
-          constant_bool(class_symbol.type.get_bool(ID_is_annotation)));
-        // isArray:
-        args.push_back(constant_bool(class_is_array));
-        // isInterface:
-        args.push_back(
-          constant_bool(class_symbol.type.get_bool(ID_interface)));
-        // isSynthetic:
-        args.push_back(
-          constant_bool(class_symbol.type.get_bool(ID_synthetic)));
-        // isLocalClass:
-        args.push_back(nondet_bool);
-        // isMemberClass:
-        args.push_back(nondet_bool);
-        // isEnum:
-        args.push_back(
-          constant_bool(class_symbol.type.get_bool(ID_enumeration)));
+        code_function_callt initializer_call(
+          class_literal_init_method->symbol_expr(),
+          {// this:
+           address_of_exprt(sym.symbol_expr()),
+           // name:
+           address_of_exprt(class_name_literal),
+           // isAnnotation:
+           constant_bool(java_class_type.get_is_annotation()),
+           // isArray:
+           constant_bool(class_is_array),
+           // isInterface:
+           constant_bool(java_class_type.get_interface()),
+           // isSynthetic:
+           constant_bool(java_class_type.get_synthetic()),
+           // isLocalClass:
+           nondet_bool,
+           // isMemberClass:
+           nondet_bool,
+           // isEnum:
+           constant_bool(java_class_type.get_is_enumeration())});
 
         // First initialize the object as prior to a constructor:
         namespacet ns(symbol_table);
 
-        exprt zero_object =
-          zero_initializer(
-            sym.type, source_locationt(), ns, message_handler);
+        auto zero_object = zero_initializer(sym.type, source_locationt(), ns);
+        if(!zero_object.has_value())
+        {
+          messaget message(message_handler);
+          message.error() << "failed to zero-initialize " << sym.name
+                          << messaget::eom;
+          throw 0;
+        }
         set_class_identifier(
-          to_struct_expr(zero_object), ns, to_symbol_type(sym.type));
+          to_struct_expr(*zero_object), ns, to_struct_tag_type(sym.type));
 
-        code_block.copy_to_operands(
-          code_assignt(sym.symbol_expr(), zero_object));
+        code_block.add(
+          std::move(code_assignt(sym.symbol_expr(), *zero_object)));
 
         // Then call the init function:
-        code_block.move_to_operands(initializer_call);
+        code_block.add(std::move(initializer_call));
       }
-      else if(sym.value.is_nil() && sym.type!=empty_typet())
+      else if(sym.value.is_nil() && sym.type != java_void_type())
       {
         const bool is_class_model =
           has_suffix(id2string(sym.name), "@class_model");
@@ -218,9 +236,9 @@ static void java_static_lifetime_init(
                                     is_non_null_library_global(sym.name) ||
                                     assume_init_pointers_not_null;
 
-        object_factory_parameterst parameters = object_factory_parameters;
+        java_object_factory_parameterst parameters = object_factory_parameters;
         if(not_allow_null && !is_class_model)
-          parameters.max_nonnull_tree_depth = 1;
+          parameters.min_null_tree_depth = 1;
 
         gen_nondet_init(
           sym.symbol_expr(),
@@ -228,10 +246,11 @@ static void java_static_lifetime_init(
           symbol_table,
           source_location,
           false,
-          allocation_typet::GLOBAL,
+          lifetimet::STATIC_GLOBAL,
           parameters,
           pointer_type_selector,
-          update_in_placet::NO_UPDATE_IN_PLACE);
+          update_in_placet::NO_UPDATE_IN_PLACE,
+          message_handler);
       }
       else if(sym.value.is_not_nil())
       {
@@ -241,6 +260,7 @@ static void java_static_lifetime_init(
       }
     }
   }
+  initialize_symbol.value = std::move(code_block);
 }
 
 /// Checks whether the given symbol is a valid java main method
@@ -252,38 +272,30 @@ bool is_java_main(const symbolt &function)
 {
   bool named_main = has_suffix(id2string(function.name), JAVA_MAIN_METHOD);
   const java_method_typet &function_type = to_java_method_type(function.type);
-  const typet &string_array_type = java_type_from_string("[Ljava/lang/String;");
+  const auto string_array_type = java_type_from_string("[Ljava/lang/String;");
   // checks whether the function is static and has a single String[] parameter
   bool is_static = !function_type.has_this();
   // this should be implied by the signature
   const java_method_typet::parameterst &parameters = function_type.parameters();
   bool has_correct_type = function_type.return_type().id() == ID_empty &&
                           parameters.size() == 1 &&
-                          parameters[0].type().full_eq(string_array_type);
+                          parameters[0].type().full_eq(*string_array_type);
   bool public_access = function_type.get(ID_access) == ID_public;
   return named_main && is_static && has_correct_type && public_access;
 }
 
-///  Extends \p init_code with code that allocates the objects used as test
-///  arguments for the function under test (\p function) and
-///  non-deterministically initializes them.
-///
-///  All the code generated by this function goes to __CPROVER__start, just
-///  before the call to the method under test.
-///
-///  \returns A std::vector of symbol_exprt, one per parameter of \p function,
-///  containing the objects that can be used as arguments for \p function.
-exprt::operandst java_build_arguments(
+std::pair<code_blockt, std::vector<exprt>> java_build_arguments(
   const symbolt &function,
-  code_blockt &init_code,
   symbol_table_baset &symbol_table,
   bool assume_init_pointers_not_null,
-  object_factory_parameterst object_factory_parameters,
-  const select_pointer_typet &pointer_type_selector)
+  java_object_factory_parameterst object_factory_parameters,
+  const select_pointer_typet &pointer_type_selector,
+  message_handlert &message_handler)
 {
-  const java_method_typet::parameterst &parameters =
-    to_java_method_type(function.type).parameters();
+  const java_method_typet &function_type = to_java_method_type(function.type);
+  const java_method_typet::parameterst &parameters = function_type.parameters();
 
+  code_blockt init_code;
   exprt::operandst main_arguments;
   main_arguments.resize(parameters.size());
 
@@ -294,7 +306,7 @@ exprt::operandst java_build_arguments(
 
   // we iterate through all the parameters of the function under test, allocate
   // an object for that parameter (recursively allocating other objects
-  // necessary to initialize it), and declare such object as an ID_input
+  // necessary to initialize it), and mark such object using `code_inputt`.
   for(std::size_t param_number=0;
       param_number<parameters.size();
       param_number++)
@@ -307,119 +319,196 @@ exprt::operandst java_build_arguments(
     // be null
     bool is_this=(param_number==0) && parameters[param_number].get_this();
 
-    object_factory_parameterst parameters = object_factory_parameters;
+    if(is_this && function_type.get_is_constructor())
+    {
+      const symbol_exprt result = fresh_java_symbol(
+                                    p.type(),
+                                    "this_parameter",
+                                    function.location,
+                                    function.name,
+                                    symbol_table)
+                                    .symbol_expr();
+      main_arguments[param_number] = result;
+      init_code.add(code_declt{result});
+      init_code.add(
+        code_assignt{result, side_effect_exprt(ID_java_new, p.type())});
+      continue;
+    }
+
+    java_object_factory_parameterst factory_parameters =
+      object_factory_parameters;
     // only pointer must be non-null
     if(assume_init_pointers_not_null || is_this)
-      parameters.max_nonnull_tree_depth = 1;
+      factory_parameters.min_null_tree_depth = 1;
     // in main() also the array elements of the argument must be non-null
     if(is_main)
-      parameters.max_nonnull_tree_depth = 2;
+      factory_parameters.min_null_tree_depth = 2;
 
-    parameters.function_id = goto_functionst::entry_point();
+    factory_parameters.function_id = goto_functionst::entry_point();
 
-    // generate code to allocate and non-deterministicaly initialize the
-    // argument
-    main_arguments[param_number] = object_factory(
-      p.type(),
-      base_name,
-      init_code,
-      symbol_table,
-      parameters,
-      allocation_typet::LOCAL,
-      function.location,
-      pointer_type_selector);
+    namespacet ns(symbol_table);
+
+    // Generate code to allocate and non-deterministicaly initialize the
+    // argument, if the argument has different possible object types (e.g., from
+    // casts in the function body), then choose one in a non-deterministic way.
+    const auto alternatives =
+      pointer_type_selector.get_parameter_alternative_types(
+        function.name, p.get_identifier(), ns);
+    if(alternatives.empty())
+    {
+      main_arguments[param_number] = object_factory(
+        p.type(),
+        base_name,
+        init_code,
+        symbol_table,
+        factory_parameters,
+        lifetimet::DYNAMIC,
+        function.location,
+        pointer_type_selector,
+        message_handler);
+    }
+    else
+    {
+      INVARIANT(!is_this, "We cannot have different types for `this` here");
+      // create a non-deterministic switch between all possible values for the
+      // type of the parameter.
+
+      // the idea is to get a new symbol for the parameter value `tmp`
+
+      // then add a non-deterministic switch over all possible input types,
+      // construct the object type at hand and assign to `tmp`
+
+      // switch(...)
+      // {
+      //   case obj1:
+      //     tmp_expr = object_factory(...)
+      //     param = tmp_expr
+      //     break
+      //   ...
+      // }
+      // method(..., param, ...)
+      //
+
+      const symbolt result_symbol = fresh_java_symbol(
+        p.type(),
+        "nondet_parameter_" + std::to_string(param_number),
+        function.location,
+        function.name,
+        symbol_table);
+      main_arguments[param_number] = result_symbol.symbol_expr();
+
+      std::vector<codet> cases;
+      cases.reserve(alternatives.size());
+      for(const auto &type : alternatives)
+      {
+        code_blockt init_code_for_type;
+        exprt init_expr_for_parameter = object_factory(
+          java_reference_type(type),
+          id2string(base_name) + "_alternative_" +
+            id2string(type.get_identifier()),
+          init_code_for_type,
+          symbol_table,
+          factory_parameters,
+          lifetimet::DYNAMIC,
+          function.location,
+          pointer_type_selector,
+          message_handler);
+        init_code_for_type.add(
+          code_assignt(
+            result_symbol.symbol_expr(),
+            typecast_exprt(init_expr_for_parameter, p.type())));
+        cases.push_back(init_code_for_type);
+      }
+
+      init_code.add(
+        generate_nondet_switch(
+          id2string(function.name) + "_" + std::to_string(param_number),
+          cases,
+          java_int_type(),
+          ID_java,
+          function.location,
+          symbol_table));
+    }
 
     // record as an input
-    codet input(ID_input);
-    input.operands().resize(2);
-    input.op0()=
-      address_of_exprt(
-        index_exprt(
-          string_constantt(base_name),
-          from_integer(0, index_type())));
-    input.op1()=main_arguments[param_number];
-    input.add_source_location()=function.location;
-
-    init_code.move_to_operands(input);
+    init_code.add(
+      code_inputt{base_name, main_arguments[param_number], function.location});
   }
 
-  return main_arguments;
+  return make_pair(init_code, main_arguments);
 }
 
-void java_record_outputs(
+/// Mark return value, pointer type parameters and the exception as outputs.
+static code_blockt java_record_outputs(
   const symbolt &function,
   const exprt::operandst &main_arguments,
-  code_blockt &init_code,
   symbol_table_baset &symbol_table)
+{
+  code_blockt init_code;
+
+  if(auto return_value = record_return_value(function, symbol_table))
+  {
+    init_code.add(std::move(*return_value));
+  }
+
+  init_code.append(
+    record_pointer_parameters(function, main_arguments, symbol_table));
+
+  init_code.add(record_exception(function, symbol_table));
+
+  return init_code;
+}
+
+static optionalt<codet> record_return_value(
+  const symbolt &function,
+  const symbol_table_baset &symbol_table)
+{
+  if(to_java_method_type(function.type).return_type() == java_void_type())
+    return {};
+
+  const symbolt &return_symbol =
+    symbol_table.lookup_ref(JAVA_ENTRY_POINT_RETURN_SYMBOL);
+
+  return code_outputt{
+    return_symbol.base_name, return_symbol.symbol_expr(), function.location};
+}
+
+static code_blockt record_pointer_parameters(
+  const symbolt &function,
+  const std::vector<exprt> &arguments,
+  const symbol_table_baset &symbol_table)
 {
   const java_method_typet::parameterst &parameters =
     to_java_method_type(function.type).parameters();
 
-  exprt::operandst result;
-  result.reserve(parameters.size()+1);
+  code_blockt init_code;
 
-  bool has_return_value =
-    to_java_method_type(function.type).return_type() != empty_typet();
-
-  if(has_return_value)
-  {
-    // record return value
-    codet output(ID_output);
-    output.operands().resize(2);
-
-    const symbolt &return_symbol=
-      *symbol_table.lookup(JAVA_ENTRY_POINT_RETURN_SYMBOL);
-
-    output.op0()=
-      address_of_exprt(
-        index_exprt(
-          string_constantt(return_symbol.base_name),
-          from_integer(0, index_type())));
-    output.op1()=return_symbol.symbol_expr();
-    output.add_source_location()=function.location;
-
-    init_code.move_to_operands(output);
-  }
-
-  for(std::size_t param_number=0;
-      param_number<parameters.size();
+  for(std::size_t param_number = 0; param_number < parameters.size();
       param_number++)
   {
-    const symbolt &p_symbol=
-      *symbol_table.lookup(parameters[param_number].get_identifier());
+    const symbolt &p_symbol =
+      symbol_table.lookup_ref(parameters[param_number].get_identifier());
 
-    if(p_symbol.type.id()==ID_pointer)
-    {
-      // record as an output
-      codet output(ID_output);
-      output.operands().resize(2);
-      output.op0()=
-        address_of_exprt(
-          index_exprt(
-            string_constantt(p_symbol.base_name),
-            from_integer(0, index_type())));
-      output.op1()=main_arguments[param_number];
-      output.add_source_location()=function.location;
+    if(!can_cast_type<pointer_typet>(p_symbol.type))
+      continue;
 
-      init_code.move_to_operands(output);
-    }
+    init_code.add(code_outputt{
+      p_symbol.base_name, arguments[param_number], function.location});
   }
+  return init_code;
+}
+
+static codet record_exception(
+  const symbolt &function,
+  const symbol_table_baset &symbol_table)
+{
+  // retrieve the exception variable
+  const symbolt &exc_symbol =
+    symbol_table.lookup_ref(JAVA_ENTRY_POINT_EXCEPTION_SYMBOL);
 
   // record exceptional return variable as output
-  codet output(ID_output);
-  output.operands().resize(2);
-
-  // retrieve the exception variable
-  const symbolt exc_symbol=*symbol_table.lookup(
-    JAVA_ENTRY_POINT_EXCEPTION_SYMBOL);
-
-  output.op0()=address_of_exprt(
-    index_exprt(string_constantt(exc_symbol.base_name),
-                from_integer(0, index_type())));
-  output.op1()=exc_symbol.symbol_expr();
-  output.add_source_location()=function.location;
-
-  init_code.move_to_operands(output);
+  return code_outputt{
+    exc_symbol.base_name, exc_symbol.symbol_expr(), function.location};
 }
 
 main_function_resultt get_main_symbol(
@@ -430,16 +519,13 @@ main_function_resultt get_main_symbol(
   messaget message(message_handler);
 
   // find main symbol
-  if(config.main!="")
+  if(config.main.has_value())
   {
-    // Add java:: prefix
-    std::string main_identifier="java::"+config.main;
-
     std::string error_message;
-    irep_idt main_symbol_id=
-      resolve_friendly_method_name(config.main, symbol_table, error_message);
+    irep_idt main_symbol_id = resolve_friendly_method_name(
+      config.main.value(), symbol_table, error_message);
 
-    if(main_symbol_id==irep_idt())
+    if(main_symbol_id.empty())
     {
       message.error()
         << "main symbol resolution failed: " << error_message << messaget::eom;
@@ -455,9 +541,6 @@ main_function_resultt get_main_symbol(
   }
   else
   {
-    // no function given, we look for the main class
-    assert(config.main=="");
-
     // are we given a main class?
     if(main_class.empty())
     {
@@ -482,49 +565,16 @@ main_function_resultt get_main_symbol(
   }
 }
 
-/// Given the \p symbol_table and the \p main_class to test, this function
-/// generates a new function __CPROVER__start that calls the method under tests.
-///
-/// If __CPROVER__start is already in the `symbol_table`, it silently returns.
-/// Otherwise it finds the method under test using `get_main_symbol` and
-/// constructs a body for __CPROVER__start which does as follows:
-///
-/// 1. Allocates and initializes the parameters of the method under test.
-/// 2. Call it and save its return variable in the variable 'return'.
-/// 3. Declare variable 'return' as an output variable (codet with id
-///    ID_output), together with other objects possibly altered by the execution
-///    the method under test (in `java_record_outputs`)
-///
-/// When \p assume_init_pointers_not_null is false, the generated parameter
-/// initialization code will non-deterministically set input parameters to
-/// either null or a stack-allocated object. Observe that the null/non-null
-/// setting only applies to the parameter itself, and is not propagated to other
-/// pointers that it might be necessary to initialize in the object tree rooted
-/// at the parameter.
-/// Parameter \p max_nondet_array_length provides the maximum length for an
-/// array used as part of the input to the method under test, and
-/// \p max_nondet_tree_depth defines the maximum depth of the object tree
-/// created for such inputs. This maximum depth is used **in conjunction** with
-/// the so-called "recursive type set" (see field `recursive_set` in class
-/// java_object_factoryt) to bound the depth of the object tree for the
-/// parameter. Only when
-/// - the depth of the tree is >= max_nondet_tree_depth **AND**
-/// - the type of the object under initialization is already found in the
-///   recursive set
-/// then that object is not initalized and the reference pointing to it is
-/// (deterministically) set to null. This is a source of underapproximation in
-/// our approach to test generation, and should perhaps be fixed in the future.
-///
-/// \returns true if error occurred on entry point search
 bool java_entry_point(
   symbol_table_baset &symbol_table,
   const irep_idt &main_class,
   message_handlert &message_handler,
   bool assume_init_pointers_not_null,
   bool assert_uncaught_exceptions,
-  const object_factory_parameterst &object_factory_parameters,
+  const java_object_factory_parameterst &object_factory_parameters,
   const select_pointer_typet &pointer_type_selector,
-  bool string_refinement_enabled)
+  bool string_refinement_enabled,
+  const build_argumentst &build_arguments)
 {
   // check if the entry point is already there
   if(symbol_table.symbols.find(goto_functionst::entry_point())!=
@@ -540,48 +590,24 @@ bool java_entry_point(
 
   assert(symbol.type.id()==ID_code);
 
-  create_initialize(symbol_table);
-
-  java_static_lifetime_init(
-    symbol_table,
-    symbol.location,
-    assume_init_pointers_not_null,
-    object_factory_parameters,
-    pointer_type_selector,
-    string_refinement_enabled,
-    message_handler);
-
   return generate_java_start_function(
     symbol,
     symbol_table,
     message_handler,
-    assume_init_pointers_not_null,
     assert_uncaught_exceptions,
     object_factory_parameters,
-    pointer_type_selector);
+    pointer_type_selector,
+    build_arguments);
 }
 
-/// Generate a _start function for a specific function. See
-/// java_entry_point for more details.
-/// \param symbol: The symbol representing the function to call
-/// \param symbol_table: Global symbol table
-/// \param message_handler: Where to write output to
-/// \param assume_init_pointers_not_null: When creating pointers, assume they
-///   always take a non-null value.
-/// \param max_nondet_array_length: The length of the arrays to create when
-///   filling them
-/// \param max_nondet_tree_depth: defines the maximum depth of the object tree
-///   (see java_entry_points documentation for details)
-/// \param pointer_type_selector: Logic for substituting types of pointers
-/// \returns true if error occurred on entry point search, false otherwise
 bool generate_java_start_function(
   const symbolt &symbol,
   symbol_table_baset &symbol_table,
   message_handlert &message_handler,
-  bool assume_init_pointers_not_null,
   bool assert_uncaught_exceptions,
-  const object_factory_parameterst &object_factory_parameters,
-  const select_pointer_typet &pointer_type_selector)
+  const java_object_factory_parameterst &object_factory_parameters,
+  const select_pointer_typet &pointer_type_selector,
+  const build_argumentst &build_arguments)
 {
   messaget message(message_handler);
   code_blockt init_code;
@@ -598,12 +624,10 @@ bool generate_java_start_function(
       return true; // give up with error
     }
 
-    code_function_callt call_init;
-    call_init.lhs().make_nil();
+    code_function_callt call_init(init_it->second.symbol_expr());
     call_init.add_source_location()=symbol.location;
-    call_init.function()=init_it->second.symbol_expr();
 
-    init_code.move_to_operands(call_init);
+    init_code.add(std::move(call_init));
   }
 
   // build call to the main method, of the form
@@ -611,26 +635,24 @@ bool generate_java_start_function(
   // where return is a new variable
   // and arg1 ... argn are constructed below as well
 
-  code_function_callt call_main;
-
   source_locationt loc=symbol.location;
   loc.set_function(symbol.name);
   source_locationt &dloc=loc;
 
   // function to call
+  code_function_callt call_main(symbol.symbol_expr());
   call_main.add_source_location()=dloc;
-  call_main.function()=symbol.symbol_expr();
   call_main.function().add_source_location()=dloc;
 
   // if the method return type is not void, store return value in a new variable
   // named 'return'
-  if(to_java_method_type(symbol.type).return_type() != empty_typet())
+  if(to_java_method_type(symbol.type).return_type() != java_void_type())
   {
     auxiliary_symbolt return_symbol;
     return_symbol.mode=ID_java;
     return_symbol.is_static_lifetime=false;
     return_symbol.name=JAVA_ENTRY_POINT_RETURN_SYMBOL;
-    return_symbol.base_name="return";
+    return_symbol.base_name = "return'";
     return_symbol.type = to_java_method_type(symbol.type).return_type();
 
     symbol_table.add(return_symbol);
@@ -642,26 +664,20 @@ bool generate_java_start_function(
   exc_symbol.mode=ID_java;
   exc_symbol.name=JAVA_ENTRY_POINT_EXCEPTION_SYMBOL;
   exc_symbol.base_name=exc_symbol.name;
-  exc_symbol.type=java_reference_type(empty_typet());
+  exc_symbol.type = java_reference_type(java_void_type());
   symbol_table.add(exc_symbol);
 
   // Zero-initialise the top-level exception catch variable:
-  init_code.copy_to_operands(
-    code_assignt(
-      exc_symbol.symbol_expr(),
-      null_pointer_exprt(to_pointer_type(exc_symbol.type))));
+  init_code.add(code_assignt(
+    exc_symbol.symbol_expr(),
+    null_pointer_exprt(to_pointer_type(exc_symbol.type))));
 
   // create code that allocates the objects used as test arguments and
   // non-deterministically initializes them
-  exprt::operandst main_arguments=
-    java_build_arguments(
-      symbol,
-      init_code,
-      symbol_table,
-      assume_init_pointers_not_null,
-      object_factory_parameters,
-      pointer_type_selector);
-  call_main.arguments()=main_arguments;
+  const std::pair<code_blockt, std::vector<exprt>> main_arguments =
+    build_arguments(symbol, symbol_table);
+  init_code.append(main_arguments.first);
+  call_main.arguments() = main_arguments.second;
 
   // Create target labels for the toplevel exception handler:
   code_labelt toplevel_catch("toplevel_catch", code_skipt());
@@ -679,29 +695,30 @@ bool generate_java_start_function(
   irept catch_type_list(ID_exception_list);
   irept catch_target_list(ID_label);
 
-  call_block.move_to_operands(push_universal_handler);
+  call_block.add(std::move(push_universal_handler));
 
   // we insert the call to the method AFTER the argument initialization code
-  call_block.move_to_operands(call_main);
+  call_block.add(std::move(call_main));
 
   // Pop the handler:
   code_pop_catcht pop_handler;
-  call_block.move_to_operands(pop_handler);
-  init_code.move_to_operands(call_block);
+  call_block.add(std::move(pop_handler));
+  init_code.add(std::move(call_block));
 
   // Normal return: skip the exception handler:
-  init_code.copy_to_operands(code_gotot(after_catch.get_label()));
+  init_code.add(code_gotot(after_catch.get_label()));
 
   // Exceptional return: catch and assign to exc_symbol.
   code_landingpadt landingpad(exc_symbol.symbol_expr());
-  init_code.copy_to_operands(toplevel_catch);
-  init_code.move_to_operands(landingpad);
+  init_code.add(std::move(toplevel_catch));
+  init_code.add(std::move(landingpad));
 
   // Converge normal and exceptional return:
-  init_code.move_to_operands(after_catch);
+  init_code.add(std::move(after_catch));
 
-  // declare certain (which?) variables as test outputs
-  java_record_outputs(symbol, main_arguments, init_code, symbol_table);
+  // Mark return value, pointer type parameters and the exception as outputs.
+  init_code.append(
+    java_record_outputs(symbol, main_arguments.second, symbol_table));
 
   // add uncaught-exception check if requested
   if(assert_uncaught_exceptions)
@@ -715,7 +732,7 @@ bool generate_java_start_function(
   symbolt new_symbol;
 
   new_symbol.name=goto_functionst::entry_point();
-  new_symbol.type = java_method_typet({}, empty_typet());
+  new_symbol.type = java_method_typet({}, java_void_type());
   new_symbol.value.swap(init_code);
   new_symbol.mode=ID_java;
 
