@@ -21,6 +21,7 @@ Date:   March 2017
 #include <util/string_expr.h>
 
 #include <util/ieee_float.h> // should get rid of this
+#include <util/optional.h>
 
 #include <array>
 #include <unordered_set>
@@ -37,18 +38,17 @@ public:
   java_string_library_preprocesst()
     : char_type(java_char_type()),
       index_type(java_int_type()),
-      refined_string_type(index_type, char_type)
+      refined_string_type(index_type, pointer_type(char_type))
   {
   }
 
   void initialize_known_type_table();
   void initialize_conversion_table();
-  void initialize_refined_string_type();
 
   bool implements_function(const irep_idt &function_id) const;
   void get_all_function_names(std::unordered_set<irep_idt> &methods) const;
 
-  exprt
+  codet
   code_for_function(const symbolt &symbol, symbol_table_baset &symbol_table);
 
   codet replace_character_call(code_function_callt call)
@@ -145,49 +145,31 @@ private:
   // A set tells us what java types should be considered as string objects
   std::unordered_set<irep_idt> string_types;
 
-  codet make_equals_function_code(
+  code_blockt make_float_to_string_code(
     const java_method_typet &type,
     const source_locationt &loc,
     const irep_idt &function_id,
     symbol_table_baset &symbol_table);
 
-  codet make_float_to_string_code(
+  code_blockt make_copy_string_code(
     const java_method_typet &type,
     const source_locationt &loc,
     const irep_idt &function_id,
     symbol_table_baset &symbol_table);
 
-  codet make_string_to_char_array_code(
-    const java_method_typet &type,
-    const source_locationt &loc,
-    const irep_idt &function_id,
-    symbol_tablet &symbol_table);
-
-  codet make_string_format_code(
+  code_blockt make_copy_constructor_code(
     const java_method_typet &type,
     const source_locationt &loc,
     const irep_idt &function_id,
     symbol_table_baset &symbol_table);
 
-  codet make_copy_string_code(
+  code_returnt make_string_length_code(
     const java_method_typet &type,
     const source_locationt &loc,
     const irep_idt &function_id,
     symbol_table_baset &symbol_table);
 
-  codet make_copy_constructor_code(
-    const java_method_typet &type,
-    const source_locationt &loc,
-    const irep_idt &function_id,
-    symbol_table_baset &symbol_table);
-
-  codet make_string_length_code(
-    const java_method_typet &type,
-    const source_locationt &loc,
-    const irep_idt &function_id,
-    symbol_table_baset &symbol_table);
-
-  codet make_object_get_class_code(
+  code_blockt make_class_identifier_code(
     const java_method_typet &type,
     const source_locationt &loc,
     const irep_idt &function_id,
@@ -197,6 +179,7 @@ private:
   exprt::operandst process_parameters(
     const java_method_typet::parameterst &params,
     const source_locationt &loc,
+    const irep_idt &function_name,
     symbol_table_baset &symbol_table,
     code_blockt &init_code);
 
@@ -205,6 +188,7 @@ private:
     java_string_library_preprocesst &preprocess,
     const exprt &deref,
     const source_locationt &loc,
+    const irep_idt &function_id,
     symbol_tablet &symbol_table,
     code_blockt &init_code);
 
@@ -212,38 +196,32 @@ private:
     const exprt &deref,
     const source_locationt &loc,
     symbol_table_baset &symbol_table,
+    const irep_idt &function_name,
     code_blockt &init_code);
 
   exprt::operandst process_operands(
     const exprt::operandst &operands,
     const source_locationt &loc,
-    symbol_table_baset &symbol_table,
-    code_blockt &init_code);
-
-  exprt::operandst process_equals_function_operands(
-    const exprt::operandst &operands,
-    const source_locationt &loc,
+    const irep_idt &function_name,
     symbol_table_baset &symbol_table,
     code_blockt &init_code);
 
   refined_string_exprt replace_char_array(
     const exprt &array_pointer,
     const source_locationt &loc,
+    const irep_idt &function_name,
     symbol_table_baset &symbol_table,
     code_blockt &code);
 
   symbol_exprt fresh_string(
     const typet &type,
     const source_locationt &loc,
+    const irep_idt &function_id,
     symbol_table_baset &symbol_table);
 
-  symbol_exprt fresh_array(
-    const typet &type,
-    const source_locationt &loc,
-    symbol_tablet &symbol_table);
-
   refined_string_exprt decl_string_expr(
     const source_locationt &loc,
+    const irep_idt &function_id,
     symbol_table_baset &symbol_table,
     code_blockt &code);
 
@@ -260,21 +238,14 @@ private:
     symbol_table_baset &symbol_table,
     code_blockt &code);
 
-  exprt allocate_fresh_array(
-    const typet &type,
-    const source_locationt &loc,
-    const irep_idt &function_id,
-    symbol_tablet &symbol_table,
-    code_blockt &code);
-
   codet code_return_function_application(
-    const irep_idt &function_name,
+    const irep_idt &function_id,
     const exprt::operandst &arguments,
     const typet &type,
     symbol_table_baset &symbol_table);
 
   refined_string_exprt string_expr_of_function(
-    const irep_idt &function_name,
+    const irep_idt &function_id,
     const exprt::operandst &arguments,
     const source_locationt &loc,
     symbol_table_baset &symbol_table,
@@ -284,12 +255,14 @@ private:
     const exprt &lhs,
     const exprt &rhs_array,
     const exprt &rhs_length,
-    const symbol_table_baset &symbol_table);
+    const symbol_table_baset &symbol_table,
+    bool is_constructor);
 
   codet code_assign_string_expr_to_java_string(
     const exprt &lhs,
     const refined_string_exprt &rhs,
-    const symbol_table_baset &symbol_table);
+    const symbol_table_baset &symbol_table,
+    bool is_constructor);
 
   void code_assign_java_string_to_string_expr(
     const refined_string_exprt &lhs,
@@ -304,59 +277,35 @@ private:
     symbol_table_baset &symbol_table,
     code_blockt &code);
 
-  codet make_function_from_call(
-    const irep_idt &function_name,
-    const java_method_typet &type,
-    const source_locationt &loc,
-    symbol_table_baset &symbol_table);
-
-  codet make_init_function_from_call(
-    const irep_idt &function_name,
-    const java_method_typet &type,
-    const source_locationt &loc,
-    symbol_table_baset &symbol_table,
-    bool ignore_first_arg = true);
-
-  codet make_assign_and_return_function_from_call(
-    const irep_idt &function_name,
-    const java_method_typet &type,
-    const source_locationt &loc,
-    symbol_table_baset &symbol_table);
-
-  codet make_assign_function_from_call(
-    const irep_idt &function_name,
-    const java_method_typet &type,
-    const source_locationt &loc,
-    symbol_table_baset &symbol_table);
-
-  codet make_string_returning_function_from_call(
-    const irep_idt &function_name,
-    const java_method_typet &type,
-    const source_locationt &loc,
-    symbol_table_baset &symbol_table);
-
-  exprt make_argument_for_format(
-    const exprt &argv,
-    std::size_t index,
-    const struct_typet &structured_type,
-    const source_locationt &loc,
+  code_blockt make_function_from_call(
     const irep_idt &function_id,
-    symbol_table_baset &symbol_table,
-    code_blockt &code);
-
-  exprt get_primitive_value_of_object(
-    const exprt &object,
-    irep_idt type_name,
-    const source_locationt &loc,
-    symbol_table_baset &symbol_table,
-    code_blockt &code);
-
-  exprt get_object_at_index(const exprt &argv, std::size_t index);
-
-  codet make_init_from_array_code(
     const java_method_typet &type,
     const source_locationt &loc,
+    symbol_table_baset &symbol_table);
+
+  code_blockt make_init_function_from_call(
     const irep_idt &function_id,
+    const java_method_typet &type,
+    const source_locationt &loc,
+    symbol_table_baset &symbol_table,
+    bool is_constructor = true);
+
+  code_blockt make_assign_and_return_function_from_call(
+    const irep_idt &function_id,
+    const java_method_typet &type,
+    const source_locationt &loc,
+    symbol_table_baset &symbol_table);
+
+  code_blockt make_assign_function_from_call(
+    const irep_idt &function_id,
+    const java_method_typet &type,
+    const source_locationt &loc,
+    symbol_table_baset &symbol_table);
+
+  code_blockt make_string_returning_function_from_call(
+    const irep_idt &function_id,
+    const java_method_typet &type,
+    const source_locationt &loc,
     symbol_table_baset &symbol_table);
 };
 
@@ -371,6 +320,7 @@ void add_pointer_to_array_association(
   const exprt &array,
   symbol_table_baset &symbol_table,
   const source_locationt &loc,
+  const irep_idt &function_id,
   code_blockt &code);
 
 void add_array_to_length_association(
@@ -378,14 +328,16 @@ void add_array_to_length_association(
   const exprt &length,
   symbol_table_baset &symbol_table,
   const source_locationt &loc,
+  const irep_idt &function_id,
   code_blockt &code);
 
 void add_character_set_constraint(
   const exprt &pointer,
   const exprt &length,
-  const irep_idt &char_set,
+  const irep_idt &char_range,
   symbol_table_baset &symbol_table,
   const source_locationt &loc,
+  const irep_idt &function_id,
   code_blockt &code);
 
 #endif // CPROVER_JAVA_BYTECODE_JAVA_STRING_LIBRARY_PREPROCESS_H
