@@ -8,40 +8,20 @@ Author: Chris Smowton, chris.smowton@diffblue.com
 
 #include "java_string_literals.h"
 #include "java_root_class.h"
+#include "java_string_literal_expr.h"
 #include "java_types.h"
 #include "java_utils.h"
 
+#include <linking/static_lifetime_init.h>
 #include <util/arith_tools.h>
 #include <util/expr_initializer.h>
 #include <util/namespace.h>
+#include <util/string_utils.h>
 #include <util/unicode.h>
 
 #include <iomanip>
 #include <sstream>
 
-/// Replace non-alphanumeric characters with `_xx` escapes, where xx are hex
-/// digits. Underscores are replaced by `__`.
-/// \param to_escape: string to escape
-/// \return string with non-alphanumeric characters escaped
-static std::string escape_non_alnum(const std::string &to_escape)
-{
-  std::ostringstream escaped;
-  for(auto &ch : to_escape)
-  {
-    if(ch=='_')
-      escaped << "__";
-    else if(isalnum(ch))
-      escaped << ch;
-    else
-      escaped << '_'
-              << std::hex
-              << std::setfill('0')
-              << std::setw(2)
-              << (unsigned int)ch;
-  }
-  return escaped.str();
-}
-
 /// Convert UCS-2 or UTF-16 to an array expression.
 /// \par parameters: `in`: wide string to convert
 /// \return Returns a Java char array containing the same wchars.
@@ -49,28 +29,19 @@ static array_exprt utf16_to_array(const std::wstring &in)
 {
   const auto jchar=java_char_type();
   array_exprt ret(
-    array_typet(jchar, from_integer(in.length(), java_int_type())));
+    {}, array_typet(jchar, from_integer(in.length(), java_int_type())));
   for(const auto c : in)
     ret.copy_to_operands(from_integer(c, jchar));
   return ret;
 }
 
-/// Creates or gets an existing constant global symbol for a given string
-/// literal.
-/// \param string_expr: string literal expression to convert
-/// \param symbol_table: global symbol table. If not already present, constant
-///   global symbols will be added.
-/// \param string_refinement_enabled: if true, string refinement's string data
-///   structure will also be initialised and added to the symbol table.
-/// \return a symbol_expr corresponding to the new or existing literal symbol.
 symbol_exprt get_or_create_string_literal_symbol(
-  const exprt &string_expr,
+  const java_string_literal_exprt &string_expr,
   symbol_table_baset &symbol_table,
   bool string_refinement_enabled)
 {
-  PRECONDITION(string_expr.id() == ID_java_string_literal);
-  const irep_idt value = string_expr.get(ID_value);
-  const symbol_typet string_type("java::java.lang.String");
+  const irep_idt value = string_expr.value();
+  const struct_tag_typet string_type("java::java.lang.String");
 
   const std::string escaped_symbol_name = escape_non_alnum(id2string(value));
   const std::string escaped_symbol_name_with_prefix =
@@ -80,6 +51,13 @@ symbol_exprt get_or_create_string_literal_symbol(
   if(findit != symbol_table.symbols.end())
     return findit->second.symbol_expr();
 
+#ifndef CPROVER_INVARIANT_DO_NOT_CHECK
+  const auto initialize_function = symbol_table.lookup(INITIALIZE_FUNCTION);
+  INVARIANT(
+    !initialize_function || initialize_function->value.is_nil(),
+    "Cannot create more string literals after making " INITIALIZE_FUNCTION);
+#endif
+
   // Create a new symbol:
   symbolt new_symbol;
   new_symbol.name = escaped_symbol_name_with_prefix;
@@ -90,15 +68,16 @@ symbol_exprt get_or_create_string_literal_symbol(
   new_symbol.mode = ID_java;
   new_symbol.is_type = false;
   new_symbol.is_lvalue = true;
+  new_symbol.is_state_var = true;
   new_symbol.is_static_lifetime = true;
 
   namespacet ns(symbol_table);
 
   // Regardless of string refinement setting, at least initialize
   // the literal with @clsid = String
-  symbol_typet jlo_symbol("java::java.lang.Object");
+  struct_tag_typet jlo_symbol("java::java.lang.Object");
   const auto &jlo_struct = to_struct_type(ns.follow(jlo_symbol));
-  struct_exprt jlo_init(jlo_symbol);
+  struct_exprt jlo_init({}, jlo_symbol);
   const auto &jls_struct = to_struct_type(ns.follow(string_type));
   java_root_class_init(jlo_init, jlo_struct, "java::java.lang.String");
 
@@ -109,7 +88,7 @@ symbol_exprt get_or_create_string_literal_symbol(
     const array_exprt data =
       utf16_to_array(utf8_to_utf16_native_endian(id2string(value)));
 
-    struct_exprt literal_init(new_symbol.type);
+    struct_exprt literal_init({}, new_symbol.type);
     literal_init.operands().resize(jls_struct.components().size());
     const std::size_t jlo_nb = jls_struct.component_number("@java.lang.Object");
     literal_init.operands()[jlo_nb] = jlo_init;
@@ -173,8 +152,8 @@ symbol_exprt get_or_create_string_literal_symbol(
     // _return_value global for its side-effects.
     exprt init_comma_expr(ID_comma);
     init_comma_expr.type() = literal_init.type();
-    init_comma_expr.copy_to_operands(return_symbol.symbol_expr());
-    init_comma_expr.move_to_operands(literal_init);
+    init_comma_expr.add_to_operands(
+      return_symbol.symbol_expr(), std::move(literal_init));
     new_symbol.value = init_comma_expr;
   }
   else if(jls_struct.components().size()>=1 &&
@@ -183,8 +162,7 @@ symbol_exprt get_or_create_string_literal_symbol(
     // Case where something defined java.lang.String, so it has
     // a proper base class (always java.lang.Object in practical
     // JDKs seen so far)
-    struct_exprt literal_init(new_symbol.type);
-    literal_init.move_to_operands(jlo_init);
+    struct_exprt literal_init({std::move(jlo_init)}, new_symbol.type);
     for(const auto &comp : jls_struct.components())
     {
       if(comp.get_name()=="@java.lang.Object")
@@ -192,16 +170,19 @@ symbol_exprt get_or_create_string_literal_symbol(
       // Other members of JDK's java.lang.String we don't understand
       // without string-refinement. Just zero-init them; consider using
       // test-gen-like nondet object trees instead.
-      literal_init.copy_to_operands(
-        zero_initializer(comp.type(), string_expr.source_location(), ns));
+      const auto init =
+        zero_initializer(comp.type(), string_expr.source_location(), ns);
+      CHECK_RETURN(init.has_value());
+      literal_init.copy_to_operands(*init);
     }
     new_symbol.value = literal_init;
   }
-  else if(jls_struct.get_bool(ID_incomplete_class))
+  else if(to_java_class_type(jls_struct).get_is_stub())
   {
     // Case where java.lang.String was stubbed, and so directly defines
     // @class_identifier
     new_symbol.value = jlo_init;
+    new_symbol.value.type() = string_type;
   }
 
   bool add_failed = symbol_table.add(new_symbol);
@@ -212,3 +193,14 @@ symbol_exprt get_or_create_string_literal_symbol(
 
   return new_symbol.symbol_expr();
 }
+
+symbol_exprt get_or_create_string_literal_symbol(
+  const irep_idt &string_value,
+  symbol_table_baset &symbol_table,
+  bool string_refinement_enabled)
+{
+  return get_or_create_string_literal_symbol(
+    java_string_literal_exprt{string_value},
+    symbol_table,
+    string_refinement_enabled);
+}
