@@ -29,52 +29,61 @@ std::vector<typet> parse_list_types(
   const char opening_bracket,
   const char closing_bracket);
 
-typet java_int_type()
+signedbv_typet java_int_type()
 {
-  return signedbv_typet(32);
+  static const auto result = signedbv_typet(32);
+  return result;
 }
 
-typet java_void_type()
+empty_typet java_void_type()
 {
-  return void_typet();
+  static const auto result = empty_typet();
+  return result;
 }
 
-typet java_long_type()
+signedbv_typet java_long_type()
 {
-  return signedbv_typet(64);
+  static const auto result = signedbv_typet(64);
+  return result;
 }
 
-typet java_short_type()
+signedbv_typet java_short_type()
 {
-  return signedbv_typet(16);
+  static const auto result = signedbv_typet(16);
+  return result;
 }
 
-typet java_byte_type()
+signedbv_typet java_byte_type()
 {
-  return signedbv_typet(8);
+  static const auto result = signedbv_typet(8);
+  return result;
 }
 
-typet java_char_type()
+unsignedbv_typet java_char_type()
 {
-  return unsignedbv_typet(16);
+  static const auto result = unsignedbv_typet(16);
+  return result;
 }
 
-typet java_float_type()
+floatbv_typet java_float_type()
 {
-  return ieee_float_spect::single_precision().to_type();
+  static const auto result = ieee_float_spect::single_precision().to_type();
+  return result;
 }
 
-typet java_double_type()
+floatbv_typet java_double_type()
 {
-  return ieee_float_spect::double_precision().to_type();
+  static const auto result = ieee_float_spect::double_precision().to_type();
+  return result;
 }
 
-typet java_boolean_type()
+c_bool_typet java_boolean_type()
 {
   // The Java standard doesn't really prescribe the width
   // of a boolean. However, JNI suggests that it's 8 bits.
   // http://docs.oracle.com/javase/7/docs/technotes/guides/jni/spec/types.html
-  return c_bool_typet(8);
+  static const auto result = c_bool_typet(8);
+  return result;
 }
 
 reference_typet java_reference_type(const typet &subtype)
@@ -82,16 +91,23 @@ reference_typet java_reference_type(const typet &subtype)
   return reference_type(subtype);
 }
 
-reference_typet java_lang_object_type()
+java_reference_typet java_reference_type(const struct_tag_typet &subtype)
 {
-  return java_reference_type(symbol_typet("java::java.lang.Object"));
+  return to_java_reference_type(reference_type(subtype));
+}
+
+java_reference_typet java_lang_object_type()
+{
+  static const auto result =
+    java_reference_type(struct_tag_typet("java::java.lang.Object"));
+  return result;
 }
 
 /// Construct an array pointer type. It is a pointer to a symbol with identifier
 /// java::array[]. Its ID_element_type is set to the corresponding primitive
 /// type, or void* for arrays of references.
-/// \param subtype Character indicating the type of array
-reference_typet java_array_type(const char subtype)
+/// \param subtype: Character indicating the type of array
+java_reference_typet java_array_type(const char subtype)
 {
   std::string subtype_str;
 
@@ -117,16 +133,16 @@ reference_typet java_array_type(const char subtype)
 
   irep_idt class_name="array["+subtype_str+"]";
 
-  symbol_typet symbol_type("java::"+id2string(class_name));
-  symbol_type.set(ID_C_base_name, class_name);
-  symbol_type.set(ID_element_type, java_type_from_char(subtype));
+  struct_tag_typet struct_tag_type("java::" + id2string(class_name));
+  struct_tag_type.set(ID_C_base_name, class_name);
+  struct_tag_type.set(ID_element_type, java_type_from_char(subtype));
 
-  return java_reference_type(symbol_type);
+  return java_reference_type(struct_tag_type);
 }
 
 /// Return a const reference to the element type of a given java array type
-/// \param array_symbol The java array type
-const typet &java_array_element_type(const symbol_typet &array_symbol)
+/// \param array_symbol: The java array type
+const typet &java_array_element_type(const struct_tag_typet &array_symbol)
 {
   DATA_INVARIANT(
     is_java_array_tag(array_symbol.get_identifier()),
@@ -135,8 +151,8 @@ const typet &java_array_element_type(const symbol_typet &array_symbol)
 }
 
 /// Return a non-const reference to the element type of a given java array type
-/// \param array_symbol The java array type
-typet &java_array_element_type(symbol_typet &array_symbol)
+/// \param array_symbol: The java array type
+typet &java_array_element_type(struct_tag_typet &array_symbol)
 {
   DATA_INVARIANT(
     is_java_array_tag(array_symbol.get_identifier()),
@@ -149,28 +165,70 @@ bool is_java_array_type(const typet &type)
 {
   if(
     !can_cast_type<pointer_typet>(type) ||
-    !can_cast_type<symbol_typet>(type.subtype()))
+    !can_cast_type<struct_tag_typet>(type.subtype()))
   {
     return false;
   }
-  const auto &subtype_symbol = to_symbol_type(type.subtype());
-  return is_java_array_tag(subtype_symbol.get_identifier());
+  const auto &subtype_struct_tag = to_struct_tag_type(type.subtype());
+  return is_java_array_tag(subtype_struct_tag.get_identifier());
 }
 
 /// Checks whether the given type is a multi-dimensional array pointer type,
-// i.e., a pointer to an array type with element type also being a pointer to an
-/// array type.
+/// i.e., a pointer to an array type with element type also being a pointer to
+/// an array type.
 bool is_multidim_java_array_type(const typet &type)
 {
-  return is_java_array_type(type) &&
-         is_java_array_type(
-           java_array_element_type(to_symbol_type(type.subtype())));
+  return is_java_array_type(type) && is_java_array_type(java_array_element_type(
+                                       to_struct_tag_type(type.subtype())));
+}
+
+/// Returns the underlying element type and array dimensionality of Java struct
+/// \p type. If this is not an array type the return value will be `{type, 0}`.
+std::pair<typet, std::size_t>
+java_array_dimension_and_element_type(const struct_tag_typet &type)
+{
+  std::size_t array_dimensions = 0;
+  typet underlying_type;
+  for(underlying_type = java_reference_type(type);
+      is_java_array_type(underlying_type);
+      underlying_type =
+        java_array_element_type(to_struct_tag_type(underlying_type.subtype())))
+  {
+    ++array_dimensions;
+  }
+
+  return {underlying_type, array_dimensions};
+}
+
+/// \param pointer: pointer to an array[reference] object
+/// \return member expression to access its array dimension field
+exprt get_array_dimension_field(const exprt &pointer)
+{
+  PRECONDITION(pointer.type().id() == ID_pointer);
+
+  struct_tag_typet java_reference_array_type(JAVA_REFERENCE_ARRAY_CLASSID);
+  exprt deref = dereference_exprt(typecast_exprt::conditional_cast(
+    pointer, java_reference_type(java_reference_array_type)));
+  return member_exprt(deref, JAVA_ARRAY_DIMENSION_FIELD_NAME, java_int_type());
+}
+
+/// \param pointer: pointer to an array[reference] object
+/// \return member expression to access its array element_type field
+exprt get_array_element_type_field(const exprt &pointer)
+{
+  PRECONDITION(pointer.type().id() == ID_pointer);
+
+  struct_tag_typet java_reference_array_type(JAVA_REFERENCE_ARRAY_CLASSID);
+  exprt deref = dereference_exprt(typecast_exprt::conditional_cast(
+    pointer, java_reference_type(java_reference_array_type)));
+  return member_exprt(
+    deref, JAVA_ARRAY_ELEMENT_CLASSID_FIELD_NAME, string_typet());
 }
 
 /// See above
-/// \param tag Tag of a struct
+/// \param tag: Tag of a struct
 /// \return True if the given string is a Java array tag, i.e., has a prefix
-/// of java::array[
+///   of java::array[
 bool is_java_array_tag(const irep_idt& tag)
 {
   return has_prefix(id2string(tag), "java::array[");
@@ -199,8 +257,10 @@ typet java_type_from_char(char t)
   case 'f': return java_float_type();
   case 'd': return java_double_type();
   case 'z': return java_boolean_type();
-  case 'a': return java_reference_type(void_typet());
-  default: UNREACHABLE; return nil_typet();
+  case 'a':
+    return java_reference_type(java_void_type());
+  default:
+    UNREACHABLE;
   }
 }
 
@@ -228,7 +288,7 @@ exprt java_bytecode_promotion(const exprt &expr)
 }
 
 /// Take a list of generic type arguments and parse them into the generic type.
-/// \param generic_type [out]: The existing generic type to add the information
+/// \param [out] generic_type: The existing generic type to add the information
 ///   to
 /// \param type_arguments: The string representing the generic type arguments
 ///   for a signature. For example `<TT;Ljava/lang/Foo;LList<LInteger;>;>`
@@ -333,6 +393,32 @@ std::vector<typet> parse_list_types(
   const std::string class_name_prefix,
   const char opening_bracket,
   const char closing_bracket)
+{
+  // Loop over the types in the given string, parsing each one in turn
+  // and adding to the type_list
+  std::vector<typet> type_list;
+  for(const std::string &raw_type :
+      parse_raw_list_types(src, opening_bracket, closing_bracket))
+  {
+    auto new_type = java_type_from_string(raw_type, class_name_prefix);
+    INVARIANT(new_type.has_value(), "Failed to parse type");
+    type_list.push_back(std::move(*new_type));
+  }
+  return type_list;
+}
+
+/// Given a substring of a descriptor or signature that contains one or more
+/// types parse out the individual type strings.
+/// \param src: The input string that is wrapped in either ( ) or < >
+/// \param opening_bracket: For checking string is passed in as expected, the
+///   opening bracket (i.e. '(' or '<').
+/// \param closing_bracket: For checking string is passed in as expected, the
+///   closing bracket (i.e. ')' or '>').
+/// \return A vector of strings that represent the bytecode type.
+std::vector<std::string> parse_raw_list_types(
+  const std::string src,
+  const char opening_bracket,
+  const char closing_bracket)
 {
   PRECONDITION(src.size() >= 2);
   PRECONDITION(src[0] == opening_bracket);
@@ -340,7 +426,7 @@ std::vector<typet> parse_list_types(
 
   // Loop over the types in the given string, parsing each one in turn
   // and adding to the type_list
-  std::vector<typet> type_list;
+  std::vector<std::string> type_list;
   for(std::size_t i = 1; i < src.size() - 1; i++)
   {
     size_t start = i;
@@ -369,10 +455,7 @@ std::vector<typet> parse_list_types(
     }
 
     std::string sub_str = src.substr(start, i - start + 1);
-    const typet &new_type = java_type_from_string(sub_str, class_name_prefix);
-    INVARIANT(new_type != nil_typet(), "Failed to parse type");
-
-    type_list.push_back(new_type);
+    type_list.emplace_back(sub_str);
   }
   return type_list;
 }
@@ -394,13 +477,13 @@ build_class_name(const std::string &src, const std::string &class_name_prefix)
 
   std::string container_class = gather_full_class_name(src);
   std::string identifier = "java::" + container_class;
-  symbol_typet symbol_type(identifier);
-  symbol_type.set(ID_C_base_name, container_class);
+  struct_tag_typet struct_tag_type(identifier);
+  struct_tag_type.set(ID_C_base_name, container_class);
 
   std::size_t f_pos = src.find('<', 1);
   if(f_pos != std::string::npos)
   {
-    java_generic_typet result(symbol_type);
+    java_generic_typet result(struct_tag_type);
     // get generic type information
     do
     {
@@ -415,10 +498,10 @@ build_class_name(const std::string &src, const std::string &class_name_prefix)
       // Look for the next generic type info (if it exists)
       f_pos = src.find('<', e_pos + 1);
     } while(f_pos != std::string::npos);
-    return result;
+    return std::move(result);
   }
 
-  return java_reference_type(symbol_type);
+  return java_reference_type(struct_tag_type);
 }
 
 /// Finds the closing semi-colon ending a ClassTypeSignature that starts at
@@ -427,8 +510,8 @@ build_class_name(const std::string &src, const std::string &class_name_prefix)
 /// \param starting_point: The string position where the opening 'L' we want to
 ///   find the closing ';' for.
 /// \return The string position corresponding to the matching ';'. For example:
-/// LA;, we would return 2. For LA<TT;>; we would return 7.
-/// See unit/java_bytecode/java_util_tests.cpp for more examples.
+///   LA;, we would return 2. For LA<TT;>; we would return 7.
+///   See unit/java_bytecode/java_util_tests.cpp for more examples.
 size_t find_closing_semi_colon_for_reference_type(
   const std::string src,
   size_t starting_point)
@@ -454,6 +537,13 @@ size_t find_closing_semi_colon_for_reference_type(
   return next_semi_colon;
 }
 
+java_reference_typet java_reference_array_type(const struct_tag_typet &subtype)
+{
+  java_reference_typet result = java_array_type('a');
+  result.subtype().set(ID_element_type, java_reference_type(subtype));
+  return result;
+}
+
 /// Transforms a string representation of a Java type into an internal type
 /// representation thereof.
 ///
@@ -466,13 +556,13 @@ size_t find_closing_semi_colon_for_reference_type(
 /// \param src: the string representation as used in the class file
 /// \param class_name_prefix: name of class to append to generic type
 ///   variables/parameters
-/// \returns internal type representation for GOTO programs
-typet java_type_from_string(
+/// \return internal type representation for GOTO programs
+optionalt<typet> java_type_from_string(
   const std::string &src,
   const std::string &class_name_prefix)
 {
   if(src.empty())
-    return nil_typet();
+    return {};
 
   // a java type is encoded in different ways
   //  - a method type is encoded as '(...)R' where the parenthesis include the
@@ -516,15 +606,15 @@ typet java_type_from_string(
           "Cannot currently parse bounds on generic types");
       }
 
-      const typet &method_type=java_type_from_string(
-        src.substr(closing_generic+1, std::string::npos), class_name_prefix);
+      auto method_type = java_type_from_string(
+        src.substr(closing_generic + 1, std::string::npos), class_name_prefix);
 
       // This invariant being violated means that tkiley has not understood
       // part of the signature spec.
       // Only class and method signatures can start with a '<' and classes are
       // handled separately.
       INVARIANT(
-        method_type.id()==ID_code,
+        method_type.has_value() && method_type->id() == ID_code,
         "This should correspond to method signatures only");
 
       return method_type;
@@ -533,9 +623,9 @@ typet java_type_from_string(
     {
       std::size_t e_pos=src.rfind(')');
       if(e_pos==std::string::npos)
-        return nil_typet();
+        return {};
 
-      typet return_type = java_type_from_string(
+      auto return_type = java_type_from_string(
         std::string(src, e_pos + 1, std::string::npos), class_name_prefix);
 
       std::vector<typet> param_types =
@@ -549,7 +639,7 @@ typet java_type_from_string(
         std::back_inserter(parameters),
         [](const typet &type) { return java_method_typet::parametert(type); });
 
-      return java_method_typet(std::move(parameters), std::move(return_type));
+      return java_method_typet(std::move(parameters), std::move(*return_type));
     }
 
   case '[': // array type
@@ -557,18 +647,22 @@ typet java_type_from_string(
       // If this is a reference array, we generate a plain array[reference]
       // with void* members, but note the real type in ID_element_type.
       if(src.size()<=1)
-        return nil_typet();
+        return {};
       char subtype_letter=src[1];
-      const typet subtype=
-        java_type_from_string(src.substr(1, std::string::npos),
-                              class_name_prefix);
+      auto subtype = java_type_from_string(
+        src.substr(1, std::string::npos), class_name_prefix);
       if(subtype_letter=='L' || // [L denotes a reference array of some sort.
          subtype_letter=='[' || // Array-of-arrays
          subtype_letter=='T')   // Array of generic types
         subtype_letter='A';
-      typet tmp=java_array_type(std::tolower(subtype_letter));
-      tmp.subtype().set(ID_element_type, subtype);
-      return tmp;
+      subtype_letter = std::tolower(subtype_letter);
+      if(subtype_letter == 'a')
+      {
+        return java_reference_array_type(
+          to_struct_tag_type(subtype->subtype()));
+      }
+      else
+        return java_array_type(subtype_letter);
     }
 
   case 'B': return java_byte_type();
@@ -588,7 +682,8 @@ typet java_type_from_string(
     irep_idt type_var_name(class_name_prefix+"::"+src.substr(1, src.size()-2));
     return java_generic_parametert(
       type_var_name,
-      to_symbol_type(java_type_from_string("Ljava/lang/Object;").subtype()));
+      to_struct_tag_type(
+        java_type_from_string("Ljava/lang/Object;")->subtype()));
   }
   case 'L':
     {
@@ -604,7 +699,7 @@ typet java_type_from_string(
     throw unsupported_java_class_signature_exceptiont("wild card generic");
   }
   default:
-    return nil_typet();
+    return {};
   }
 }
 
@@ -615,13 +710,13 @@ char java_char_from_type(const typet &type)
   if(id==ID_signedbv)
   {
     const size_t width=to_signedbv_type(type).get_width();
-    if(to_signedbv_type(java_int_type()).get_width()==width)
+    if(java_int_type().get_width() == width)
       return 'i';
-    else if(to_signedbv_type(java_long_type()).get_width()==width)
+    else if(java_long_type().get_width() == width)
       return 'l';
-    else if(to_signedbv_type(java_short_type()).get_width()==width)
+    else if(java_short_type().get_width() == width)
       return 's';
-    else if(to_signedbv_type(java_byte_type()).get_width()==width)
+    else if(java_byte_type().get_width() == width)
       return 'b';
   }
   else if(id==ID_unsignedbv)
@@ -629,9 +724,9 @@ char java_char_from_type(const typet &type)
   else if(id==ID_floatbv)
   {
     const size_t width(to_floatbv_type(type).get_width());
-    if(to_floatbv_type(java_float_type()).get_width()==width)
+    if(java_float_type().get_width() == width)
       return 'f';
-    else if(to_floatbv_type(java_double_type()).get_width()==width)
+    else if(java_double_type().get_width() == width)
       return 'd';
   }
   else if(id==ID_c_bool)
@@ -648,7 +743,7 @@ char java_char_from_type(const typet &type)
 ///
 /// For example for `HashMap<K, V>` a vector with two elements would be returned
 ///
-/// \returns vector of java types representing the generic type variables
+/// \return vector of java types representing the generic type variables
 std::vector<typet> java_generic_type_from_string(
   const std::string &class_name,
   const std::string &src)
@@ -688,7 +783,8 @@ std::vector<typet> java_generic_type_from_string(
 
     java_generic_parametert type_var_type(
       type_var_name,
-      to_symbol_type(java_type_from_string(bound_type, class_name).subtype()));
+      to_struct_tag_type(
+        java_type_from_string(bound_type, class_name)->subtype()));
 
     types.push_back(type_var_type);
     signature=signature.substr(var_sep+1, std::string::npos);
@@ -707,19 +803,19 @@ static std::string slash_to_dot(const std::string &src)
   return result;
 }
 
-symbol_typet java_classname(const std::string &id)
+struct_tag_typet java_classname(const std::string &id)
 {
   if(!id.empty() && id[0]=='[')
-    return to_symbol_type(java_type_from_string(id).subtype());
+    return to_struct_tag_type(java_type_from_string(id)->subtype());
 
   std::string class_name=id;
 
   class_name=slash_to_dot(class_name);
   irep_idt identifier="java::"+class_name;
-  symbol_typet symbol_type(identifier);
-  symbol_type.set(ID_C_base_name, class_name);
+  struct_tag_typet struct_tag_type(identifier);
+  struct_tag_type.set(ID_C_base_name, class_name);
 
-  return symbol_type;
+  return struct_tag_type;
 }
 
 /// Programmatic documentation of the structure of a Java array (of either
@@ -737,7 +833,9 @@ symbol_typet java_classname(const std::string &id)
 /// \return True if it is a Java array type, false otherwise
 bool is_valid_java_array(const struct_typet &type)
 {
-  bool correct_num_components=type.components().size()==3;
+  bool correct_num_components =
+    type.components().size() ==
+    (type.get_tag() == JAVA_REFERENCE_ARRAY_CLASSID ? 5 : 3);
   if(!correct_num_components)
     return false;
 
@@ -745,42 +843,61 @@ bool is_valid_java_array(const struct_typet &type)
   const struct_union_typet::componentt base_class_component=
     type.components()[0];
 
-  bool base_component_valid=true;
-  base_component_valid&=base_class_component.get_name()=="@java.lang.Object";
+  if(base_class_component.get_name() != "@java.lang.Object")
+    return false;
 
-  bool length_component_valid=true;
   const struct_union_typet::componentt length_component=
     type.components()[1];
-  length_component_valid&=length_component.get_name()=="length";
-  length_component_valid&=length_component.type()==java_int_type();
+  if(length_component.get_name() != "length")
+    return false;
+  if(length_component.type() != java_int_type())
+    return false;
 
-  bool data_component_valid=true;
   const struct_union_typet::componentt data_component=
     type.components()[2];
-  data_component_valid&=data_component.get_name()=="data";
-  data_component_valid&=data_component.type().id()==ID_pointer;
+  if(data_component.get_name() != "data")
+    return false;
+  if(data_component.type().id() != ID_pointer)
+    return false;
 
-  return correct_num_components &&
-    base_component_valid &&
-    length_component_valid &&
-    data_component_valid;
+  if(type.get_tag() == JAVA_REFERENCE_ARRAY_CLASSID)
+  {
+    const struct_union_typet::componentt array_element_type_component =
+      type.components()[3];
+    if(
+      array_element_type_component.get_name() !=
+      JAVA_ARRAY_ELEMENT_CLASSID_FIELD_NAME)
+      return false;
+    if(array_element_type_component.type() != string_typet())
+      return false;
+
+    const struct_union_typet::componentt array_dimension_component =
+      type.components()[4];
+    if(array_dimension_component.get_name() != JAVA_ARRAY_DIMENSION_FIELD_NAME)
+      return false;
+    if(array_dimension_component.type() != java_int_type())
+      return false;
+  }
+
+  return true;
 }
 
 /// Compares the types, including checking element types if both types are
 /// arrays.
-/// \param type1 First type to compare
-/// \param type2 Second type to compare
+/// \param type1: First type to compare
+/// \param type2: Second type to compare
 /// \return True if the types are equal, including elemnt types if they are
-/// both arrays
+///   both arrays
 bool equal_java_types(const typet &type1, const typet &type2)
 {
   bool arrays_with_same_element_type = true;
   if(
     type1.id() == ID_pointer && type2.id() == ID_pointer &&
-    type1.subtype().id() == ID_symbol && type2.subtype().id() == ID_symbol)
+    type1.subtype().id() == ID_struct_tag &&
+    type2.subtype().id() == ID_struct_tag)
   {
-    const symbol_typet &subtype_symbol1 = to_symbol_type(type1.subtype());
-    const symbol_typet &subtype_symbol2 = to_symbol_type(type2.subtype());
+    const auto &subtype_symbol1 = to_struct_tag_type(type1.subtype());
+    const auto &subtype_symbol2 = to_struct_tag_type(type2.subtype());
     if(
       subtype_symbol1.get_identifier() == subtype_symbol2.get_identifier() &&
       is_java_array_tag(subtype_symbol1.get_identifier()))
@@ -797,6 +914,31 @@ bool equal_java_types(const typet &type1, const typet &type2)
   return (type1 == type2 && arrays_with_same_element_type);
 }
 
+std::vector<java_generic_parametert>
+get_all_generic_parameters(const typet &type)
+{
+  std::vector<java_generic_parametert> generic_parameters;
+  if(is_java_implicitly_generic_class_type(type))
+  {
+    const java_implicitly_generic_class_typet &implicitly_generic_class =
+      to_java_implicitly_generic_class_type(to_java_class_type(type));
+    generic_parameters.insert(
+      generic_parameters.end(),
+      implicitly_generic_class.implicit_generic_types().begin(),
+      implicitly_generic_class.implicit_generic_types().end());
+  }
+  if(is_java_generic_class_type(type))
+  {
+    const java_generic_class_typet &generic_class =
+      to_java_generic_class_type(to_java_class_type(type));
+    generic_parameters.insert(
+      generic_parameters.end(),
+      generic_class.generic_types().begin(),
+      generic_class.generic_types().end());
+  }
+  return generic_parameters;
+}
+
 void get_dependencies_from_generic_parameters_rec(
   const typet &t,
   std::set<irep_idt> &refs)
@@ -823,15 +965,15 @@ void get_dependencies_from_generic_parameters_rec(
       get_dependencies_from_generic_parameters_rec(param.type(), refs);
   }
 
-  // symbol type
-  else if(t.id() == ID_symbol)
+  // struct tag
+  else if(t.id() == ID_struct_tag)
   {
-    const symbol_typet &symbol_type = to_symbol_type(t);
-    const irep_idt class_name(symbol_type.get_identifier());
+    const auto &struct_tag_type = to_struct_tag_type(t);
+    const irep_idt class_name(struct_tag_type.get_identifier());
     if(is_java_array_tag(class_name))
     {
       get_dependencies_from_generic_parameters(
-        java_array_element_type(symbol_type), refs);
+        java_array_element_type(struct_tag_type), refs);
     }
     else
       refs.insert(strip_java_namespace_prefix(class_name));
@@ -843,7 +985,7 @@ void get_dependencies_from_generic_parameters_rec(
 /// must be loaded but only appear as generic type argument, not as a field
 /// reference.
 /// \param signature: the string representation of the signature to analyze
-/// \param refs [out]: the set to insert the names of the found dependencies
+/// \param [out] refs: the set to insert the names of the found dependencies
 void get_dependencies_from_generic_parameters(
   const std::string &signature,
   std::set<irep_idt> &refs)
@@ -863,9 +1005,9 @@ void get_dependencies_from_generic_parameters(
     // class signature without bounds and without wildcards
     else if(signature.find('*') == std::string::npos)
     {
-      get_dependencies_from_generic_parameters_rec(
-        java_type_from_string(signature, erase_type_arguments(signature)),
-        refs);
+      auto type_from_string =
+        java_type_from_string(signature, erase_type_arguments(signature));
+      get_dependencies_from_generic_parameters_rec(*type_from_string, refs);
     }
   }
   catch(unsupported_java_class_signature_exceptiont &)
@@ -879,7 +1021,7 @@ void get_dependencies_from_generic_parameters(
 /// used to get information about class dependencies that must be loaded but
 /// only appear as generic type argument, not as a field reference.
 /// \param t: the type to analyze
-/// \param refs [out]: the set to insert the names of the found dependencies
+/// \param [out] refs: the set to insert the names of the found dependencies
 void get_dependencies_from_generic_parameters(
   const typet &t,
   std::set<irep_idt> &refs)
@@ -892,25 +1034,27 @@ void get_dependencies_from_generic_parameters(
 /// This assumes that the class named \p class_name_prefix extends or implements
 /// the class \p type, and that \p base_ref corresponds to a generic class.
 /// For instance since HashMap<K,V> extends Map<K,V> we would call
-/// `java_generic_symbol_typet(symbol_typet("Map"), "Ljava/util/Map<TK;TV;>;",
-/// "java.util.HashMap")` which generates a symbol type with identifier "Map",
-/// and two generic types with identifier "java.util.HashMap::K" and
-/// "java.util.HashMap::V" respectively.
-java_generic_symbol_typet::java_generic_symbol_typet(
-  const symbol_typet &type,
+/// `java_generic_struct_tag_typet(struct_tag_typet("Map"),
+/// "Ljava/util/Map<TK;TV;>;", "java.util.HashMap")` which generates a symbol
+/// type with identifier "Map", and two generic types with identifier
+/// "java.util.HashMap::K" and "java.util.HashMap::V" respectively.
+java_generic_struct_tag_typet::java_generic_struct_tag_typet(
+  const struct_tag_typet &type,
   const std::string &base_ref,
   const std::string &class_name_prefix)
-  : symbol_typet(type)
+  : struct_tag_typet(type)
 {
   set(ID_C_java_generic_symbol, true);
-  const typet &base_type = java_type_from_string(base_ref, class_name_prefix);
-  PRECONDITION(is_java_generic_type(base_type));
-  const java_generic_typet &gen_base_type = to_java_generic_type(base_type);
+  const auto base_type = java_type_from_string(base_ref, class_name_prefix);
+  PRECONDITION(is_java_generic_type(*base_type));
+  const java_generic_typet &gen_base_type = to_java_generic_type(*base_type);
   INVARIANT(
-    type.get_identifier() == to_symbol_type(gen_base_type.subtype()).get_identifier(),
-    "identifier of "+type.pretty()+"\n and identifier of type "+
-    gen_base_type.subtype().pretty()+"\ncreated by java_type_from_string for "+
-    base_ref+" should be equal");
+    type.get_identifier() ==
+      to_struct_tag_type(gen_base_type.subtype()).get_identifier(),
+    "identifier of " + type.pretty() + "\n and identifier of type " +
+      gen_base_type.subtype().pretty() +
+      "\ncreated by java_type_from_string for " + base_ref +
+      " should be equal");
   generic_types().insert(
     generic_types().end(),
     gen_base_type.generic_type_arguments().begin(),
@@ -919,18 +1063,17 @@ java_generic_symbol_typet::java_generic_symbol_typet(
 
 /// Check if this symbol has the given generic type. If yes, return its index
 /// in the vector of generic types.
-/// \param type The parameter type we are looking for.
+/// \param type: The parameter type we are looking for.
 /// \return The index of the type in the vector of generic types.
-optionalt<size_t> java_generic_symbol_typet::generic_type_index(
+optionalt<size_t> java_generic_struct_tag_typet::generic_type_index(
   const java_generic_parametert &type) const
 {
   const auto &type_variable = type.get_name();
   const auto &generics = generic_types();
   for(std::size_t i = 0; i < generics.size(); ++i)
   {
-    if(
-      is_java_generic_parameter(generics[i]) &&
-      to_java_generic_parameter(generics[i]).get_name() == type_variable)
+    auto param = type_try_dynamic_cast<java_generic_parametert>(generics[i]);
+    if(param && param->get_name() == type_variable)
       return i;
   }
   return {};
@@ -938,6 +1081,8 @@ optionalt<size_t> java_generic_symbol_typet::generic_type_index(
 
 std::string pretty_java_type(const typet &type)
 {
+  if(type == java_void_type())
+    return "void";
   if(type == java_int_type())
     return "int";
   else if(type == java_long_type())
@@ -958,12 +1103,13 @@ std::string pretty_java_type(const typet &type)
     return "byte";
   else if(is_reference(type))
   {
-    if(type.subtype().id() == ID_symbol)
+    if(type.subtype().id() == ID_struct_tag)
     {
-      const auto &symbol_type = to_symbol_type(type.subtype());
-      const irep_idt &id = symbol_type.get_identifier();
+      const auto &struct_tag_type = to_struct_tag_type(type.subtype());
+      const irep_idt &id = struct_tag_type.get_identifier();
       if(is_java_array_tag(id))
-        return pretty_java_type(java_array_element_type(symbol_type)) + "[]";
+        return pretty_java_type(java_array_element_type(struct_tag_type)) +
+               "[]";
       else
         return id2string(strip_java_namespace_prefix(id));
     }
