@@ -30,6 +30,11 @@ Date:   December 2016
 
 #include <analyses/uncaught_exceptions_analysis.h>
 
+#include <linking/static_lifetime_init.h>
+
+#include "java_expr.h"
+#include "java_types.h"
+
 /// Lowers high-level exception descriptions into low-level operations suitable
 /// for symex and other analyses that don't understand the THROW or CATCH GOTO
 /// instructions.
@@ -46,7 +51,7 @@ Date:   December 2016
 /// (in instruction->code.exception_list()) and a corresponding GOTO program
 /// target for each (in instruction->targets).
 /// Thrown instructions are currently always matched to tags using
-/// java_instanceof, optionally lowered to a check on the @class_identifier
+/// java_instanceof, optionally lowered to a check on the `@class_identifier`
 /// field, so a language frontend wanting to use this class must use
 /// exceptions with a Java-compatible structure.
 ///
@@ -87,25 +92,43 @@ public:
 
   explicit remove_exceptionst(
     symbol_table_baset &_symbol_table,
+    const class_hierarchyt *_class_hierarchy,
     function_may_throwt _function_may_throw,
-    bool remove_added_instanceof,
-    message_handlert &message_handler)
+    bool _remove_added_instanceof,
+    message_handlert &_message_handler)
     : symbol_table(_symbol_table),
+      class_hierarchy(_class_hierarchy),
       function_may_throw(_function_may_throw),
-      remove_added_instanceof(remove_added_instanceof),
-      message_handler(message_handler)
+      remove_added_instanceof(_remove_added_instanceof),
+      message_handler(_message_handler)
   {
+    if(remove_added_instanceof)
+    {
+      INVARIANT(
+        class_hierarchy != nullptr,
+        "remove_exceptions needs a class hierarchy to remove instanceof "
+        "statements (either supply one, or don't use REMOVE_ADDED_INSTANCEOF)");
+    }
   }
 
   void operator()(goto_functionst &goto_functions);
-  void operator()(goto_programt &goto_program);
+  void
+  operator()(const irep_idt &function_identifier, goto_programt &goto_program);
 
 protected:
   symbol_table_baset &symbol_table;
+  const class_hierarchyt *class_hierarchy;
   function_may_throwt function_may_throw;
   bool remove_added_instanceof;
   message_handlert &message_handler;
 
+  enum class instrumentation_resultt
+  {
+    DID_NOTHING,
+    ADDED_CODE_WITHOUT_MAY_THROW,
+    ADDED_CODE_WITH_MAY_THROW,
+  };
+
   symbol_exprt get_inflight_exception_global();
 
   bool function_or_callees_may_throw(const goto_programt &) const;
@@ -122,24 +145,28 @@ protected:
     std::size_t &universal_catch);
 
   void add_exception_dispatch_sequence(
+    const irep_idt &function_identifier,
     goto_programt &goto_program,
     const goto_programt::targett &instr_it,
     const stack_catcht &stack_catch,
-    const std::vector<exprt> &locals);
+    const std::vector<symbol_exprt> &locals);
 
   bool instrument_throw(
+    const irep_idt &function_identifier,
     goto_programt &goto_program,
     const goto_programt::targett &,
     const stack_catcht &,
-    const std::vector<exprt> &);
+    const std::vector<symbol_exprt> &);
 
-  bool instrument_function_call(
+  instrumentation_resultt instrument_function_call(
+    const irep_idt &function_identifier,
     goto_programt &goto_program,
     const goto_programt::targett &,
     const stack_catcht &,
-    const std::vector<exprt> &);
+    const std::vector<symbol_exprt> &);
 
   void instrument_exceptions(
+    const irep_idt &function_identifier,
     goto_programt &goto_program);
 };
 
@@ -190,7 +217,7 @@ bool remove_exceptionst::function_or_callees_may_throw(
 /// Translates an exception landing-pad into instructions that copy the
 /// in-flight exception pointer to a nominated expression, then clear the
 /// in-flight exception (i.e. null the pointer), hence marking it caught.
-/// \param goto_program: body of the function containing this landingpad
+/// \param [out] goto_program: body of the function containing this landingpad
 ///   instruction
 /// \param instr_it: iterator pointing to the landingpad instruction.
 ///   Will be overwritten.
@@ -213,27 +240,26 @@ void remove_exceptionst::instrument_exception_handler(
     const symbol_exprt thrown_global_symbol=
       get_inflight_exception_global();
     // next we reset the exceptional return to NULL
-    null_pointer_exprt null_voidptr((pointer_type(empty_typet())));
+    null_pointer_exprt null_voidptr((pointer_type(java_void_type())));
 
     // add the assignment @inflight_exception = NULL
-    goto_programt::targett t_null=goto_program.insert_after(instr_it);
-    t_null->make_assignment();
-    t_null->source_location=instr_it->source_location;
-    t_null->code=code_assignt(
-      thrown_global_symbol,
-      null_voidptr);
-    t_null->function=instr_it->function;
+    goto_program.insert_after(
+      instr_it,
+      goto_programt::make_assignment(
+        code_assignt(thrown_global_symbol, null_voidptr),
+        instr_it->source_location));
 
     // add the assignment exc = @inflight_exception (before the null assignment)
-    goto_programt::targett t_exc=goto_program.insert_after(instr_it);
-    t_exc->make_assignment();
-    t_exc->source_location=instr_it->source_location;
-    t_exc->code=code_assignt(
-      thrown_exception_local,
-      typecast_exprt(thrown_global_symbol, thrown_exception_local.type()));
-    t_exc->function=instr_it->function;
+    goto_program.insert_after(
+      instr_it,
+      goto_programt::make_assignment(
+        code_assignt(
+          thrown_exception_local,
+          typecast_exprt(thrown_global_symbol, thrown_exception_local.type())),
+        instr_it->source_location));
   }
-  instr_it->make_skip();
+
+  instr_it->turn_into_skip();
 }
 
 /// Find the innermost universal exception handler for the current
@@ -250,13 +276,13 @@ void remove_exceptionst::instrument_exception_handler(
 /// we note  that try1{ try2 {} catch2c {} catch2d {}} catch1a() {} catch1b{}
 /// must catch in the following order: 2c 2d 1a 1b hence the numerical index
 /// is reversed whereas the letter ordering remains the same.
-/// @param stack_catch exception table
-/// @param goto_program program being evaluated
-/// @param[out] universal_try returns the try block
-///        corresponding to the desired exception handler
-/// @param[out] universal_catch returns the catch block
-///        corresponding to the exception desired exception handler
-/// @return the desired exception handler
+/// \param stack_catch: exception table
+/// \param goto_program: program being evaluated
+/// \param [out] universal_try: returns the try block corresponding to the
+///   desired exception handler
+/// \param [out] universal_catch: returns the catch block corresponding to the
+///   exception desired exception handler
+/// \return the desired exception handler
 goto_programt::targett remove_exceptionst::find_universal_exception(
   const remove_exceptionst::stack_catcht &stack_catch,
   goto_programt &goto_program,
@@ -289,16 +315,18 @@ goto_programt::targett remove_exceptionst::find_universal_exception(
 /// if (exception instanceof ExnA) then goto handlerA
 /// else if (exception instanceof ExnB) then goto handlerB
 /// else goto universal_handler or (dead locals; function exit)
+/// \param function_identifier: name of the function containing \p instr_it
 /// \param goto_program: body of the function to which instr_it belongs
 /// \param instr_it: throw or call instruction that may be an
 ///   exception source
 /// \param stack_catch: exception handlers currently registered
 /// \param locals: local variables to kill on a function-exit edge
 void remove_exceptionst::add_exception_dispatch_sequence(
+  const irep_idt &function_identifier,
   goto_programt &goto_program,
   const goto_programt::targett &instr_it,
   const remove_exceptionst::stack_catcht &stack_catch,
-  const std::vector<exprt> &locals)
+  const std::vector<symbol_exprt> &locals)
 {
   // Jump to the universal handler or function end, as appropriate.
   // This will appear after the GOTO-based dynamic dispatch below
@@ -332,46 +360,50 @@ void remove_exceptionst::add_exception_dispatch_sequence(
       if(!stack_catch[i][j].first.empty())
       {
         // Normal exception handler, make an instanceof check.
-        goto_programt::targett t_exc=goto_program.insert_after(instr_it);
-        t_exc->make_goto(new_state_pc);
-        t_exc->source_location=instr_it->source_location;
-        t_exc->function=instr_it->function;
+        goto_programt::targett t_exc = goto_program.insert_after(
+          instr_it,
+          goto_programt::make_goto(
+            new_state_pc, true_exprt(), instr_it->source_location));
 
         // use instanceof to check that this is the correct handler
-        symbol_typet type(stack_catch[i][j].first);
-        type_exprt expr(type);
+        struct_tag_typet type(stack_catch[i][j].first);
 
-        binary_predicate_exprt check(exc_thrown, ID_java_instanceof, expr);
+        java_instanceof_exprt check(exc_thrown, type);
         t_exc->guard=check;
 
         if(remove_added_instanceof)
-          remove_instanceof(t_exc, goto_program, symbol_table, message_handler);
+        {
+          remove_instanceof(
+            function_identifier,
+            t_exc,
+            goto_program,
+            symbol_table,
+            *class_hierarchy,
+            message_handler);
+        }
       }
     }
   }
 
-  default_dispatch->make_goto(default_target);
-  default_dispatch->source_location=instr_it->source_location;
-  default_dispatch->function=instr_it->function;
+  *default_dispatch = goto_programt::make_goto(
+    default_target, true_exprt(), instr_it->source_location);
 
   // add dead instructions
   for(const auto &local : locals)
   {
-    goto_programt::targett t_dead=goto_program.insert_after(instr_it);
-    t_dead->make_dead();
-    t_dead->code=code_deadt(local);
-    t_dead->source_location=instr_it->source_location;
-    t_dead->function=instr_it->function;
+    goto_program.insert_after(
+      instr_it, goto_programt::make_dead(local, instr_it->source_location));
   }
 }
 
 /// instruments each throw with conditional GOTOS to the corresponding
 /// exception handlers
 bool remove_exceptionst::instrument_throw(
+  const irep_idt &function_identifier,
   goto_programt &goto_program,
   const goto_programt::targett &instr_it,
   const remove_exceptionst::stack_catcht &stack_catch,
-  const std::vector<exprt> &locals)
+  const std::vector<symbol_exprt> &locals)
 {
   PRECONDITION(instr_it->type==THROW);
 
@@ -379,7 +411,7 @@ bool remove_exceptionst::instrument_throw(
     uncaught_exceptions_domaint::get_exception_symbol(instr_it->code);
 
   add_exception_dispatch_sequence(
-    goto_program, instr_it, stack_catch, locals);
+    function_identifier, goto_program, instr_it, stack_catch, locals);
 
   // find the symbol where the thrown exception should be stored:
   symbol_exprt exc_thrown =
@@ -398,11 +430,13 @@ bool remove_exceptionst::instrument_throw(
 
 /// instruments each function call that may escape exceptions with conditional
 /// GOTOS to the corresponding exception handlers
-bool remove_exceptionst::instrument_function_call(
+remove_exceptionst::instrumentation_resultt
+remove_exceptionst::instrument_function_call(
+  const irep_idt &function_identifier,
   goto_programt &goto_program,
   const goto_programt::targett &instr_it,
   const stack_catcht &stack_catch,
-  const std::vector<exprt> &locals)
+  const std::vector<symbol_exprt> &locals)
 {
   PRECONDITION(instr_it->type==FUNCTION_CALL);
 
@@ -410,7 +444,8 @@ bool remove_exceptionst::instrument_function_call(
   goto_programt::targett next_it=instr_it;
   next_it++;
 
-  code_function_callt &function_call=to_code_function_call(instr_it->code);
+  const code_function_callt &function_call = instr_it->get_function_call();
+
   DATA_INVARIANT(
     function_call.function().id()==ID_symbol,
     "identified expected to be a symbol");
@@ -421,43 +456,48 @@ bool remove_exceptionst::instrument_function_call(
   {
     equal_exprt no_exception_currently_in_flight(
       get_inflight_exception_global(),
-      null_pointer_exprt(pointer_type(empty_typet())));
+      null_pointer_exprt(pointer_type(java_void_type())));
 
     if(symbol_table.lookup_ref(callee_id).type.get_bool(ID_C_must_not_throw))
     {
       // Function is annotated must-not-throw, but we can't prove that here.
       // Insert an ASSUME(@inflight_exception == null):
-      goto_programt::targett assume_null = goto_program.insert_after(instr_it);
-      assume_null->make_assumption(no_exception_currently_in_flight);
+      goto_program.insert_after(
+        instr_it,
+        goto_programt::make_assumption(no_exception_currently_in_flight));
+
+      return instrumentation_resultt::ADDED_CODE_WITHOUT_MAY_THROW;
     }
     else
     {
       add_exception_dispatch_sequence(
-        goto_program, instr_it, stack_catch, locals);
+        function_identifier, goto_program, instr_it, stack_catch, locals);
 
       // add a null check (so that instanceof can be applied)
-      goto_programt::targett t_null=goto_program.insert_after(instr_it);
-      t_null->make_goto(next_it);
-      t_null->source_location=instr_it->source_location;
-      t_null->function=instr_it->function;
-      t_null->guard=no_exception_currently_in_flight;
-    }
+      goto_program.insert_after(
+        instr_it,
+        goto_programt::make_goto(
+          next_it,
+          no_exception_currently_in_flight,
+          instr_it->source_location));
 
-    return true;
+      return instrumentation_resultt::ADDED_CODE_WITH_MAY_THROW;
+    }
   }
 
-  return false;
+  return instrumentation_resultt::DID_NOTHING;
 }
 
 /// instruments throws, function calls that may escape exceptions and exception
 /// handlers. Additionally, it re-computes the live-range of local variables in
 /// order to add DEAD instructions.
 void remove_exceptionst::instrument_exceptions(
+  const irep_idt &function_identifier,
   goto_programt &goto_program)
 {
   stack_catcht stack_catch; // stack of try-catch blocks
-  std::vector<std::vector<exprt>> stack_locals; // stack of local vars
-  std::vector<exprt> locals;
+  std::vector<std::vector<symbol_exprt>> stack_locals; // stack of local vars
+  std::vector<symbol_exprt> locals;
 
   if(goto_program.empty())
     return;
@@ -466,12 +506,13 @@ void remove_exceptionst::instrument_exceptions(
     function_or_callees_may_throw(goto_program);
 
   bool did_something = false;
+  bool added_goto_instruction = false;
 
   Forall_goto_program_instructions(instr_it, goto_program)
   {
     if(instr_it->is_decl())
     {
-      code_declt decl=to_code_decl(instr_it->code);
+      code_declt decl = instr_it->get_decl();
       locals.push_back(decl.symbol());
     }
     // Is it a handler push/pop or catch landing-pad?
@@ -543,21 +584,35 @@ void remove_exceptionst::instrument_exceptions(
           false,
           "CATCH opcode should be one of push-catch, pop-catch, landingpad");
       }
-      instr_it->make_skip();
+
+      instr_it->turn_into_skip();
       did_something = true;
     }
     else if(instr_it->type==THROW)
     {
-      did_something |=
-        instrument_throw(goto_program, instr_it, stack_catch, locals);
+      did_something = instrument_throw(
+        function_identifier, goto_program, instr_it, stack_catch, locals);
     }
     else if(instr_it->type==FUNCTION_CALL)
     {
-      did_something |=
-        instrument_function_call(goto_program, instr_it, stack_catch, locals);
+      instrumentation_resultt result =
+        instrument_function_call(
+          function_identifier, goto_program, instr_it, stack_catch, locals);
+      did_something = result != instrumentation_resultt::DID_NOTHING;
+      added_goto_instruction =
+        result == instrumentation_resultt::ADDED_CODE_WITH_MAY_THROW;
     }
   }
 
+  // INITIALIZE_FUNCTION should not contain any exception handling branches for
+  // two reasons: (1) with symex-driven lazy loading it means that code that
+  // references @inflight_exception might be generated before
+  // @inflight_exception is initialized; (2) symex can analyze
+  // INITIALIZE_FUNCTION much faster if it doesn't contain any branching.
+  INVARIANT(
+    function_identifier != INITIALIZE_FUNCTION || !added_goto_instruction,
+    INITIALIZE_FUNCTION " should not contain any exception handling branches");
+
   if(did_something)
     remove_skip(goto_program);
 }
@@ -565,33 +620,34 @@ void remove_exceptionst::instrument_exceptions(
 void remove_exceptionst::operator()(goto_functionst &goto_functions)
 {
   Forall_goto_functions(it, goto_functions)
-    instrument_exceptions(it->second.body);
+    instrument_exceptions(it->first, it->second.body);
 }
 
-void remove_exceptionst::operator()(goto_programt &goto_program)
+void remove_exceptionst::
+operator()(const irep_idt &function_identifier, goto_programt &goto_program)
 {
-  instrument_exceptions(goto_program);
+  instrument_exceptions(function_identifier, goto_program);
 }
 
 /// removes throws/CATCH-POP/CATCH-PUSH
-void remove_exceptions(
+void remove_exceptions_using_instanceof(
   symbol_table_baset &symbol_table,
   goto_functionst &goto_functions,
-  message_handlert &message_handler,
-  remove_exceptions_typest type)
+  message_handlert &message_handler)
 {
   const namespacet ns(symbol_table);
   std::map<irep_idt, std::set<irep_idt>> exceptions_map;
+
   uncaught_exceptions(goto_functions, ns, exceptions_map);
+
   remove_exceptionst::function_may_throwt function_may_throw =
     [&exceptions_map](const irep_idt &id) {
       return !exceptions_map[id].empty();
     };
+
   remove_exceptionst remove_exceptions(
-    symbol_table,
-    function_may_throw,
-    type == remove_exceptions_typest::REMOVE_ADDED_INSTANCEOF,
-    message_handler);
+    symbol_table, nullptr, function_may_throw, false, message_handler);
+
   remove_exceptions(goto_functions);
 }
 
@@ -601,37 +657,116 @@ void remove_exceptions(
 /// because we can't inspect other functions to determine whether they throw
 /// or not, and therefore must assume they do and always introduce post-call
 /// exception dispatch.
+/// \param function_identifier: name of the goto function being processed
 /// \param goto_program: program to remove exceptions from
 /// \param symbol_table: global symbol table. The `@inflight_exception` global
 ///   may be added if not already present. It will not be initialised; that is
 ///   the caller's responsibility.
 /// \param message_handler: logging output
-/// \param type: specifies whether instanceof operations generated by this pass
-///   should be lowered to class-identifier comparisons (using
-///   remove_instanceof).
-void remove_exceptions(
+void remove_exceptions_using_instanceof(
+  const irep_idt &function_identifier,
   goto_programt &goto_program,
   symbol_table_baset &symbol_table,
-  message_handlert &message_handler,
-  remove_exceptions_typest type)
+  message_handlert &message_handler)
+{
+  remove_exceptionst::function_may_throwt any_function_may_throw =
+    [](const irep_idt &) { return true; };
+
+  remove_exceptionst remove_exceptions(
+    symbol_table, nullptr, any_function_may_throw, false, message_handler);
+
+  remove_exceptions(function_identifier, goto_program);
+}
+
+/// removes throws/CATCH-POP/CATCH-PUSH, replacing them with explicit exception
+/// propagation.
+/// \param goto_model: model to remove exceptions from. The
+///   `@inflight_exception` global may be added to its symbol table if not
+///   already present. It will not be initialised; that is the caller's
+///   responsibility.
+/// \param message_handler: logging output
+void remove_exceptions_using_instanceof(
+  goto_modelt &goto_model,
+  message_handlert &message_handler)
+{
+  remove_exceptions_using_instanceof(
+    goto_model.symbol_table, goto_model.goto_functions, message_handler);
+}
+
+/// removes throws/CATCH-POP/CATCH-PUSH
+void remove_exceptions(
+  symbol_table_baset &symbol_table,
+  goto_functionst &goto_functions,
+  const class_hierarchyt &class_hierarchy,
+  message_handlert &message_handler)
+{
+  const namespacet ns(symbol_table);
+  std::map<irep_idt, std::set<irep_idt>> exceptions_map;
+
+  uncaught_exceptions(goto_functions, ns, exceptions_map);
+
+  remove_exceptionst::function_may_throwt function_may_throw =
+    [&exceptions_map](const irep_idt &id) {
+      return !exceptions_map[id].empty();
+    };
+
+  remove_exceptionst remove_exceptions(
+    symbol_table, &class_hierarchy, function_may_throw, true, message_handler);
+
+  remove_exceptions(goto_functions);
+}
+
+/// removes throws/CATCH-POP/CATCH-PUSH from a single GOTO program, replacing
+/// them with explicit exception propagation.
+/// Note this is somewhat less accurate than the whole-goto-model version,
+/// because we can't inspect other functions to determine whether they throw
+/// or not, and therefore must assume they do and always introduce post-call
+/// exception dispatch.
+/// \param function_identifier: name of the goto function being processed
+/// \param goto_program: program to remove exceptions from
+/// \param symbol_table: global symbol table. The `@inflight_exception` global
+///   may be added if not already present. It will not be initialised; that is
+///   the caller's responsibility.
+/// \param class_hierarchy: class hierarchy analysis of symbol_table.
+///   Only needed if type == REMOVE_ADDED_INSTANCEOF; otherwise may be null.
+/// \param message_handler: logging output
+void remove_exceptions(
+  const irep_idt &function_identifier,
+  goto_programt &goto_program,
+  symbol_table_baset &symbol_table,
+  const class_hierarchyt &class_hierarchy,
+  message_handlert &message_handler)
 {
   remove_exceptionst::function_may_throwt any_function_may_throw =
     [](const irep_idt &) { return true; };
 
   remove_exceptionst remove_exceptions(
     symbol_table,
+    &class_hierarchy,
     any_function_may_throw,
-    type == remove_exceptions_typest::REMOVE_ADDED_INSTANCEOF,
+    true,
     message_handler);
-  remove_exceptions(goto_program);
+
+  remove_exceptions(function_identifier, goto_program);
 }
 
-/// removes throws/CATCH-POP/CATCH-PUSH
+/// removes throws/CATCH-POP/CATCH-PUSH, replacing them with explicit exception
+/// propagation.
+/// \param goto_model: model to remove exceptions from. The
+///   `@inflight_exception` global may be added to its symbol table if not
+///   already present. It will not be initialised; that is the caller's
+///   responsibility.
+/// \param class_hierarchy: class hierarchy analysis of symbol_table.
+///   Only needed if type == REMOVE_ADDED_INSTANCEOF; otherwise may be null.
+/// \param message_handler: logging output
 void remove_exceptions(
   goto_modelt &goto_model,
-  message_handlert &message_handler,
-  remove_exceptions_typest type)
+  const class_hierarchyt &class_hierarchy,
+  message_handlert &message_handler)
 {
   remove_exceptions(
-    goto_model.symbol_table, goto_model.goto_functions, message_handler, type);
+    goto_model.symbol_table,
+    goto_model.goto_functions,
+    class_hierarchy,
+    message_handler);
 }
