@@ -14,7 +14,7 @@ Author: Diffblue Ltd.
 #include <java_bytecode/java_object_factory.h>
 #include <java_bytecode/java_pointer_casts.h>
 
-#include <util/fresh_symbol.h>
+#include "java_utils.h"
 #include <util/invariant_utils.h>
 #include <util/namespace.h>
 #include <util/std_code.h>
@@ -26,7 +26,7 @@ public:
   java_simple_method_stubst(
     symbol_table_baset &_symbol_table,
     bool _assume_non_null,
-    const object_factory_parameterst &_object_factory_parameters,
+    const java_object_factory_parameterst &_object_factory_parameters,
     message_handlert &_message_handler)
     : symbol_table(_symbol_table),
       assume_non_null(_assume_non_null),
@@ -52,7 +52,7 @@ public:
 protected:
   symbol_table_baset &symbol_table;
   bool assume_non_null;
-  const object_factory_parameterst &object_factory_parameters;
+  const java_object_factory_parameterst &object_factory_parameters;
   message_handlert &message_handler;
 };
 
@@ -66,6 +66,8 @@ protected:
 ///   type.
 /// \param ptr: pointer to the memory to initialize
 /// \param loc: source location to set for the opaque method stub
+/// \param function_id: name of the function we're generated stub code for; used
+///   to ensure any generated temporaries are created in the correct scope.
 /// \param [out] parent_block: The parent block in which the new instructions
 ///   will be added.
 /// \param insert_before_index: The position in which the new instructions
@@ -106,11 +108,11 @@ void java_simple_method_stubst::create_method_stub_at(
   // If it's a constructor the thing we're constructing has already
   // been allocated by this point.
   if(is_constructor)
-    to_init = dereference_exprt(to_init, expected_base);
+    to_init = dereference_exprt(to_init);
 
-  object_factory_parameterst parameters = object_factory_parameters;
+  java_object_factory_parameterst parameters = object_factory_parameters;
   if(assume_non_null)
-    parameters.max_nonnull_tree_depth = 1;
+    parameters.min_null_tree_depth = 1;
 
   // Generate new instructions.
   code_blockt new_instructions;
@@ -121,20 +123,21 @@ void java_simple_method_stubst::create_method_stub_at(
     symbol_table,
     loc,
     is_constructor,
-    allocation_typet::DYNAMIC,
+    lifetimet::DYNAMIC,
     parameters,
     update_in_place ? update_in_placet::MUST_UPDATE_IN_PLACE
-                    : update_in_placet::NO_UPDATE_IN_PLACE);
+                    : update_in_placet::NO_UPDATE_IN_PLACE,
+    message_handler);
 
   // Insert new_instructions into parent block.
-  if(!new_instructions.operands().empty())
+  if(!new_instructions.statements().empty())
   {
-    auto insert_position = parent_block.operands().begin();
+    auto insert_position = parent_block.statements().begin();
     std::advance(insert_position, insert_before_index);
-    parent_block.operands().insert(
+    parent_block.statements().insert(
       insert_position,
-      new_instructions.operands().begin(),
-      new_instructions.operands().end());
+      new_instructions.statements().begin(),
+      new_instructions.statements().end());
   }
 }
 
@@ -145,12 +148,29 @@ void java_simple_method_stubst::create_method_stub_at(
 void java_simple_method_stubst::create_method_stub(symbolt &symbol)
 {
   code_blockt new_instructions;
-  const java_method_typet &required_type = to_java_method_type(symbol.type);
+  java_method_typet &required_type = to_java_method_type(symbol.type);
 
   // synthetic source location that contains the opaque function name only
   source_locationt synthesized_source_location;
   synthesized_source_location.set_function(symbol.name);
 
+  // make sure all parameters are named
+  for(auto &parameter : required_type.parameters())
+  {
+    if(parameter.get_identifier().empty())
+    {
+      symbolt &parameter_symbol = fresh_java_symbol(
+        parameter.type(),
+        "#anon",
+        synthesized_source_location,
+        symbol.name,
+        symbol_table);
+      parameter_symbol.is_parameter = true;
+      parameter.set_base_name(parameter_symbol.base_name);
+      parameter.set_identifier(parameter_symbol.name);
+    }
+  }
+
   // Initialize the return value or `this` parameter under construction.
   // Note symbol.type is required_type, but with write access
   // Probably required_type should not be const
@@ -159,18 +179,18 @@ void java_simple_method_stubst::create_method_stub(symbolt &symbol)
   {
     const auto &this_argument = required_type.parameters()[0];
     const typet &this_type = this_argument.type();
-    symbolt &init_symbol = get_fresh_aux_symbol(
+    symbolt &init_symbol = fresh_java_symbol(
       this_type,
-      id2string(symbol.name),
       "to_construct",
       synthesized_source_location,
-      ID_java,
+      symbol.name,
       symbol_table);
     const symbol_exprt &init_symbol_expression = init_symbol.symbol_expr();
     code_assignt get_argument(
-      init_symbol_expression, symbol_exprt(this_argument.get_identifier()));
+      init_symbol_expression,
+      symbol_exprt(this_argument.get_identifier(), this_type));
     get_argument.add_source_location() = synthesized_source_location;
-    new_instructions.copy_to_operands(get_argument);
+    new_instructions.add(get_argument);
     create_method_stub_at(
       this_type,
       init_symbol_expression,
@@ -184,21 +204,20 @@ void java_simple_method_stubst::create_method_stub(symbolt &symbol)
   else
   {
     const typet &required_return_type = required_type.return_type();
-    if(required_return_type != empty_typet())
+    if(required_return_type != java_void_type())
     {
-      symbolt &to_return_symbol = get_fresh_aux_symbol(
+      symbolt &to_return_symbol = fresh_java_symbol(
         required_return_type,
-        id2string(symbol.name),
         "to_return",
         synthesized_source_location,
-        ID_java,
+        symbol.name,
         symbol_table);
       const exprt &to_return = to_return_symbol.symbol_expr();
       if(to_return_symbol.type.id() != ID_pointer)
       {
-        object_factory_parameterst parameters = object_factory_parameters;
+        java_object_factory_parameterst parameters = object_factory_parameters;
         if(assume_non_null)
-          parameters.max_nonnull_tree_depth = 1;
+          parameters.min_null_tree_depth = 1;
 
         gen_nondet_init(
           to_return,
@@ -206,9 +225,10 @@ void java_simple_method_stubst::create_method_stub(symbolt &symbol)
           symbol_table,
           source_locationt(),
           false,
-          allocation_typet::LOCAL, // Irrelevant as type is primitive
+          lifetimet::AUTOMATIC_LOCAL, // Irrelevant as type is primitive
           parameters,
-          update_in_placet::NO_UPDATE_IN_PLACE);
+          update_in_placet::NO_UPDATE_IN_PLACE,
+          message_handler);
       }
       else
         create_method_stub_at(
@@ -220,7 +240,7 @@ void java_simple_method_stubst::create_method_stub(symbolt &symbol)
           0,
           false,
           false);
-      new_instructions.copy_to_operands(code_returnt(to_return));
+      new_instructions.add(code_returnt(to_return));
     }
   }
 
@@ -232,7 +252,7 @@ void java_simple_method_stubst::create_method_stub(symbolt &symbol)
 /// \param symname: Symbol name to consider stubbing
 void java_simple_method_stubst::check_method_stub(const irep_idt &symname)
 {
-  const symbolt &sym = *symbol_table.lookup(symname);
+  const symbolt &sym = symbol_table.lookup_ref(symname);
   if(!sym.is_type && sym.value.id() == ID_nil &&
     sym.type.id() == ID_code &&
     // do not stub internal locking calls as 'create_method_stub' does not
@@ -245,7 +265,7 @@ void java_simple_method_stubst::check_method_stub(const irep_idt &symname)
     sym.name !=
       "java::java.lang.Object.monitorexit:(Ljava/lang/Object;)V")
   {
-    create_method_stub(*symbol_table.get_writeable(symname));
+    create_method_stub(symbol_table.get_writeable_ref(symname));
   }
 }
 
@@ -253,7 +273,7 @@ void java_generate_simple_method_stub(
   const irep_idt &function_name,
   symbol_table_baset &symbol_table,
   bool assume_non_null,
-  const object_factory_parameterst &object_factory_parameters,
+  const java_object_factory_parameterst &object_factory_parameters,
   message_handlert &message_handler)
 {
   java_simple_method_stubst stub_generator(
@@ -274,7 +294,7 @@ void java_generate_simple_method_stub(
 void java_generate_simple_method_stubs(
   symbol_table_baset &symbol_table,
   bool assume_non_null,
-  const object_factory_parameterst &object_factory_parameters,
+  const java_object_factory_parameterst &object_factory_parameters,
   message_handlert &message_handler)
 {
   // The intent here is to apply stub_generator.check_method_stub() to
