@@ -12,63 +12,71 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_JBMC_JBMC_PARSE_OPTIONS_H
 #define CPROVER_JBMC_JBMC_PARSE_OPTIONS_H
 
-#include <util/ui_message.h>
 #include <util/parse_options.h>
 #include <util/timestamper.h>
+#include <util/ui_message.h>
+#include <util/validation_interface.h>
 
 #include <langapi/language.h>
 
 #include <analyses/goto_check.h>
 
-#include <cbmc/bmc.h>
+#include <goto-checker/bmc_util.h>
 
-#include <goto-instrument/cover.h>
 #include <goto-programs/class_hierarchy.h>
 #include <goto-programs/goto_trace.h>
-#include <goto-programs/lazy_goto_model.h>
 #include <goto-programs/show_properties.h>
 
 #include <goto-symex/path_storage.h>
 
+#include <solvers/strings/string_refinement.h>
+
 #include <java_bytecode/java_bytecode_language.h>
+#include <java_bytecode/lazy_goto_model.h>
+
+#include <json/json_interface.h>
+#include <xmllang/xml_interface.h>
 
-class bmct;
 class goto_functionst;
 class optionst;
 
 // clang-format off
 #define JBMC_OPTIONS \
   OPT_BMC \
-  "(preprocess)(slice-by-trace):" \
+  "(preprocess)" \
   OPT_FUNCTIONS \
   "(no-simplify)(full-slice)" \
   OPT_REACHABILITY_SLICER \
   "(debug-level):(no-propagation)(no-simplify-if)" \
   "(document-subgoals)(outfile):" \
   "(object-bits):" \
-  "(classpath):(cp):(main-class):" \
+  "(classpath):(cp):" \
+  OPT_JAVA_JAR \
+  "(main-class):" \
+  OPT_JAVA_GOTO_BINARY \
   "(no-assertions)(no-assumptions)" \
-  "(xml-ui)(json-ui)" \
-  "(smt1)(smt2)(fpa)(cvc3)(cvc4)(boolector)(yices)(z3)(opensmt)(mathsat)" \
+  OPT_XML_INTERFACE \
+  OPT_JSON_INTERFACE \
+  "(smt1)(smt2)(fpa)(cvc3)(cvc4)(boolector)(yices)(z3)(mathsat)" \
   "(no-sat-preprocessor)" \
   "(beautify)" \
   "(dimacs)(refine)(max-node-refinement):(refine-arrays)(refine-arithmetic)"\
-  "(refine-strings)" /* will go away */ \
-  "(no-refine-strings)" \
-  "(string-printable)" \
-  "(string-max-input-length):" /* will go away */ \
-  "(max-nondet-string-length):" \
+  OPT_STRING_REFINEMENT \
   "(16)(32)(64)(LP64)(ILP64)(LLP64)(ILP32)(LP32)" \
   OPT_SHOW_GOTO_FUNCTIONS \
   OPT_SHOW_CLASS_HIERARCHY \
   "(show-loops)" \
-  "(show-symbol-table)(show-parse-tree)" \
+  "(show-symbol-table)" \
+  "(list-symbols)" \
+  "(show-parse-tree)" \
   OPT_SHOW_PROPERTIES \
   "(drop-unused-functions)" \
   "(property):(stop-on-fail)(trace)" \
   "(verbosity):" \
+  "(nondet-static)" \
+  OPT_JAVA_TRACE_VALIDATION \
   "(version)" \
-  "(cover):(symex-coverage-report):" \
+  "(symex-coverage-report):" \
   OPT_TIMESTAMP \
   "(i386-linux)(i386-macos)(i386-win32)(win32)(winx64)" \
   "(ppc-macos)" \
@@ -77,15 +85,14 @@ class optionst;
   OPT_FLUSH \
   JAVA_BYTECODE_LANGUAGE_OPTIONS \
   "(java-unwind-enum-static)" \
-  "(localize-faults)(localize-faults-method):" \
+  "(localize-faults)" \
   "(java-threading)" \
   OPT_GOTO_TRACE \
+  OPT_VALIDATE \
   "(symex-driven-lazy-loading)"
 // clang-format on
 
-class jbmc_parse_optionst:
-  public parse_options_baset,
-  public messaget
+class jbmc_parse_optionst : public parse_options_baset
 {
 public:
   virtual int doit() override;
@@ -118,18 +125,22 @@ public:
     bool body_available);
 
 protected:
-  ui_message_handlert ui_message_handler;
-  std::unique_ptr<cover_configt> cover_config;
-  path_strategy_choosert path_strategy_chooser;
-  object_factory_parameterst object_factory_params;
+  java_object_factory_parameterst object_factory_params;
   bool stub_objects_are_not_null;
 
+  std::unique_ptr<class_hierarchyt> class_hierarchy;
+
   void get_command_line_options(optionst &);
   int get_goto_program(
-    std::unique_ptr<goto_modelt> &goto_model, const optionst &);
+    std::unique_ptr<abstract_goto_modelt> &goto_model,
+    const optionst &);
   bool show_loaded_functions(const abstract_goto_modelt &goto_model);
+  bool show_loaded_symbols(const abstract_goto_modelt &goto_model);
 
-  bool set_properties(goto_modelt &goto_model);
+  /// See java_bytecode_languaget::method_context.
+  /// The two fields are initialized in exactly the same way.
+  /// TODO Refactor this so it only needs to be computed once, in one place.
+  optionalt<prefix_filtert> method_context;
 };
 
 #endif // CPROVER_JBMC_JBMC_PARSE_OPTIONS_H
