@@ -27,6 +27,7 @@ Author: Peter Schrammel
 #include <goto-programs/adjust_float_expressions.h>
 #include <goto-programs/goto_convert_functions.h>
 #include <goto-programs/goto_inline.h>
+#include <goto-programs/initialize_goto_model.h>
 #include <goto-programs/instrument_preconditions.h>
 #include <goto-programs/link_to_library.h>
 #include <goto-programs/loop_ids.h>
@@ -39,13 +40,12 @@ Author: Peter Schrammel
 #include <goto-programs/remove_unused_functions.h>
 #include <goto-programs/remove_vector.h>
 #include <goto-programs/remove_virtual_functions.h>
+#include <goto-programs/rewrite_union.h>
 #include <goto-programs/set_properties.h>
 #include <goto-programs/show_properties.h>
 #include <goto-programs/string_abstraction.h>
 #include <goto-programs/string_instrumentation.h>
 
-#include <goto-symex/rewrite_union.h>
-
 #include <goto-instrument/cover.h>
 
 #include <pointer-analysis/add_failed_symbols.h>
@@ -62,21 +62,11 @@ Author: Peter Schrammel
 #include <goto-diff/unified_diff.h>
 
 jdiff_parse_optionst::jdiff_parse_optionst(int argc, const char **argv)
-  : parse_options_baset(JDIFF_OPTIONS, argc, argv),
-    jdiff_languagest(cmdline, ui_message_handler),
-    ui_message_handler(cmdline, std::string("JDIFF ") + CBMC_VERSION),
-    languages2(cmdline, ui_message_handler)
-{
-}
-
-::jdiff_parse_optionst::jdiff_parse_optionst(
-  int argc,
-  const char **argv,
-  const std::string &extra_options)
-  : parse_options_baset(JDIFF_OPTIONS + extra_options, argc, argv),
-    jdiff_languagest(cmdline, ui_message_handler),
-    ui_message_handler(cmdline, std::string("JDIFF ") + CBMC_VERSION),
-    languages2(cmdline, ui_message_handler)
+  : parse_options_baset(
+      JDIFF_OPTIONS,
+      argc,
+      argv,
+      std::string("JDIFF ") + CBMC_VERSION)
 {
 }
 
@@ -93,6 +83,7 @@ void jdiff_parse_optionst::get_command_line_options(optionst &options)
   // we always require these options
   cmdline.set("no-lazy-methods");
   cmdline.set("no-refine-strings");
+  parse_java_language_options(cmdline, options);
 
   if(cmdline.isset("cover"))
     parse_cover_options(cmdline, options);
@@ -106,9 +97,6 @@ void jdiff_parse_optionst::get_command_line_options(optionst &options)
   if(cmdline.isset("debug-level"))
     options.set_option("debug-level", cmdline.get_value("debug-level"));
 
-  if(cmdline.isset("slice-by-trace"))
-    options.set_option("slice-by-trace", cmdline.get_value("slice-by-trace"));
-
   if(cmdline.isset("unwindset"))
     options.set_option("unwindset", cmdline.get_value("unwindset"));
 
@@ -200,35 +188,37 @@ int jdiff_parse_optionst::doit()
 
   optionst options;
   get_command_line_options(options);
-  eval_verbosity(
+  messaget::eval_verbosity(
     cmdline.get_value("verbosity"), messaget::M_STATISTICS, ui_message_handler);
 
   //
   // Print a banner
   //
-  status() << "JDIFF version " << CBMC_VERSION << " " << sizeof(void *) * 8
-           << "-bit " << config.this_architecture() << " "
-           << config.this_operating_system() << eom;
+  log.status() << "JDIFF version " << CBMC_VERSION << " " << sizeof(void *) * 8
+               << "-bit " << config.this_architecture() << " "
+               << config.this_operating_system() << messaget::eom;
 
   if(cmdline.args.size() != 2)
   {
-    error() << "Please provide two programs to compare" << eom;
+    log.error() << "Please provide two programs to compare" << messaget::eom;
     return CPROVER_EXIT_INCORRECT_TASK;
   }
 
-  goto_modelt goto_model1, goto_model2;
+  register_languages();
 
-  int get_goto_program_ret = get_goto_program(options, *this, goto_model1);
-  if(get_goto_program_ret != -1)
-    return get_goto_program_ret;
-  get_goto_program_ret = get_goto_program(options, languages2, goto_model2);
-  if(get_goto_program_ret != -1)
-    return get_goto_program_ret;
+  goto_modelt goto_model1 =
+    initialize_goto_model({cmdline.args[0]}, ui_message_handler, options);
+  if(process_goto_program(options, goto_model1))
+    return CPROVER_EXIT_INTERNAL_ERROR;
+  goto_modelt goto_model2 =
+    initialize_goto_model({cmdline.args[1]}, ui_message_handler, options);
+  if(process_goto_program(options, goto_model2))
+    return CPROVER_EXIT_INTERNAL_ERROR;
 
   if(cmdline.isset("show-loops"))
   {
-    show_loop_ids(get_ui(), goto_model1);
-    show_loop_ids(get_ui(), goto_model2);
+    show_loop_ids(ui_message_handler.get_ui(), goto_model1);
+    show_loop_ids(ui_message_handler.get_ui(), goto_model2);
     return CPROVER_EXIT_SUCCESS;
   }
 
@@ -237,15 +227,9 @@ int jdiff_parse_optionst::doit()
     cmdline.isset("list-goto-functions"))
   {
     show_goto_functions(
-      goto_model1,
-      get_message_handler(),
-      ui_message_handler.get_ui(),
-      cmdline.isset("list-goto-functions"));
+      goto_model1, ui_message_handler, cmdline.isset("list-goto-functions"));
     show_goto_functions(
-      goto_model2,
-      get_message_handler(),
-      ui_message_handler.get_ui(),
-      cmdline.isset("list-goto-functions"));
+      goto_model2, ui_message_handler, cmdline.isset("list-goto-functions"));
     return CPROVER_EXIT_SUCCESS;
   }
 
@@ -274,86 +258,34 @@ int jdiff_parse_optionst::doit()
   }
 
   java_syntactic_difft sd(
-    goto_model1, goto_model2, options, get_message_handler());
-  sd.set_ui(get_ui());
+    goto_model1, goto_model2, options, ui_message_handler);
   sd();
   sd.output_functions();
 
   return CPROVER_EXIT_SUCCESS;
 }
 
-int jdiff_parse_optionst::get_goto_program(
-  const optionst &options,
-  jdiff_languagest &languages,
-  goto_modelt &goto_model)
-{
-  status() << "Reading program from `" << cmdline.args[0] << "'" << eom;
-
-  if(is_goto_binary(cmdline.args[0]))
-  {
-    if(read_goto_binary(
-         cmdline.args[0],
-         goto_model.symbol_table,
-         goto_model.goto_functions,
-         languages.get_message_handler()))
-      return CPROVER_EXIT_INCORRECT_TASK;
-
-    config.set(cmdline);
-
-    // This one is done.
-    cmdline.args.erase(cmdline.args.begin());
-  }
-  else
-  {
-    // This is a a workaround to make parse() think that there is only
-    // one source file.
-    std::string arg2("");
-    if(cmdline.args.size() == 2)
-    {
-      arg2 = cmdline.args[1];
-      cmdline.args.erase(--cmdline.args.end());
-    }
-
-    if(languages.parse() || languages.typecheck() || languages.final())
-      return CPROVER_EXIT_INCORRECT_TASK;
-
-    // we no longer need any parse trees or language files
-    languages.clear_parse();
-
-    status() << "Generating GOTO Program" << eom;
-
-    goto_model.symbol_table = languages.symbol_table;
-    goto_convert(
-      goto_model.symbol_table, goto_model.goto_functions, ui_message_handler);
-
-    if(process_goto_program(options, goto_model))
-      return CPROVER_EXIT_INTERNAL_ERROR;
-
-    // if we had a second argument then we will handle it next
-    if(arg2 != "")
-      cmdline.args[0] = arg2;
-  }
-
-  return -1; // no error, continue
-}
-
 bool jdiff_parse_optionst::process_goto_program(
   const optionst &options,
   goto_modelt &goto_model)
 {
-  try
   {
     // remove function pointers
-    status() << "Removal of function pointers and virtual functions" << eom;
+    log.status() << "Removing function pointers and virtual functions"
+                 << messaget::eom;
     remove_function_pointers(
-      get_message_handler(), goto_model, cmdline.isset("pointer-check"));
+      ui_message_handler, goto_model, cmdline.isset("pointer-check"));
 
     // Java virtual functions -> explicit dispatch tables:
     remove_virtual_functions(goto_model);
-    // remove catch and throw
-    remove_exceptions(goto_model, get_message_handler());
+
+    // remove Java throw and catch
+    // This introduces instanceof, so order is important:
+    remove_exceptions_using_instanceof(goto_model, ui_message_handler);
+
     // Java instanceof -> clsid comparison:
-    remove_instanceof(goto_model, get_message_handler());
+    class_hierarchyt class_hierarchy(goto_model.symbol_table);
+    remove_instanceof(goto_model, class_hierarchy, ui_message_handler);
 
     mm_io(goto_model);
 
@@ -367,7 +299,7 @@ bool jdiff_parse_optionst::process_goto_program(
     rewrite_union(goto_model);
 
     // add generic checks
-    status() << "Generic Property Instrumentation" << eom;
+    log.status() << "Generic Property Instrumentation" << messaget::eom;
     goto_check(options, goto_model);
 
     // checks don't know about adjusted float expressions
@@ -386,7 +318,9 @@ bool jdiff_parse_optionst::process_goto_program(
     // instrument cover goals
     if(cmdline.isset("cover"))
     {
-      if(instrument_cover_goals(options, goto_model, get_message_handler()))
+      const auto cover_config =
+        get_cover_config(options, goto_model.symbol_table, ui_message_handler);
+      if(instrument_cover_goals(cover_config, goto_model, ui_message_handler))
         return true;
     }
 
@@ -402,31 +336,6 @@ bool jdiff_parse_optionst::process_goto_program(
     goto_model.goto_functions.update();
   }
 
-  catch(const char *e)
-  {
-    error() << e << eom;
-    return true;
-  }
-
-  catch(const std::string &e)
-  {
-    error() << e << eom;
-    return true;
-  }
-
-  catch(int e)
-  {
-    error() << "Numeric exception: " << e << eom;
-    return true;
-  }
-
-  catch(const std::bad_alloc &)
-  {
-    error() << "Out of memory" << eom;
-    exit(CPROVER_EXIT_INTERNAL_OUT_OF_MEMORY);
-    return true;
-  }
-
   return false;
 }
 
@@ -435,10 +344,10 @@ void jdiff_parse_optionst::help()
 {
   // clang-format off
   std::cout << '\n' << banner_string("JDIFF", CBMC_VERSION) << '\n'
+            << align_center_with_border("Copyright (C) 2016-2018") << '\n'
+            << align_center_with_border("Daniel Kroening, Peter Schrammel") << '\n' // NOLINT(*)
+            << align_center_with_border("kroening@kroening.com") << '\n'
             <<
-    "* *                Copyright (C) 2016-2018                  * *\n"
-    "* *            Daniel Kroening, Peter Schrammel             * *\n"
-    "* *                 kroening@kroening.com                   * *\n"
     "\n"
     "Usage:                       Purpose:\n"
     "\n"
