@@ -14,6 +14,7 @@ Author: Peter Schrammel
 
 #include <analyses/goto_check.h>
 
+#include <util/options.h>
 #include <util/parse_options.h>
 #include <util/timestamper.h>
 #include <util/ui_message.h>
@@ -22,10 +23,7 @@ Author: Peter Schrammel
 #include <goto-programs/show_goto_functions.h>
 #include <goto-programs/show_properties.h>
 
-#include "jdiff_languages.h"
-
 class goto_modelt;
-class optionst;
 
 // clang-format off
 #define JDIFF_OPTIONS \
@@ -42,33 +40,20 @@ class optionst;
   "(compact-output)"
 // clang-format on
 
-class jdiff_parse_optionst : public parse_options_baset, public jdiff_languagest
+class jdiff_parse_optionst : public parse_options_baset
 {
 public:
   virtual int doit();
   virtual void help();
 
   jdiff_parse_optionst(int argc, const char **argv);
-  jdiff_parse_optionst(
-    int argc,
-    const char **argv,
-    const std::string &extra_options);
 
 protected:
-  ui_message_handlert ui_message_handler;
-  jdiff_languagest languages2;
+  void register_languages();
 
-  virtual void get_command_line_options(optionst &options);
+  void get_command_line_options(optionst &options);
 
-  virtual int get_goto_program(
-    const optionst &options,
-    jdiff_languagest &languages,
-    goto_modelt &goto_model);
-
-  virtual bool
-  process_goto_program(const optionst &options, goto_modelt &goto_model);
-
-  void preprocessing();
+  bool process_goto_program(const optionst &options, goto_modelt &goto_model);
 };
 
 #endif // CPROVER_JDIFF_JDIFF_PARSE_OPTIONS_H
