@@ -1,25 +1,27 @@
 /*******************************************************************\
 
- Module: Unit test utilities
+Module: Unit test utilities
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
 #include "require_goto_statements.h"
 
-#include <testing-utils/catch.hpp>
+#include <testing-utils/use_catch.h>
 
 #include <algorithm>
-#include <util/expr_iterator.h>
 #include <goto-programs/goto_functions.h>
+#include <goto-programs/show_goto_functions.h>
 #include <java_bytecode/java_types.h>
+#include <util/expr_iterator.h>
+#include <util/expr_util.h>
 #include <util/suffix.h>
 
 /// Expand value of a function to include all child codets
 /// \param function_value: The value of the function (e.g. got by looking up
 ///  the function in the symbol table and getting the value)
-/// \return: All ID_code statements in the tree rooted at \p function_value
+/// \return All ID_code statements in the tree rooted at \p function_value
 std::vector<codet>
 require_goto_statements::get_all_statements(const exprt &function_value)
 {
@@ -37,7 +39,7 @@ require_goto_statements::get_all_statements(const exprt &function_value)
   return statements;
 }
 
-/// \param symbol_table Symbol table for the test
+/// \param symbol_table: Symbol table for the test
 /// \return All codet statements of the __CPROVER_start function
 const std::vector<codet>
 require_goto_statements::require_entry_point_statements(
@@ -59,16 +61,18 @@ require_goto_statements::require_entry_point_statements(
 /// \param structure_name: The name of variable of type struct
 /// \param superclass_name: The name of the superclass (if given)
 /// \param component_name: The name of the component of the superclass that
-/// should be assigned
-/// \return: All the assignments to that component.
+/// \param symbol_table: A symbol table to enable type lookups
+///   should be assigned
+/// \return All the assignments to that component.
 require_goto_statements::pointer_assignment_locationt
 require_goto_statements::find_struct_component_assignments(
   const std::vector<codet> &statements,
   const irep_idt &structure_name,
   const optionalt<irep_idt> &superclass_name,
-  const irep_idt &component_name)
+  const irep_idt &component_name,
+  const symbol_tablet &symbol_table)
 {
-  pointer_assignment_locationt locations;
+  pointer_assignment_locationt locations{};
 
   for(const auto &assignment : statements)
   {
@@ -93,13 +97,17 @@ require_goto_statements::find_struct_component_assignments(
             member_expr.compound().id() == ID_member)
           {
             const member_exprt &superclass_expr =
-              to_member_expr(member_expr.op0());
+              to_member_expr(member_expr.compound());
             const irep_idt supercomponent_name =
               "@" + id2string(superclass_name.value());
 
+            object_descriptor_exprt ode;
+            const namespacet ns(symbol_table);
+            ode.build(superclass_expr, ns);
             if(
               superclass_expr.get_component_name() == supercomponent_name &&
-              superclass_expr.symbol().get_identifier() == structure_name)
+              to_symbol_expr(to_dereference_expr(ode.root_object()).pointer())
+                  .get_identifier() == structure_name)
             {
               if(
                 code_assign.rhs() ==
@@ -120,10 +128,14 @@ require_goto_statements::find_struct_component_assignments(
           // member_exprt member_expr:
           // - component name: \p component_name
           // - operand (component of): symbol for \p structure_name
+
+          object_descriptor_exprt ode;
+          const namespacet ns(symbol_table);
+          ode.build(member_expr, ns);
           if(
-            member_expr.op().id() == ID_symbol &&
-            member_expr.symbol().get_identifier() == structure_name &&
-            member_expr.get_component_name() == component_name)
+            member_expr.get_component_name() == component_name &&
+            to_symbol_expr(to_dereference_expr(ode.root_object()).pointer())
+                .get_identifier() == structure_name)
           {
             if(
               code_assign.rhs() ==
@@ -144,8 +156,8 @@ require_goto_statements::find_struct_component_assignments(
 }
 
 /// Find assignment statements that set this->{component_name}
-/// \param statements The statements to look through
-/// \param component_name The name of the component whose assignments we are
+/// \param statements: The statements to look through
+/// \param component_name: The name of the component whose assignments we are
 ///   looking for.
 /// \return A collection of all non-null assignments to this component
 ///   and, if present, a null assignment.
@@ -167,21 +179,25 @@ require_goto_statements::find_this_component_assignment(
         const member_exprt &member_expr = to_member_expr(code_assign.lhs());
         if(
           member_expr.get_component_name() == component_name &&
-          member_expr.op().id() == ID_dereference &&
-          member_expr.op().op0().id() == ID_symbol &&
-          has_suffix(
-            id2string(to_symbol_expr(member_expr.op().op0()).get_identifier()),
-            "this"))
+          member_expr.op().id() == ID_dereference)
         {
+          const auto &pointer = to_dereference_expr(member_expr.op()).pointer();
           if(
-            code_assign.rhs() ==
-            null_pointer_exprt(to_pointer_type(code_assign.lhs().type())))
+            pointer.id() == ID_symbol &&
+            has_suffix(
+              id2string(to_symbol_expr(pointer).get_identifier()),
+              id2string(ID_this)))
           {
-            locations.null_assignment = code_assign;
-          }
-          else
-          {
-            locations.non_null_assignments.push_back(code_assign);
+            if(
+              code_assign.rhs() ==
+              null_pointer_exprt(to_pointer_type(code_assign.lhs().type())))
+            {
+              locations.null_assignment = code_assign;
+            }
+            else
+            {
+              locations.non_null_assignments.push_back(code_assign);
+            }
           }
         }
       }
@@ -194,8 +210,8 @@ require_goto_statements::find_this_component_assignment(
 /// instructions.
 /// \param pointer_name: The name of the variable
 /// \param instructions: The instructions to look through
-/// \return: A structure that contains the null assignment if found, and a
-/// vector of all other assignments
+/// \return A structure that contains the null assignment if found, and a
+///   vector of all other assignments
 require_goto_statements::pointer_assignment_locationt
 require_goto_statements::find_pointer_assignments(
   const irep_idt &pointer_name,
@@ -282,24 +298,97 @@ const code_declt &require_goto_statements::require_declaration_of_name(
   throw no_decl_found_exceptiont(variable_name.c_str());
 }
 
+/// Get the unique non-null expression assigned to a symbol. The symbol may have
+/// many null assignments, but only one non-null assignment.
+/// \param entry_point_instructions: A vector of instructions
+/// \param symbol_identifier: The identifier of the symbol we are considering
+/// \return The unique non-null expression assigned to the symbol
+const exprt &get_unique_non_null_expression_assigned_to_symbol(
+  const std::vector<codet> &entry_point_instructions,
+  const irep_idt &symbol_identifier)
+{
+  const auto &assignments = require_goto_statements::find_pointer_assignments(
+                              symbol_identifier, entry_point_instructions)
+                              .non_null_assignments;
+  REQUIRE(assignments.size() == 1);
+  return assignments[0].rhs();
+}
+
+/// Get the unique symbol assigned to a symbol, if one exists. There must be
+/// a unique non-null assignment to the symbol, and it is either another symbol,
+/// in which case we return that symbol expression, or something else, which
+/// case we return a null pointer.
+/// \param entry_point_instructions: A vector of instructions
+/// \param symbol_identifier: The identifier of the symbol
+/// \return The unique symbol assigned to \p input_symbol_identifier, or a null
+///   pointer if no symbols are assigned to it
+const symbol_exprt *try_get_unique_symbol_assigned_to_symbol(
+  const std::vector<codet> &entry_point_instructions,
+  const irep_idt &symbol_identifier)
+{
+  const auto &expr = get_unique_non_null_expression_assigned_to_symbol(
+    entry_point_instructions, symbol_identifier);
+
+  return expr_try_dynamic_cast<symbol_exprt>(skip_typecast(expr));
+}
+
+/// Follow the chain of non-null assignments until we find a symbol that
+/// hasn't ever had another symbol assigned to it. For example, if this code is
+/// ```
+/// a = 5 + g(7)
+/// b = a
+/// c = b
+/// ```
+/// then given input c we return a.
+/// \param entry_point_instructions: A vector of instructions
+/// \param input_symbol_identifier: The identifier of the symbol we are
+///   currently considering
+/// \return The identifier of the symbol which is (possibly indirectly) assigned
+///   to \p input_symbol_identifier and which does not have any symbol assigned
+///   to it
+static const irep_idt &
+get_ultimate_source_symbol(
+  const std::vector<codet> &entry_point_instructions,
+  const irep_idt &input_symbol_identifier)
+{
+  const symbol_exprt *symbol_assigned_to_input_symbol =
+    try_get_unique_symbol_assigned_to_symbol(
+      entry_point_instructions, input_symbol_identifier);
+
+  if(symbol_assigned_to_input_symbol)
+  {
+    return get_ultimate_source_symbol(
+      entry_point_instructions,
+      symbol_assigned_to_input_symbol->get_identifier());
+  }
+
+  return input_symbol_identifier;
+}
+
 /// Checks that the component of the structure (possibly inherited from
 /// the superclass) is assigned an object of the given type.
-/// \param structure_name The name the variable
-/// \param superclass_name The name of its superclass (if given)
-/// \param component_name The name of the field of the superclass
-/// \param component_type_name The name of the required type of the field
-/// \param typecast_name The name of the type to which the object is cast (if
-/// there is a typecast)
-/// \param entry_point_instructions The statements to look through
-/// \return The identifier of the variable assigned to the field
+/// \param structure_name: The name the variable
+/// \param superclass_name: The name of its superclass (if given)
+/// \param component_name: The name of the field of the superclass
+/// \param component_type_name: The name of the required type of the field
+/// \param typecast_name: The name of the type to which the object is cast (if
+///   there is a typecast)
+/// \param entry_point_instructions: The statements to look through
+/// \param symbol_table: A symbol table to enable type lookups
+/// \return The identifier of the ultimate source symbol assigned to the field,
+///   which will be used for future calls to
+///   `require_struct_component_assignment`.
 const irep_idt &require_goto_statements::require_struct_component_assignment(
   const irep_idt &structure_name,
   const optionalt<irep_idt> &superclass_name,
   const irep_idt &component_name,
   const irep_idt &component_type_name,
   const optionalt<irep_idt> &typecast_name,
-  const std::vector<codet> &entry_point_instructions)
+  const std::vector<codet> &entry_point_instructions,
+  const symbol_tablet &symbol_table)
 {
+  namespacet ns(symbol_table);
+
   // First we need to find the assignments to the component belonging to
   // the structure_name object
   const auto &component_assignments =
@@ -307,75 +396,48 @@ const irep_idt &require_goto_statements::require_struct_component_assignment(
       entry_point_instructions,
       structure_name,
       superclass_name,
-      component_name);
+      component_name,
+      symbol_table);
+  INFO(
+    "looking for component assignment " << component_name << " in "
+                                        << structure_name);
   REQUIRE(component_assignments.non_null_assignments.size() == 1);
 
-  // We are expecting that the resulting statement can be of two forms:
-  // 1. structure_name.(@superclass_name if given).component =
-  //     (struct cast_type_name *) tmp_object_factory$1;
-  //   followed by a direct assignment like this:
-  //     tmp_object_factory$1 = &tmp_object_factory$2;
-  // 2. structure_name.component = &tmp_object_factory$1;
-  exprt component_assignment_rhs_expr =
-    component_assignments.non_null_assignments[0].rhs();
+  // We are expecting the non-null assignment to be of the form:
+  //   malloc_site->(@superclass_name if given.)field =
+  //     (possible typecast) malloc_site$0;
+  const symbol_exprt *rhs_symbol_expr = expr_try_dynamic_cast<symbol_exprt>(
+    skip_typecast(component_assignments.non_null_assignments[0].rhs()));
+  REQUIRE(rhs_symbol_expr);
 
-  // If the rhs is a typecast (case 1 above), deconstruct it to get the name of
-  // the variable and find the assignment to it
-  if(component_assignment_rhs_expr.id() == ID_typecast)
-  {
-    const auto &component_assignment_rhs =
-      to_typecast_expr(component_assignment_rhs_expr);
-
-    // Check the type we are casting to
-    if(typecast_name.has_value())
-    {
-      REQUIRE(component_assignment_rhs.type().id() == ID_pointer);
-      REQUIRE(
-        to_symbol_type(
-          to_pointer_type(component_assignment_rhs.type()).subtype())
-          .get(ID_identifier) == typecast_name.value());
-    }
-
-    const auto &component_reference_tmp_name =
-      to_symbol_expr(component_assignment_rhs.op()).get_identifier();
-    const auto &component_reference_assignments =
-      require_goto_statements::find_pointer_assignments(
-        component_reference_tmp_name, entry_point_instructions)
-        .non_null_assignments;
-    REQUIRE(component_reference_assignments.size() == 1);
-    component_assignment_rhs_expr = component_reference_assignments[0].rhs();
-  }
-
-  // The rhs assigns an address of a variable, get its name
-  const auto &component_reference_assignment_rhs =
-    to_address_of_expr(component_assignment_rhs_expr);
-  const auto &component_tmp_name =
-    to_symbol_expr(component_reference_assignment_rhs.op()).get_identifier();
+  const irep_idt &symbol_identifier = get_ultimate_source_symbol(
+    entry_point_instructions, rhs_symbol_expr->get_identifier());
 
   // After we have found the declaration of the final assignment's
   // right hand side, then we want to identify that the type
   // is the one we expect, e.g.:
-  // struct java.lang.Integer { __CPROVER_string @class_identifier; }
-  // tmp_object_factory$2;
+  // struct java.lang.Integer *malloc_site$0;
   const auto &component_declaration =
     require_goto_statements::require_declaration_of_name(
-      component_tmp_name, entry_point_instructions);
-  REQUIRE(component_declaration.symbol().type().id() == ID_struct);
+      symbol_identifier, entry_point_instructions);
+  const typet &component_type =
+    to_pointer_type(component_declaration.symbol().type()).subtype();
+  REQUIRE(component_type.id() == ID_struct_tag);
   const auto &component_struct =
-    to_struct_type(component_declaration.symbol().type());
+    ns.follow_tag(to_struct_tag_type(component_type));
   REQUIRE(component_struct.get(ID_name) == component_type_name);
 
-  return component_tmp_name;
+  return symbol_identifier;
 }
 
 /// Checks that the array component of the structure (possibly inherited from
 /// the superclass) is assigned an array with given element type.
-/// \param structure_name The name the variable
-/// \param superclass_name The name of its superclass (if given)
-/// \param array_component_name The name of the array field of the superclass
-/// \param array_type_name The type of the array, e.g., java::array[reference]
-/// \param array_component_element_type_name The element type of the array
-/// \param entry_point_instructions The statements to look through
+/// \param structure_name: The name the variable
+/// \param superclass_name: The name of its superclass (if given)
+/// \param array_component_name: The name of the array field of the superclass
+/// \param array_type_name: The type of the array, e.g., java::array[reference]
+/// \param entry_point_instructions: The statements to look through
+/// \param symbol_table: A symbol table to enable type lookups
 /// \return The identifier of the variable assigned to the field
 const irep_idt &
 require_goto_statements::require_struct_array_component_assignment(
@@ -383,8 +445,8 @@ require_goto_statements::require_struct_array_component_assignment(
   const optionalt<irep_idt> &superclass_name,
   const irep_idt &array_component_name,
   const irep_idt &array_type_name,
-  const irep_idt &array_component_element_type_name,
-  const std::vector<codet> &entry_point_instructions)
+  const std::vector<codet> &entry_point_instructions,
+  const symbol_tablet &symbol_table)
 {
   // First we need to find the assignments to the component belonging to
   // the structure_name object
@@ -393,7 +455,8 @@ require_goto_statements::require_struct_array_component_assignment(
       entry_point_instructions,
       structure_name,
       superclass_name,
-      array_component_name);
+      array_component_name,
+      symbol_table);
   REQUIRE(component_assignments.non_null_assignments.size() == 1);
 
   // We are expecting that the resulting statement is of form:
@@ -413,7 +476,8 @@ require_goto_statements::require_struct_array_component_assignment(
   // Check the type we are casting to matches the array type
   REQUIRE(component_assignment_rhs.type().id() == ID_pointer);
   REQUIRE(
-    to_symbol_type(to_pointer_type(component_assignment_rhs.type()).subtype())
+    to_struct_tag_type(
+      to_pointer_type(component_assignment_rhs.type()).subtype())
       .get(ID_identifier) == array_type_name);
 
   // Get the tmp_object name and find assignments to it, there should be only
@@ -443,8 +507,8 @@ require_goto_statements::require_struct_array_component_assignment(
 
 /// Checks that the input argument (of method under test) with given name is
 /// assigned a single non-null object in the entry point function.
-/// \param argument_name Name of the input argument of method under test
-/// \param entry_point_statements The statements to look through
+/// \param argument_name: Name of the input argument of method under test
+/// \param entry_point_statements: The statements to look through
 /// \return The identifier of the variable assigned to the input argument
 const irep_idt &
 require_goto_statements::require_entry_point_argument_assignment(
@@ -453,10 +517,10 @@ require_goto_statements::require_entry_point_argument_assignment(
 {
   // Trace the creation of the object that is being supplied as the input
   // argument to the function under test
-  const pointer_assignment_locationt &argument_assignments =
+  const pointer_assignment_locationt argument_assignments =
     find_pointer_assignments(
-      id2string(goto_functionst::entry_point()) + "::" +
-        id2string(argument_name),
+      id2string(goto_functionst::entry_point()) +
+        "::" + id2string(argument_name),
       entry_point_statements);
 
   // There should be at most one assignment to it
@@ -464,14 +528,13 @@ require_goto_statements::require_entry_point_argument_assignment(
 
   // The following finds the name of the tmp object that gets assigned
   // to the input argument. There must be one such assignment only,
-  // and usually looks like this:
-  //   argument_name = &tmp_object_factory$1;
+  // and it usually looks like this:
+  //   argument_name = tmp_object_factory$1;
   const auto &argument_assignment =
     argument_assignments.non_null_assignments[0];
-  const auto &argument_tmp_name =
-    to_symbol_expr(to_address_of_expr(argument_assignment.rhs()).object())
-      .get_identifier();
-  return argument_tmp_name;
+  const symbol_exprt &argument_symbol =
+    to_symbol_expr(skip_typecast(argument_assignment.rhs()));
+  return argument_symbol.get_identifier();
 }
 
 /// Verify that a collection of statements contains a function call to a
@@ -502,6 +565,6 @@ std::vector<code_function_callt> require_goto_statements::find_function_calls(
         }
       }
     }
-  }
+    }
   return function_calls;
 }
