@@ -1,15 +1,14 @@
 /*******************************************************************\
 
- Module: Unit test utilities
+Module: Unit test utilities
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
 #include "require_type.h"
 
-#include <testing-utils/catch.hpp>
-#include <util/base_type.h>
+#include <testing-utils/use_catch.h>
 #include <util/namespace.h>
 #include <util/symbol_table.h>
 
@@ -26,18 +25,35 @@ pointer_typet require_type::require_pointer(
   const pointer_typet &pointer = to_pointer_type(type);
 
   if(subtype)
-  {
-    namespacet ns{symbol_tablet{}};
-    base_type_eq(pointer.subtype(), subtype.value(), ns);
-  }
+    REQUIRE(pointer.subtype() == subtype.value());
+
   return pointer;
 }
 
+/// Checks that a class has a component with a specific name
+/// \param java_class_type: The class that should have the component
+/// \param component_name: The name of the component
+/// \return The component with the specified name
+java_class_typet::componentt require_type::require_component(
+  const java_class_typet &java_class_type,
+  const irep_idt &component_name)
+{
+  const auto &component = std::find_if(
+    java_class_type.components().begin(),
+    java_class_type.components().end(),
+    [&component_name](const java_class_typet::componentt &component) {
+      return component.get_name() == component_name;
+    });
+
+  REQUIRE(component != java_class_type.components().end());
+  return *component;
+}
+
 /// Checks a struct like type has a component with a specific name
 /// \param struct_type: The structure that should have the component
 /// \param component_name: The name of the component
 /// \return The component with the specified name
-struct_union_typet::componentt require_type::require_component(
+struct_typet::componentt require_type::require_component(
   const struct_typet &struct_type,
   const irep_idt &component_name)
 {
@@ -63,9 +79,9 @@ code_typet require_type::require_code(const typet &type)
 
 /// Verify a given type is an code_typet, and that the code it represents
 /// accepts a given number of parameters
-/// \param type The type to check
-/// \param num_params check the the given code_typet expects this
-/// number of parameters
+/// \param type: The type to check
+/// \param num_params: check the the given code_typet expects this
+///   number of parameters
 /// \return The type cast to a code_typet
 code_typet
 require_type::require_code(const typet &type, const size_t num_params)
@@ -86,9 +102,9 @@ java_method_typet require_type::require_java_method(const typet &type)
 
 /// Verify a given type is an java_method_typet, and that the code it represents
 /// accepts a given number of parameters
-/// \param type The type to check
-/// \param num_params check the the given java_method_typet expects this
-/// number of parameters
+/// \param type: The type to check
+/// \param num_params: check the the given java_method_typet expects this
+///   number of parameters
 /// \return The type cast to a java_method_typet
 java_method_typet
 require_type::require_java_method(const typet &type, const size_t num_params)
@@ -101,8 +117,8 @@ require_type::require_java_method(const typet &type, const size_t num_params)
 /// Verify that a function has a parameter of a specific name.
 /// \param function_type: The type of the function
 /// \param param_name: The name of the parameter
-/// \return: A reference to the parameter structure corresponding to this
-/// parameter name.
+/// \return A reference to the parameter structure corresponding to this
+///   parameter name.
 code_typet::parametert require_type::require_parameter(
   const code_typet &function_type,
   const irep_idt &param_name)
@@ -120,8 +136,8 @@ code_typet::parametert require_type::require_parameter(
 
 /// Helper function for testing that java generic type arguments match
 /// a given expectation.
-/// \param type_argument The generic type argument to test
-/// \param expected The expected value of the argument
+/// \param type_argument: The generic type argument to test
+/// \param expected: The expected value of the argument
 /// \return true if the generic type argument meets the expectations
 bool require_java_generic_type_argument_expectation(
   const reference_typet &type_argument,
@@ -131,7 +147,7 @@ bool require_java_generic_type_argument_expectation(
   {
   case require_type::type_argument_kindt::Var:
   {
-    REQUIRE(is_java_generic_parameter((type_argument)));
+    REQUIRE(is_java_generic_parameter(type_argument));
     java_generic_parametert parameter =
       to_java_generic_parameter(type_argument);
     REQUIRE(parameter.type_variable().get_identifier() == expected.description);
@@ -140,7 +156,9 @@ bool require_java_generic_type_argument_expectation(
   case require_type::type_argument_kindt::Inst:
   {
     REQUIRE(!is_java_generic_parameter(type_argument));
-    REQUIRE(type_argument.subtype() == symbol_typet(expected.description));
+    REQUIRE(
+      to_struct_tag_type(type_argument.subtype()).get_identifier() ==
+      expected.description);
     return true;
   }
   }
@@ -150,7 +168,7 @@ bool require_java_generic_type_argument_expectation(
 }
 
 /// Verify a given type is a java_generic_type
-/// \param type The type to check
+/// \param type: The type to check
 /// \return The type, cast to a java_generic_typet
 java_generic_typet require_type::require_java_generic_type(const typet &type)
 {
@@ -166,8 +184,8 @@ java_generic_typet require_type::require_java_generic_type(const typet &type)
 ///   {{require_type::type_argument_kindt::Inst, "java::java.lang.Integer"},
 ///    {require_type::type_argument_kindt::Var, "T"}})
 ///
-/// \param type The type to check
-/// \param type_expectations A set of type argument kinds and identifiers
+/// \param type: The type to check
+/// \param type_expectations: A set of type argument kinds and identifiers
 ///  which should be expected as the type arguments of the given generic type.
 /// \return The given type, cast to a java_generic_typet
 java_generic_typet require_type::require_java_generic_type(
@@ -191,7 +209,7 @@ java_generic_typet require_type::require_java_generic_type(
 }
 
 /// Verify a given type is a java_generic_parameter, e.g., `T`
-/// \param type The type to check
+/// \param type: The type to check
 /// \return The type, cast to a java_generic_parametert
 java_generic_parametert
 require_type::require_java_generic_parameter(const typet &type)
@@ -203,8 +221,8 @@ require_type::require_java_generic_parameter(const typet &type)
 /// Verify a given type is a java_generic_parametert with the given name.
 /// Expected usage is something like this:
 /// require_java_generic_parameter(parameter, "java::Generic::T")
-/// \param type The type to check
-/// \param parameter String with the parameter name.
+/// \param type: The type to check
+/// \param parameter: String with the parameter name.
 /// \return The given type, cast to a java_generic_parametert
 java_generic_parametert require_type::require_java_generic_parameter(
   const typet &type,
@@ -221,13 +239,13 @@ java_generic_parametert require_type::require_java_generic_parameter(
 }
 
 /// Test a type to ensure it is not a java generics type.
-/// \param type The type to test
-/// \param expect_subtype Optionally, also test that the subtype of the given
-/// type matches this parameter
+/// \param type: The type to test
+/// \param expect_subtype: Optionally, also test that the subtype of the given
+///   type matches this parameter
 /// \return The value passed in the first argument
 const typet &require_type::require_java_non_generic_type(
   const typet &type,
-  const optionalt<symbol_typet> &expect_subtype)
+  const optionalt<struct_tag_typet> &expect_subtype)
 {
   REQUIRE(!is_java_generic_parameter(type));
   REQUIRE(!is_java_generic_type(type));
@@ -237,36 +255,36 @@ const typet &require_type::require_java_non_generic_type(
 }
 
 /// Checks that the given type is a complete class.
-/// \param class_type type of the class
+/// \param class_type: type of the class
 /// \return class_type of the class
 class_typet require_type::require_complete_class(const typet &class_type)
 {
   REQUIRE(class_type.id() == ID_struct);
 
-  const class_typet &class_class_type = to_class_type(class_type);
-  REQUIRE(class_class_type.is_class());
-  REQUIRE_FALSE(class_class_type.get_bool(ID_incomplete_class));
+  const java_class_typet &java_class_type = to_java_class_type(class_type);
+  REQUIRE(java_class_type.is_class());
+  REQUIRE_FALSE(java_class_type.get_is_stub());
 
-  return class_class_type;
+  return java_class_type;
 }
 
 /// Checks that the given type is an incomplete class.
-/// \param class_type type of the class
+/// \param class_type: type of the class
 /// \return class_type of the class
 class_typet require_type::require_incomplete_class(const typet &class_type)
 {
   REQUIRE(class_type.id() == ID_struct);
 
-  const class_typet &class_class_type = to_class_type(class_type);
-  REQUIRE(class_class_type.is_class());
-  REQUIRE(class_class_type.get_bool(ID_incomplete_class));
+  const java_class_typet &java_class_type = to_java_class_type(class_type);
+  REQUIRE(java_class_type.is_class());
+  REQUIRE(java_class_type.get_is_stub());
 
-  return class_class_type;
+  return java_class_type;
 }
 
 /// Verify that a class is a valid java generic class.
 /// \param class_type: the class
-/// \return: A reference to the java generic class type.
+/// \return A reference to the java generic class type.
 java_generic_class_typet
 require_type::require_java_generic_class(const typet &class_type)
 {
@@ -287,7 +305,7 @@ require_type::require_java_generic_class(const typet &class_type)
 /// specified list of variables.
 /// \param class_type: the class
 /// \param type_variables: vector of type variables
-/// \return: A reference to the java generic class type.
+/// \return A reference to the java generic class type.
 java_generic_class_typet require_type::require_java_generic_class(
   const typet &class_type,
   const std::initializer_list<irep_idt> &type_variables)
@@ -315,7 +333,7 @@ java_generic_class_typet require_type::require_java_generic_class(
 
 /// Verify that a class is a complete, valid java generic class.
 /// \param class_type: the class
-/// \return: A reference to the java generic class type.
+/// \return A reference to the java generic class type.
 java_generic_class_typet
 require_type::require_complete_java_generic_class(const typet &class_type)
 {
@@ -327,7 +345,7 @@ require_type::require_complete_java_generic_class(const typet &class_type)
 /// specified list of variables.
 /// \param class_type: the class
 /// \param type_variables: vector of type variables
-/// \return: A reference to the java generic class type.
+/// \return A reference to the java generic class type.
 java_generic_class_typet require_type::require_complete_java_generic_class(
   const typet &class_type,
   const std::initializer_list<irep_idt> &type_variables)
@@ -338,7 +356,7 @@ java_generic_class_typet require_type::require_complete_java_generic_class(
 
 /// Verify that a class is a valid java implicitly generic class.
 /// \param class_type: the class
-/// \return: A reference to the java generic class type.
+/// \return A reference to the java generic class type.
 java_implicitly_generic_class_typet
 require_type::require_java_implicitly_generic_class(const typet &class_type)
 {
@@ -359,8 +377,8 @@ require_type::require_java_implicitly_generic_class(const typet &class_type)
 /// Verify that a class is a valid java generic class with the
 /// specified list of variables.
 /// \param class_type: the class
-/// \param type_variables: vector of type variables
-/// \return: A reference to the java generic class type.
+/// \param implicit_type_variables: vector of type variables
+/// \return A reference to the java generic class type.
 java_implicitly_generic_class_typet
 require_type::require_java_implicitly_generic_class(
   const typet &class_type,
@@ -374,24 +392,19 @@ require_type::require_java_implicitly_generic_class(
     &implicit_generic_type_vars =
       java_implicitly_generic_class_type.implicit_generic_types();
   REQUIRE(implicit_generic_type_vars.size() == implicit_type_variables.size());
-  REQUIRE(
-    std::equal(
-      implicit_type_variables.begin(),
-      implicit_type_variables.end(),
-      implicit_generic_type_vars.begin(),
-      [](
-        const irep_idt &type_var_name,
-        const java_generic_parametert &param) { //NOLINT
-        REQUIRE(is_java_generic_parameter(param));
-        return param.type_variable().get_identifier() == type_var_name;
-      }));
-
+  auto param = implicit_generic_type_vars.begin();
+  auto type_var_name = implicit_type_variables.begin();
+  for(; param != implicit_generic_type_vars.end(); ++param, ++type_var_name)
+  {
+    REQUIRE(is_java_generic_parameter(*param));
+    REQUIRE(param->type_variable().get_identifier() == *type_var_name);
+  }
   return java_implicitly_generic_class_type;
 }
 
 /// Verify that a class is a complete, valid java implicitly generic class.
 /// \param class_type: the class
-/// \return: A reference to the java generic class type.
+/// \return A reference to the java generic class type.
 java_implicitly_generic_class_typet
 require_type::require_complete_java_implicitly_generic_class(
   const typet &class_type)
@@ -403,8 +416,8 @@ require_type::require_complete_java_implicitly_generic_class(
 /// Verify that a class is a complete, valid java generic class with the
 /// specified list of variables.
 /// \param class_type: the class
-/// \param type_variables: vector of type variables
-/// \return: A reference to the java generic class type.
+/// \param implicit_type_variables: vector of type variables
+/// \return A reference to the java generic class type.
 java_implicitly_generic_class_typet
 require_type::require_complete_java_implicitly_generic_class(
   const typet &class_type,
@@ -417,7 +430,7 @@ require_type::require_complete_java_implicitly_generic_class(
 
 /// Verify that a class is a valid nongeneric java class
 /// \param class_type: the class
-/// \return: A reference to the java generic class type.
+/// \return A reference to the java generic class type.
 java_class_typet
 require_type::require_java_non_generic_class(const typet &class_type)
 {
@@ -435,7 +448,7 @@ require_type::require_java_non_generic_class(const typet &class_type)
 
 /// Verify that a class is a complete, valid nongeneric java class
 /// \param class_type: the class
-/// \return: A reference to the java generic class type.
+/// \return A reference to the java generic class type.
 java_class_typet
 require_type::require_complete_java_non_generic_class(const typet &class_type)
 {
@@ -447,51 +460,61 @@ require_type::require_complete_java_non_generic_class(const typet &class_type)
 /// \param type: The type to check
 /// \param identifier: The identifier the symbol type should have
 /// \return The cast version of the input type
-const symbol_typet &
-require_type::require_symbol(const typet &type, const irep_idt &identifier)
+const struct_tag_typet &
+require_type::require_struct_tag(const typet &type, const irep_idt &identifier)
 {
-  REQUIRE(type.id() == ID_symbol);
-  const symbol_typet &result = to_symbol_type(type);
-  if(identifier != "")
+  REQUIRE(type.id() == ID_struct_tag);
+  const struct_tag_typet &result = to_struct_tag_type(type);
+  if(!identifier.empty())
   {
     REQUIRE(result.get_identifier() == identifier);
   }
   return result;
 }
 
+pointer_typet
+require_type::require_pointer_to_tag(const typet &type, const irep_idt &tag)
+{
+  const auto pointer_type = require_type::require_pointer(type, {});
+  require_type::require_struct_tag(pointer_type.subtype(), tag);
+  return pointer_type;
+}
+
 /// Verify a given type is a java generic symbol type
-/// \param type The type to check
-/// \param identifier The identifier to match
-/// \return The type, cast to a java_generic_symbol_typet
-java_generic_symbol_typet require_type::require_java_generic_symbol_type(
+/// \param type: The type to check
+/// \param identifier: The identifier to match
+/// \return The type, cast to a java_generic_struct_tag_typet
+java_generic_struct_tag_typet
+require_type::require_java_generic_struct_tag_type(
   const typet &type,
   const std::string &identifier)
 {
-  symbol_typet symbol_type = require_symbol(type, identifier);
-  REQUIRE(is_java_generic_symbol_type(type));
-  return to_java_generic_symbol_type(type);
+  struct_tag_typet struct_tag_type = require_struct_tag(type, identifier);
+  REQUIRE(is_java_generic_struct_tag_type(type));
+  return to_java_generic_struct_tag_type(type);
 }
 
 /// Verify a given type is a java generic symbol type, checking
 /// that it's associated type arguments match a given set of identifiers.
 /// Expected usage is something like this:
 ///
-/// require_java_generic_symbol_type(type, "java::Generic",
+/// require_java_generic_struct_tag_type(type, "java::Generic",
 ///   {{require_type::type_argument_kindt::Inst, "java::java.lang.Integer"},
 ///    {require_type::type_argument_kindt::Var, "T"}})
 ///
-/// \param type The type to check
-/// \param identifier The identifier to match
-/// \param type_expectations A set of type argument kinds and identifiers
+/// \param type: The type to check
+/// \param identifier: The identifier to match
+/// \param type_expectations: A set of type argument kinds and identifiers
 ///  which should be expected as the type arguments of the given generic type
-/// \return The given type, cast to a java_generic_symbol_typet
-java_generic_symbol_typet require_type::require_java_generic_symbol_type(
+/// \return The given type, cast to a java_generic_struct_tag_typet
+java_generic_struct_tag_typet
+require_type::require_java_generic_struct_tag_type(
   const typet &type,
   const std::string &identifier,
   const require_type::expected_type_argumentst &type_expectations)
 {
-  const java_generic_symbol_typet &generic_base_type =
-    require_java_generic_symbol_type(type, identifier);
+  const java_generic_struct_tag_typet &generic_base_type =
+    require_java_generic_struct_tag_type(type, identifier);
 
   const java_generic_typet::generic_type_argumentst &generic_type_arguments =
     generic_base_type.generic_types();
@@ -508,8 +531,8 @@ java_generic_symbol_typet require_type::require_java_generic_symbol_type(
 
 /// Verify that the lambda method handles of a class match the given
 /// expectation.
-/// \param class_struct class type to be verified
-/// \param expected_identifiers expected list of lambda method handle
+/// \param class_type: class type to be verified
+/// \param expected_identifiers: expected list of lambda method handle
 ///   references
 /// \return lambda method handles of the class
 require_type::java_lambda_method_handlest
@@ -521,15 +544,14 @@ require_type::require_lambda_method_handles(
     class_type.lambda_method_handles();
   REQUIRE(lambda_method_handles.size() == expected_identifiers.size());
 
-  REQUIRE(
-    std::equal(
-      lambda_method_handles.begin(),
-      lambda_method_handles.end(),
-      expected_identifiers.begin(),
-      [](
-        const symbol_exprt &lambda_method_handle,
-        const std::string &expected_identifier) { //NOLINT
-        return lambda_method_handle.get_identifier() == expected_identifier;
-      }));
+  REQUIRE(std::equal(
+    lambda_method_handles.begin(),
+    lambda_method_handles.end(),
+    expected_identifiers.begin(),
+    [](
+      const irept &lambda_method_handle,
+      const std::string &expected_identifier) { //NOLINT
+      return lambda_method_handle.id() == expected_identifier;
+    }));
   return lambda_method_handles;
 }
