@@ -1,8 +1,8 @@
 /*******************************************************************\
 
- Module: Unit test utilities
+Module: Unit test utilities
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
@@ -25,8 +25,16 @@ namespace require_type
 pointer_typet
 require_pointer(const typet &type, const optionalt<typet> &subtype);
 
-const symbol_typet &
-require_symbol(const typet &type, const irep_idt &identifier = "");
+const struct_tag_typet &
+require_struct_tag(const typet &type, const irep_idt &identifier = "");
+
+pointer_typet require_pointer_to_tag(
+  const typet &type,
+  const irep_idt &identifier = irep_idt());
+
+java_class_typet::componentt require_component(
+  const java_class_typet &java_class_type,
+  const irep_idt &component_name);
 
 struct_typet::componentt require_component(
   const struct_typet &struct_type,
@@ -71,7 +79,7 @@ java_generic_parametert require_java_generic_parameter(
 
 const typet &require_java_non_generic_type(
   const typet &type,
-  const optionalt<symbol_typet> &expect_subtype);
+  const optionalt<struct_tag_typet> &expect_subtype);
 
 class_typet require_complete_class(const typet &class_type);
 
@@ -110,11 +118,11 @@ java_class_typet require_java_non_generic_class(const typet &class_type);
 java_class_typet
 require_complete_java_non_generic_class(const typet &class_type);
 
-java_generic_symbol_typet require_java_generic_symbol_type(
+java_generic_struct_tag_typet require_java_generic_struct_tag_type(
   const typet &type,
   const std::string &identifier);
 
-java_generic_symbol_typet require_java_generic_symbol_type(
+java_generic_struct_tag_typet require_java_generic_struct_tag_type(
   const typet &type,
   const std::string &identifier,
   const require_type::expected_type_argumentst &type_expectations);
