@@ -1,13 +1,13 @@
 /*******************************************************************\
 
- Module: Unit tests for class_hierarchyt output functions
+Module: Unit tests for class_hierarchyt output functions
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
-#include <testing-utils/catch.hpp>
 #include <java-testing-utils/load_java_class.h>
+#include <testing-utils/use_catch.h>
 
 #include <goto-programs/class_hierarchy.h>
 
