@@ -1,16 +1,16 @@
 /*******************************************************************\
 
- Module: Unit tests for instantiating generic classes.
+Module: Unit tests for instantiating generic classes.
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
-#include <testing-utils/catch.hpp>
 #include <java-testing-utils/load_java_class.h>
 #include <java-testing-utils/require_goto_statements.h>
-#include <util/config.h>
 #include <java-testing-utils/require_type.h>
 #include <testing-utils/require_symbol.h>
+#include <testing-utils/use_catch.h>
+#include <util/config.h>
 
 // NOTE: To inspect these tests at any point, use expr2java.
 // A good way to verify the validity of a test is to iterate
@@ -21,6 +21,8 @@ SCENARIO(
   "Instantiate generic parameters to methods or fields used within",
   "[core][goto_program_generics][generic_parameters_test]")
 {
+  config.ansi_c.set_LP64();
+
   GIVEN("A class with a generic field")
   {
     const symbol_tablet &symbol_table = load_java_class(
@@ -39,7 +41,7 @@ SCENARIO(
       //   this = &tmp_object_factory$1;
       const irep_idt &tmp_object_name =
         require_goto_statements::require_entry_point_argument_assignment(
-          "this", entry_point_code);
+          ID_this, entry_point_code);
 
       THEN("Object 'this' has field 'field_input' of type Wrapper")
       {
@@ -50,7 +52,8 @@ SCENARIO(
             "field_input",
             "java::Wrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
 
         THEN("Object 'field_input' has field 'field' of type IWrapper")
         {
@@ -60,7 +63,8 @@ SCENARIO(
             "field",
             "java::IWrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
         THEN("Object 'field_input' has field 'array_field' of type IWrapper[]")
         {
@@ -68,9 +72,9 @@ SCENARIO(
             field_input_name,
             {},
             "array_field",
-            "java::array[reference]",
-            "java::IWrapper",
-            entry_point_code);
+            JAVA_REFERENCE_ARRAY_CLASSID,
+            entry_point_code,
+            symbol_table);
         }
       }
     }
@@ -100,7 +104,7 @@ SCENARIO(
       // for, so we need to extract its identifier, and start looking for that.
       const auto &tmp_object_name =
         require_goto_statements::require_entry_point_argument_assignment(
-          "this", entry_point_code);
+          ID_this, entry_point_code);
 
       THEN("Type of multiple generic fields should be right")
       {
@@ -114,7 +118,7 @@ SCENARIO(
         const java_generic_typet &type =
           require_type::require_java_generic_type(component.type());
         require_type::require_pointer(
-          type.generic_type_arguments()[0], symbol_typet{"java::BWrapper"});
+          type.generic_type_arguments()[0], struct_tag_typet{"java::BWrapper"});
       }
 
       THEN("Object 'this' has field 'field_input1' of type Wrapper")
@@ -126,7 +130,8 @@ SCENARIO(
             "field_input1",
             "java::Wrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
 
         THEN("Object 'field_input1' has field 'field' of type IWrapper")
         {
@@ -136,7 +141,8 @@ SCENARIO(
             "field",
             "java::IWrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
       }
 
@@ -149,7 +155,8 @@ SCENARIO(
             "field_input2",
             "java::Wrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
 
         THEN("Object 'field_input2' has field 'field' of type BWrapper")
         {
@@ -159,7 +166,8 @@ SCENARIO(
             "field",
             "java::BWrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
       }
     }
@@ -182,7 +190,7 @@ SCENARIO(
 
       const auto &tmp_object_name =
         require_goto_statements::require_entry_point_argument_assignment(
-          "this", entry_point_code);
+          ID_this, entry_point_code);
 
       THEN("Object 'this' has field 'field_input1' of type Wrapper")
       {
@@ -193,7 +201,8 @@ SCENARIO(
             "field_input1",
             "java::Wrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
 
         THEN("Object 'field_input1' has field 'field' of type Wrapper")
         {
@@ -204,12 +213,19 @@ SCENARIO(
               "field",
               "java::Wrapper",
               {},
-              entry_point_code);
+              entry_point_code,
+              symbol_table);
 
           THEN("Object 'field' has field 'field' of type IWrapper")
           {
             require_goto_statements::require_struct_component_assignment(
-              field_name, {}, "field", "java::IWrapper", {}, entry_point_code);
+              field_name,
+              {},
+              "field",
+              "java::IWrapper",
+              {},
+              entry_point_code,
+              symbol_table);
           }
         }
       }
@@ -232,7 +248,7 @@ SCENARIO(
 
       const auto &tmp_object_name =
         require_goto_statements::require_entry_point_argument_assignment(
-          "this", entry_point_code);
+          ID_this, entry_point_code);
 
       THEN("Object 'this' has field 'field_input' of type PairWrapper")
       {
@@ -243,7 +259,8 @@ SCENARIO(
             "field_input",
             "java::PairWrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
 
         THEN("Object 'field_input' has field 'key' of type IWrapper")
         {
@@ -253,7 +270,8 @@ SCENARIO(
             "first",
             "java::IWrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
 
         THEN("Object 'field_input' has field 'value' of type IWrapper")
@@ -264,7 +282,8 @@ SCENARIO(
             "second",
             "java::IWrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
       }
     }
@@ -299,9 +318,9 @@ SCENARIO(
 
         // Trace the assignments back to the declaration of the generic type
         // and verify that it is what we expect.
-        const auto &tmp_object_struct =
-          to_struct_type(tmp_object_declaration.symbol().type());
-        REQUIRE(tmp_object_struct.get_tag() == "Wrapper");
+        const auto &tmp_object_struct_tag = to_struct_tag_type(
+          to_pointer_type(tmp_object_declaration.symbol().type()).subtype());
+        REQUIRE(tmp_object_struct_tag.get_identifier() == "java::Wrapper");
 
         THEN("Object 'v' has field 'field' of type IWrapper")
         {
@@ -311,7 +330,8 @@ SCENARIO(
             "field",
             "java::IWrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
       }
     }
@@ -348,9 +368,9 @@ SCENARIO(
 
         // Trace the assignments back to the declaration of the generic type
         // and verify that it is what we expect.
-        const auto &tmp_object_struct =
-          to_struct_type(tmp_object_declaration.symbol().type());
-        REQUIRE(tmp_object_struct.get_tag() == "Wrapper");
+        const auto &tmp_object_struct_tag = to_struct_tag_type(
+          to_pointer_type(tmp_object_declaration.symbol().type()).subtype());
+        REQUIRE(tmp_object_struct_tag.get_identifier() == "java::Wrapper");
 
         THEN(
           "Object 'v' has field 'field' of type Object (upper bound of the "
@@ -362,7 +382,8 @@ SCENARIO(
             "field",
             "java::java.lang.Object",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
       }
     }
@@ -397,11 +418,11 @@ SCENARIO(
 
         // Trace the assignments back to the declaration of the generic type
         // and verify that it is what we expect.
-        const auto &tmp_object_struct =
-          to_struct_type(tmp_object_declaration.symbol().type());
+        const auto &tmp_object_struct_tag = to_struct_tag_type(
+          to_pointer_type(tmp_object_declaration.symbol().type()).subtype());
         REQUIRE(
-          tmp_object_struct.get_tag() ==
-          "GenericFields$GenericInnerOuter$Outer");
+          tmp_object_struct_tag.get_identifier() ==
+          "java::GenericFields$GenericInnerOuter$Outer");
 
         THEN("Object 'v' has field 'field' of type InnerClass")
         {
@@ -412,7 +433,8 @@ SCENARIO(
               "field",
               "java::GenericFields$GenericInnerOuter$Outer$InnerClass",
               {},
-              entry_point_code);
+              entry_point_code,
+              symbol_table);
 
           THEN("Object 'field' has field 't' of type Integer")
           {
@@ -422,7 +444,8 @@ SCENARIO(
               "t",
               "java::java.lang.Integer",
               {},
-              entry_point_code);
+              entry_point_code,
+              symbol_table);
           }
         }
       }
@@ -460,11 +483,11 @@ SCENARIO(
 
         // Trace the assignments back to the declaration of the generic type
         // and verify that it is what we expect.
-        const auto &tmp_object_struct =
-          to_struct_type(tmp_object_declaration.symbol().type());
+        const auto &tmp_object_struct_tag = to_struct_tag_type(
+          to_pointer_type(tmp_object_declaration.symbol().type()).subtype());
         REQUIRE(
-          tmp_object_struct.get_tag() ==
-          "GenericFields$GenericRewriteParameter$A");
+          tmp_object_struct_tag.get_identifier() ==
+          "java::GenericFields$GenericRewriteParameter$A");
 
         THEN("Object 'v' has field 'value' of type Integer")
         {
@@ -474,7 +497,8 @@ SCENARIO(
             "value",
             "java::java.lang.Integer",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
 
         THEN("Object 'v' has field 'field' of type A")
@@ -486,7 +510,8 @@ SCENARIO(
               "field",
               "java::GenericFields$GenericRewriteParameter$A",
               {},
-              entry_point_code);
+              entry_point_code,
+              symbol_table);
 
           THEN("Object 'field' has field 'value' of type Boolean")
           {
@@ -496,7 +521,8 @@ SCENARIO(
               "value",
               "java::java.lang.Boolean",
               {},
-              entry_point_code);
+              entry_point_code,
+              symbol_table);
           }
         }
       }
@@ -539,7 +565,7 @@ SCENARIO(
 
       const irep_idt &tmp_object_name =
         require_goto_statements::require_entry_point_argument_assignment(
-          "this", entry_point_code);
+          ID_this, entry_point_code);
 
       THEN("Object 'this' has field 'f' of type UnsupportedWrapper2")
       {
@@ -554,7 +580,8 @@ SCENARIO(
             "f",
             "java::UnsupportedWrapper2",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
 
         THEN("Object 'f' has unspecialized field 'field'")
         {
@@ -567,7 +594,8 @@ SCENARIO(
             "field",
             "java::java.lang.Object",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
       }
     }
@@ -601,7 +629,7 @@ SCENARIO(
 
       const irep_idt &tmp_object_name =
         require_goto_statements::require_entry_point_argument_assignment(
-          "this", entry_point_code);
+          ID_this, entry_point_code);
 
       THEN("Object 'this' has field 'f' of type OpaqueWrapper")
       {
@@ -612,7 +640,8 @@ SCENARIO(
             "f",
             "java::OpaqueWrapper",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
 
         THEN("Object 'f' has unspecialized field 'field'")
         {
@@ -622,7 +651,8 @@ SCENARIO(
             "field",
             "java::java.lang.Object",
             {},
-            entry_point_code);
+            entry_point_code,
+            symbol_table);
         }
       }
     }
