@@ -1,20 +1,20 @@
 /*******************************************************************\
 
- Module: Unit tests for converting invokedynamic instructions into codet
+Module: Unit tests for converting invokedynamic instructions into codet
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
-#include <testing-utils/catch.hpp>
+#include <goto-programs/class_hierarchy.h>
 #include <java-testing-utils/load_java_class.h>
 #include <java-testing-utils/require_goto_statements.h>
-#include <testing-utils/require_expr.h>
 #include <java-testing-utils/require_type.h>
-#include <testing-utils/run_test_with_compilers.h>
+#include <testing-utils/require_expr.h>
 #include <testing-utils/require_symbol.h>
+#include <testing-utils/run_test_with_compilers.h>
+#include <testing-utils/use_catch.h>
 #include <util/expr_iterator.h>
-#include <goto-programs/class_hierarchy.h>
 
 struct lambda_assignment_test_datat
 {
@@ -53,8 +53,7 @@ void validate_lambda_assignment(
   const typecast_exprt &rhs_value =
     require_expr::require_typecast(lambda_assignment.rhs());
 
-  const symbol_exprt &rhs_symbol =
-    require_expr::require_symbol(rhs_value.op0());
+  const symbol_exprt &rhs_symbol = require_expr::require_symbol(rhs_value.op());
 
   const irep_idt &tmp_object_symbol = rhs_symbol.get_identifier();
 
@@ -72,8 +71,8 @@ void validate_lambda_assignment(
   const pointer_typet &lambda_temp_type =
     require_type::require_pointer(side_effect_expr.type(), {});
 
-  const symbol_typet &lambda_implementor_type =
-    require_type::require_symbol(lambda_temp_type.subtype());
+  const struct_tag_typet &lambda_implementor_type =
+    require_type::require_struct_tag(lambda_temp_type.subtype());
 
   const irep_idt &tmp_class_identifier =
     lambda_implementor_type.get_identifier();
@@ -107,12 +106,6 @@ void validate_lambda_assignment(
     Catch::Matchers::Vector::ContainsElementMatcher<irep_idt>{
       tmp_class_identifier});
 
-  const java_class_typet::componentt super_class_component =
-    require_type::require_component(tmp_lambda_class_type, "@java.lang.Object");
-
-  const symbol_typet &super_class_type = require_type::require_symbol(
-    super_class_component.type(), "java::java.lang.Object");
-
   THEN("The function in the class should call the lambda method")
   {
     const irep_idt method_identifier =
@@ -247,11 +240,11 @@ SCENARIO(
 
           symbol_exprt integer_param{"primitive", java_int_type()};
           symbol_exprt ref_param{"reference",
-                                 java_type_from_string("Ljava/lang/Object;")};
+                                 *java_type_from_string("Ljava/lang/Object;")};
           // NOLINTNEXTLINE(whitespace/braces)
           symbol_exprt generic_param{
             "specalisedGeneric",
-            java_type_from_string("LDummyGeneric<Ljava/lang/Interger;>;")};
+            *java_type_from_string("LDummyGeneric<Ljava/lang/Interger;>;")};
           test_data.expected_params = {integer_param, ref_param, generic_param};
           test_data.should_return_value = false;
 
@@ -273,14 +266,14 @@ SCENARIO(
             "java::LocalLambdas.lambda$test$2:"
             "[I[Ljava/lang/Object;[LDummyGeneric;)V";
 
-          symbol_exprt integer_param{"primitive", java_type_from_string("[I")};
+          symbol_exprt integer_param{"primitive", *java_type_from_string("[I")};
           symbol_exprt ref_param{"reference",
-                                 java_type_from_string("[Ljava/lang/Object;")};
+                                 *java_type_from_string("[Ljava/lang/Object;")};
 
           // NOLINTNEXTLINE(whitespace/braces)
           symbol_exprt generic_param{
             "specalisedGeneric",
-            java_type_from_string("[LDummyGeneric<Ljava/lang/Interger;>;")};
+            *java_type_from_string("[LDummyGeneric<Ljava/lang/Interger;>;")};
           test_data.expected_params = {integer_param, ref_param, generic_param};
           test_data.should_return_value = false;
 
@@ -435,11 +428,11 @@ SCENARIO(
 
           symbol_exprt integer_param{"primitive", java_int_type()};
           symbol_exprt ref_param{"reference", // NOLINT(whitespace/braces)
-                                 java_type_from_string("Ljava/lang/Object;")};
+                                 *java_type_from_string("Ljava/lang/Object;")};
           // NOLINTNEXTLINE(whitespace/braces)
           symbol_exprt generic_param{
             "specalisedGeneric",
-            java_type_from_string("LDummyGeneric<Ljava/lang/Interger;>;")};
+            *java_type_from_string("LDummyGeneric<Ljava/lang/Interger;>;")};
           test_data.expected_params = {integer_param, ref_param, generic_param};
 
           test_data.should_return_value = false;
@@ -458,13 +451,13 @@ SCENARIO(
             "java::MemberLambdas.lambda$new$2:"
             "([I[Ljava/lang/Object;[LDummyGeneric;)V";
 
-          symbol_exprt integer_param{"primitive", java_type_from_string("[I")};
+          symbol_exprt integer_param{"primitive", *java_type_from_string("[I")};
           symbol_exprt ref_param{"reference", // NOLINT(whitespace/braces)
-                                 java_type_from_string("[Ljava/lang/Object;")};
+                                 *java_type_from_string("[Ljava/lang/Object;")};
           // NOLINTNEXTLINE(whitespace/braces)
           symbol_exprt generic_param{
             "specalisedGeneric",
-            java_type_from_string("[LDummyGeneric<Ljava/lang/Interger;>;")};
+            *java_type_from_string("[LDummyGeneric<Ljava/lang/Interger;>;")};
           test_data.expected_params = {integer_param, ref_param, generic_param};
           test_data.should_return_value = false;
 
@@ -607,11 +600,11 @@ SCENARIO(
 
           symbol_exprt integer_param{"primitive", java_int_type()};
           symbol_exprt ref_param{"reference",
-                                 java_type_from_string("Ljava/lang/Object;")};
+                                 *java_type_from_string("Ljava/lang/Object;")};
           // NOLINTNEXTLINE(whitespace/braces)
           symbol_exprt generic_param{
             "specalisedGeneric",
-            java_type_from_string("LDummyGeneric<Ljava/lang/Interger;>;")};
+            *java_type_from_string("LDummyGeneric<Ljava/lang/Interger;>;")};
           test_data.expected_params = {integer_param, ref_param, generic_param};
 
           test_data.should_return_value = false;
@@ -630,13 +623,13 @@ SCENARIO(
             "java::StaticLambdas.lambda$static$2:"
             "([I[Ljava/lang/Object;[LDummyGeneric;)V";
 
-          symbol_exprt integer_param{"primitive", java_type_from_string("[I")};
+          symbol_exprt integer_param{"primitive", *java_type_from_string("[I")};
           symbol_exprt ref_param{"reference",
-                                 java_type_from_string("[Ljava/lang/Object;")};
+                                 *java_type_from_string("[Ljava/lang/Object;")};
           // NOLINTNEXTLINE(whitespace/braces)
           symbol_exprt generic_param{
             "specalisedGeneric",
-            java_type_from_string("[LDummyGeneric<Ljava/lang/Interger;>;")};
+            *java_type_from_string("[LDummyGeneric<Ljava/lang/Interger;>;")};
           test_data.expected_params = {integer_param, ref_param, generic_param};
           test_data.should_return_value = false;
 
