@@ -1,14 +1,14 @@
 /*******************************************************************\
 
- Module: Unit tests for parsing generic classes
+Module: Unit tests for parsing generic classes
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
-#include <testing-utils/catch.hpp>
 #include <java-testing-utils/load_java_class.h>
 #include <java-testing-utils/require_type.h>
+#include <testing-utils/use_catch.h>
 
 SCENARIO(
   "parse_generic_class_with_generic_inner_classes_fields",
@@ -36,8 +36,8 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field");
 
-      require_type::require_pointer(
-        field_component.type(), symbol_typet(class_prefix + "$InnerClass"));
+      require_type::require_pointer_to_tag(
+        field_component.type(), class_prefix + "$InnerClass");
 
       THEN("The pointer should be generic")
       {
@@ -54,9 +54,8 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field2");
 
-      require_type::require_pointer(
-        field_component.type(),
-        symbol_typet(class_prefix + "$GenericInnerClass"));
+      require_type::require_pointer_to_tag(
+        field_component.type(), class_prefix + "$GenericInnerClass");
 
       THEN("The pointer should be generic")
       {
@@ -76,9 +75,8 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field3");
 
-      require_type::require_pointer(
-        field_component.type(),
-        symbol_typet(class_prefix + "$GenericInnerClass"));
+      require_type::require_pointer_to_tag(
+        field_component.type(), class_prefix + "$GenericInnerClass");
 
       THEN("The pointer should be generic")
       {
@@ -96,10 +94,9 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field4");
 
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         field_component.type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerClass");
 
       THEN("The pointer should be generic")
       {
@@ -119,10 +116,9 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field5");
 
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         field_component.type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerClass");
 
       THEN("The pointer should be generic")
       {
@@ -140,10 +136,9 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field6");
 
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         field_component.type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass");
 
       THEN("The pointer should be generic")
       {
@@ -165,10 +160,9 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field7");
 
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         field_component.type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass");
 
       THEN("The pointer should be generic")
       {
@@ -187,9 +181,8 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field8");
 
-      require_type::require_pointer(
-        field_component.type(),
-        symbol_typet(class_prefix + "$TwoParamInnerClass"));
+      require_type::require_pointer_to_tag(
+        field_component.type(), class_prefix + "$TwoParamInnerClass");
 
       THEN("The pointer should be generic")
       {
@@ -211,9 +204,8 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field9");
 
-      require_type::require_pointer(
-        field_component.type(),
-        symbol_typet(class_prefix + "$TwoParamInnerClass"));
+      require_type::require_pointer_to_tag(
+        field_component.type(), class_prefix + "$TwoParamInnerClass");
 
       THEN("The pointer should be generic")
       {
@@ -235,9 +227,8 @@ SCENARIO(
       const struct_typet::componentt &field_component =
         require_type::require_component(java_generic_class, "field10");
 
-      require_type::require_pointer(
-        field_component.type(),
-        symbol_typet(class_prefix + "$TwoParamInnerClass"));
+      require_type::require_pointer_to_tag(
+        field_component.type(), class_prefix + "$TwoParamInnerClass");
 
       THEN("The pointer should be generic")
       {
@@ -261,10 +252,9 @@ SCENARIO(
       //   GenericInnerClass<GenericClassWithGenericInnerClasses<Integer>>.
       //     DoublyNestedInnerGenericClass<T> field11;
 
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         field_component.type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass");
 
       THEN("The pointer should be GenericClassWithGenericInnerClasses")
       {
@@ -323,8 +313,8 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
-        param_type.type(), symbol_typet(class_prefix + "$InnerClass"));
+      require_type::require_pointer_to_tag(
+        param_type.type(), class_prefix + "$InnerClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"}});
@@ -352,16 +342,16 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
-        param_type.type(), symbol_typet(class_prefix + "$InnerClass"));
+      require_type::require_pointer_to_tag(
+        param_type.type(), class_prefix + "$InnerClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"}});
 
       const auto &param_type2 =
         require_type::require_parameter(function_call, "input2");
-      require_type::require_pointer(
-        param_type2.type(), symbol_typet(class_prefix + "$InnerClass"));
+      require_type::require_pointer_to_tag(
+        param_type2.type(), class_prefix + "$InnerClass");
       require_type::require_java_generic_type(
         param_type2.type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"}});
@@ -388,8 +378,8 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
-        param_type.type(), symbol_typet(class_prefix + "$GenericInnerClass"));
+      require_type::require_pointer_to_tag(
+        param_type.type(), class_prefix + "$GenericInnerClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var,
@@ -419,8 +409,8 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
-        param_type.type(), symbol_typet(class_prefix + "$GenericInnerClass"));
+      require_type::require_pointer_to_tag(
+        param_type.type(), class_prefix + "$GenericInnerClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"},
@@ -449,10 +439,9 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         param_type.type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var,
@@ -483,10 +472,9 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         param_type.type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"},
@@ -515,10 +503,9 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         param_type.type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var,
@@ -551,10 +538,9 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         param_type.type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"},
@@ -583,8 +569,8 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
-        param_type.type(), symbol_typet(class_prefix + "$TwoParamInnerClass"));
+      require_type::require_pointer_to_tag(
+        param_type.type(), class_prefix + "$TwoParamInnerClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var,
@@ -616,8 +602,8 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
-        param_type.type(), symbol_typet(class_prefix + "$TwoParamInnerClass"));
+      require_type::require_pointer_to_tag(
+        param_type.type(), class_prefix + "$TwoParamInnerClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var,
@@ -649,8 +635,8 @@ SCENARIO(
     {
       const auto &param_type =
         require_type::require_parameter(function_call, "input");
-      require_type::require_pointer(
-        param_type.type(), symbol_typet(class_prefix + "$TwoParamInnerClass"));
+      require_type::require_pointer_to_tag(
+        param_type.type(), class_prefix + "$TwoParamInnerClass");
       require_type::require_java_generic_type(
         param_type.type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"},
@@ -677,9 +663,8 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
-        function_call.return_type(),
-        symbol_typet(class_prefix + "$InnerClass"));
+      require_type::require_pointer_to_tag(
+        function_call.return_type(), class_prefix + "$InnerClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"}});
@@ -704,9 +689,8 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
-        function_call.return_type(),
-        symbol_typet(class_prefix + "$GenericInnerClass"));
+      require_type::require_pointer_to_tag(
+        function_call.return_type(), class_prefix + "$GenericInnerClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var,
@@ -734,9 +718,8 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
-        function_call.return_type(),
-        symbol_typet(class_prefix + "$GenericInnerClass"));
+      require_type::require_pointer_to_tag(
+        function_call.return_type(), class_prefix + "$GenericInnerClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"},
@@ -763,10 +746,10 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         function_call.return_type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerClass"));
+
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var,
@@ -795,10 +778,9 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         function_call.return_type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"},
@@ -825,10 +807,9 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         function_call.return_type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var,
@@ -859,10 +840,9 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
+      require_type::require_pointer_to_tag(
         function_call.return_type(),
-        symbol_typet(
-          class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass"));
+        class_prefix + "$GenericInnerClass$DoublyNestedInnerGenericClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"},
@@ -890,9 +870,8 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
-        function_call.return_type(),
-        symbol_typet(class_prefix + "$TwoParamInnerClass"));
+      require_type::require_pointer_to_tag(
+        function_call.return_type(), class_prefix + "$TwoParamInnerClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var,
@@ -922,9 +901,8 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
-        function_call.return_type(),
-        symbol_typet(class_prefix + "$TwoParamInnerClass"));
+      require_type::require_pointer_to_tag(
+        function_call.return_type(), class_prefix + "$TwoParamInnerClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var,
@@ -954,9 +932,8 @@ SCENARIO(
 
     THEN("The return type should be correct")
     {
-      require_type::require_pointer(
-        function_call.return_type(),
-        symbol_typet(class_prefix + "$TwoParamInnerClass"));
+      require_type::require_pointer_to_tag(
+        function_call.return_type(), class_prefix + "$TwoParamInnerClass");
       require_type::require_java_generic_type(
         function_call.return_type(),
         {{require_type::type_argument_kindt::Var, class_prefix + "::T"},
