@@ -1,14 +1,14 @@
 /*******************************************************************\
 
- Module: Unit tests for parsing generic classes
+Module: Unit tests for parsing generic classes
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
-#include <testing-utils/catch.hpp>
 #include <java-testing-utils/load_java_class.h>
 #include <java-testing-utils/require_type.h>
+#include <testing-utils/use_catch.h>
 
 SCENARIO(
   "parse_signature_descriptor_mismatch",
@@ -27,9 +27,8 @@ SCENARIO(
     REQUIRE(new_symbol_table.has_symbol(inner_prefix));
 
     const symbolt &inner_symbol = new_symbol_table.lookup_ref(inner_prefix);
-    const class_typet &inner_class_type =
-      require_type::require_complete_java_implicitly_generic_class(
-        inner_symbol.type);
+    require_type::require_complete_java_implicitly_generic_class(
+      inner_symbol.type);
   }
 
   THEN(
