@@ -1,8 +1,8 @@
 /*******************************************************************\
 
- Module: Unit tests for parsing generic classes
+Module: Unit tests for parsing generic classes
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
@@ -13,8 +13,8 @@
 #include <java-testing-utils/require_parse_tree.h>
 
 #include <java_bytecode/java_bytecode_parser.h>
-#include <testing-utils/catch.hpp>
 #include <testing-utils/message.h>
+#include <testing-utils/use_catch.h>
 
 #include <java_bytecode/java_bytecode_parse_tree.h>
 #include <java_bytecode/java_types.h>
@@ -35,6 +35,7 @@ SCENARIO(
       optionalt<java_bytecode_parse_treet> parse_tree = java_bytecode_parse(
         "./java_bytecode/java_bytecode_parse_lambdas/lambda_examples/" +
           compiler + "_classes/StaticLambdas.class",
+        "StaticLambdas",
         null_message_handler);
       WHEN("Parsing that class")
       {
@@ -285,7 +286,7 @@ SCENARIO(
           REQUIRE(id2string(lambda_method.descriptor) == method_type);
 
           const reference_typet dummy_generic_reference_type =
-            java_reference_type(symbol_typet{"java.lang.Object"});
+            java_reference_type(struct_tag_typet{"java.lang.Object"});
 
           // NOLINTNEXTLINE(whitespace/braces)
           fieldref_exprt fieldref{
@@ -318,7 +319,7 @@ SCENARIO(
           REQUIRE(id2string(lambda_method.descriptor) == method_type);
 
           const reference_typet dummy_generic_reference_type =
-            java_reference_type(symbol_typet{"DummyGeneric"});
+            java_reference_type(struct_tag_typet{"DummyGeneric"});
 
           fieldref_exprt fieldref{dummy_generic_reference_type,
                                   "staticSpecalisedGeneric",
@@ -345,6 +346,7 @@ SCENARIO(
         optionalt<java_bytecode_parse_treet> parse_tree = java_bytecode_parse(
           "./java_bytecode/java_bytecode_parse_lambdas/lambda_examples/" +
             compiler + "_classes/LocalLambdas.class",
+          "LocalLambdas",
           null_message_handler);
         WHEN("Parsing that class")
         {
@@ -653,6 +655,7 @@ SCENARIO(
         optionalt<java_bytecode_parse_treet> parse_tree = java_bytecode_parse(
           "./java_bytecode/java_bytecode_parse_lambdas/lambda_examples/" +
             compiler + "_classes/MemberLambdas.class",
+          "MemberLambdas",
           null_message_handler);
         WHEN("Parsing that class")
         {
@@ -912,7 +915,7 @@ SCENARIO(
             REQUIRE_FALSE(lambda_method.is_static);
 
             const reference_typet dummy_generic_reference_type =
-              java_reference_type(symbol_typet{"java.lang.Object"});
+              java_reference_type(struct_tag_typet{"java.lang.Object"});
 
             // NOLINTNEXTLINE(whitespace/braces)
             const fieldref_exprt reference_fieldref{
@@ -951,7 +954,7 @@ SCENARIO(
             REQUIRE_FALSE(lambda_method.is_static);
 
             const reference_typet dummy_generic_reference_type =
-              java_reference_type(symbol_typet{"DummyGeneric"});
+              java_reference_type(struct_tag_typet{"DummyGeneric"});
 
             // NOLINTNEXTLINE(whitespace/braces)
             const fieldref_exprt generic_reference_fieldref{
@@ -987,6 +990,7 @@ SCENARIO(
         optionalt<java_bytecode_parse_treet> parse_tree = java_bytecode_parse(
           "./java_bytecode/java_bytecode_parse_lambdas/lambda_examples/" +
             compiler + "_classes/OuterMemberLambdas$Inner.class",
+          "OuterMemberLambdas$Inner",
           null_message_handler);
         WHEN("Parsing that class")
         {
@@ -999,7 +1003,7 @@ SCENARIO(
 
           // Field ref for getting the outer class
           const reference_typet outer_class_reference_type =
-            java_reference_type(symbol_typet{"OuterMemberLambdas"});
+            java_reference_type(struct_tag_typet{"OuterMemberLambdas"});
           // NOLINTNEXTLINE(whitespace/braces)
           const fieldref_exprt outer_fieldref{
             outer_class_reference_type, "this$0", "OuterMemberLambdas$Inner"};
@@ -1065,7 +1069,7 @@ SCENARIO(
             REQUIRE_FALSE(lambda_method.is_static);
 
             const reference_typet dummy_generic_reference_type =
-              java_reference_type(symbol_typet{"java.lang.Object"});
+              java_reference_type(struct_tag_typet{"java.lang.Object"});
 
             // NOLINTNEXTLINE(whitespace/braces)
             const fieldref_exprt reference_fieldref{
@@ -1107,7 +1111,7 @@ SCENARIO(
             REQUIRE_FALSE(lambda_method.is_static);
 
             const reference_typet dummy_generic_reference_type =
-              java_reference_type(symbol_typet{"DummyGeneric"});
+              java_reference_type(struct_tag_typet{"DummyGeneric"});
 
             // NOLINTNEXTLINE(whitespace/braces)
             const fieldref_exprt generic_reference_fieldref{
