@@ -1,8 +1,8 @@
 /*******************************************************************\
 
- Module: Unit tests for converting annotations
+Module: Unit tests for converting annotations
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
@@ -11,7 +11,7 @@
 #include <java_bytecode/java_bytecode_convert_method.h>
 #include <java_bytecode/java_bytecode_parse_tree.h>
 #include <java_bytecode/java_types.h>
-#include <testing-utils/catch.hpp>
+#include <testing-utils/use_catch.h>
 
 SCENARIO(
   "java_bytecode_parse_attributes",
@@ -600,14 +600,14 @@ SCENARIO(
 
   GIVEN("A method that may or may not throw exceptions")
   {
-    const symbol_tablet &new_symbol_table = load_java_class(
+    const symbol_tablet &new_symbol_table2 = load_java_class(
       "ThrowsExceptions", "./java_bytecode/java_bytecode_parser");
     WHEN("Parsing the exceptions attribute for a method that throws exceptions")
     {
       THEN("We should be able to get the list of exceptions it throws")
       {
         const symbolt &method_symbol =
-          new_symbol_table.lookup_ref("java::ThrowsExceptions.test:()V");
+          new_symbol_table2.lookup_ref("java::ThrowsExceptions.test:()V");
         const java_method_typet method =
           to_java_method_type(method_symbol.type);
         const std::vector<irep_idt> exceptions = method.throws_exceptions();
@@ -629,7 +629,7 @@ SCENARIO(
     {
       THEN("We should be able to get the list of exceptions it throws")
       {
-        const symbolt &method_symbol = new_symbol_table.lookup_ref(
+        const symbolt &method_symbol = new_symbol_table2.lookup_ref(
           "java::ThrowsExceptions.testNoExceptions:()V");
         const java_method_typet method =
           to_java_method_type(method_symbol.type);
