@@ -1,22 +1,23 @@
 /*******************************************************************\
 
- Module: Java string library preprocess.
-         Test for converting an expression to a string expression.
+Module: Java string library preprocess.
+        Test for converting an expression to a string expression.
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
-#include <testing-utils/catch.hpp>
-#include <util/expr.h>
-#include <util/std_code.h>
-#include <util/namespace.h>
-#include <java_bytecode/java_object_factory.h>
+#include <iostream>
 #include <java_bytecode/java_bytecode_language.h>
+#include <java_bytecode/java_object_factory.h>
 #include <java_bytecode/java_root_class.h>
 #include <langapi/language_util.h>
 #include <langapi/mode.h>
-#include <iostream>
+#include <testing-utils/message.h>
+#include <testing-utils/use_catch.h>
+#include <util/expr.h>
+#include <util/namespace.h>
+#include <util/std_code.h>
 
 SCENARIO(
   "Generate string object",
@@ -29,9 +30,12 @@ SCENARIO(
     register_language(new_java_bytecode_language);
 
     // Add java.lang.Object to symbol table
+    java_class_typet jlo_class_type;
+    jlo_class_type.set_tag("java.lang.Object");
+    jlo_class_type.set_name("java::java.lang.Object");
     symbolt jlo_sym;
     jlo_sym.name = "java::java.lang.Object";
-    jlo_sym.type = struct_typet();
+    jlo_sym.type = jlo_class_type;
     jlo_sym.is_type = true;
     java_root_class(jlo_sym);
     bool failed = symbol_table.add(jlo_sym);
@@ -43,10 +47,10 @@ SCENARIO(
     namespacet ns(symbol_table);
 
     // Declare a String named arg
-    symbol_typet java_string_type("java::java.lang.String");
+    struct_tag_typet java_string_type("java::java.lang.String");
     symbol_exprt expr("arg", java_string_type);
 
-    object_factory_parameterst object_factory_parameters;
+    java_object_factory_parameterst object_factory_parameters;
     object_factory_parameters.max_nondet_string_length = 20;
     object_factory_parameters.function_id = "test";
 
@@ -59,9 +63,10 @@ SCENARIO(
         symbol_table,
         loc,
         false,
-        allocation_typet::DYNAMIC,
+        lifetimet::DYNAMIC,
         object_factory_parameters,
-        update_in_placet::NO_UPDATE_IN_PLACE);
+        update_in_placet::NO_UPDATE_IN_PLACE,
+        null_message_handler);
 
       THEN("Code is produced")
       {
@@ -77,27 +82,28 @@ SCENARIO(
               std::regex_replace(line, spaces, " "), numbers, ""));
         }
 
-        const std::vector<std::string> reference_code = { // NOLINT
+        // clang-format off
+        // NOLINTNEXTLINE
+        const std::vector<std::string> reference_code = {
           "int tmp_object_factory;",
           "tmp_object_factory = NONDET(int);",
-          "__CPROVER_assume(tmp_object_factory >= 0);",
-          "__CPROVER_assume(tmp_object_factory <= 20);",
-          "char (*string_data_pointer)[INFINITY()];",
-          "string_data_pointer = "
+          CPROVER_PREFIX "assume(tmp_object_factory >= 0);",
+          CPROVER_PREFIX "assume(tmp_object_factory <= 20);",
+          "char (*nondet_infinite_array_pointer)[INFINITY()];",
+          "nondet_infinite_array_pointer = "
             "ALLOCATE(char [INFINITY()], INFINITY(), false);",
-          "char nondet_infinite_array[INFINITY()];",
-          "nondet_infinite_array = NONDET(char [INFINITY()]);",
-          "*string_data_pointer = nondet_infinite_array;",
+          "*nondet_infinite_array_pointer = NONDET(char [INFINITY()]);",
           "int return_array;",
           "return_array = cprover_associate_array_to_pointer_func"
-            "(*string_data_pointer, *string_data_pointer);",
+            "(*nondet_infinite_array_pointer, *nondet_infinite_array_pointer);",
           "int return_array;",
           "return_array = cprover_associate_length_to_array_func"
-            "(*string_data_pointer, tmp_object_factory);",
+            "(*nondet_infinite_array_pointer, tmp_object_factory);",
           "arg = { .@java.lang.Object={ .@class_identifier"
             "=\"java::java.lang.String\" },"
             " .length=tmp_object_factory, "
-            ".data=*string_data_pointer };"};
+            ".data=*nondet_infinite_array_pointer };"};
+        // clang-format on
 
         for(std::size_t i = 0;
             i < code_string.size() && i < reference_code.size();
