@@ -1,15 +1,15 @@
 /*******************************************************************\
 
- Module: Parameter indices test
+Module: Parameter indices test
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
 #include <java_bytecode/java_types.h>
 
-#include <testing-utils/catch.hpp>
 #include <java-testing-utils/load_java_class.h>
+#include <testing-utils/use_catch.h>
 
 void check_consistency(const symbolt &symbol)
 {
