@@ -2,8 +2,11 @@
 
 #include <inttypes.h>
 
-#define STATIC_ASSERT(condition) \
-  int some_array##__LINE__[(condition) ? 1 : -1]
+#  define CONCAT(a, b) a##b
+#  define CONCAT2(a, b) CONCAT(a, b)
+
+#  define STATIC_ASSERT(condition)                                             \
+    int CONCAT2(some_array, __LINE__)[(condition) ? 1 : -1]
 
 // Debian package openvswitch
 enum __attribute__((__packed__)) ofpact_type {
