@@ -1,5 +1,8 @@
-#define STATIC_ASSERT(condition) \
-  int array##__LINE__[(condition) ? 1 : -1]
+#define CONCAT(a, b) a##b
+#define CONCAT2(a, b) CONCAT(a, b)
+
+#define STATIC_ASSERT(condition)                                               \
+  int CONCAT2(some_array, __LINE__)[(condition) ? 1 : -1]
 
 #ifdef __GNUC__
 #ifndef __clang__
