@@ -3,7 +3,8 @@ typedef float _Float32;
 typedef double _Float32x;
 typedef double _Float64;
 typedef long double _Float64x;
+typedef long double _Float128;
 typedef long double _Float128x;
 
 // But this type should:
-_Float128 f128;
+__float128 f128;
