@@ -1,8 +1,11 @@
 #include <wchar.h>
 #include <stdlib.h> // for size_t
 
-#define STATIC_ASSERT(condition) \
-  int some_array##__LINE__[(condition) ? 1 : -1];
+#define CONCAT(a, b) a##b
+#define CONCAT2(a, b) CONCAT(a, b)
+
+#define STATIC_ASSERT(condition)                                               \
+  int CONCAT2(some_array, __LINE__)[(condition) ? 1 : -1]
 
 // check size_t
 STATIC_ASSERT(sizeof(void *)==sizeof(size_t));
