@@ -9,7 +9,11 @@ volatile int flag1 = 0, flag2 = 0; // boolean flags
 void* thr1(void * arg) { // frontend produces 12 transitions from this thread. It would be better if it would produce only 8!
   flag1 = 1;
   turn = 1;
+#ifdef __GNUC__
   __asm__ __volatile__ ("mfence": : :"memory");
+#else
+  __CPROVER_fence("WWfence", "RRfence", "RWfence", "WRfence");
+#endif
   __CPROVER_assume(! (flag2==1 && turn==1) );
   // begin: critical section
   x = 0;
@@ -21,7 +25,11 @@ void* thr1(void * arg) { // frontend produces 12 transitions from this thread. I
 void* thr2(void * arg) {
   flag2 = 1;
   turn = 0;
+#ifdef __GNUC__
   __asm__ __volatile__ ("mfence": : :"memory");
+#else
+  __CPROVER_fence("WWfence", "RRfence", "RWfence", "WRfence");
+#endif
   __CPROVER_assume(! (flag1==1 && turn==0) );
   // begin: critical section
   x = 1;
