@@ -14,7 +14,7 @@ int main()
 
     assert(__CPROVER_string_length(s) == i + 5);
     assert(__CPROVER_string_issuffix(__CPROVER_string_literal("po"),s));
-    assert(__CPROVER_char_at(s, i) == __CPROVER_char_literal("p"));
+    assert(__CPROVER_char_at(s, i) == 'p');
     assert(__CPROVER_string_issuffix(__CPROVER_string_literal("p!o"), s));
 
     return 0;
