@@ -23,9 +23,13 @@ class custom_bitvector_analysist;
 class custom_bitvector_domaint:public ai_domain_baset
 {
 public:
-  void
-  transform(locationt from, locationt to, ai_baset &ai, const namespacet &ns)
-    final override;
+  void transform(
+    const irep_idt &function_from,
+    locationt from,
+    const irep_idt &function_to,
+    locationt to,
+    ai_baset &ai,
+    const namespacet &ns) final override;
 
   void output(
     std::ostream &out,
