@@ -28,26 +28,39 @@ public:
   {
   }
 
-  void
-  transform(locationt from, locationt to, ai_baset &ai, const namespacet &ns)
-    final override;
+  /// Abstract Interpretation domain transform function.
+  void transform(
+    const irep_idt &function_from,
+    locationt from,
+    const irep_idt &function_to,
+    locationt to,
+    ai_baset &ai,
+    const namespacet &ns) final override;
 
+  /// Abstract Interpretation domain output function.
+  /// \param out: A stream to dump domain state on.
+  /// \param ai: A reference to the currently active
+  ///   abstract interpreter.
+  /// \param ns: A namespace to resolve symbols during output.
   void output(
     std::ostream &out,
     const ai_baset &ai,
     const namespacet &ns) const final override;
 
+  /// Abstract Interpretation domain merge function.
   bool merge(
     const global_may_alias_domaint &b,
     locationt from,
     locationt to);
 
+  /// Clear list of aliases, and mark domain as bottom.
   void make_bottom() final override
   {
     aliases.clear();
     has_values=tvt(false);
   }
 
+  /// Clear list of aliases, and mark domain as top.
   void make_top() final override
   {
     aliases.clear();
@@ -59,6 +72,7 @@ public:
     make_top();
   }
 
+  /// Returns true if domain is bottom.
   bool is_bottom() const final override
   {
     DATA_INVARIANT(!has_values.is_false() || (aliases.size()==0),
@@ -66,6 +80,7 @@ public:
     return has_values.is_false();
   }
 
+  /// Returns false if domain is top.
   bool is_top() const final override
   {
     DATA_INVARIANT(!has_values.is_true() || (aliases.size()==0),
@@ -84,6 +99,10 @@ private:
   void get_rhs_aliases_address_of(const exprt &, std::set<irep_idt> &);
 };
 
+/// This is a may analysis (i.e. aliasing that may occur during execution,
+/// but is not a given), for global variables. Implemented as a
+/// Steensgaard-style analysis, with the Union-find algorithm, for
+/// efficiency reasons.
 class global_may_alias_analysist:public ait<global_may_alias_domaint>
 {
 protected:
