@@ -7,7 +7,9 @@ Author: Daniel Kroening, kroening@kroening.com
 \*******************************************************************/
 
 /// \file
-/// Interval Analysis
+/// Interval Analysis -- implements the functionality necessary for performing
+/// abstract interpretation over interval domain for goto programs. The result
+/// of the analysis is an instrumented program.
 
 #include "interval_analysis.h"
 
@@ -15,17 +17,23 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include "interval_domain.h"
 
+/// Instruments all "post-branch" instructions with assumptions about variable
+/// intervals. For each such instruction, the function evaluates all variables
+/// referenced within the input goto_function as intervals, transforms these
+/// intervals into expressions and instruments the instruction with their
+/// conjunction.
+/// Example: interval [5,10] (for variable "x") translates into conjunction
+/// 5 <= x && x <= 10.
+/// \param interval_analysis: Interval domain to be used for variable evaluation
+/// \param [out] goto_function: Goto function to be analysed and instrumented.
 void instrument_intervals(
   const ait<interval_domaint> &interval_analysis,
   goto_functionst::goto_functiont &goto_function)
 {
   std::set<symbol_exprt> symbols;
 
-  forall_goto_program_instructions(i_it, goto_function.body)
-  {
-    find_symbols(i_it->code, symbols);
-    find_symbols(i_it->guard, symbols);
-  }
+  for(const auto &i : goto_function.body.instructions)
+    i.apply([&symbols](const exprt &e) { find_symbols(e, symbols); });
 
   Forall_goto_program_instructions(i_it, goto_function.body)
   {
@@ -35,9 +43,9 @@ void instrument_intervals(
     }
     else
     {
-      goto_programt::const_targett previous=i_it;
-      previous--;
-      if(previous->is_goto() && !previous->guard.is_true())
+      goto_programt::const_targett previous = std::prev(i_it);
+
+      if(previous->is_goto() && !previous->get_condition().is_true())
       {
         // we follow a branch, instrument
       }
@@ -68,14 +76,16 @@ void instrument_intervals(
     {
       goto_programt::targett t=i_it;
       goto_function.body.insert_before_swap(i_it);
-      t->make_assumption(conjunction(assertion));
+      *t = goto_programt::make_assumption(conjunction(assertion));
       i_it++; // goes to original instruction
       t->source_location=i_it->source_location;
-      t->function=i_it->function;
     }
   }
 }
 
+/// Initialises the abstract interpretation over interval domain and
+/// instruments instructions using interval assumptions.
+/// \param [out] goto_model: Input code as goto_model.
 void interval_analysis(goto_modelt &goto_model)
 {
   ait<interval_domaint> interval_analysis;
