@@ -12,20 +12,56 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "invariant_propagation.h"
 
 #include <util/simplify_expr.h>
-#include <util/base_type.h>
 #include <util/symbol_table.h>
 #include <util/std_expr.h>
 
+/// Pass the necessary arguments to the invariant_set_domaint's when constructed
+class invariant_set_domain_factoryt
+  : public ai_domain_factoryt<invariant_set_domaint>
+{
+public:
+  explicit invariant_set_domain_factoryt(invariant_propagationt &_ip) : ip(_ip)
+  {
+  }
+
+  std::unique_ptr<statet> make(locationt l) const override
+  {
+    auto p = util_make_unique<invariant_set_domaint>(
+      ip.value_sets, ip.object_store, ip.ns);
+    CHECK_RETURN(p->is_bottom());
+
+    return std::unique_ptr<statet>(p.release());
+  }
+
+private:
+  invariant_propagationt &ip;
+};
+
+invariant_propagationt::invariant_propagationt(
+  const namespacet &_ns,
+  value_setst &_value_sets)
+  : ait<invariant_set_domaint>(
+      util_make_unique<invariant_set_domain_factoryt>(*this)),
+    ns(_ns),
+    value_sets(_value_sets),
+    object_store(_ns)
+{
+}
+
 void invariant_propagationt::make_all_true()
 {
-  for(auto &state : state_map)
-    state.second.invariant_set.make_true();
+  for(auto &state :
+      static_cast<location_sensitive_storaget &>(*storage).internal())
+    static_cast<invariant_set_domaint &>(*(state.second))
+      .invariant_set.make_true();
 }
 
 void invariant_propagationt::make_all_false()
 {
-  for(auto &state : state_map)
-    state.second.invariant_set.make_false();
+  for(auto &state :
+      static_cast<location_sensitive_storaget &>(*storage).internal())
+    static_cast<invariant_set_domaint &>(*(state.second))
+      .invariant_set.make_false();
 }
 
 void invariant_propagationt::add_objects(
@@ -90,12 +126,13 @@ void invariant_propagationt::get_objects_rec(
   const exprt &src,
   std::list<exprt> &dest)
 {
-  const typet &t=ns.follow(src.type());
+  const typet &t = src.type();
 
-  if(t.id()==ID_struct ||
-     t.id()==ID_union)
+  if(
+    t.id() == ID_struct || t.id() == ID_union || t.id() == ID_struct_tag ||
+    t.id() == ID_union_tag)
   {
-    const struct_typet &struct_type=to_struct_type(t);
+    const struct_union_typet &struct_type = to_struct_union_type(ns.follow(t));
 
     for(const auto &component : struct_type.components())
     {
@@ -184,13 +221,12 @@ bool invariant_propagationt::check_type(const typet &type) const
 {
   if(type.id()==ID_pointer)
     return true;
-  else if(type.id()==ID_struct ||
-          type.id()==ID_union)
+  else if(
+    type.id() == ID_struct || type.id() == ID_union ||
+    type.id() == ID_struct_tag || type.id() == ID_union_tag)
     return false;
   else if(type.id()==ID_array)
     return false;
-  else if(type.id()==ID_symbol)
-    return check_type(ns.follow(type));
   else if(type.id()==ID_unsignedbv ||
           type.id()==ID_signedbv)
     return true;
@@ -200,35 +236,15 @@ bool invariant_propagationt::check_type(const typet &type) const
   return false;
 }
 
-void invariant_propagationt::initialize(const goto_programt &goto_program)
+void invariant_propagationt::initialize(
+  const irep_idt &function,
+  const goto_programt &goto_program)
 {
-  baset::initialize(goto_program);
-
-  forall_goto_program_instructions(it, goto_program)
-  {
-    invariant_sett &s = (*this)[it].invariant_set;
-
-    if(it==goto_program.instructions.begin())
-      s.make_true();
-    else
-      s.make_false();
-
-    s.set_value_sets(value_sets);
-    s.set_object_store(object_store);
-    s.set_namespace(ns);
-  }
+  baset::initialize(function, goto_program);
 
   add_objects(goto_program);
 }
 
-void invariant_propagationt::initialize(const goto_functionst &goto_functions)
-{
-  baset::initialize(goto_functions);
-
-  forall_goto_functions(f_it, goto_functions)
-    initialize(f_it->second.body);
-}
-
 void invariant_propagationt::simplify(goto_functionst &goto_functions)
 {
   Forall_goto_functions(it, goto_functions)
@@ -249,12 +265,12 @@ void invariant_propagationt::simplify(goto_programt &goto_program)
 
     const invariant_sett &invariant_set = d.invariant_set;
 
-    exprt simplified_guard(i_it->guard);
+    exprt simplified_guard(i_it->get_condition());
 
     invariant_set.simplify(simplified_guard);
     ::simplify(simplified_guard, ns);
 
     if(invariant_set.implies(simplified_guard).is_true())
-      i_it->guard=true_exprt();
+      i_it->set_condition(true_exprt());
   }
 }
