@@ -17,27 +17,21 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "ai.h"
 #include "invariant_set_domain.h"
 
+class invariant_set_domain_factoryt;
+
 class invariant_propagationt:public
   ait<invariant_set_domaint>
 {
 public:
-  invariant_propagationt(
-    const namespacet &_ns,
-    value_setst &_value_sets):
-    ait<invariant_set_domaint>(),
-    ns(_ns),
-    value_sets(_value_sets),
-    object_store(_ns)
-  {
-  }
+  invariant_propagationt(const namespacet &_ns, value_setst &_value_sets);
 
   const invariant_sett &lookup(locationt l) const
   {
     return (*this)[l].invariant_set;
   }
 
-  virtual void initialize(const goto_programt &goto_program);
-  virtual void initialize(const goto_functionst &goto_functions);
+  void initialize(const irep_idt &function, const goto_programt &goto_program)
+    override;
 
   void make_all_true();
   void make_all_false();
@@ -48,6 +42,14 @@ public:
   typedef ait<invariant_set_domaint> baset;
 
 protected:
+  // Each invariant_set_domain needs access to a few of the fields of the
+  // invariant_propagation object.  This is a historic design that predates
+  // the current interfaces.  Removing it would require a substantial refactor.
+  // A minimally-intrusive work around is for the domain factory to be a
+  // friend of the analyser object and create domains with references to the
+  // relevant fields.
+  friend invariant_set_domain_factoryt;
+
   const namespacet &ns;
   value_setst &value_sets;
 
