@@ -20,7 +20,11 @@ Author: Daniel Kroening, kroening@kroening.com
 class invariant_set_domaint:public ai_domain_baset
 {
 public:
-  invariant_set_domaint():has_values(false)
+  invariant_set_domaint(
+    value_setst &value_sets,
+    inv_object_storet &object_store,
+    const namespacet &ns)
+    : has_values(false), invariant_set(value_sets, object_store, ns)
   {
   }
 
@@ -49,11 +53,13 @@ public:
     if(has_values.is_known())
       out << has_values.to_string() << '\n';
     else
-      invariant_set.output("", out);
+      invariant_set.output(out);
   }
 
   virtual void transform(
+    const irep_idt &function_from,
     locationt from_l,
+    const irep_idt &function_to,
     locationt to_l,
     ai_baset &ai,
     const namespacet &ns) final override;
