@@ -8,12 +8,15 @@ Date: October 2012
 
 \*******************************************************************/
 
-#ifndef CPROVER_GOTO_PROGRAMS_IS_THREADED_H
-#define CPROVER_GOTO_PROGRAMS_IS_THREADED_H
+/// \file
+/// Over-approximate Concurrency for Threaded Goto Programs
+
+#ifndef CPROVER_ANALYSES_IS_THREADED_H
+#define CPROVER_ANALYSES_IS_THREADED_H
 
 #include <set>
 
-#include <goto-programs/goto_functions.h>
+#include <goto-programs/goto_model.h>
 
 class is_threadedt
 {
@@ -24,11 +27,22 @@ public:
     compute(goto_functions);
   }
 
-  inline bool operator()(const goto_programt::const_targett t) const
+  explicit is_threadedt(
+    const goto_modelt &goto_model)
+  {
+    compute(goto_model.goto_functions);
+  }
+
+  bool operator()(const goto_programt::const_targett t) const
   {
     return is_threaded_set.find(t)!=is_threaded_set.end();
   }
-  
+
+  bool operator()(void) const
+  {
+    return !is_threaded_set.empty();
+  }
+
 protected:
   typedef std::set<goto_programt::const_targett> is_threaded_sett;
   is_threaded_sett is_threaded_set;
@@ -37,4 +51,4 @@ protected:
     const goto_functionst &goto_functions);
 };
 
-#endif
+#endif // CPROVER_ANALYSES_IS_THREADED_H
