@@ -22,7 +22,8 @@ Author: Daniel Kroening, kroening@kroening.com
 exprt::operandst build_function_environment(
   const code_typet::parameterst &parameters,
   code_blockt &init_code,
-  symbol_tablet &symbol_table)
+  symbol_tablet &symbol_table,
+  const c_object_factory_parameterst &object_factory_parameters)
 {
   exprt::operandst main_arguments;
   main_arguments.resize(parameters.size());
@@ -35,14 +36,14 @@ exprt::operandst build_function_environment(
     const irep_idt base_name=p.get_base_name().empty()?
       ("argument#"+std::to_string(param_number)):p.get_base_name();
 
-    main_arguments[param_number]=
-      c_nondet_symbol_factory(
-        init_code,
-        symbol_table,
-        base_name,
-        p.type(),
-        p.source_location(),
-        true);
+    main_arguments[param_number] = c_nondet_symbol_factory(
+      init_code,
+      symbol_table,
+      base_name,
+      p.type(),
+      p.source_location(),
+      object_factory_parameters,
+      lifetimet::AUTOMATIC_LOCAL);
   }
 
   return main_arguments;
@@ -53,27 +54,16 @@ void record_function_outputs(
   code_blockt &init_code,
   symbol_tablet &symbol_table)
 {
-  bool has_return_value=
-    to_code_type(function.type).return_type()!=empty_typet();
+  bool has_return_value =
+    to_code_type(function.type).return_type() != void_type();
 
   if(has_return_value)
   {
+    const symbolt &return_symbol = symbol_table.lookup_ref("return'");
+
     // record return value
-    codet output(ID_output);
-    output.operands().resize(2);
-
-    const symbolt &return_symbol=*symbol_table.lookup("return'");
-
-    output.op0()=
-      address_of_exprt(
-        index_exprt(
-          string_constantt(return_symbol.base_name),
-          from_integer(0, index_type())));
-
-    output.op1()=return_symbol.symbol_expr();
-    output.add_source_location()=function.location;
-
-    init_code.move_to_operands(output);
+    init_code.add(code_outputt{
+      return_symbol.base_name, return_symbol.symbol_expr(), function.location});
   }
 
   #if 0
@@ -101,7 +91,7 @@ void record_function_outputs(
       output.op1()=symbol.symbol_expr();
       output.add_source_location()=p.source_location();
 
-      init_code.move_to_operands(output);
+      init_code.add(std::move(output));
     }
 
     i++;
@@ -111,7 +101,8 @@ void record_function_outputs(
 
 bool ansi_c_entry_point(
   symbol_tablet &symbol_table,
-  message_handlert &message_handler)
+  message_handlert &message_handler,
+  const c_object_factory_parameterst &object_factory_parameters)
 {
   // check if entry point is already there
   if(symbol_table.symbols.find(goto_functionst::entry_point())!=
@@ -120,12 +111,13 @@ bool ansi_c_entry_point(
 
   irep_idt main_symbol;
 
-  // find main symbol
-  if(config.main!="")
+  // find main symbol, if any is given
+  if(config.main.has_value())
   {
     std::list<irep_idt> matches;
 
-    forall_symbol_base_map(it, symbol_table.symbol_base_map, config.main)
+    forall_symbol_base_map(
+      it, symbol_table.symbol_base_map, config.main.value())
     {
       // look it up
       symbol_tablet::symbolst::const_iterator s_it=
@@ -141,15 +133,15 @@ bool ansi_c_entry_point(
     if(matches.empty())
     {
       messaget message(message_handler);
-      message.error() << "main symbol `" << config.main
-                      << "' not found" << messaget::eom;
+      message.error() << "main symbol '" << config.main.value() << "' not found"
+                      << messaget::eom;
       return true; // give up
     }
 
     if(matches.size()>=2)
     {
       messaget message(message_handler);
-      message.error() << "main symbol `" << config.main
+      message.error() << "main symbol '" << config.main.value()
                       << "' is ambiguous" << messaget::eom;
       return true;
     }
@@ -172,33 +164,38 @@ bool ansi_c_entry_point(
   if(symbol.value.is_nil())
   {
     messaget message(message_handler);
-    message.error() << "main symbol `" << id2string(main_symbol)
+    message.error() << "main symbol '" << id2string(main_symbol)
                     << "' has no body" << messaget::eom;
     return false; // give up
   }
 
-  if(static_lifetime_init(symbol_table, symbol.location, message_handler))
-    return true;
+  static_lifetime_init(symbol_table, symbol.location);
 
-  return generate_ansi_c_start_function(symbol, symbol_table, message_handler);
+  return generate_ansi_c_start_function(
+    symbol, symbol_table, message_handler, object_factory_parameters);
 }
 
-
 /// Generate a _start function for a specific function
 /// \param symbol: The symbol for the function that should be
 ///   used as the entry point
 /// \param symbol_table: The symbol table for the program. The new _start
 ///   function symbol will be added to this table
 /// \param message_handler: The message handler
+/// \param object_factory_parameters: configuration parameters for the object
+///   factory
 /// \return Returns false if the _start method was generated correctly
 bool generate_ansi_c_start_function(
   const symbolt &symbol,
   symbol_tablet &symbol_table,
-  message_handlert &message_handler)
+  message_handlert &message_handler,
+  const c_object_factory_parameterst &object_factory_parameters)
 {
   PRECONDITION(!symbol.value.is_nil());
   code_blockt init_code;
 
+  // add 'HIDE' label
+  init_code.add(code_labelt(CPROVER_PREFIX "HIDE", code_skipt()));
+
   // build call to initialization function
 
   {
@@ -213,28 +210,25 @@ bool generate_ansi_c_start_function(
       return true;
     }
 
-    code_function_callt call_init;
-    call_init.lhs().make_nil();
+    code_function_callt call_init(init_it->second.symbol_expr());
     call_init.add_source_location()=symbol.location;
-    call_init.function()=init_it->second.symbol_expr();
 
-    init_code.move_to_operands(call_init);
+    init_code.add(std::move(call_init));
   }
 
   // build call to main function
 
-  code_function_callt call_main;
+  code_function_callt call_main(symbol.symbol_expr());
   call_main.add_source_location()=symbol.location;
-  call_main.function()=symbol.symbol_expr();
   call_main.function().add_source_location()=symbol.location;
 
-  if(to_code_type(symbol.type).return_type()!=empty_typet())
+  if(to_code_type(symbol.type).return_type() != void_type())
   {
     auxiliary_symbolt return_symbol;
     return_symbol.mode=ID_C;
     return_symbol.is_static_lifetime=false;
     return_symbol.name="return'";
-    return_symbol.base_name="return";
+    return_symbol.base_name = "return'";
     return_symbol.type=to_code_type(symbol.type).return_type();
 
     symbol_table.add(return_symbol);
@@ -254,17 +248,65 @@ bool generate_ansi_c_start_function(
     {
       namespacet ns(symbol_table);
 
-      const symbolt &argc_symbol=ns.lookup("argc'");
+      {
+        symbolt argc_symbol;
+
+        argc_symbol.base_name = "argc'";
+        argc_symbol.name = "argc'";
+        argc_symbol.type = signed_int_type();
+        argc_symbol.is_static_lifetime = true;
+        argc_symbol.is_lvalue = true;
+        argc_symbol.mode = ID_C;
+
+        auto r = symbol_table.insert(argc_symbol);
+        if(!r.second && r.first != argc_symbol)
+        {
+          messaget message(message_handler);
+          message.error() << "argc already exists but is not usable"
+                          << messaget::eom;
+          return true;
+        }
+      }
+
+      const symbolt &argc_symbol = ns.lookup("argc'");
+
+      {
+        // we make the type of this thing an array of pointers
+        // need to add one to the size -- the array is terminated
+        // with NULL
+        const exprt one_expr = from_integer(1, argc_symbol.type);
+        const plus_exprt size_expr(argc_symbol.symbol_expr(), one_expr);
+        const array_typet argv_type(pointer_type(char_type()), size_expr);
+
+        symbolt argv_symbol;
+
+        argv_symbol.base_name = "argv'";
+        argv_symbol.name = "argv'";
+        argv_symbol.type = argv_type;
+        argv_symbol.is_static_lifetime = true;
+        argv_symbol.is_lvalue = true;
+        argv_symbol.mode = ID_C;
+
+        auto r = symbol_table.insert(argv_symbol);
+        if(!r.second && r.first != argv_symbol)
+        {
+          messaget message(message_handler);
+          message.error() << "argv already exists but is not usable"
+                          << messaget::eom;
+          return true;
+        }
+      }
+
       const symbolt &argv_symbol=ns.lookup("argv'");
 
       {
         // assume argc is at least one
         exprt one=from_integer(1, argc_symbol.type);
 
-        const binary_relation_exprt ge(argc_symbol.symbol_expr(), ID_ge, one);
+        binary_relation_exprt ge(
+          argc_symbol.symbol_expr(), ID_ge, std::move(one));
 
-        code_assumet assumption(ge);
-        init_code.move_to_operands(assumption);
+        init_code.add(code_assumet(std::move(ge)));
       }
 
       {
@@ -274,48 +316,63 @@ bool generate_ansi_c_start_function(
 
         exprt bound_expr=from_integer(upper_bound, argc_symbol.type);
 
-        const binary_relation_exprt le(
-          argc_symbol.symbol_expr(), ID_le, bound_expr);
+        binary_relation_exprt le(
+          argc_symbol.symbol_expr(), ID_le, std::move(bound_expr));
 
-        code_assumet assumption(le);
-        init_code.move_to_operands(assumption);
+        init_code.add(code_assumet(std::move(le)));
       }
 
-      {
-        // record argc as an input
-        codet input(ID_input);
-        input.operands().resize(2);
-        input.op0()=address_of_exprt(
-          index_exprt(string_constantt("argc"), from_integer(0, index_type())));
-        input.op1()=argc_symbol.symbol_expr();
-        init_code.move_to_operands(input);
-      }
+      // record argc as an input
+      init_code.add(code_inputt{"argc", argc_symbol.symbol_expr()});
 
       if(parameters.size()==3)
       {
+        {
+          symbolt envp_size_symbol;
+          envp_size_symbol.base_name = "envp_size'";
+          envp_size_symbol.name = "envp_size'";
+          envp_size_symbol.type = size_type();
+          envp_size_symbol.is_static_lifetime = true;
+          envp_size_symbol.mode = ID_C;
+
+          if(!symbol_table.insert(std::move(envp_size_symbol)).second)
+          {
+            messaget message(message_handler);
+            message.error()
+              << "failed to insert envp_size symbol" << messaget::eom;
+            return true;
+          }
+        }
+
         const symbolt &envp_size_symbol=ns.lookup("envp_size'");
 
-        // assume envp_size is INTMAX-1
-        mp_integer max;
+        {
+          symbolt envp_symbol;
+          envp_symbol.base_name = "envp'";
+          envp_symbol.name = "envp'";
+          envp_symbol.type = array_typet(
+            pointer_type(char_type()), envp_size_symbol.symbol_expr());
+          envp_symbol.is_static_lifetime = true;
+          envp_symbol.mode = ID_C;
 
-        if(envp_size_symbol.type.id()==ID_signedbv)
-        {
-          max=to_signedbv_type(envp_size_symbol.type).largest();
+          if(!symbol_table.insert(std::move(envp_symbol)).second)
+          {
+            messaget message(message_handler);
+            message.error() << "failed to insert envp symbol" << messaget::eom;
+            return true;
+          }
         }
-        else if(envp_size_symbol.type.id()==ID_unsignedbv)
-        {
-          max=to_unsignedbv_type(envp_size_symbol.type).largest();
-        }
-        else
-          UNREACHABLE;
+
+        // assume envp_size is INTMAX-1
+        const mp_integer max =
+          to_integer_bitvector_type(envp_size_symbol.type).largest();
 
         exprt max_minus_one=from_integer(max-1, envp_size_symbol.type);
 
-        const binary_relation_exprt le(
-          envp_size_symbol.symbol_expr(), ID_le, max_minus_one);
+        binary_relation_exprt le(
+          envp_size_symbol.symbol_expr(), ID_le, std::move(max_minus_one));
 
-        code_assumet assumption(le);
-        init_code.move_to_operands(assumption);
+        init_code.add(code_assumet(le));
       }
 
       {
@@ -326,10 +383,10 @@ bool generate_ansi_c_start_function(
         zero_string.type().subtype()=char_type();
         zero_string.type().set(ID_size, "infinity");
         const index_exprt index(zero_string, from_integer(0, uint_type()));
-        exprt address_of=address_of_exprt(index, pointer_type(char_type()));
-
-        if(argv_symbol.type.subtype()!=address_of.type())
-          address_of.make_typecast(argv_symbol.type.subtype());
+        exprt address_of =
+          typecast_exprt::conditional_cast(
+            address_of_exprt(index, pointer_type(char_type())),
+            argv_symbol.type.subtype());
 
         // assign argv[*] to the address of a string-object
         array_of_exprt array_of(address_of, argv_symbol.type);
@@ -350,7 +407,7 @@ bool generate_ansi_c_start_function(
         // disable bounds check on that one
         index_expr.set("bounds_check", false);
 
-        init_code.copy_to_operands(code_assignt(index_expr, null));
+        init_code.add(code_assignt(index_expr, null));
       }
 
       if(parameters.size()==3)
@@ -359,18 +416,16 @@ bool generate_ansi_c_start_function(
         const symbolt &envp_size_symbol=ns.lookup("envp_size'");
 
         // assume envp[envp_size] is NULL
-        const null_pointer_exprt null(
-          to_pointer_type(envp_symbol.type.subtype()));
+        null_pointer_exprt null(to_pointer_type(envp_symbol.type.subtype()));
 
         index_exprt index_expr(
           envp_symbol.symbol_expr(), envp_size_symbol.symbol_expr());
         // disable bounds check on that one
         index_expr.set("bounds_check", false);
 
-        const equal_exprt is_null(index_expr, null);
+        equal_exprt is_null(std::move(index_expr), std::move(null));
 
-        code_assumet assumption2(is_null);
-        init_code.move_to_operands(assumption2);
+        init_code.add(code_assumet(is_null));
       }
 
       {
@@ -384,40 +439,36 @@ bool generate_ansi_c_start_function(
         exprt &op0=operands[0];
         exprt &op1=operands[1];
 
-        op0=argc_symbol.symbol_expr();
+        op0 = typecast_exprt::conditional_cast(
+          argc_symbol.symbol_expr(), parameters[0].type());
 
         {
-          const exprt &arg1=parameters[1];
-          const pointer_typet &pointer_type=
-            to_pointer_type(arg1.type());
-
           index_exprt index_expr(
-            argv_symbol.symbol_expr(),
-            from_integer(0, index_type()),
-            pointer_type.subtype());
+            argv_symbol.symbol_expr(), from_integer(0, index_type()));
 
           // disable bounds check on that one
           index_expr.set("bounds_check", false);
 
-          op1=address_of_exprt(index_expr, pointer_type);
+          const pointer_typet &pointer_type =
+            to_pointer_type(parameters[1].type());
+
+          op1 = typecast_exprt::conditional_cast(
+            address_of_exprt(index_expr), pointer_type);
         }
 
         // do we need envp?
         if(parameters.size()==3)
         {
           const symbolt &envp_symbol=ns.lookup("envp'");
-          exprt &op2=operands[2];
-
-          const exprt &arg2=parameters[2];
-          const pointer_typet &pointer_type=
-            to_pointer_type(arg2.type());
 
           index_exprt index_expr(
-            envp_symbol.symbol_expr(),
-            from_integer(0, index_type()),
-            pointer_type.subtype());
+            envp_symbol.symbol_expr(), from_integer(0, index_type()));
 
-          op2=address_of_exprt(index_expr, pointer_type);
+          const pointer_typet &pointer_type =
+            to_pointer_type(parameters[2].type());
+
+          operands[2] = typecast_exprt::conditional_cast(
+            address_of_exprt(index_expr), pointer_type);
         }
       }
     }
@@ -427,14 +478,11 @@ bool generate_ansi_c_start_function(
   else
   {
     // produce nondet arguments
-    call_main.arguments()=
-      build_function_environment(
-        parameters,
-        init_code,
-        symbol_table);
+    call_main.arguments() = build_function_environment(
+      parameters, init_code, symbol_table, object_factory_parameters);
   }
 
-  init_code.move_to_operands(call_main);
+  init_code.add(std::move(call_main));
 
   // TODO: add read/modified (recursively in call graph) globals as INPUT/OUTPUT
 
@@ -444,14 +492,13 @@ bool generate_ansi_c_start_function(
   symbolt new_symbol;
 
   new_symbol.name=goto_functionst::entry_point();
-  new_symbol.type = code_typet({}, empty_typet());
+  new_symbol.type = code_typet({}, void_type());
   new_symbol.value.swap(init_code);
   new_symbol.mode=symbol.mode;
 
   if(!symbol_table.insert(std::move(new_symbol)).second)
   {
-    messaget message;
-    message.set_message_handler(message_handler);
+    messaget message(message_handler);
     message.error() << "failed to insert main symbol" << messaget::eom;
     return true;
   }
