@@ -43,7 +43,7 @@ bool ansi_c_languaget::preprocess(
   std::ostream &outstream)
 {
   // stdin?
-  if(path=="")
+  if(path.empty())
     return c_preprocess(instream, outstream, get_message_handler());
 
   return c_preprocess(path, outstream, get_message_handler());
@@ -76,7 +76,6 @@ bool ansi_c_languaget::parse(
   ansi_c_parser.set_message_handler(get_message_handler());
   ansi_c_parser.for_has_scope=config.ansi_c.for_has_scope;
   ansi_c_parser.ts_18661_3_Floatn_types=config.ansi_c.ts_18661_3_Floatn_types;
-  ansi_c_parser.Float128_type = config.ansi_c.Float128_type;
   ansi_c_parser.cpp98=false; // it's not C++
   ansi_c_parser.cpp11=false; // it's not C++
   ansi_c_parser.mode=config.ansi_c.mode;
@@ -105,7 +104,8 @@ bool ansi_c_languaget::parse(
 
 bool ansi_c_languaget::typecheck(
   symbol_tablet &symbol_table,
-  const std::string &module)
+  const std::string &module,
+  const bool keep_file_local)
 {
   symbol_tablet new_symbol_table;
 
@@ -118,7 +118,8 @@ bool ansi_c_languaget::typecheck(
     return true;
   }
 
-  remove_internal_symbols(new_symbol_table);
+  remove_internal_symbols(
+    new_symbol_table, this->get_message_handler(), keep_file_local);
 
   if(linking(symbol_table, new_symbol_table, get_message_handler()))
     return true;
@@ -129,16 +130,9 @@ bool ansi_c_languaget::typecheck(
 bool ansi_c_languaget::generate_support_functions(
   symbol_tablet &symbol_table)
 {
-  // Note this can happen here because C doesn't currently
-  // support lazy loading at the symbol-table level, and therefore
-  // function bodies have already been populated and external stub
-  // symbols created during the typecheck() phase. If it gains lazy
-  // loading support then stubs will need to be created during
-  // function body parsing, or else generate-stubs moved to the
-  // final phase.
-  generate_opaque_method_stubs(symbol_table);
   // This creates __CPROVER_start and __CPROVER_initialize:
-  return ansi_c_entry_point(symbol_table, get_message_handler());
+  return ansi_c_entry_point(
+    symbol_table, get_message_handler(), object_factory_params);
 }
 
 void ansi_c_languaget::show_parse(std::ostream &out)
@@ -220,7 +214,9 @@ bool ansi_c_languaget::to_expr(
   if(expr.id()==ID_typecast &&
      expr.type().id()==ID_empty &&
      expr.operands().size()==1)
-    expr=expr.op0();
+  {
+    expr = to_typecast_expr(expr).op();
+  }
 
   return result;
 }
