@@ -17,10 +17,28 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <langapi/language.h>
 
 #include "ansi_c_parse_tree.h"
+#include "c_object_factory_parameters.h"
+
+// clang-format off
+#define OPT_ANSI_C_LANGUAGE \
+  "(max-nondet-tree-depth):" \
+  "(min-null-tree-depth):"
+
+#define HELP_ANSI_C_LANGUAGE \
+  " --max-nondet-tree-depth N    limit size of nondet (e.g. input) object tree;\n" /* NOLINT(*) */\
+  "                              at level N pointers are set to null\n" \
+  " --min-null-tree-depth N      minimum level at which a pointer can first be\n" /* NOLINT(*) */\
+  "                              NULL in a recursively nondet initialized struct\n" /* NOLINT(*) */
+// clang-format on
 
 class ansi_c_languaget:public languaget
 {
 public:
+  void set_language_options(const optionst &options) override
+  {
+    object_factory_params.set(options);
+  }
+
   bool preprocess(
     std::istream &instream,
     const std::string &path,
@@ -35,7 +53,19 @@ public:
 
   bool typecheck(
     symbol_tablet &symbol_table,
-    const std::string &module) override;
+    const std::string &module,
+    const bool keep_file_local) override;
+
+  bool can_keep_file_local() override
+  {
+    return true;
+  }
+
+  bool
+  typecheck(symbol_tablet &symbol_table, const std::string &module) override
+  {
+    return typecheck(symbol_table, module, true);
+  }
 
   void show_parse(std::ostream &out) override;
 
@@ -75,6 +105,8 @@ public:
 protected:
   ansi_c_parse_treet parse_tree;
   std::string parse_path;
+
+  c_object_factory_parameterst object_factory_params;
 };
 
 std::unique_ptr<languaget> new_ansi_c_language();
