@@ -6,34 +6,18 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <ostream>
-
 #include "ansi_c_scope.h"
 
-/*******************************************************************\
-
-Function: ansi_c_scopet::print
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
+#include <ostream>
 
 void ansi_c_scopet::print(std::ostream &out) const
 {
   out << "Prefix: " << prefix << "\n";
 
-  for(ansi_c_scopet::name_mapt::const_iterator
-      n_it=name_map.begin();
-      n_it!=name_map.end();
-      n_it++)
+  for(const auto &name : name_map)
   {
-    out << "  ID: " << n_it->first
-        << " CLASS: " << n_it->second.id_class
+    out << "  ID: " << name.first
+        << " CLASS: " << static_cast<int>(name.second.id_class)
         << "\n";
   }
 }
-
