@@ -6,8 +6,9 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#ifndef CPROVER_C_PREPROCESS_H
-#define CPROVER_C_PREPROCESS_H
+
+#ifndef CPROVER_ANSI_C_C_PREPROCESS_H
+#define CPROVER_ANSI_C_C_PREPROCESS_H
 
 #include <iosfwd>
 #include <string>
@@ -18,7 +19,7 @@ bool c_preprocess(
   const std::string &path,
   std::ostream &outstream,
   message_handlert &message_handler);
- 
+
 bool c_preprocess(
   std::istream &instream,
   std::ostream &outstream,
@@ -26,5 +27,5 @@ bool c_preprocess(
 
 // returns 'true' in case of error
 bool test_c_preprocessor(message_handlert &message_handler);
- 
-#endif
+
+#endif // CPROVER_ANSI_C_C_PREPROCESS_H
