@@ -36,14 +36,13 @@ public:
     is_inline=false;
     is_weak=false;
     is_used = false;
-    is_always_inline = false;
     alias.clear();
     asm_label.clear();
     section.clear();
   }
 
-  bool is_typedef, is_extern, is_static, is_register, is_inline,
-    is_thread_local, is_weak, is_used, is_always_inline;
+  bool is_typedef, is_extern, is_static, is_register,
+       is_inline, is_thread_local, is_weak, is_used;
 
   // __attribute__((alias("foo")))
   irep_idt alias;
@@ -54,7 +53,6 @@ public:
 
   bool operator==(const c_storage_spect &other) const
   {
-    // clang-format off
     return is_typedef==other.is_typedef &&
            is_extern==other.is_extern &&
            is_static==other.is_static &&
@@ -63,11 +61,9 @@ public:
            is_inline==other.is_inline &&
            is_weak==other.is_weak &&
            is_used == other.is_used &&
-           is_always_inline == other.is_always_inline &&
            alias==other.alias &&
            asm_label==other.asm_label &&
            section==other.section;
-    // clang-format on
   }
 
   bool operator!=(const c_storage_spect &other) const
@@ -85,7 +81,6 @@ public:
     is_thread_local |=other.is_thread_local;
     is_weak         |=other.is_weak;
     is_used         |=other.is_used;
-    is_always_inline |= other.is_always_inline;
     if(alias.empty())
       alias=other.alias;
     if(asm_label.empty())
