@@ -11,10 +11,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include "c_typecheck_base.h"
 
-#include <util/invariant.h>
-#include <util/std_types.h>
-#include <util/prefix.h>
+#include <util/c_types.h>
 #include <util/config.h>
+#include <util/invariant.h>
+#include <util/prefix.h>
+#include <util/std_types.h>
 
 #include "expr2c.h"
 #include "type2name.h"
@@ -38,8 +39,8 @@ void c_typecheck_baset::move_symbol(symbolt &symbol, symbolt *&new_symbol)
   if(symbol_table.move(symbol, new_symbol))
   {
     error().source_location=symbol.location;
-    error() << "failed to move symbol `" << symbol.name
-            << "' into symbol table" << eom;
+    error() << "failed to move symbol '" << symbol.name << "' into symbol table"
+            << eom;
     throw 0;
   }
 }
@@ -67,6 +68,15 @@ void c_typecheck_baset::typecheck_symbol(symbolt &symbol)
     // and have static lifetime
     new_name=root_name;
     symbol.is_static_lifetime=true;
+
+    if(symbol.value.is_not_nil())
+    {
+      // According to the C standard this should be an error, but at least some
+      // versions of Visual Studio insist to use this in their C library, and
+      // GCC just warns as well.
+      warning().source_location = symbol.value.find_source_location();
+      warning() << "`extern' symbol should not have an initializer" << eom;
+    }
   }
   else if(!is_function && symbol.value.id()==ID_code)
   {
@@ -76,21 +86,15 @@ void c_typecheck_baset::typecheck_symbol(symbolt &symbol)
   }
 
   // set the pretty name
-  if(symbol.is_type &&
-     (final_type.id()==ID_struct ||
-      final_type.id()==ID_incomplete_struct))
+  if(symbol.is_type && final_type.id() == ID_struct)
   {
     symbol.pretty_name="struct "+id2string(symbol.base_name);
   }
-  else if(symbol.is_type &&
-          (final_type.id()==ID_union ||
-           final_type.id()==ID_incomplete_union))
+  else if(symbol.is_type && final_type.id() == ID_union)
   {
     symbol.pretty_name="union "+id2string(symbol.base_name);
   }
-  else if(symbol.is_type &&
-          (final_type.id()==ID_c_enum ||
-           final_type.id()==ID_incomplete_c_enum))
+  else if(symbol.is_type && final_type.id() == ID_c_enum)
   {
     symbol.pretty_name="enum "+id2string(symbol.base_name);
   }
@@ -116,12 +120,12 @@ void c_typecheck_baset::typecheck_symbol(symbolt &symbol)
     if(old_it->second.is_type!=symbol.is_type)
     {
       error().source_location=symbol.location;
-      error() << "redeclaration of `" << symbol.display_name()
+      error() << "redeclaration of '" << symbol.display_name()
               << "' as a different kind of symbol" << eom;
       throw 0;
     }
 
-    symbolt & existing_symbol=*symbol_table.get_writeable(symbol.name);
+    symbolt &existing_symbol = symbol_table.get_writeable_ref(symbol.name);
     if(symbol.is_type)
       typecheck_redefinition_type(existing_symbol, symbol);
     else
@@ -140,16 +144,14 @@ void c_typecheck_baset::typecheck_new_symbol(symbolt &symbol)
   {
     if(symbol.value.is_not_nil() &&
        !symbol.is_macro)
+    {
       typecheck_function_body(symbol);
+    }
     else
     {
       // we don't need the identifiers
-      code_typet &code_type=to_code_type(symbol.type);
-      for(code_typet::parameterst::iterator
-          it=code_type.parameters().begin();
-          it!=code_type.parameters().end();
-          it++)
-        it->set_identifier(irep_idt());
+      for(auto &parameter : to_code_type(symbol.type).parameters())
+        parameter.set_identifier(irep_idt());
     }
   }
   else if(!symbol.is_macro)
@@ -167,12 +169,21 @@ void c_typecheck_baset::typecheck_redefinition_type(
   const typet &final_new=follow(new_symbol.type);
 
   // see if we had something incomplete before
-  if(final_old.id()==ID_incomplete_struct ||
-     final_old.id()==ID_incomplete_union ||
-     final_old.id()==ID_incomplete_c_enum)
+  if(
+    (final_old.id() == ID_struct &&
+     to_struct_type(final_old).is_incomplete()) ||
+    (final_old.id() == ID_union && to_union_type(final_old).is_incomplete()) ||
+    (final_old.id() == ID_c_enum && to_c_enum_type(final_old).is_incomplete()))
   {
-    // new one complete?
-    if("incomplete_"+final_new.id_string()==final_old.id_string())
+    // is the new one complete?
+    if(
+      final_new.id() == final_old.id() &&
+      ((final_new.id() == ID_struct &&
+        !to_struct_type(final_new).is_incomplete()) ||
+       (final_new.id() == ID_union &&
+        !to_union_type(final_new).is_incomplete()) ||
+       (final_new.id() == ID_c_enum &&
+        !to_c_enum_type(final_new).is_incomplete())))
     {
       // overwrite location
       old_symbol.location=new_symbol.location;
@@ -181,24 +192,27 @@ void c_typecheck_baset::typecheck_redefinition_type(
       old_symbol.type.swap(new_symbol.type);
     }
     else if(new_symbol.type.id()==old_symbol.type.id())
-      return;
+      return; // ignore
     else
     {
       error().source_location=new_symbol.location;
-      error() << "conflicting definition of type symbol `"
-              << new_symbol.display_name()
-              << '\'' << eom;
+      error() << "conflicting definition of type symbol '"
+              << new_symbol.display_name() << '\'' << eom;
       throw 0;
     }
   }
   else
   {
     // see if new one is just a tag
-    if(final_new.id()==ID_incomplete_struct ||
-       final_new.id()==ID_incomplete_union ||
-       final_new.id()==ID_incomplete_c_enum)
+    if(
+      (final_new.id() == ID_struct &&
+       to_struct_type(final_new).is_incomplete()) ||
+      (final_new.id() == ID_union &&
+       to_union_type(final_new).is_incomplete()) ||
+      (final_new.id() == ID_c_enum &&
+       to_c_enum_type(final_new).is_incomplete()))
     {
-      if("incomplete_"+final_old.id_string()==final_new.id_string())
+      if(final_old.id() == final_new.id())
       {
         // just ignore silently
       }
@@ -206,9 +220,8 @@ void c_typecheck_baset::typecheck_redefinition_type(
       {
         // arg! new tag type
         error().source_location=new_symbol.location;
-        error() << "conflicting definition of tag symbol `"
-                << new_symbol.display_name()
-                << '\'' << eom;
+        error() << "conflicting definition of tag symbol '"
+                << new_symbol.display_name() << '\'' << eom;
         throw 0;
       }
     }
@@ -232,8 +245,8 @@ void c_typecheck_baset::typecheck_redefinition_type(
       if(follow(new_symbol.type)!=follow(old_symbol.type))
       {
         error().source_location=new_symbol.location;
-        error() << "type symbol `"
-                << new_symbol.display_name() << "' defined twice:\n"
+        error() << "type symbol '" << new_symbol.display_name()
+                << "' defined twice:\n"
                 << "Original: " << to_string(old_symbol.type) << "\n"
                 << "     New: " << to_string(new_symbol.type) << eom;
         throw 0;
@@ -270,8 +283,7 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
   }
 
   // do initializer, this may change the type
-  if(follow(new_symbol.type).id()!=ID_code &&
-     !new_symbol.is_macro)
+  if(new_symbol.type.id() != ID_code && !new_symbol.is_macro)
     do_initializer(new_symbol);
 
   const typet &final_new=follow(new_symbol.type);
@@ -295,15 +307,10 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
 
   if(final_new.id()==ID_code)
   {
-    bool inlined=
-       (new_symbol.type.get_bool(ID_C_inlined) ||
-        old_symbol.type.get_bool(ID_C_inlined));
-
     if(final_old.id()!=ID_code)
     {
       error().source_location=new_symbol.location;
-      error() << "error: function symbol `"
-              << new_symbol.display_name()
+      error() << "error: function symbol '" << new_symbol.display_name()
               << "' redefined with a different type:\n"
               << "Original: " << to_string(old_symbol.type) << "\n"
               << "     New: " << to_string(new_symbol.type) << eom;
@@ -313,6 +320,8 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
     code_typet &old_ct=to_code_type(old_symbol.type);
     code_typet &new_ct=to_code_type(new_symbol.type);
 
+    const bool inlined = old_ct.get_inlined() || new_ct.get_inlined();
+
     if(old_ct.has_ellipsis() && !new_ct.has_ellipsis())
       old_ct=new_ct;
     else if(!old_ct.has_ellipsis() && new_ct.has_ellipsis())
@@ -320,8 +329,8 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
 
     if(inlined)
     {
-      old_symbol.type.set(ID_C_inlined, true);
-      new_symbol.type.set(ID_C_inlined, true);
+      old_ct.set_inlined(true);
+      new_ct.set_inlined(true);
     }
 
     // do body
@@ -334,7 +343,7 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
         // definition is marked as "extern inline"
 
         if(
-          old_symbol.type.get_bool(ID_C_inlined) &&
+          old_ct.get_inlined() &&
           (config.ansi_c.mode == configt::ansi_ct::flavourt::GCC ||
            config.ansi_c.mode == configt::ansi_ct::flavourt::CLANG ||
            config.ansi_c.mode == configt::ansi_ct::flavourt::ARM ||
@@ -345,13 +354,9 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
           old_symbol.is_file_local=new_symbol.is_file_local;
 
           // remove parameter declarations to avoid conflicts
-          const code_typet::parameterst &old_p=old_ct.parameters();
-          for(code_typet::parameterst::const_iterator
-              p_it=old_p.begin();
-              p_it!=old_p.end();
-              p_it++)
+          for(const auto &old_parameter : old_ct.parameters())
           {
-            const irep_idt &identifier=p_it->get_identifier();
+            const irep_idt &identifier = old_parameter.get_identifier();
 
             symbol_tablet::symbolst::const_iterator p_s_it=
               symbol_table.symbols.find(identifier);
@@ -362,7 +367,7 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
         else
         {
           error().source_location=new_symbol.location;
-          error() << "function body `" << new_symbol.display_name()
+          error() << "function body '" << new_symbol.display_name()
                   << "' defined twice" << eom;
           throw 0;
         }
@@ -405,33 +410,22 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
        to_array_type(final_new).size().is_not_nil() &&
        final_old.subtype()==final_new.subtype())
     {
-      // we don't do symbol types for arrays anymore
-      PRECONDITION(old_symbol.type.id()!=ID_symbol);
       old_symbol.type=new_symbol.type;
     }
-    else if((final_old.id()==ID_incomplete_c_enum ||
-             final_old.id()==ID_c_enum) &&
-            (final_new.id()==ID_incomplete_c_enum ||
-             final_new.id()==ID_c_enum))
-    {
-      // this is ok for now
-    }
-    else if(final_old.id()==ID_pointer &&
-            follow(final_old).subtype().id()==ID_code &&
-            to_code_type(follow(final_old).subtype()).has_ellipsis() &&
-            final_new.id()==ID_pointer &&
-            follow(final_new).subtype().id()==ID_code)
+    else if(
+      final_old.id() == ID_pointer && final_old.subtype().id() == ID_code &&
+      to_code_type(final_old.subtype()).has_ellipsis() &&
+      final_new.id() == ID_pointer && final_new.subtype().id() == ID_code)
     {
       // to allow
       // int (*f) ();
       // int (*f) (int)=0;
       old_symbol.type=new_symbol.type;
     }
-    else if(final_old.id()==ID_pointer &&
-            follow(final_old).subtype().id()==ID_code &&
-            final_new.id()==ID_pointer &&
-            follow(final_new).subtype().id()==ID_code &&
-            to_code_type(follow(final_new).subtype()).has_ellipsis())
+    else if(
+      final_old.id() == ID_pointer && final_old.subtype().id() == ID_code &&
+      final_new.id() == ID_pointer && final_new.subtype().id() == ID_code &&
+      to_code_type(final_new.subtype()).has_ellipsis())
     {
       // to allow
       // int (*f) (int)=0;
@@ -440,7 +434,7 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
     else
     {
       error().source_location=new_symbol.location;
-      error() << "symbol `" << new_symbol.display_name()
+      error() << "symbol '" << new_symbol.display_name()
               << "' redefined with a different type:\n"
               << "Original: " << to_string(old_symbol.type) << "\n"
               << "     New: " << to_string(new_symbol.type) << eom;
@@ -468,19 +462,17 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
       }
       else
       {
-        if(new_symbol.is_macro &&
-           (final_new.id()==ID_incomplete_c_enum ||
-            final_new.id()==ID_c_enum) &&
-            old_symbol.value.is_constant() &&
-            new_symbol.value.is_constant() &&
-            old_symbol.value.get(ID_value)==new_symbol.value.get(ID_value))
+        if(
+          new_symbol.is_macro && final_new.id() == ID_c_enum &&
+          old_symbol.value.is_constant() && new_symbol.value.is_constant() &&
+          old_symbol.value.get(ID_value) == new_symbol.value.get(ID_value))
         {
           // ignore
         }
         else
         {
           warning().source_location=new_symbol.value.find_source_location();
-          warning() << "symbol `" << new_symbol.display_name()
+          warning() << "symbol '" << new_symbol.display_name()
                     << "' already has an initial value" << eom;
         }
       }
@@ -523,31 +515,27 @@ void c_typecheck_baset::typecheck_function_body(symbolt &symbol)
   unsigned anon_counter=0;
 
   // Add the parameter declarations into the symbol table.
-  code_typet::parameterst &parameters=code_type.parameters();
-  for(code_typet::parameterst::iterator
-      p_it=parameters.begin();
-      p_it!=parameters.end();
-      p_it++)
+  for(auto &p : code_type.parameters())
   {
     // may be anonymous
-    if(p_it->get_base_name().empty())
+    if(p.get_base_name().empty())
     {
       irep_idt base_name="#anon"+std::to_string(anon_counter++);
-      p_it->set_base_name(base_name);
+      p.set_base_name(base_name);
     }
 
     // produce identifier
-    irep_idt base_name=p_it->get_base_name();
+    irep_idt base_name = p.get_base_name();
     irep_idt identifier=id2string(symbol.name)+"::"+id2string(base_name);
 
-    p_it->set_identifier(identifier);
+    p.set_identifier(identifier);
 
     parameter_symbolt p_symbol;
 
-    p_symbol.type=p_it->type();
+    p_symbol.type = p.type();
     p_symbol.name=identifier;
     p_symbol.base_name=base_name;
-    p_symbol.location=p_it->source_location();
+    p_symbol.location = p.source_location();
 
     symbolt *new_p_symbol;
     move_symbol(p_symbol, new_p_symbol);
@@ -556,18 +544,13 @@ void c_typecheck_baset::typecheck_function_body(symbolt &symbol)
   // typecheck the body code
   typecheck_code(to_code(symbol.value));
 
-  // special case for main()
-  if(symbol.name==ID_main)
-    add_argc_argv(symbol);
-
   // check the labels
-  for(std::map<irep_idt, source_locationt>::const_iterator
-      it=labels_used.begin(); it!=labels_used.end(); it++)
+  for(const auto &label : labels_used)
   {
-    if(labels_defined.find(it->first)==labels_defined.end())
+    if(labels_defined.find(label.first) == labels_defined.end())
     {
-      error().source_location=it->second;
-      error() << "branching label `" << it->first
+      error().source_location = label.second;
+      error() << "branching label '" << label.first
               << "' is not defined in function" << eom;
       throw 0;
     }
@@ -623,12 +606,9 @@ void c_typecheck_baset::apply_asm_label(
   {
     const code_typet &code_type=to_code_type(symbol.type);
 
-    for(code_typet::parameterst::const_iterator
-        p_it=code_type.parameters().begin();
-        p_it!=code_type.parameters().end();
-        ++p_it)
+    for(const auto &p : code_type.parameters())
     {
-      const irep_idt &p_bn=p_it->get_base_name();
+      const irep_idt &p_bn = p.get_base_name();
       if(p_bn.empty())
         continue;
 
@@ -647,9 +627,9 @@ void c_typecheck_baset::typecheck_declaration(
 {
   if(declaration.get_is_static_assert())
   {
-    assert(declaration.operands().size()==2);
-    typecheck_expr(declaration.op0());
-    typecheck_expr(declaration.op1());
+    auto &static_assert_expr = to_binary_expr(declaration);
+    typecheck_expr(static_assert_expr.op0());
+    typecheck_expr(static_assert_expr.op1());
   }
   else
   {
@@ -660,9 +640,9 @@ void c_typecheck_baset::typecheck_declaration(
     typecheck_type(declaration.type());
 
     // mark as 'already typechecked'
-    make_already_typechecked(declaration.type());
+    already_typechecked_typet::make_already_typechecked(declaration.type());
 
-    codet contract;
+    irept contract;
 
     {
       exprt spec_requires=
@@ -675,12 +655,9 @@ void c_typecheck_baset::typecheck_declaration(
     }
 
     // Now do declarators, if any.
-    for(ansi_c_declarationt::declaratorst::iterator
-        d_it=declaration.declarators().begin();
-        d_it!=declaration.declarators().end();
-        d_it++)
+    for(auto &declarator : declaration.declarators())
     {
-      c_storage_spect full_spec(declaration.full_type(*d_it));
+      c_storage_spect full_spec(declaration.full_type(declarator));
       full_spec|=c_storage_spec;
 
       declaration.set_is_inline(full_spec.is_inline);
@@ -693,7 +670,7 @@ void c_typecheck_baset::typecheck_declaration(
       declaration.set_is_used(full_spec.is_used);
 
       symbolt symbol;
-      declaration.to_symbol(*d_it, symbol);
+      declaration.to_symbol(declarator, symbol);
       current_symbol=symbol;
 
       // now check other half of type
@@ -714,9 +691,9 @@ void c_typecheck_baset::typecheck_declaration(
         // also cater for renaming/placement in sections
         const auto &renaming_entry = asm_label_map.find(full_spec.alias);
         if(renaming_entry == asm_label_map.end())
-          symbol.value = symbol_exprt(full_spec.alias);
+          symbol.value = symbol_exprt::typeless(full_spec.alias);
         else
-          symbol.value = symbol_exprt(renaming_entry->second);
+          symbol.value = symbol_exprt::typeless(renaming_entry->second);
         symbol.is_macro=true;
       }
 
@@ -746,24 +723,24 @@ void c_typecheck_baset::typecheck_declaration(
       }
 
       irep_idt identifier=symbol.name;
-      d_it->set_name(identifier);
+      declarator.set_name(identifier);
 
       typecheck_symbol(symbol);
 
       // add code contract (if any); we typecheck this after the
       // function body done above, so as to have parameter symbols
       // available
-      symbolt &new_symbol=*symbol_table.get_writeable(identifier);
+      symbolt &new_symbol = symbol_table.get_writeable_ref(identifier);
 
-      typecheck_spec_expr(contract, ID_C_spec_requires);
+      typecheck_spec_expr(static_cast<codet &>(contract), ID_C_spec_requires);
 
-      typet ret_type=empty_typet();
+      typet ret_type = void_type();
       if(new_symbol.type.id()==ID_code)
         ret_type=to_code_type(new_symbol.type).return_type();
       assert(parameter_map.empty());
       if(ret_type.id()!=ID_empty)
-        parameter_map["__CPROVER_return_value"]=ret_type;
-      typecheck_spec_expr(contract, ID_C_spec_ensures);
+        parameter_map[CPROVER_PREFIX "return_value"] = ret_type;
+      typecheck_spec_expr(static_cast<codet &>(contract), ID_C_spec_ensures);
       parameter_map.clear();
 
       if(contract.find(ID_C_spec_requires).is_not_nil())
