@@ -10,10 +10,14 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_ANSI_C_EXPR2C_CLASS_H
 #define CPROVER_ANSI_C_EXPR2C_CLASS_H
 
+#include "expr2c.h"
+
 #include <string>
 #include <unordered_map>
 #include <unordered_set>
 
+#include <util/byte_operators.h>
+#include <util/mathematical_expr.h>
 #include <util/std_code.h>
 #include <util/std_expr.h>
 
@@ -23,7 +27,13 @@ class namespacet;
 class expr2ct
 {
 public:
-  explicit expr2ct(const namespacet &_ns):ns(_ns), sizeof_nesting(0) { }
+  explicit expr2ct(
+    const namespacet &_ns,
+    const expr2c_configurationt &configuration =
+      expr2c_configurationt::default_configuration)
+    : ns(_ns), configuration(configuration), sizeof_nesting(0)
+  {
+  }
   virtual ~expr2ct() { }
 
   virtual std::string convert(const typet &src);
@@ -31,8 +41,12 @@ public:
 
   void get_shorthands(const exprt &expr);
 
+  std::string
+  convert_with_identifier(const typet &src, const std::string &identifier);
+
 protected:
   const namespacet &ns;
+  const expr2c_configurationt &configuration;
 
   virtual std::string convert_rec(
     const typet &src,
@@ -83,8 +97,10 @@ protected:
     unsigned &precedence);
 
   std::string convert_binary(
-    const exprt &src, const std::string &symbol,
-    unsigned precedence, bool full_parentheses);
+    const binary_exprt &,
+    const std::string &symbol,
+    unsigned precedence,
+    bool full_parentheses);
 
   std::string convert_multi_ary(
     const exprt &src, const std::string &symbol,
@@ -105,21 +121,23 @@ protected:
   std::string convert_array_of(const exprt &src, unsigned precedence);
 
   std::string convert_trinary(
-    const exprt &src, const std::string &symbol1,
-    const std::string &symbol2, unsigned precedence);
+    const ternary_exprt &src,
+    const std::string &symbol1,
+    const std::string &symbol2,
+    unsigned precedence);
 
   std::string convert_overflow(
     const exprt &src, unsigned &precedence);
 
   std::string convert_quantifier(
-    const exprt &src, const std::string &symbol,
+    const quantifier_exprt &,
+    const std::string &symbol,
     unsigned precedence);
 
   std::string convert_with(
     const exprt &src, unsigned precedence);
 
-  std::string convert_update(
-    const exprt &src, unsigned precedence);
+  std::string convert_update(const update_exprt &, unsigned precedence);
 
   std::string convert_member_designator(
     const exprt &src);
@@ -130,33 +148,27 @@ protected:
   std::string convert_index(
     const exprt &src, unsigned precedence);
 
-  std::string convert_byte_extract(
-    const exprt &src,
-    unsigned precedence);
+  std::string
+  convert_byte_extract(const byte_extract_exprt &, unsigned precedence);
 
-  std::string convert_byte_update(
-    const exprt &src,
-    unsigned precedence);
+  std::string
+  convert_byte_update(const byte_update_exprt &, unsigned precedence);
 
-  std::string convert_extractbit(
-    const exprt &src,
-    unsigned precedence);
+  std::string convert_extractbit(const extractbit_exprt &, unsigned precedence);
 
-  std::string convert_extractbits(
-    const exprt &src,
-    unsigned precedence);
+  std::string
+  convert_extractbits(const extractbits_exprt &src, unsigned precedence);
 
   std::string convert_unary(
-    const exprt &src, const std::string &symbol,
+    const unary_exprt &,
+    const std::string &symbol,
     unsigned precedence);
 
   std::string convert_unary_post(
     const exprt &src, const std::string &symbol,
     unsigned precedence);
 
-  std::string convert_function(
-    const exprt &src, const std::string &symbol,
-    unsigned precedence);
+  std::string convert_function(const exprt &src, const std::string &symbol);
 
   std::string convert_complex(
     const exprt &src,
@@ -175,8 +187,6 @@ protected:
   std::string convert_code_switch_case(const code_switch_caset &src, unsigned indent);
   std::string convert_code_asm(const code_asmt &src, unsigned indent);
   std::string convert_code_assign(const code_assignt &src, unsigned indent);
-  std::string convert_code_free(const codet &src, unsigned indent);
-  std::string convert_code_init(const codet &src, unsigned indent);
   // NOLINTNEXTLINE(whitespace/line_length)
   std::string convert_code_ifthenelse(const code_ifthenelset &src, unsigned indent);
   std::string convert_code_for(const code_fort &src, unsigned indent);
@@ -188,9 +198,9 @@ protected:
   std::string convert_code_goto(const codet &src, unsigned indent);
   std::string convert_code_assume(const codet &src, unsigned indent);
   std::string convert_code_assert(const codet &src, unsigned indent);
-  std::string convert_code_break(const codet &src, unsigned indent);
+  std::string convert_code_break(unsigned indent);
   std::string convert_code_switch(const codet &src, unsigned indent);
-  std::string convert_code_continue(const codet &src, unsigned indent);
+  std::string convert_code_continue(unsigned indent);
   std::string convert_code_decl(const codet &src, unsigned indent);
   std::string convert_code_decl_block(const codet &src, unsigned indent);
   std::string convert_code_dead(const codet &src, unsigned indent);
@@ -209,10 +219,10 @@ protected:
   virtual std::string convert_with_precedence(
     const exprt &src, unsigned &precedence);
 
-  // NOLINTNEXTLINE(whitespace/line_length)
-  std::string convert_function_application(const function_application_exprt &src, unsigned &precedence);
-  // NOLINTNEXTLINE(whitespace/line_length)
-  std::string convert_side_effect_expr_function_call(const side_effect_expr_function_callt &src, unsigned &precedence);
+  std::string
+  convert_function_application(const function_application_exprt &src);
+  std::string convert_side_effect_expr_function_call(
+    const side_effect_expr_function_callt &src);
   std::string convert_allocate(const exprt &src, unsigned &precedence);
   std::string convert_nondet(const exprt &src, unsigned &precedence);
   std::string convert_prob_coin(const exprt &src, unsigned &precedence);
@@ -220,18 +230,15 @@ protected:
   // NOLINTNEXTLINE(whitespace/line_length)
   std::string convert_statement_expression(const exprt &src, unsigned &precendence);
 
-  virtual std::string convert_symbol(const exprt &src, unsigned &precedence);
-  std::string convert_predicate_symbol(const exprt &src, unsigned &precedence);
-  // NOLINTNEXTLINE(whitespace/line_length)
-  std::string convert_predicate_next_symbol(const exprt &src, unsigned &precedence);
-  // NOLINTNEXTLINE(whitespace/line_length)
-  std::string convert_predicate_passive_symbol(const exprt &src, unsigned &precedence);
-  // NOLINTNEXTLINE(whitespace/line_length)
-  std::string convert_nondet_symbol(const nondet_symbol_exprt &, unsigned &precedence);
-  std::string convert_quantified_symbol(const exprt &src, unsigned &precedence);
-  std::string convert_nondet_bool(const exprt &src, unsigned &precedence);
+  virtual std::string convert_symbol(const exprt &src);
+  std::string convert_predicate_symbol(const exprt &src);
+  std::string convert_predicate_next_symbol(const exprt &src);
+  std::string convert_predicate_passive_symbol(const exprt &src);
+  std::string convert_nondet_symbol(const nondet_symbol_exprt &);
+  std::string convert_quantified_symbol(const exprt &src);
+  std::string convert_nondet_bool();
   std::string convert_object_descriptor(const exprt &src, unsigned &precedence);
-  std::string convert_literal(const exprt &src, unsigned &precedence);
+  std::string convert_literal(const exprt &src);
   // NOLINTNEXTLINE(whitespace/line_length)
   virtual std::string convert_constant(const constant_exprt &src, unsigned &precedence);
   virtual std::string convert_constant_bool(bool boolean_value);
@@ -241,11 +248,10 @@ protected:
   virtual std::string convert_struct(const exprt &src, unsigned &precedence);
   std::string convert_union(const exprt &src, unsigned &precedence);
   std::string convert_vector(const exprt &src, unsigned &precedence);
-  std::string convert_array(const exprt &src, unsigned &precedence);
+  std::string convert_array(const exprt &src);
   std::string convert_array_list(const exprt &src, unsigned &precedence);
-  std::string convert_initializer_list(const exprt &src, unsigned &precedence);
-  // NOLINTNEXTLINE(whitespace/line_length)
-  std::string convert_designated_initializer(const exprt &src, unsigned &precedence);
+  std::string convert_initializer_list(const exprt &src);
+  std::string convert_designated_initializer(const exprt &src);
   std::string convert_concatenation(const exprt &src, unsigned &precedence);
   std::string convert_sizeof(const exprt &src, unsigned &precedence);
   std::string convert_let(const let_exprt &, unsigned precedence);
