@@ -14,13 +14,13 @@ Author: Daniel Kroening, kroening@kroening.com
 // These are gcc-specific; most are not implemented by clang
 // https://gcc.gnu.org/onlinedocs/gcc/Floating-Types.html
 
-bitvector_typet gcc_float16_type();
-bitvector_typet gcc_float32_type();
-bitvector_typet gcc_float32x_type();
-bitvector_typet gcc_float64_type();
-bitvector_typet gcc_float64x_type();
-bitvector_typet gcc_float128_type();
-bitvector_typet gcc_float128x_type();
+floatbv_typet gcc_float16_type();
+floatbv_typet gcc_float32_type();
+floatbv_typet gcc_float32x_type();
+floatbv_typet gcc_float64_type();
+floatbv_typet gcc_float64x_type();
+floatbv_typet gcc_float128_type();
+floatbv_typet gcc_float128x_type();
 unsignedbv_typet gcc_unsigned_int128_type();
 signedbv_typet gcc_signed_int128_type();
 
