@@ -6,25 +6,17 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
+/// \file
+/// ANSI-C Language Conversion
+
+#include "preprocessor_line.h"
+
 #include <cctype>
 
 #include <util/string2int.h>
 #include <util/parser.h>
 
 #include "literals/unescape_string.h"
-#include "preprocessor_line.h"
-
-/*******************************************************************\
-
-Function: preprocessor_line
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
 
 void preprocessor_line(
   const char *text,
@@ -32,12 +24,13 @@ void preprocessor_line(
 {
   const char *ptr=text;
   std::string line_number;
-  
+
   // skip WS
   while(*ptr==' ' || *ptr=='\t') ptr++;
 
   // skip #
-  if(*ptr!='#') return;
+  if(*ptr!='#')
+    return;
   ptr++;
 
   // skip WS
@@ -58,7 +51,7 @@ void preprocessor_line(
     line_number+=*ptr;
     ptr++;
   }
-  
+
   // skip until "
   while(*ptr!='\n' && *ptr!='"') ptr++;
 
@@ -67,9 +60,9 @@ void preprocessor_line(
   // skip "
   if(*ptr!='"')
     return;
-  
+
   ptr++;
-  
+
   std::string file_name_tmp;
 
   // get file name
