@@ -17,8 +17,10 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <memory>
 
 #include <util/config.h>
+#include <util/exception_utils.h>
 #include <util/exit_codes.h>
 #include <util/invariant.h>
+#include <util/make_unique.h>
 #include <util/unicode.h>
 #include <util/version.h>
 
@@ -26,64 +28,84 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <ansi-c/c_preprocess.h>
 #include <ansi-c/cprover_library.h>
+#include <ansi-c/gcc_version.h>
+
+#include <assembler/remove_asm.h>
 
 #include <cpp/cprover_library.h>
 
+#include <goto-checker/all_properties_verifier.h>
+#include <goto-checker/all_properties_verifier_with_fault_localization.h>
+#include <goto-checker/all_properties_verifier_with_trace_storage.h>
+#include <goto-checker/bmc_util.h>
+#include <goto-checker/cover_goals_verifier_with_trace_storage.h>
+#include <goto-checker/multi_path_symex_checker.h>
+#include <goto-checker/multi_path_symex_only_checker.h>
+#include <goto-checker/properties.h>
+#include <goto-checker/single_loop_incremental_symex_checker.h>
+#include <goto-checker/single_path_symex_checker.h>
+#include <goto-checker/single_path_symex_only_checker.h>
+#include <goto-checker/stop_on_fail_verifier.h>
+#include <goto-checker/stop_on_fail_verifier_with_fault_localization.h>
+
 #include <goto-programs/adjust_float_expressions.h>
 #include <goto-programs/initialize_goto_model.h>
 #include <goto-programs/instrument_preconditions.h>
-#include <goto-programs/goto_convert_functions.h>
-#include <goto-programs/goto_inline.h>
 #include <goto-programs/link_to_library.h>
 #include <goto-programs/loop_ids.h>
 #include <goto-programs/mm_io.h>
 #include <goto-programs/read_goto_binary.h>
+#include <goto-programs/remove_complex.h>
 #include <goto-programs/remove_function_pointers.h>
 #include <goto-programs/remove_returns.h>
-#include <goto-programs/remove_vector.h>
-#include <goto-programs/remove_complex.h>
-#include <goto-programs/remove_asm.h>
-#include <goto-programs/remove_unused_functions.h>
 #include <goto-programs/remove_skip.h>
+#include <goto-programs/remove_unused_functions.h>
+#include <goto-programs/remove_vector.h>
+#include <goto-programs/rewrite_union.h>
 #include <goto-programs/set_properties.h>
 #include <goto-programs/show_goto_functions.h>
-#include <goto-programs/show_symbol_table.h>
 #include <goto-programs/show_properties.h>
+#include <goto-programs/show_symbol_table.h>
 #include <goto-programs/string_abstraction.h>
 #include <goto-programs/string_instrumentation.h>
+#include <goto-programs/validate_goto_model.h>
 
-#include <goto-symex/rewrite_union.h>
-
-#include <goto-instrument/reachability_slicer.h>
+#include <goto-instrument/cover.h>
 #include <goto-instrument/full_slicer.h>
 #include <goto-instrument/nondet_static.h>
-#include <goto-instrument/cover.h>
+#include <goto-instrument/reachability_slicer.h>
+
+#include <goto-symex/path_storage.h>
 
 #include <pointer-analysis/add_failed_symbols.h>
 
 #include <langapi/mode.h>
 
-#include "xml_interface.h"
+#include "c_test_input_generator.h"
 
 cbmc_parse_optionst::cbmc_parse_optionst(int argc, const char **argv)
-  : parse_options_baset(CBMC_OPTIONS, argc, argv),
-    xml_interfacet(cmdline),
-    messaget(ui_message_handler),
-    ui_message_handler(cmdline, std::string("CBMC ") + CBMC_VERSION),
-    path_strategy_chooser()
+  : parse_options_baset(
+      CBMC_OPTIONS,
+      argc,
+      argv,
+      std::string("CBMC ") + CBMC_VERSION)
 {
+  json_interface(cmdline, ui_message_handler);
+  xml_interface(cmdline, ui_message_handler);
 }
 
 ::cbmc_parse_optionst::cbmc_parse_optionst(
   int argc,
   const char **argv,
   const std::string &extra_options)
-  : parse_options_baset(CBMC_OPTIONS + extra_options, argc, argv),
-    xml_interfacet(cmdline),
-    messaget(ui_message_handler),
-    ui_message_handler(cmdline, std::string("CBMC ") + CBMC_VERSION),
-    path_strategy_chooser()
+  : parse_options_baset(
+      CBMC_OPTIONS + extra_options,
+      argc,
+      argv,
+      std::string("CBMC ") + CBMC_VERSION)
 {
+  json_interface(cmdline, ui_message_handler);
+  xml_interface(cmdline, ui_message_handler);
 }
 
 void cbmc_parse_optionst::set_default_options(optionst &options)
@@ -95,8 +117,10 @@ void cbmc_parse_optionst::set_default_options(optionst &options)
   options.set_option("pretty-names", true);
   options.set_option("propagation", true);
   options.set_option("sat-preprocessor", true);
+  options.set_option("simple-slice", true);
   options.set_option("simplify", true);
   options.set_option("simplify-if", true);
+  options.set_option("show-goto-symex-steps", false);
 
   // Other default
   options.set_option("arrays-uf", "auto");
@@ -111,26 +135,52 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
   }
 
   cbmc_parse_optionst::set_default_options(options);
+  parse_c_object_factory_options(cmdline, options);
+
+  if(cmdline.isset("function"))
+    options.set_option("function", cmdline.get_value("function"));
 
   if(cmdline.isset("cover") && cmdline.isset("unwinding-assertions"))
   {
-    error() << "--cover and --unwinding-assertions must not be given together"
-            << eom;
+    log.error()
+      << "--cover and --unwinding-assertions must not be given together"
+      << messaget::eom;
     exit(CPROVER_EXIT_USAGE_ERROR);
   }
 
+  if(cmdline.isset("max-field-sensitivity-array-size"))
+  {
+    options.set_option(
+      "max-field-sensitivity-array-size",
+      cmdline.get_value("max-field-sensitivity-array-size"));
+  }
+
+  if(cmdline.isset("no-array-field-sensitivity"))
+  {
+    if(cmdline.isset("max-field-sensitivity-array-size"))
+    {
+      log.error()
+        << "--no-array-field-sensitivity and --max-field-sensitivity-array-size"
+        << " must not be given together" << messaget::eom;
+      exit(CPROVER_EXIT_USAGE_ERROR);
+    }
+    options.set_option("no-array-field-sensitivity", true);
+  }
+
   if(cmdline.isset("partial-loops") && cmdline.isset("unwinding-assertions"))
   {
-    error() << "--partial-loops and --unwinding-assertions must not be given "
-            << "together" << eom;
+    log.error()
+      << "--partial-loops and --unwinding-assertions must not be given "
+      << "together" << messaget::eom;
     exit(CPROVER_EXIT_USAGE_ERROR);
   }
 
   if(cmdline.isset("reachability-slice") &&
      cmdline.isset("reachability-slice-fb"))
   {
-    error() << "--reachability-slice and --reachability-slice-fb must not be "
-            << "given together" << eom;
+    log.error()
+      << "--reachability-slice and --reachability-slice-fb must not be "
+      << "given together" << messaget::eom;
     exit(CPROVER_EXIT_USAGE_ERROR);
   }
 
@@ -139,11 +189,11 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
 
   if(cmdline.isset("show-symex-strategies"))
   {
-    std::cout << path_strategy_chooser.show_strategies();
+    log.status() << show_path_strategies() << messaget::eom;
     exit(CPROVER_EXIT_SUCCESS);
   }
 
-  path_strategy_chooser.set_path_strategy_options(cmdline, options, *this);
+  parse_path_strategy_options(cmdline, options, ui_message_handler);
 
   if(cmdline.isset("program-only"))
     options.set_option("program-only", true);
@@ -160,6 +210,15 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
   if(cmdline.isset("c89"))
     config.ansi_c.set_c89();
 
+  if(cmdline.isset("symex-complexity-limit"))
+    options.set_option(
+      "symex-complexity-limit", cmdline.get_value("symex-complexity-limit"));
+
+  if(cmdline.isset("symex-complexity-failed-child-loops-limit"))
+    options.set_option(
+      "symex-complexity-failed-child-loops-limit",
+      cmdline.get_value("symex-complexity-failed-child-loops-limit"));
+
   if(cmdline.isset("c99"))
     config.ansi_c.set_c99();
 
@@ -201,18 +260,17 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
      cmdline.isset("outfile"))
     options.set_option("stop-on-fail", true);
 
-  if(cmdline.isset("trace") ||
-     cmdline.isset("stop-on-fail"))
+  if(
+    cmdline.isset("trace") || cmdline.isset("compact-trace") ||
+    cmdline.isset("stack-trace") || cmdline.isset("stop-on-fail") ||
+    (ui_message_handler.get_ui() != ui_message_handlert::uit::PLAIN &&
+     !cmdline.isset("cover")))
+  {
     options.set_option("trace", true);
+  }
 
   if(cmdline.isset("localize-faults"))
     options.set_option("localize-faults", true);
-  if(cmdline.isset("localize-faults-method"))
-  {
-    options.set_option(
-      "localize-faults-method",
-      cmdline.get_value("localize-faults-method"));
-  }
 
   if(cmdline.isset("unwind"))
     options.set_option("unwind", cmdline.get_value("unwind"));
@@ -224,10 +282,13 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
     options.set_option("debug-level", cmdline.get_value("debug-level"));
 
   if(cmdline.isset("slice-by-trace"))
-    options.set_option("slice-by-trace", cmdline.get_value("slice-by-trace"));
+  {
+    log.error() << "--slice-by-trace has been removed" << messaget::eom;
+    exit(CPROVER_EXIT_USAGE_ERROR);
+  }
 
   if(cmdline.isset("unwindset"))
-    options.set_option("unwindset", cmdline.get_value("unwindset"));
+    options.set_option("unwindset", cmdline.get_values("unwindset"));
 
   // constant propagation
   if(cmdline.isset("no-propagation"))
@@ -255,7 +316,10 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
 
   // generate unwinding assertions
   if(cmdline.isset("unwinding-assertions"))
+  {
     options.set_option("unwinding-assertions", true);
+    options.set_option("paths-symex-explore-all", true);
+  }
 
   if(cmdline.isset("partial-loops"))
     options.set_option("partial-loops", true);
@@ -299,9 +363,6 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
   {
     options.set_option("refine-strings", true);
     options.set_option("string-printable", cmdline.isset("string-printable"));
-    if(cmdline.isset("string-max-length"))
-      options.set_option(
-        "string-max-length", cmdline.get_value("string-max-length"));
   }
 
   if(cmdline.isset("max-node-refinement"))
@@ -309,11 +370,35 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
       "max-node-refinement",
       cmdline.get_value("max-node-refinement"));
 
+  if(cmdline.isset("incremental-loop"))
+  {
+    options.set_option(
+      "incremental-loop", cmdline.get_value("incremental-loop"));
+    options.set_option("refine", true);
+    options.set_option("refine-arrays", true);
+
+    if(cmdline.isset("unwind-min"))
+      options.set_option("unwind-min", cmdline.get_value("unwind-min"));
+
+    if(cmdline.isset("unwind-max"))
+      options.set_option("unwind-max", cmdline.get_value("unwind-max"));
+
+    if(cmdline.isset("ignore-properties-before-unwind-min"))
+      options.set_option("ignore-properties-before-unwind-min", true);
+
+    if(cmdline.isset("paths"))
+    {
+      log.error() << "--paths not supported with --incremental-loop"
+                  << messaget::eom;
+      exit(CPROVER_EXIT_USAGE_ERROR);
+    }
+  }
+
   // SMT Options
 
   if(cmdline.isset("smt1"))
   {
-    error() << "--smt1 is no longer supported" << eom;
+    log.error() << "--smt1 is no longer supported" << messaget::eom;
     exit(CPROVER_EXIT_USAGE_ERROR);
   }
 
@@ -331,6 +416,12 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
     options.set_option("smt2", true);
   }
 
+  if(cmdline.isset("cprover-smt2"))
+  {
+    options.set_option("cprover-smt2", true), solver_set = true;
+    options.set_option("smt2", true);
+  }
+
   if(cmdline.isset("mathsat"))
   {
     options.set_option("mathsat", true), solver_set=true;
@@ -389,9 +480,25 @@ void cbmc_parse_optionst::get_command_line_options(optionst &options)
   }
 
   if(cmdline.isset("symex-coverage-report"))
+  {
     options.set_option(
       "symex-coverage-report",
       cmdline.get_value("symex-coverage-report"));
+    options.set_option("paths-symex-explore-all", true);
+  }
+
+  if(cmdline.isset("validate-ssa-equation"))
+  {
+    options.set_option("validate-ssa-equation", true);
+  }
+
+  if(cmdline.isset("validate-goto-model"))
+  {
+    options.set_option("validate-goto-model", true);
+  }
+
+  if(cmdline.isset("show-goto-symex-steps"))
+    options.set_option("show-goto-symex-steps", true);
 
   PARSE_OPTIONS_GOTO_TRACE(cmdline, options);
 }
@@ -410,32 +517,17 @@ int cbmc_parse_optionst::doit()
   //
 
   optionst options;
-  try
-  {
-    get_command_line_options(options);
-  }
+  get_command_line_options(options);
 
-  catch(const char *error_msg)
-  {
-    error() << error_msg << eom;
-    return CPROVER_EXIT_EXCEPTION;
-  }
-
-  catch(const std::string &error_msg)
-  {
-    error() << error_msg << eom;
-    return CPROVER_EXIT_EXCEPTION;
-  }
-
-  eval_verbosity(
+  messaget::eval_verbosity(
     cmdline.get_value("verbosity"), messaget::M_STATISTICS, ui_message_handler);
 
   //
   // Print a banner
   //
-  status() << "CBMC version " << CBMC_VERSION << " " << sizeof(void *) * 8
-           << "-bit " << config.this_architecture() << " "
-           << config.this_operating_system() << eom;
+  log.status() << "CBMC version " << CBMC_VERSION << " " << sizeof(void *) * 8
+               << "-bit " << config.this_architecture() << " "
+               << config.this_operating_system() << messaget::eom;
 
   //
   // Unwinding of transition systems is done by hw-cbmc.
@@ -444,30 +536,40 @@ int cbmc_parse_optionst::doit()
   if(cmdline.isset("module") ||
      cmdline.isset("gen-interface"))
   {
-    error() << "This version of CBMC has no support for "
-               " hardware modules. Please use hw-cbmc." << eom;
+    log.error() << "This version of CBMC has no support for "
+                   " hardware modules. Please use hw-cbmc."
+                << messaget::eom;
     return CPROVER_EXIT_USAGE_ERROR;
   }
 
   register_languages();
 
+  // configure gcc, if required
+  if(config.ansi_c.preprocessor == configt::ansi_ct::preprocessort::GCC)
+  {
+    gcc_versiont gcc_version;
+    gcc_version.get("gcc");
+    configure_gcc(gcc_version);
+  }
+
   if(cmdline.isset("test-preprocessor"))
-    return test_c_preprocessor(get_message_handler())
+    return test_c_preprocessor(ui_message_handler)
              ? CPROVER_EXIT_PREPROCESSOR_TEST_FAILED
              : CPROVER_EXIT_SUCCESS;
 
   if(cmdline.isset("preprocess"))
   {
-    preprocessing();
+    preprocessing(options);
     return CPROVER_EXIT_SUCCESS;
   }
 
   if(cmdline.isset("show-parse-tree"))
   {
-    if(cmdline.args.size()!=1 ||
-       is_goto_binary(cmdline.args[0]))
+    if(
+      cmdline.args.size() != 1 ||
+      is_goto_binary(cmdline.args[0], ui_message_handler))
     {
-      error() << "Please give exactly one source file" << eom;
+      log.error() << "Please give exactly one source file" << messaget::eom;
       return CPROVER_EXIT_INCORRECT_TASK;
     }
 
@@ -481,8 +583,8 @@ int cbmc_parse_optionst::doit()
 
     if(!infile)
     {
-      error() << "failed to open input file `"
-              << filename << "'" << eom;
+      log.error() << "failed to open input file '" << filename << "'"
+                  << messaget::eom;
       return CPROVER_EXIT_INCORRECT_TASK;
     }
 
@@ -491,19 +593,19 @@ int cbmc_parse_optionst::doit()
 
     if(language==nullptr)
     {
-      error() << "failed to figure out type of file `"
-              <<  filename << "'" << eom;
+      log.error() << "failed to figure out type of file '" << filename << "'"
+                  << messaget::eom;
       return CPROVER_EXIT_INCORRECT_TASK;
     }
 
-    language->get_language_options(cmdline);
-    language->set_message_handler(get_message_handler());
+    language->set_language_options(options);
+    language->set_message_handler(ui_message_handler);
 
-    status() << "Parsing " << filename << eom;
+    log.status() << "Parsing " << filename << messaget::eom;
 
     if(language->parse(infile, filename))
     {
-      error() << "PARSING ERROR" << eom;
+      log.error() << "PARSING ERROR" << messaget::eom;
       return CPROVER_EXIT_INCORRECT_TASK;
     }
 
@@ -512,7 +614,7 @@ int cbmc_parse_optionst::doit()
   }
 
   int get_goto_program_ret =
-    get_goto_program(goto_model, options, cmdline, *this, ui_message_handler);
+    get_goto_program(goto_model, options, cmdline, ui_message_handler);
 
   if(get_goto_program_ret!=-1)
     return get_goto_program_ret;
@@ -520,50 +622,149 @@ int cbmc_parse_optionst::doit()
   if(cmdline.isset("show-claims") || // will go away
      cmdline.isset("show-properties")) // use this one
   {
-    show_properties(
-      goto_model, get_message_handler(), ui_message_handler.get_ui());
+    show_properties(goto_model, ui_message_handler);
     return CPROVER_EXIT_SUCCESS;
   }
 
   if(set_properties())
     return CPROVER_EXIT_SET_PROPERTIES_FAILED;
 
-  return bmct::do_language_agnostic_bmc(
-    path_strategy_chooser,
-    options,
-    goto_model,
-    ui_message_handler.get_ui(),
-    *this);
+  if(
+    options.get_bool_option("program-only") ||
+    options.get_bool_option("show-vcc"))
+  {
+    if(options.get_bool_option("paths"))
+    {
+      all_properties_verifiert<single_path_symex_only_checkert> verifier(
+        options, ui_message_handler, goto_model);
+      (void)verifier();
+    }
+    else
+    {
+      all_properties_verifiert<multi_path_symex_only_checkert> verifier(
+        options, ui_message_handler, goto_model);
+      (void)verifier();
+    }
+
+    return CPROVER_EXIT_SUCCESS;
+  }
+
+  if(
+    options.get_bool_option("dimacs") || !options.get_option("outfile").empty())
+  {
+    if(options.get_bool_option("paths"))
+    {
+      stop_on_fail_verifiert<single_path_symex_checkert> verifier(
+        options, ui_message_handler, goto_model);
+      (void)verifier();
+    }
+    else
+    {
+      stop_on_fail_verifiert<multi_path_symex_checkert> verifier(
+        options, ui_message_handler, goto_model);
+      (void)verifier();
+    }
+
+    return CPROVER_EXIT_SUCCESS;
+  }
+
+  if(options.is_set("cover"))
+  {
+    cover_goals_verifier_with_trace_storaget<multi_path_symex_checkert>
+      verifier(options, ui_message_handler, goto_model);
+    (void)verifier();
+    verifier.report();
+
+    c_test_input_generatort test_generator(ui_message_handler, options);
+    test_generator(verifier.get_traces());
+
+    return CPROVER_EXIT_SUCCESS;
+  }
+
+  std::unique_ptr<goto_verifiert> verifier = nullptr;
+
+  if(options.is_set("incremental-loop"))
+  {
+    if(options.get_bool_option("stop-on-fail"))
+    {
+      verifier = util_make_unique<
+        stop_on_fail_verifiert<single_loop_incremental_symex_checkert>>(
+        options, ui_message_handler, goto_model);
+    }
+    else
+    {
+      verifier = util_make_unique<all_properties_verifier_with_trace_storaget<
+        single_loop_incremental_symex_checkert>>(
+        options, ui_message_handler, goto_model);
+    }
+  }
+  else if(
+    options.get_bool_option("stop-on-fail") && options.get_bool_option("paths"))
+  {
+    verifier =
+      util_make_unique<stop_on_fail_verifiert<single_path_symex_checkert>>(
+        options, ui_message_handler, goto_model);
+  }
+  else if(
+    options.get_bool_option("stop-on-fail") &&
+    !options.get_bool_option("paths"))
+  {
+    if(options.get_bool_option("localize-faults"))
+    {
+      verifier =
+        util_make_unique<stop_on_fail_verifier_with_fault_localizationt<
+          multi_path_symex_checkert>>(options, ui_message_handler, goto_model);
+    }
+    else
+    {
+      verifier =
+        util_make_unique<stop_on_fail_verifiert<multi_path_symex_checkert>>(
+          options, ui_message_handler, goto_model);
+    }
+  }
+  else if(
+    !options.get_bool_option("stop-on-fail") &&
+    options.get_bool_option("paths"))
+  {
+    verifier = util_make_unique<
+      all_properties_verifier_with_trace_storaget<single_path_symex_checkert>>(
+      options, ui_message_handler, goto_model);
+  }
+  else if(
+    !options.get_bool_option("stop-on-fail") &&
+    !options.get_bool_option("paths"))
+  {
+    if(options.get_bool_option("localize-faults"))
+    {
+      verifier =
+        util_make_unique<all_properties_verifier_with_fault_localizationt<
+          multi_path_symex_checkert>>(options, ui_message_handler, goto_model);
+    }
+    else
+    {
+      verifier = util_make_unique<
+        all_properties_verifier_with_trace_storaget<multi_path_symex_checkert>>(
+        options, ui_message_handler, goto_model);
+    }
+  }
+  else
+  {
+    UNREACHABLE;
+  }
+
+  const resultt result = (*verifier)();
+  verifier->report();
+
+  return result_to_exit_code(result);
 }
 
 bool cbmc_parse_optionst::set_properties()
 {
-  try
-  {
-    if(cmdline.isset("claim")) // will go away
-      ::set_properties(goto_model, cmdline.get_values("claim"));
+  if(cmdline.isset("claim")) // will go away
+    ::set_properties(goto_model, cmdline.get_values("claim"));
 
-    if(cmdline.isset("property")) // use this one
-      ::set_properties(goto_model, cmdline.get_values("property"));
-  }
-
-  catch(const char *e)
-  {
-    error() << e << eom;
-    return true;
-  }
-
-  catch(const std::string &e)
-  {
-    error() << e << eom;
-    return true;
-  }
-
-  catch(int e)
-  {
-    error() << "Numeric exception : " << e << eom;
-    return true;
-  }
+  if(cmdline.isset("property")) // use this one
+    ::set_properties(goto_model, cmdline.get_values("property"));
 
   return false;
 }
@@ -572,133 +773,84 @@ int cbmc_parse_optionst::get_goto_program(
   goto_modelt &goto_model,
   const optionst &options,
   const cmdlinet &cmdline,
-  messaget &log,
   ui_message_handlert &ui_message_handler)
 {
+  messaget log{ui_message_handler};
   if(cmdline.args.empty())
   {
-    log.error() << "Please provide a program to verify" << log.eom;
+    log.error() << "Please provide a program to verify" << messaget::eom;
     return CPROVER_EXIT_INCORRECT_TASK;
   }
 
-  try
+  goto_model = initialize_goto_model(cmdline.args, ui_message_handler, options);
+
+  if(cmdline.isset("show-symbol-table"))
   {
-    goto_model = initialize_goto_model(cmdline, ui_message_handler);
-
-    if(cmdline.isset("show-symbol-table"))
-    {
-      show_symbol_table(goto_model, ui_message_handler.get_ui());
-      return CPROVER_EXIT_SUCCESS;
-    }
-
-    if(cbmc_parse_optionst::process_goto_program(goto_model, options, log))
-      return CPROVER_EXIT_INTERNAL_ERROR;
-
-    // show it?
-    if(cmdline.isset("show-loops"))
-    {
-      show_loop_ids(ui_message_handler.get_ui(), goto_model);
-      return CPROVER_EXIT_SUCCESS;
-    }
-
-    // show it?
-    if(
-      cmdline.isset("show-goto-functions") ||
-      cmdline.isset("list-goto-functions"))
-    {
-      show_goto_functions(
-        goto_model,
-        ui_message_handler,
-        ui_message_handler.get_ui(),
-        cmdline.isset("list-goto-functions"));
-      return CPROVER_EXIT_SUCCESS;
-    }
-
-    log.status() << config.object_bits_info() << log.eom;
+    show_symbol_table(goto_model, ui_message_handler);
+    return CPROVER_EXIT_SUCCESS;
   }
 
-  catch(const char *e)
+  if(cbmc_parse_optionst::process_goto_program(goto_model, options, log))
+    return CPROVER_EXIT_INTERNAL_ERROR;
+
+  if(cmdline.isset("validate-goto-model"))
   {
-    log.error() << e << log.eom;
-    return CPROVER_EXIT_EXCEPTION;
+    goto_model.validate();
   }
 
-  catch(const std::string &e)
+  // show it?
+  if(cmdline.isset("show-loops"))
   {
-    log.error() << e << log.eom;
-    return CPROVER_EXIT_EXCEPTION;
+    show_loop_ids(ui_message_handler.get_ui(), goto_model);
+    return CPROVER_EXIT_SUCCESS;
   }
 
-  catch(int e)
+  // show it?
+  if(
+    cmdline.isset("show-goto-functions") ||
+    cmdline.isset("list-goto-functions"))
   {
-    log.error() << "Numeric exception : " << e << log.eom;
-    return CPROVER_EXIT_EXCEPTION;
+    show_goto_functions(
+      goto_model, ui_message_handler, cmdline.isset("list-goto-functions"));
+    return CPROVER_EXIT_SUCCESS;
   }
 
-  catch(const std::bad_alloc &)
-  {
-    log.error() << "Out of memory" << log.eom;
-    return CPROVER_EXIT_INTERNAL_OUT_OF_MEMORY;
-  }
+  log.status() << config.object_bits_info() << messaget::eom;
 
   return -1; // no error, continue
 }
 
-void cbmc_parse_optionst::preprocessing()
+void cbmc_parse_optionst::preprocessing(const optionst &options)
 {
-  try
+  if(cmdline.args.size() != 1)
   {
-    if(cmdline.args.size()!=1)
-    {
-      error() << "Please provide one program to preprocess" << eom;
-      return;
-    }
-
-    std::string filename=cmdline.args[0];
-
-    std::ifstream infile(filename);
-
-    if(!infile)
-    {
-      error() << "failed to open input file" << eom;
-      return;
-    }
-
-    std::unique_ptr<languaget> language=get_language_from_filename(filename);
-    language->get_language_options(cmdline);
-
-    if(language==nullptr)
-    {
-      error() << "failed to figure out type of file" << eom;
-      return;
-    }
-
-    language->set_message_handler(get_message_handler());
-
-    if(language->preprocess(infile, filename, std::cout))
-      error() << "PREPROCESSING ERROR" << eom;
+    log.error() << "Please provide one program to preprocess" << messaget::eom;
+    return;
   }
 
-  catch(const char *e)
+  std::string filename = cmdline.args[0];
+
+  std::ifstream infile(filename);
+
+  if(!infile)
   {
-    error() << e << eom;
+    log.error() << "failed to open input file" << messaget::eom;
+    return;
   }
 
-  catch(const std::string &e)
+  std::unique_ptr<languaget> language = get_language_from_filename(filename);
+  language->set_language_options(options);
+
+  if(language == nullptr)
   {
-    error() << e << eom;
+    log.error() << "failed to figure out type of file" << messaget::eom;
+    return;
   }
 
-  catch(int e)
-  {
-    error() << "Numeric exception : " << e << eom;
-  }
+  language->set_message_handler(ui_message_handler);
 
-  catch(const std::bad_alloc &)
-  {
-    error() << "Out of memory" << eom;
-    exit(CPROVER_EXIT_INTERNAL_OUT_OF_MEMORY);
-  }
+  if(language->preprocess(infile, filename, std::cout))
+    log.error() << "PREPROCESSING ERROR" << messaget::eom;
 }
 
 bool cbmc_parse_optionst::process_goto_program(
@@ -706,159 +858,135 @@ bool cbmc_parse_optionst::process_goto_program(
   const optionst &options,
   messaget &log)
 {
-  try
+  // Remove inline assembler; this needs to happen before
+  // adding the library.
+  remove_asm(goto_model);
+
+  // add the library
+  log.status() << "Adding CPROVER library (" << config.ansi_c.arch << ")"
+               << messaget::eom;
+  link_to_library(
+    goto_model, log.get_message_handler(), cprover_cpp_library_factory);
+  link_to_library(
+    goto_model, log.get_message_handler(), cprover_c_library_factory);
+
+  if(options.get_bool_option("string-abstraction"))
+    string_instrumentation(goto_model, log.get_message_handler());
+
+  // remove function pointers
+  log.status() << "Removal of function pointers and virtual functions"
+               << messaget::eom;
+  remove_function_pointers(
+    log.get_message_handler(),
+    goto_model,
+    options.get_bool_option("pointer-check"));
+
+  mm_io(goto_model);
+
+  // instrument library preconditions
+  instrument_preconditions(goto_model);
+
+  // remove returns, gcc vectors, complex
+  remove_returns(goto_model);
+  remove_vector(goto_model);
+  remove_complex(goto_model);
+  rewrite_union(goto_model);
+
+  // add generic checks
+  log.status() << "Generic Property Instrumentation" << messaget::eom;
+  goto_check(options, goto_model);
+
+  // checks don't know about adjusted float expressions
+  adjust_float_expressions(goto_model);
+
+  // ignore default/user-specified initialization
+  // of variables with static lifetime
+  if(options.get_bool_option("nondet-static"))
   {
-    // Remove inline assembler; this needs to happen before
-    // adding the library.
-    remove_asm(goto_model);
-
-    // add the library
-    log.status() << "Adding CPROVER library (" << config.ansi_c.arch << ")"
-                 << eom;
-    link_to_library(
-      goto_model, log.get_message_handler(), cprover_cpp_library_factory);
-    link_to_library(
-      goto_model, log.get_message_handler(), cprover_c_library_factory);
-
-    if(options.get_bool_option("string-abstraction"))
-      string_instrumentation(goto_model, log.get_message_handler());
-
-    // remove function pointers
-    log.status() << "Removal of function pointers and virtual functions" << eom;
-    remove_function_pointers(
-      log.get_message_handler(),
-      goto_model,
-      options.get_bool_option("pointer-check"));
-
-    mm_io(goto_model);
-
-    // instrument library preconditions
-    instrument_preconditions(goto_model);
-
-    // remove returns, gcc vectors, complex
-    remove_returns(goto_model);
-    remove_vector(goto_model);
-    remove_complex(goto_model);
-    rewrite_union(goto_model);
-
-    // add generic checks
-    log.status() << "Generic Property Instrumentation" << eom;
-    goto_check(options, goto_model);
-
-    // checks don't know about adjusted float expressions
-    adjust_float_expressions(goto_model);
-
-    // ignore default/user-specified initialization
-    // of variables with static lifetime
-    if(options.get_bool_option("nondet-static"))
-    {
-      log.status() << "Adding nondeterministic initialization "
-                      "of static/global variables"
-                   << eom;
-      nondet_static(goto_model);
-    }
-
-    if(options.get_bool_option("string-abstraction"))
-    {
-      log.status() << "String Abstraction" << eom;
-      string_abstraction(goto_model, log.get_message_handler());
-    }
-
-    // add failed symbols
-    // needs to be done before pointer analysis
-    add_failed_symbols(goto_model.symbol_table);
-
-    // recalculate numbers, etc.
-    goto_model.goto_functions.update();
-
-    // add loop ids
-    goto_model.goto_functions.compute_loop_numbers();
-
-    if(options.get_bool_option("drop-unused-functions"))
-    {
-      // Entry point will have been set before and function pointers removed
-      log.status() << "Removing unused functions" << eom;
-      remove_unused_functions(goto_model, log.get_message_handler());
-    }
-
-    // remove skips such that trivial GOTOs are deleted and not considered
-    // for coverage annotation:
-    remove_skip(goto_model);
-
-    // instrument cover goals
-    if(options.is_set("cover"))
-    {
-      if(instrument_cover_goals(options, goto_model, log.get_message_handler()))
-        return true;
-    }
-
-    // label the assertions
-    // This must be done after adding assertions and
-    // before using the argument of the "property" option.
-    // Do not re-label after using the property slicer because
-    // this would cause the property identifiers to change.
-    label_properties(goto_model);
-
-    // reachability slice?
-    if(options.get_bool_option("reachability-slice-fb"))
-    {
-      log.status() << "Performing a forwards-backwards reachability slice"
-                   << eom;
-      if(options.is_set("property"))
-        reachability_slicer(
-          goto_model, options.get_list_option("property"), true);
-      else
-        reachability_slicer(goto_model, true);
-    }
-
-    if(options.get_bool_option("reachability-slice"))
-    {
-      log.status() << "Performing a reachability slice" << eom;
-      if(options.is_set("property"))
-        reachability_slicer(goto_model, options.get_list_option("property"));
-      else
-        reachability_slicer(goto_model);
-    }
-
-    // full slice?
-    if(options.get_bool_option("full-slice"))
-    {
-      log.status() << "Performing a full slice" << eom;
-      if(options.is_set("property"))
-        property_slicer(goto_model, options.get_list_option("property"));
-      else
-        full_slicer(goto_model);
-    }
-
-    // remove any skips introduced since coverage instrumentation
-    remove_skip(goto_model);
+    log.status() << "Adding nondeterministic initialization "
+                    "of static/global variables"
+                 << messaget::eom;
+    nondet_static(goto_model);
   }
 
-  catch(const char *e)
+  if(options.get_bool_option("string-abstraction"))
   {
-    log.error() << e << eom;
-    return true;
+    log.status() << "String Abstraction" << messaget::eom;
+    string_abstraction(goto_model, log.get_message_handler());
   }
 
-  catch(const std::string &e)
+  // add failed symbols
+  // needs to be done before pointer analysis
+  add_failed_symbols(goto_model.symbol_table);
+
+  // recalculate numbers, etc.
+  goto_model.goto_functions.update();
+
+  // add loop ids
+  goto_model.goto_functions.compute_loop_numbers();
+
+  if(options.get_bool_option("drop-unused-functions"))
   {
-    log.error() << e << eom;
-    return true;
+    // Entry point will have been set before and function pointers removed
+    log.status() << "Removing unused functions" << messaget::eom;
+    remove_unused_functions(goto_model, log.get_message_handler());
   }
 
-  catch(int e)
+  // remove skips such that trivial GOTOs are deleted and not considered
+  // for coverage annotation:
+  remove_skip(goto_model);
+
+  // instrument cover goals
+  if(options.is_set("cover"))
   {
-    log.error() << "Numeric exception : " << e << eom;
-    return true;
+    const auto cover_config = get_cover_config(
+      options, goto_model.symbol_table, log.get_message_handler());
+    if(instrument_cover_goals(
+         cover_config, goto_model, log.get_message_handler()))
+      return true;
   }
 
-  catch(const std::bad_alloc &)
+  // label the assertions
+  // This must be done after adding assertions and
+  // before using the argument of the "property" option.
+  // Do not re-label after using the property slicer because
+  // this would cause the property identifiers to change.
+  label_properties(goto_model);
+
+  // reachability slice?
+  if(options.get_bool_option("reachability-slice-fb"))
   {
-    log.error() << "Out of memory" << eom;
-    exit(CPROVER_EXIT_INTERNAL_OUT_OF_MEMORY);
-    return true;
+    log.status() << "Performing a forwards-backwards reachability slice"
+                 << messaget::eom;
+    if(options.is_set("property"))
+      reachability_slicer(
+        goto_model, options.get_list_option("property"), true);
+    else
+      reachability_slicer(goto_model, true);
   }
 
+  if(options.get_bool_option("reachability-slice"))
+  {
+    log.status() << "Performing a reachability slice" << messaget::eom;
+    if(options.is_set("property"))
+      reachability_slicer(goto_model, options.get_list_option("property"));
+    else
+      reachability_slicer(goto_model);
+  }
+
+  // full slice?
+  if(options.get_bool_option("full-slice"))
+  {
+    log.status() << "Performing a full slice" << messaget::eom;
+    if(options.is_set("property"))
+      property_slicer(goto_model, options.get_list_option("property"));
+    else
+      full_slicer(goto_model);
+  }
+
+  // remove any skips introduced since coverage instrumentation
+  remove_skip(goto_model);
+
   return false;
 }
 
@@ -867,12 +995,12 @@ void cbmc_parse_optionst::help()
 {
   // clang-format off
   std::cout << '\n' << banner_string("CBMC", CBMC_VERSION) << '\n'
+            << align_center_with_border("Copyright (C) 2001-2018") << '\n'
+            << align_center_with_border("Daniel Kroening, Edmund Clarke") << '\n' // NOLINT(*)
+            << align_center_with_border("Carnegie Mellon University, Computer Science Department") << '\n' // NOLINT(*)
+            << align_center_with_border("kroening@kroening.com") << '\n' // NOLINT(*)
+            << align_center_with_border("Protected in part by U.S. patent 7,225,417") << '\n' // NOLINT(*)
             <<
-    "* *                 Copyright (C) 2001-2018                 * *\n"
-    "* *              Daniel Kroening, Edmund Clarke             * *\n"
-    "* * Carnegie Mellon University, Computer Science Department * *\n"
-    "* *                 kroening@kroening.com                   * *\n"
-    "* *        Protected in part by U.S. patent 7,225,417       * *\n"
     "\n"
     "Usage:                       Purpose:\n"
     "\n"
@@ -924,13 +1052,13 @@ void cbmc_parse_optionst::help()
     " --round-to-plus-inf          rounding towards plus infinity\n"
     " --round-to-minus-inf         rounding towards minus infinity\n"
     " --round-to-zero              rounding towards zero\n"
+    HELP_ANSI_C_LANGUAGE
     HELP_FUNCTIONS
     "\n"
     "Program representations:\n"
     " --show-parse-tree            show parse tree\n"
     " --show-symbol-table          show loaded symbol table\n"
     HELP_SHOW_GOTO_FUNCTIONS
-    " --drop-unused-functions      drop functions trivially unreachable from main function\n" // NOLINT(*)
     "\n"
     "Program instrumentation options:\n"
     HELP_GOTO_CHECK
@@ -940,7 +1068,9 @@ void cbmc_parse_optionst::help()
     " --cover CC                   create test-suite with coverage criterion CC\n" // NOLINT(*)
     " --mm MM                      memory consistency model for concurrent programs\n" // NOLINT(*)
     HELP_REACHABILITY_SLICER
+    HELP_REACHABILITY_SLICER_FB
     " --full-slice                 run full slicer (experimental)\n" // NOLINT(*)
+    " --drop-unused-functions      drop functions trivially unreachable from main function\n" // NOLINT(*)
     "\n"
     "Semantic transformations:\n"
     // NOLINTNEXTLINE(whitespace/line_length)
@@ -954,28 +1084,24 @@ void cbmc_parse_optionst::help()
     " --dimacs                     generate CNF in DIMACS format\n"
     " --beautify                   beautify the counterexample (greedy heuristic)\n" // NOLINT(*)
     " --localize-faults            localize faults (experimental)\n"
-    " --smt1                       use default SMT1 solver (obsolete)\n"
     " --smt2                       use default SMT2 solver (Z3)\n"
     " --boolector                  use Boolector\n"
-    " --mathsat                    use MathSAT\n"
+    " --cprover-smt2               use CPROVER SMT2 solver\n"
     " --cvc4                       use CVC4\n"
+    " --mathsat                    use MathSAT\n"
     " --yices                      use Yices\n"
     " --z3                         use Z3\n"
     " --refine                     use refinement procedure (experimental)\n"
-    " --refine-strings             use string refinement (experimental)\n"
-    " --string-printable           add constraint that strings are printable (experimental)\n" // NOLINT(*)
-    " --string-max-input-length    add constraint on the length of input strings\n" // NOLINT(*)
-    " --string-max-length          add constraint on the length of strings"
-    "                              (deprecated: use string-max-input-length instead)\n" // NOLINT(*)
+    HELP_STRING_REFINEMENT_CBMC
     " --outfile filename           output formula to given file\n"
     " --arrays-uf-never            never turn arrays into uninterpreted functions\n" // NOLINT(*)
     " --arrays-uf-always           always turn arrays into uninterpreted functions\n" // NOLINT(*)
     "\n"
     "Other options:\n"
     " --version                    show version and exit\n"
-    " --xml-ui                     use XML-formatted output\n"
-    " --xml-interface              bi-directional XML interface\n"
-    " --json-ui                    use JSON-formatted output\n"
+    HELP_XML_INTERFACE
+    HELP_JSON_INTERFACE
+    HELP_VALIDATE
     HELP_GOTO_TRACE
     HELP_FLUSH
     " --verbosity #                verbosity level\n"
