@@ -12,21 +12,28 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_CBMC_CBMC_PARSE_OPTIONS_H
 #define CPROVER_CBMC_CBMC_PARSE_OPTIONS_H
 
-#include <util/ui_message.h>
+#include <ansi-c/ansi_c_language.h>
+#include <ansi-c/c_object_factory_parameters.h>
+
 #include <util/parse_options.h>
 #include <util/timestamper.h>
+#include <util/ui_message.h>
+#include <util/validation_interface.h>
 
 #include <langapi/language.h>
 
 #include <analyses/goto_check.h>
 
+#include <goto-checker/bmc_util.h>
+#include <goto-checker/solver_factory.h>
+
 #include <goto-programs/goto_trace.h>
 
-#include "bmc.h"
-#include "xml_interface.h"
-#include "cbmc_solvers.h"
+#include <solvers/strings/string_refinement.h>
+
+#include <json/json_interface.h>
+#include <xmllang/xml_interface.h>
 
-class bmct;
 class goto_functionst;
 class optionst;
 
@@ -43,15 +50,14 @@ class optionst;
   "(object-bits):" \
   OPT_GOTO_CHECK \
   "(no-assertions)(no-assumptions)" \
-  "(xml-ui)(xml-interface)(json-ui)" \
-  "(smt1)(smt2)(fpa)(cvc3)(cvc4)(boolector)(yices)(z3)(opensmt)(mathsat)" \
+  OPT_XML_INTERFACE \
+  OPT_JSON_INTERFACE \
+  "(smt1)(smt2)(fpa)(cvc3)(cvc4)(boolector)(yices)(z3)(mathsat)" \
+  "(cprover-smt2)" \
   "(no-sat-preprocessor)" \
   "(beautify)" \
   "(dimacs)(refine)(max-node-refinement):(refine-arrays)(refine-arithmetic)"\
-  "(refine-strings)" \
-  "(string-printable)" \
-  "(string-max-length):" \
-  "(string-max-input-length):" \
+  OPT_STRING_REFINEMENT_CBMC \
   "(16)(32)(64)(LP64)(ILP64)(LLP64)(ILP32)(LP32)" \
   "(little-endian)(big-endian)" \
   OPT_SHOW_GOTO_FUNCTIONS \
@@ -71,15 +77,14 @@ class optionst;
   "(string-abstraction)(no-arch)(arch):" \
   "(round-to-nearest)(round-to-plus-inf)(round-to-minus-inf)(round-to-zero)" \
   OPT_FLUSH \
-  "(localize-faults)(localize-faults-method):" \
+  "(localize-faults)" \
   OPT_GOTO_TRACE \
+  OPT_VALIDATE \
+  OPT_ANSI_C_LANGUAGE \
   "(claim):(show-claims)(floatbv)(all-claims)(all-properties)" // legacy, and will eventually disappear // NOLINT(whitespace/line_length)
 // clang-format on
 
-class cbmc_parse_optionst:
-  public parse_options_baset,
-  public xml_interfacet,
-  public messaget
+class cbmc_parse_optionst : public parse_options_baset
 {
 public:
   virtual int doit() override;
@@ -103,17 +108,14 @@ public:
     goto_modelt &,
     const optionst &,
     const cmdlinet &,
-    messaget &,
     ui_message_handlert &);
 
 protected:
   goto_modelt goto_model;
-  ui_message_handlert ui_message_handler;
-  const path_strategy_choosert path_strategy_chooser;
 
   void register_languages();
   void get_command_line_options(optionst &);
-  void preprocessing();
+  void preprocessing(const optionst &);
   bool set_properties();
 };
 
