@@ -11,16 +11,18 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 #include "cpp_declarator.h"
 
+#include <ansi-c/merged_type.h>
+
 #include <ostream>
 #include <cassert>
 
 void cpp_declaratort::output(std::ostream &out) const
 {
-  out << "  name: " << name().pretty() << "\n";
-  out << "  type: " << type().pretty() << "\n";
-  out << "  value: " << value().pretty() << "\n";
-  out << "  init_args: " << init_args().pretty() << "\n";
-  out << "  method_qualifier: " << method_qualifier().pretty() << "\n";
+  out << "  name: " << name().pretty() << '\n';
+  out << "  type: " << type().pretty() << '\n';
+  out << "  value: " << value().pretty() << '\n';
+  out << "  init_args: " << init_args().pretty() << '\n';
+  out << "  method_qualifier: " << method_qualifier().pretty() << '\n';
 }
 
 typet cpp_declaratort::merge_type(const typet &declaration_type) const
@@ -44,8 +46,8 @@ typet cpp_declaratort::merge_type(const typet &declaration_type) const
     else if(t.id()==ID_merged_type)
     {
       // the chain continues with the last one
-      assert(!t.subtypes().empty());
-      p=&t.subtypes().back();
+      auto &merged_type = to_merged_type(t);
+      p = &merged_type.last_type();
     }
     else
     {
