@@ -17,5 +17,5 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 void cpp_namespace_spect::output(std::ostream &out) const
 {
-  out << "  namespace: " << get_namespace() << "\n";
+  out << "  namespace: " << get_namespace() << '\n';
 }
