@@ -6,12 +6,14 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
-#ifndef CPROVER_CPP_PARSER_H
-#define CPROVER_CPP_PARSER_H
+/// \file
+/// C++ Parser
+
+#ifndef CPROVER_CPP_CPP_PARSER_H
+#define CPROVER_CPP_CPP_PARSER_H
 
 #include <cassert>
 
-#include <util/hash_cont.h>
 #include <util/string_hash.h>
 #include <util/parser.h>
 #include <util/expr.h>
@@ -35,35 +37,35 @@ public:
     token_buffer.clear();
     asm_block_following=false;
   }
-  
+
   cpp_parsert():
     mode(configt::ansi_ct::flavourt::ANSI),
     recognize_wchar_t(true),
-    asm_block_following(false)    
+    asm_block_following(false)
   {
   }
 
 public:
   // internal state
-  ansi_c_parsert::modet mode;  
-  
+  ansi_c_parsert::modet mode;
+
   // We can furthermore twiddle the recognition of various
   // keywords. This is honored in particular modes.
   bool recognize_wchar_t;
 
   cpp_token_buffert token_buffer;
-  
+
   cpp_tokent &current_token()
   {
     return token_buffer.current_token();
   }
-   
+
   void add_location()
   {
     token_buffer.current_token().line_no=get_line_no()-1;
     token_buffer.current_token().filename=source_location.get_file();
   }
-  
+
   // scanner
   unsigned parenthesis_counter;
   bool asm_block_following;
@@ -71,4 +73,4 @@ public:
 
 extern cpp_parsert cpp_parser;
 
-#endif
+#endif // CPROVER_CPP_CPP_PARSER_H
