@@ -40,7 +40,7 @@ void cpp_typecheckt::typecheck_enum_body(symbolt &enum_symbol)
       typecheck_expr(value);
       implicit_typecast(value, c_enum_type.subtype());
       make_constant(value);
-      if(to_integer(value, i))
+      if(to_integer(to_constant_expr(value), i))
       {
         error().source_location=value.find_source_location();
         error() << "failed to produce integer for enum constant" << eom;
@@ -58,11 +58,13 @@ void cpp_typecheckt::typecheck_enum_body(symbolt &enum_symbol)
     symbol.value=value_expr;
     symbol.location=
       static_cast<const source_locationt &>(it->find(ID_C_source_location));
-    symbol.mode=ID_cpp;
+    symbol.mode = enum_symbol.mode;
     symbol.module=module;
     symbol.type=enum_tag_type;
     symbol.is_type=false;
     symbol.is_macro=true;
+    symbol.is_file_local = true;
+    symbol.is_thread_local = true;
 
     symbolt *new_symbol;
     if(symbol_table.move(symbol, new_symbol))
@@ -133,10 +135,9 @@ void cpp_typecheckt::typecheck_enum_type(typet &type)
     if(has_body)
     {
       error().source_location=type.source_location();
-      error() << "error: enum symbol `" << base_name
+      error() << "error: enum symbol '" << base_name
               << "' declared previously\n"
-              << "location of previous definition: "
-              << symbol.location << eom;
+              << "location of previous definition: " << symbol.location << eom;
       throw 0;
     }
   }
@@ -155,11 +156,10 @@ void cpp_typecheckt::typecheck_enum_type(typet &type)
     else
     {
       typecheck_type(type.subtype());
-      if(type.subtype().id()==ID_signedbv ||
-         type.subtype().id()==ID_unsignedbv)
-      {
-      }
-      else
+      if(
+        type.subtype().id() != ID_signedbv &&
+        type.subtype().id() != ID_unsignedbv &&
+        type.subtype().id() != ID_c_bool)
       {
         error().source_location=type.source_location();
         error() << "underlying type must be integral" << eom;
@@ -197,7 +197,7 @@ void cpp_typecheckt::typecheck_enum_type(typet &type)
     scope_identifier.id_class=cpp_idt::id_classt::CLASS;
     scope_identifier.is_scope = true;
 
-    cpp_save_scopet save_scope(cpp_scopes);
+    cpp_save_scopet save_scope_before_enum_typecheck(cpp_scopes);
 
     if(new_symbol->type.get_bool(ID_C_class))
       cpp_scopes.go_to(scope_identifier);
@@ -208,8 +208,8 @@ void cpp_typecheckt::typecheck_enum_type(typet &type)
   else
   {
     error().source_location=type.source_location();
-    error() << "use of enum `" << base_name
-            << "' without previous declaration" << eom;
+    error() << "use of enum '" << base_name << "' without previous declaration"
+            << eom;
     throw 0;
   }
 
