@@ -31,8 +31,8 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
     if(symbol.value.id()!=ID_type)
     {
       error().source_location=symbol.location;
-      error() << "expected type as initializer for `"
-              << symbol.base_name << "'" << eom;
+      error() << "expected type as initializer for '" << symbol.base_name << "'"
+              << eom;
       throw 0;
     }
 
@@ -48,9 +48,8 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
     if(is_reference(symbol.type))
     {
       error().source_location=symbol.location;
-      error() << "`" << symbol.base_name
-              << "' is declared as reference but is not initialized"
-              << eom;
+      error() << "'" << symbol.base_name
+              << "' is declared as reference but is not initialized" << eom;
       throw 0;
     }
 
@@ -66,7 +65,7 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
 
     if(has_auto(symbol.type))
     {
-      cpp_convert_auto(symbol.type, symbol.value.type());
+      cpp_convert_auto(symbol.type, symbol.value.type(), get_message_handler());
       typecheck_type(symbol.type);
       implicit_typecast(symbol.value, symbol.type);
     }
@@ -75,10 +74,10 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
   }
   else if(cpp_is_pod(symbol.type))
   {
-    if(symbol.type.id() == ID_pointer &&
-       symbol.type.subtype().id() == ID_code &&
-       symbol.value.id() == ID_address_of &&
-       symbol.value.op0().id() == ID_cpp_name)
+    if(
+      symbol.type.id() == ID_pointer && symbol.type.subtype().id() == ID_code &&
+      symbol.value.id() == ID_address_of &&
+      to_address_of_expr(symbol.value).object().id() == ID_cpp_name)
     {
       // initialization of a function pointer with
       // the address of a function: use pointer type information
@@ -89,27 +88,25 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
 
       const code_typet &code_type=to_code_type(symbol.type.subtype());
 
-      for(code_typet::parameterst::const_iterator
-          ait=code_type.parameters().begin();
-          ait!=code_type.parameters().end();
-          ait++)
+      for(const auto &parameter : code_type.parameters())
       {
-        exprt new_object("new_object");
+        exprt new_object(ID_new_object, parameter.type());
         new_object.set(ID_C_lvalue, true);
-        new_object.type() = ait->type();
 
-        if(ait->get(ID_C_base_name)==ID_this)
+        if(parameter.get_this())
         {
           fargs.has_object = true;
-          new_object.type() = ait->type().subtype();
+          new_object.type() = parameter.type().subtype();
         }
 
         fargs.operands.push_back(new_object);
       }
 
-      exprt resolved_expr=resolve(
-        to_cpp_name(static_cast<irept &>(symbol.value.op0())),
-        cpp_typecheck_resolvet::wantt::BOTH, fargs);
+      exprt resolved_expr = resolve(
+        to_cpp_name(
+          static_cast<irept &>(to_address_of_expr(symbol.value).object())),
+        cpp_typecheck_resolvet::wantt::BOTH,
+        fargs);
 
       assert(symbol.type.subtype() == resolved_expr.type());
 
@@ -124,7 +121,8 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
           address_of_exprt(
             lookup(resolved_expr.get(ID_component_name)).symbol_expr());
 
-        symbol.value.type().add("to-member") = resolved_expr.op0().type();
+        symbol.value.type().add(ID_to_member) =
+          to_member_expr(resolved_expr).compound().type();
       }
       else
         UNREACHABLE;
@@ -132,9 +130,8 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
       if(symbol.type != symbol.value.type())
       {
         error().source_location=symbol.location;
-        error() << "conversion from `"
-                << to_string(symbol.value.type()) << "' to `"
-                << to_string(symbol.type) << "' " << eom;
+        error() << "conversion from '" << to_string(symbol.value.type())
+                << "' to '" << to_string(symbol.type) << "' " << eom;
         throw 0;
       }
 
@@ -143,8 +140,8 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
 
     typecheck_expr(symbol.value);
 
-    if(symbol.value.type().find("to-member").is_not_nil())
-      symbol.type.add("to-member") = symbol.value.type().find("to-member");
+    if(symbol.value.type().find(ID_to_member).is_not_nil())
+      symbol.type.add(ID_to_member) = symbol.value.type().find(ID_to_member);
 
     if(symbol.value.id()==ID_initializer_list ||
        symbol.value.id()==ID_string_constant)
@@ -156,7 +153,7 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
     }
     else if(has_auto(symbol.type))
     {
-      cpp_convert_auto(symbol.type, symbol.value.type());
+      cpp_convert_auto(symbol.type, symbol.value.type(), get_message_handler());
       typecheck_type(symbol.type);
       implicit_typecast(symbol.value, symbol.type);
     }
@@ -175,15 +172,18 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
     // we need a constructor
 
     symbol_exprt expr_symbol(symbol.name, symbol.type);
-    already_typechecked(expr_symbol);
+    already_typechecked_exprt::make_already_typechecked(expr_symbol);
 
     exprt::operandst ops;
     ops.push_back(symbol.value);
 
-    symbol.value = cpp_constructor(
-      symbol.value.source_location(),
-      expr_symbol,
-      ops);
+    auto constructor =
+      cpp_constructor(symbol.value.source_location(), expr_symbol, ops);
+
+    if(constructor.has_value())
+      symbol.value = constructor.value();
+    else
+      symbol.value = nil_exprt();
   }
 }
 
@@ -197,10 +197,17 @@ void cpp_typecheckt::zero_initializer(
 
   if(final_type.id()==ID_struct)
   {
-    forall_irep(cit, final_type.find(ID_components).get_sub())
-    {
-      const exprt &component=static_cast<const exprt &>(*cit);
+    const auto &struct_type = to_struct_type(final_type);
 
+    if(struct_type.is_incomplete())
+    {
+      error().source_location = source_location;
+      error() << "cannot zero-initialize incomplete struct" << eom;
+      throw 0;
+    }
+
+    for(const auto &component : struct_type.components())
+    {
       if(component.type().id()==ID_code)
         continue;
 
@@ -210,7 +217,7 @@ void cpp_typecheckt::zero_initializer(
       if(component.get_bool(ID_is_static))
         continue;
 
-      member_exprt member(object, component.get(ID_name), component.type());
+      member_exprt member(object, component.get_name(), component.type());
 
       // recursive call
       zero_initializer(member, component.type(), source_location, ops);
@@ -225,10 +232,8 @@ void cpp_typecheckt::zero_initializer(
     if(size_expr.id()==ID_infinity)
       return; // don't initialize
 
-    mp_integer size;
-
-    bool to_int=to_integer(size_expr, size);
-    CHECK_RETURN(!to_int);
+    const mp_integer size =
+      numeric_cast_v<mp_integer>(to_constant_expr(size_expr));
     CHECK_RETURN(size>=0);
 
     exprt::operandst empty_operands;
@@ -241,28 +246,36 @@ void cpp_typecheckt::zero_initializer(
   }
   else if(final_type.id()==ID_union)
   {
+    const auto &union_type = to_union_type(final_type);
+
+    if(union_type.is_incomplete())
+    {
+      error().source_location = source_location;
+      error() << "cannot zero-initialize incomplete union" << eom;
+      throw 0;
+    }
+
     // Select the largest component for zero-initialization
     mp_integer max_comp_size=0;
 
-    exprt comp=nil_exprt();
+    union_typet::componentt comp;
 
-    forall_irep(it, final_type.find(ID_components).get_sub())
+    for(const auto &component : union_type.components())
     {
-      const exprt &component=static_cast<const exprt &>(*it);
-
       assert(component.type().is_not_nil());
 
       if(component.type().id()==ID_code)
         continue;
 
-      exprt component_size=size_of_expr(component.type(), *this);
+      auto component_size_opt = size_of_expr(component.type(), *this);
 
-      mp_integer size_int;
-      if(!to_integer(component_size, size_int))
+      const auto size_int =
+        numeric_cast<mp_integer>(component_size_opt.value_or(nil_exprt()));
+      if(size_int.has_value())
       {
-        if(size_int>max_comp_size)
+        if(*size_int > max_comp_size)
         {
-          max_comp_size=size_int;
+          max_comp_size = *size_int;
           comp=component;
         }
       }
@@ -270,12 +283,7 @@ void cpp_typecheckt::zero_initializer(
 
     if(max_comp_size>0)
     {
-      irept name(ID_name);
-      name.set(ID_identifier, comp.get(ID_base_name));
-      name.set(ID_C_source_location, source_location);
-
-      cpp_namet cpp_name;
-      cpp_name.move_to_sub(name);
+      const cpp_namet cpp_name(comp.get_base_name(), source_location);
 
       exprt member(ID_member);
       member.copy_to_operands(object);
@@ -288,9 +296,9 @@ void cpp_typecheckt::zero_initializer(
     const unsignedbv_typet enum_type(
       to_bitvector_type(final_type.subtype()).get_width());
 
-    exprt zero(from_integer(0, enum_type));
-    zero.make_typecast(type);
-    already_typechecked(zero);
+    exprt zero =
+      typecast_exprt::conditional_cast(from_integer(0, enum_type), type);
+    already_typechecked_exprt::make_already_typechecked(zero);
 
     code_assignt assign;
     assign.lhs()=object;
@@ -299,32 +307,28 @@ void cpp_typecheckt::zero_initializer(
 
     typecheck_expr(assign.lhs());
     assign.lhs().type().set(ID_C_constant, false);
-    already_typechecked(assign.lhs());
+    already_typechecked_exprt::make_already_typechecked(assign.lhs());
 
     typecheck_code(assign);
     ops.push_back(assign);
   }
-  else if(final_type.id()==ID_incomplete_struct ||
-          final_type.id()==ID_incomplete_union)
-  {
-    error().source_location=source_location;
-    error() << "cannot zero-initialize incomplete compound" << eom;
-    throw 0;
-  }
   else
   {
-    exprt value=
-      ::zero_initializer(
-        final_type, source_location, *this, get_message_handler());
+    const auto value = ::zero_initializer(final_type, source_location, *this);
+    if(!value.has_value())
+    {
+      error().source_location = source_location;
+      error() << "cannot zero-initialize '" << to_string(final_type) << "'"
+              << eom;
+      throw 0;
+    }
 
-    code_assignt assign;
-    assign.lhs()=object;
-    assign.rhs()=value;
+    code_assignt assign(object, *value);
     assign.add_source_location()=source_location;
 
-    typecheck_expr(assign.op0());
+    typecheck_expr(assign.lhs());
     assign.lhs().type().set(ID_C_constant, false);
-    already_typechecked(assign.lhs());
+    already_typechecked_exprt::make_already_typechecked(assign.lhs());
 
     typecheck_code(assign);
     ops.push_back(assign);
