@@ -13,13 +13,6 @@ Author:
 #include <util/expr.h>
 #include <util/symbol.h>
 
-exprt cpp_symbol_expr(const symbolt &symbol);
-
-inline void already_typechecked(irept &irep)
-{
-  exprt tmp(ID_already_typechecked);
-  tmp.copy_to_operands(static_cast<exprt &>(irep));
-  irep.swap(tmp);
-}
+symbol_exprt cpp_symbol_expr(const symbolt &symbol);
 
 #endif // CPROVER_CPP_CPP_UTIL_H
