@@ -31,18 +31,16 @@ void template_mapt::apply(typet &type) const
   else if(type.id()==ID_struct ||
           type.id()==ID_union)
   {
-    irept::subt &components=type.add(ID_components).get_sub();
-
-    Forall_irep(it, components)
+    for(auto &c : to_struct_union_type(type).components())
     {
-      typet &subtype=static_cast<typet &>(it->add(ID_type));
+      typet &subtype = c.type();
       apply(subtype);
     }
   }
-  else if(type.id()==ID_symbol)
+  else if(type.id() == ID_template_parameter_symbol_type)
   {
     type_mapt::const_iterator m_it =
-      type_map.find(to_symbol_type(type).get_identifier());
+      type_map.find(to_template_parameter_symbol_type(type).get_identifier());
 
     if(m_it!=type_map.end())
     {
@@ -52,7 +50,7 @@ void template_mapt::apply(typet &type) const
   }
   else if(type.id()==ID_code)
   {
-    apply(static_cast<typet &>(type.add(ID_return_type)));
+    apply(to_code_type(type).return_type());
 
     irept::subt &parameters=type.add(ID_parameters).get_sub();
 
@@ -134,15 +132,11 @@ exprt template_mapt::lookup_expr(const irep_idt &identifier) const
 
 void template_mapt::print(std::ostream &out) const
 {
-  for(type_mapt::const_iterator it=type_map.begin();
-      it!=type_map.end();
-      it++)
-    out << it->first << " = " << it->second.pretty() << '\n';
+  for(const auto &mapping : type_map)
+    out << mapping.first << " = " << mapping.second.pretty() << '\n';
 
-  for(expr_mapt::const_iterator it=expr_map.begin();
-      it!=expr_map.end();
-      it++)
-    out << it->first << " = " << it->second.pretty() << '\n';
+  for(const auto &mapping : expr_map)
+    out << mapping.first << " = " << mapping.second.pretty() << '\n';
 }
 
 void template_mapt::build(
@@ -217,16 +211,8 @@ void template_mapt::set(
 void template_mapt::build_unassigned(
   const template_typet &template_type)
 {
-  const template_typet::template_parameterst &template_parameters=
-    template_type.template_parameters();
-
-  for(template_typet::template_parameterst::const_iterator
-      t_it=template_parameters.begin();
-      t_it!=template_parameters.end();
-      t_it++)
+  for(const auto &t : template_type.template_parameters())
   {
-    const template_parametert &t=*t_it;
-
     if(t.id()==ID_type)
     {
       typet tmp(ID_unassigned);
