@@ -21,19 +21,25 @@ Date: June 2006
 #include <util/config.h>
 #include <util/file_util.h>
 #include <util/get_base_name.h>
+#include <util/prefix.h>
+#include <util/run.h>
 #include <util/suffix.h>
+#include <util/symbol_table_builder.h>
 #include <util/tempdir.h>
+#include <util/tempfile.h>
 #include <util/unicode.h>
 #include <util/version.h>
 
-#include <ansi-c/ansi_c_language.h>
 #include <ansi-c/ansi_c_entry_point.h>
 
 #include <goto-programs/goto_convert.h>
 #include <goto-programs/goto_convert_functions.h>
+#include <goto-programs/name_mangler.h>
 #include <goto-programs/read_goto_binary.h>
+#include <goto-programs/validate_goto_model.h>
 #include <goto-programs/write_goto_binary.h>
 
+#include <langapi/language_file.h>
 #include <langapi/mode.h>
 
 #include <linking/static_lifetime_init.h>
@@ -45,46 +51,25 @@ Date: June 2006
                           "size=\"30,40\";"\
                           "ratio=compress;"
 
-// the following are for chdir
-
-#if defined(__linux__) || \
-    defined(__FreeBSD_kernel__) || \
-    defined(__GNU__) || \
-    defined(__unix__) || \
-    defined(__CYGWIN__) || \
-    defined(__MACH__)
-#include <unistd.h>
-#endif
-
-#ifdef _WIN32
-#include <direct.h>
-#include <windows.h>
-#define chdir _chdir
-#define popen _popen
-#define pclose _pclose
-#endif
-
 /// reads and source and object files, compiles and links them into goto program
 /// objects.
 /// \return true on error, false otherwise
 bool compilet::doit()
 {
-  compiled_functions.clear();
+  goto_model.goto_functions.clear();
 
-  add_compiler_specific_defines(config);
+  add_compiler_specific_defines();
 
   // Parse command line for source and object file names
-  for(std::size_t i=0; i<_cmdline.args.size(); i++)
-    if(add_input_file(_cmdline.args[i]))
+  for(const auto &arg : cmdline.args)
+    if(add_input_file(arg))
       return true;
 
-  for(std::list<std::string>::const_iterator it = libraries.begin();
-      it!=libraries.end();
-      it++)
+  for(const auto &library : libraries)
   {
-    if(!find_library(*it))
+    if(!find_library(library))
       // GCC is going to complain if this doesn't exist
-      debug() << "Library not found: " << *it << " (ignoring)" << eom;
+      debug() << "Library not found: " << library << " (ignoring)" << eom;
   }
 
   statistics() << "No. of source files: " << source_files.size() << eom;
@@ -142,7 +127,9 @@ enum class file_typet
   ELF_OBJECT
 };
 
-static file_typet detect_file_type(const std::string &file_name)
+static file_typet detect_file_type(
+  const std::string &file_name,
+  message_handlert &message_handler)
 {
   // first of all, try to open the file
   std::ifstream in(file_name);
@@ -170,7 +157,7 @@ static file_typet detect_file_type(const std::string &file_name)
   if(ext == "a")
     return file_typet::NORMAL_ARCHIVE;
 
-  if(is_goto_binary(file_name))
+  if(is_goto_binary(file_name, message_handler))
     return file_typet::GOTO_BINARY;
 
   if(hdr[0] == 0x7f && memcmp(hdr + 1, "ELF", 3) == 0)
@@ -183,21 +170,21 @@ static file_typet detect_file_type(const std::string &file_name)
 /// \return false on success, true on error.
 bool compilet::add_input_file(const std::string &file_name)
 {
-  switch(detect_file_type(file_name))
+  switch(detect_file_type(file_name, get_message_handler()))
   {
   case file_typet::FAILED_TO_OPEN_FILE:
-    warning() << "failed to open file `" << file_name
+    warning() << "failed to open file '" << file_name
               << "': " << std::strerror(errno) << eom;
     return warning_is_fatal; // generously ignore unless -Werror
 
   case file_typet::UNKNOWN:
     // unknown extension, not a goto binary, will silently ignore
-    debug() << "unknown file type in `" << file_name << "'" << eom;
+    debug() << "unknown file type in '" << file_name << "'" << eom;
     return false;
 
   case file_typet::ELF_OBJECT:
     // ELF file without goto-cc section, silently ignore
-    debug() << "ELF object without goto-cc section: `" << file_name << "'"
+    debug() << "ELF object without goto-cc section: '" << file_name << "'"
             << eom;
     return false;
 
@@ -225,71 +212,54 @@ bool compilet::add_files_from_archive(
   const std::string &file_name,
   bool thin_archive)
 {
-  std::stringstream cmd;
-  FILE *stream;
-
   std::string tstr = working_directory;
 
   if(!thin_archive)
   {
     tstr = get_temporary_directory("goto-cc.XXXXXX");
 
-    if(tstr=="")
-    {
-      error() << "Cannot create temporary directory" << eom;
-      return true;
-    }
-
     tmp_dirs.push_back(tstr);
-    if(chdir(tmp_dirs.back().c_str())!=0)
-    {
-      error() << "Cannot switch to temporary directory" << eom;
-      return true;
-    }
+    set_current_path(tmp_dirs.back());
 
     // unpack now
-    cmd << "ar x " << concat_dir_file(working_directory, file_name);
-
-    stream=popen(cmd.str().c_str(), "r");
-    pclose(stream);
-
-    cmd.clear();
-    cmd.str("");
+    int ret =
+      run("ar", {"ar", "x", concat_dir_file(working_directory, file_name)});
+    if(ret != 0)
+    {
+      error() << "Failed to extract archive " << file_name << eom;
+      return true;
+    }
   }
 
   // add the files from "ar t"
-  cmd << "ar t " << concat_dir_file(working_directory, file_name);
-
-  stream = popen(cmd.str().c_str(), "r");
-
-  if(stream != nullptr)
+  temporary_filet tmp_file_out("", "");
+  int ret = run(
+    "ar",
+    {"ar", "t", concat_dir_file(working_directory, file_name)},
+    "",
+    tmp_file_out(),
+    "");
+  if(ret != 0)
   {
-    std::string line;
-    int ch; // fgetc returns an int, not char
-    while((ch = fgetc(stream)) != EOF)
-    {
-      if(ch != '\n')
-      {
-        line += static_cast<char>(ch);
-      }
-      else
-      {
-        std::string t = concat_dir_file(tstr, line);
-
-        if(is_goto_binary(t))
-          object_files.push_back(t);
-        else
-          debug() << "Object file is not a goto binary: " << line << eom;
-
-        line = "";
-      }
-    }
-
-    pclose(stream);
+    error() << "Failed to list archive " << file_name << eom;
+    return true;
   }
 
-  if(!thin_archive && chdir(working_directory.c_str()) != 0)
-    error() << "Could not change back to working directory" << eom;
+  std::ifstream in(tmp_file_out());
+  std::string line;
+
+  while(!in.fail() && std::getline(in, line))
+  {
+    std::string t = concat_dir_file(tstr, line);
+
+    if(is_goto_binary(t, get_message_handler()))
+      object_files.push_back(t);
+    else
+      debug() << "Object file is not a goto binary: " << line << eom;
+  }
+
+  if(!thin_archive)
+    set_current_path(working_directory);
 
   return false;
 }
@@ -299,37 +269,35 @@ bool compilet::add_files_from_archive(
 /// \return true if found, false otherwise
 bool compilet::find_library(const std::string &name)
 {
-  std::string tmp;
+  std::string library_file_name;
 
-  for(std::list<std::string>::const_iterator
-      it=library_paths.begin();
-      it!=library_paths.end();
-      it++)
+  for(const auto &library_path : library_paths)
   {
-    #ifdef _WIN32
-    tmp = *it + "\\lib";
-    #else
-    tmp = *it + "/lib";
-    #endif
+    library_file_name = concat_dir_file(library_path, "lib" + name + ".a");
 
-    std::ifstream in(tmp+name+".a");
+    std::ifstream in(library_file_name);
 
     if(in.is_open())
-      return !add_input_file(tmp+name+".a");
+      return !add_input_file(library_file_name);
     else
     {
-      std::string libname=tmp+name+".so";
+      library_file_name = concat_dir_file(library_path, "lib" + name + ".so");
 
-      switch(detect_file_type(libname))
+      switch(detect_file_type(library_file_name, get_message_handler()))
       {
       case file_typet::GOTO_BINARY:
-        return !add_input_file(libname);
+        return !add_input_file(library_file_name);
 
       case file_typet::ELF_OBJECT:
-        warning() << "Warning: Cannot read ELF library " << libname << eom;
+        warning() << "Warning: Cannot read ELF library " << library_file_name
+                  << eom;
         return warning_is_fatal;
 
-      default:
+      case file_typet::THIN_ARCHIVE:
+      case file_typet::NORMAL_ARCHIVE:
+      case file_typet::SOURCE_FILE:
+      case file_typet::FAILED_TO_OPEN_FILE:
+      case file_typet::UNKNOWN:
         break;
       }
     }
@@ -344,13 +312,12 @@ bool compilet::link()
 {
   // "compile" hitherto uncompiled functions
   statistics() << "Compiling functions" << eom;
-  convert_symbols(compiled_functions);
+  convert_symbols(goto_model.goto_functions);
 
   // parse object files
   for(const auto &file_name : object_files)
   {
-    if(read_object_and_link(file_name, symbol_table,
-                            compiled_functions, get_message_handler()))
+    if(read_object_and_link(file_name, goto_model, get_message_handler()))
       return true;
   }
 
@@ -361,23 +328,28 @@ bool compilet::link()
     // new symbols may have been added to a previously linked file
     // make sure a new entry point is created that contains all
     // static initializers
-    compiled_functions.function_map.erase(INITIALIZE_FUNCTION);
+    goto_model.goto_functions.function_map.erase(INITIALIZE_FUNCTION);
 
-    symbol_table.remove(goto_functionst::entry_point());
-    compiled_functions.function_map.erase(goto_functionst::entry_point());
+    goto_model.symbol_table.remove(goto_functionst::entry_point());
+    goto_model.goto_functions.function_map.erase(
+      goto_functionst::entry_point());
 
-    if(ansi_c_entry_point(symbol_table, get_message_handler()))
+    const bool error = ansi_c_entry_point(
+      goto_model.symbol_table,
+      get_message_handler(),
+      c_object_factory_parameterst());
+
+    if(error)
       return true;
 
     // entry_point may (should) add some more functions.
-    convert_symbols(compiled_functions);
+    convert_symbols(goto_model.goto_functions);
   }
 
-  if(write_object_file(
-      output_file_executable, symbol_table, compiled_functions))
+  if(write_bin_object_file(output_file_executable, goto_model))
     return true;
 
-  return add_written_cprover_symbols(symbol_table);
+  return add_written_cprover_symbols(goto_model.symbol_table);
 }
 
 /// parses source files and writes object files, or keeps the symbols in the
@@ -417,11 +389,11 @@ bool compilet::compile()
       // output an object file for every source file
 
       // "compile" functions
-      convert_symbols(compiled_functions);
+      convert_symbols(goto_model.goto_functions);
 
       std::string cfn;
 
-      if(output_file_object=="")
+      if(output_file_object.empty())
       {
         const std::string file_name_with_obj_ext =
           get_base_name(file_name, true) + "." + object_file_extension;
@@ -434,14 +406,20 @@ bool compilet::compile()
       else
         cfn = output_file_object;
 
-      if(write_object_file(cfn, symbol_table, compiled_functions))
+      if(keep_file_local)
+      {
+        function_name_manglert<file_name_manglert> mangler(
+          get_message_handler(), goto_model, file_local_mangle_suffix);
+        mangler.mangle();
+      }
+
+      if(write_bin_object_file(cfn, goto_model))
         return true;
 
-      if(add_written_cprover_symbols(symbol_table))
+      if(add_written_cprover_symbols(goto_model.symbol_table))
         return true;
 
-      symbol_table.clear(); // clean symbol table for next source file.
-      compiled_functions.clear();
+      goto_model.clear(); // clean symbol table for next source file.
     }
   }
 
@@ -450,46 +428,48 @@ bool compilet::compile()
 
 /// parses a source file (low-level parsing)
 /// \return true on error, false otherwise
-bool compilet::parse(const std::string &file_name)
+bool compilet::parse(
+  const std::string &file_name,
+  language_filest &language_files)
 {
-  if(file_name=="-")
-    return parse_stdin();
-
-  #ifdef _MSC_VER
-  std::ifstream infile(widen(file_name));
-  #else
-  std::ifstream infile(file_name);
-  #endif
-
-  if(!infile)
-  {
-    error() << "failed to open input file `" << file_name << "'" << eom;
-    return true;
-  }
-
   std::unique_ptr<languaget> languagep;
 
   // Using '-x', the type of a file can be overridden;
   // otherwise, it's guessed from the extension.
 
-  if(override_language!="")
+  if(!override_language.empty())
   {
     if(override_language=="c++" || override_language=="c++-header")
       languagep = get_language_from_mode(ID_cpp);
     else
       languagep = get_language_from_mode(ID_C);
   }
-  else
+  else if(file_name != "-")
     languagep=get_language_from_filename(file_name);
 
   if(languagep==nullptr)
   {
-    error() << "failed to figure out type of file `" << file_name << "'" << eom;
+    error() << "failed to figure out type of file '" << file_name << "'" << eom;
     return true;
   }
 
   languagep->set_message_handler(get_message_handler());
 
+  if(file_name == "-")
+    return parse_stdin(*languagep);
+
+#ifdef _MSC_VER
+  std::ifstream infile(widen(file_name));
+#else
+  std::ifstream infile(file_name);
+#endif
+
+  if(!infile)
+  {
+    error() << "failed to open input file '" << file_name << "'" << eom;
+    return true;
+  }
+
   language_filet &lf=language_files.add_file(file_name);
   lf.language=std::move(languagep);
 
@@ -507,8 +487,8 @@ bool compilet::parse(const std::string &file_name)
 
       if(!ofs.is_open())
       {
-        error() << "failed to open output file `"
-                << cmdline.get_value('o') << "'" << eom;
+        error() << "failed to open output file '" << cmdline.get_value('o')
+                << "'" << eom;
         return true;
       }
     }
@@ -521,8 +501,7 @@ bool compilet::parse(const std::string &file_name)
 
     if(lf.language->parse(infile, file_name))
     {
-      if(get_ui()==ui_message_handlert::uit::PLAIN)
-        error() << "PARSING ERROR" << eom;
+      error() << "PARSING ERROR" << eom;
       return true;
     }
   }
@@ -532,13 +511,10 @@ bool compilet::parse(const std::string &file_name)
 }
 
 /// parses a source file (low-level parsing)
+/// \param language: source language processor
 /// \return true on error, false otherwise
-bool compilet::parse_stdin()
+bool compilet::parse_stdin(languaget &language)
 {
-  ansi_c_languaget language;
-
-  language.set_message_handler(get_message_handler());
-
   statistics() << "Parsing: (stdin)" << eom;
 
   if(mode==PREPROCESS_ONLY)
@@ -553,8 +529,8 @@ bool compilet::parse_stdin()
 
       if(!ofs.is_open())
       {
-        error() << "failed to open output file `"
-                << cmdline.get_value('o') << "'" << eom;
+        error() << "failed to open output file '" << cmdline.get_value('o')
+                << "'" << eom;
         return true;
       }
     }
@@ -565,8 +541,7 @@ bool compilet::parse_stdin()
   {
     if(language.parse(std::cin, ""))
     {
-      if(get_ui()==ui_message_handlert::uit::PLAIN)
-        error() << "PARSING ERROR" << eom;
+      error() << "PARSING ERROR" << eom;
       return true;
     }
   }
@@ -574,49 +549,43 @@ bool compilet::parse_stdin()
   return false;
 }
 
-/// writes the goto functions in the function table to a binary format object
+/// Writes the goto functions of \p src_goto_model to a binary format object
 /// file.
-/// \par parameters: file_name, functions table
-/// \return true on error, false otherwise
-bool compilet::write_object_file(
-  const std::string &file_name,
-  const symbol_tablet &lsymbol_table,
-  goto_functionst &functions)
-{
-  return write_bin_object_file(file_name, lsymbol_table, functions);
-}
-
-/// writes the goto functions in the function table to a binary format object
-/// file.
-/// \par parameters: file_name, functions table
+/// \param file_name: Target file to serialize \p src_goto_model to
+/// \param src_goto_model: goto model to serialize
 /// \return true on error, false otherwise
 bool compilet::write_bin_object_file(
   const std::string &file_name,
-  const symbol_tablet &lsymbol_table,
-  goto_functionst &functions)
+  const goto_modelt &src_goto_model)
 {
-  statistics() << "Writing binary format object `"
-               << file_name << "'" << eom;
+  if(validate_goto_model)
+  {
+    status() << "Validating goto model" << eom;
+    src_goto_model.validate();
+  }
+
+  statistics() << "Writing binary format object '" << file_name << "'" << eom;
 
   // symbols
   statistics() << "Symbols in table: "
-               << lsymbol_table.symbols.size() << eom;
+               << src_goto_model.symbol_table.symbols.size() << eom;
 
   std::ofstream outfile(file_name, std::ios::binary);
 
   if(!outfile.is_open())
   {
-    error() << "Error opening file `" << file_name << "'" << eom;
+    error() << "Error opening file '" << file_name << "'" << eom;
     return true;
   }
 
-  if(write_goto_binary(outfile, lsymbol_table, functions))
+  if(write_goto_binary(outfile, src_goto_model))
     return true;
 
-  unsigned cnt=function_body_count(functions);
+  const auto cnt = function_body_count(src_goto_model.goto_functions);
 
-  statistics() << "Functions: " << functions.function_map.size()
-               << "; " << cnt << " have a body." << eom;
+  statistics() << "Functions: "
+               << src_goto_model.goto_functions.function_map.size() << "; "
+               << cnt << " have a body." << eom;
 
   outfile.close();
   wrote_object=true;
@@ -628,29 +597,38 @@ bool compilet::write_bin_object_file(
 /// \return true on error, false otherwise
 bool compilet::parse_source(const std::string &file_name)
 {
-  if(parse(file_name))
+  language_filest language_files;
+  language_files.set_message_handler(get_message_handler());
+
+  if(parse(file_name, language_files))
     return true;
 
-  if(typecheck()) // we just want to typecheck this one file here
+  // we just typecheck one file here
+  if(language_files.typecheck(goto_model.symbol_table, keep_file_local))
+  {
+    error() << "CONVERSION ERROR" << eom;
     return true;
+  }
 
-  if((has_suffix(file_name, ".class") ||
-      has_suffix(file_name, ".jar")) &&
-     final())
+  if(language_files.final(goto_model.symbol_table))
+  {
+    error() << "CONVERSION ERROR" << eom;
     return true;
+  }
 
-  // so we remove it from the list afterwards
-  language_files.remove_file(file_name);
   return false;
 }
 
 /// constructor
 /// \return nothing
-compilet::compilet(cmdlinet &_cmdline, ui_message_handlert &mh, bool Werror):
-  language_uit(_cmdline, mh),
-  ns(symbol_table),
-  cmdline(_cmdline),
-  warning_is_fatal(Werror)
+compilet::compilet(cmdlinet &_cmdline, message_handlert &mh, bool Werror)
+  : messaget(mh),
+    ns(goto_model.symbol_table),
+    cmdline(_cmdline),
+    warning_is_fatal(Werror),
+    keep_file_local(cmdline.isset("export-function-local-symbols")),
+    file_local_mangle_suffix(
+      cmdline.isset("mangle-suffix") ? cmdline.get_value("mangle-suffix") : "")
 {
   mode=COMPILE_LINK_EXECUTABLE;
   echo_file_name=false;
@@ -664,27 +642,23 @@ compilet::~compilet()
 {
   // clean up temp dirs
 
-  for(std::list<std::string>::const_iterator it = tmp_dirs.begin();
-      it!=tmp_dirs.end();
-      it++)
-    delete_directory(*it);
+  for(const auto &dir : tmp_dirs)
+    delete_directory(dir);
 }
 
-unsigned compilet::function_body_count(const goto_functionst &functions) const
+std::size_t
+compilet::function_body_count(const goto_functionst &functions) const
 {
-  int fbs=0;
+  std::size_t count = 0;
 
-  for(goto_functionst::function_mapt::const_iterator it=
-      functions.function_map.begin();
-      it != functions.function_map.end();
-      it++)
-    if(it->second.body_available())
-      fbs++;
+  for(const auto &f : functions.function_map)
+    if(f.second.body_available())
+      count++;
 
-  return fbs;
+  return count;
 }
 
-void compilet::add_compiler_specific_defines(configt &config) const
+void compilet::add_compiler_specific_defines() const
 {
   config.ansi_c.defines.push_back(
     std::string("__GOTO_CC_VERSION__=") + CBMC_VERSION);
@@ -692,41 +666,40 @@ void compilet::add_compiler_specific_defines(configt &config) const
 
 void compilet::convert_symbols(goto_functionst &dest)
 {
-  goto_convert_functionst converter(symbol_table, get_message_handler());
+  symbol_table_buildert symbol_table_builder =
+    symbol_table_buildert::wrap(goto_model.symbol_table);
+
+  goto_convert_functionst converter(
+    symbol_table_builder, get_message_handler());
 
   // the compilation may add symbols!
 
   symbol_tablet::symbolst::size_type before=0;
 
-  while(before!=symbol_table.symbols.size())
+  while(before != symbol_table_builder.symbols.size())
   {
-    before=symbol_table.symbols.size();
+    before = symbol_table_builder.symbols.size();
 
     typedef std::set<irep_idt> symbols_sett;
     symbols_sett symbols;
 
-    for(const auto &named_symbol : symbol_table.symbols)
+    for(const auto &named_symbol : symbol_table_builder.symbols)
       symbols.insert(named_symbol.first);
 
     // the symbol table iterators aren't stable
-    for(symbols_sett::const_iterator
-        it=symbols.begin();
-        it!=symbols.end();
-        ++it)
+    for(const auto &symbol : symbols)
     {
-      symbol_tablet::symbolst::const_iterator s_it=
-        symbol_table.symbols.find(*it);
-      assert(s_it!=symbol_table.symbols.end());
+      symbol_tablet::symbolst::const_iterator s_it =
+        symbol_table_builder.symbols.find(symbol);
+      CHECK_RETURN(s_it != symbol_table_builder.symbols.end());
 
-      if(s_it->second.type.id()==ID_code &&
-         !s_it->second.is_macro &&
-         !s_it->second.is_type &&
-          s_it->second.value.id()!="compiled" &&
-          s_it->second.value.is_not_nil())
+      if(
+        s_it->second.is_function() && !s_it->second.is_compiled() &&
+        s_it->second.value.is_not_nil())
       {
         debug() << "Compiling " << s_it->first << eom;
         converter.convert_function(s_it->first, dest.function_map[s_it->first]);
-        symbol_table.get_writeable_ref(*it).value=exprt("compiled");
+        symbol_table_builder.get_writeable_ref(symbol).set_compiled();
       }
     }
   }
