@@ -17,13 +17,14 @@ Date: May 2006
 #include <util/unicode.h>
 #include <util/get_base_name.h>
 
-#include "gcc_cmdline.h"
 #include "armcc_cmdline.h"
-#include "ms_cl_cmdline.h"
-#include "ld_cmdline.h"
-#include "bcc_cmdline.h"
-#include "as_cmdline.h"
 #include "as86_cmdline.h"
+#include "as_cmdline.h"
+#include "bcc_cmdline.h"
+#include "gcc_cmdline.h"
+#include "ld_cmdline.h"
+#include "ms_cl_cmdline.h"
+#include "ms_link_cmdline.h"
 
 #include "armcc_mode.h"
 #include "as_mode.h"
@@ -31,6 +32,7 @@ Date: May 2006
 #include "gcc_mode.h"
 #include "ld_mode.h"
 #include "ms_cl_mode.h"
+#include "ms_link_mode.h"
 
 std::string to_lower_string(const std::string &s)
 {
@@ -65,15 +67,21 @@ int main(int argc, const char **argv)
   std::string base_name=get_base_name(argv[0], false);
   #endif
 
-  if(base_name=="goto-link" || base_name=="link" ||
-     base_name=="goto-cl" || base_name=="cl")
+  if(base_name == "goto-cl" || base_name == "cl")
   {
-    // this is the Visual Studio personality
+    // this is the Visual Studio CL personality
     ms_cl_cmdlinet cmdline;
     cmdline.parse_env();
     ms_cl_modet ms_cl_mode(cmdline, base_name);
     return ms_cl_mode.main(argc, argv);
   }
+  else if(base_name == "goto-link" || base_name == "link")
+  {
+    // this is the Visual Studio LINK personality
+    ms_link_cmdlinet cmdline;
+    ms_link_modet ms_link_mode(cmdline);
+    return ms_link_mode.main(argc, argv);
+  }
   else if(base_name=="goto-cw" ||
           base_name=="goto-cw-link")
   {
