@@ -22,6 +22,7 @@ Author: CM Wintersteiger, 2006
 #include <sysexits.h>
 #endif
 
+#include <util/exception_utils.h>
 #include <util/parse_options.h>
 #include <util/version.h>
 
@@ -47,10 +48,10 @@ void goto_cc_modet::help()
 {
   // clang-format off
   std::cout << '\n' << banner_string("goto-cc", CBMC_VERSION) << '\n'
+            << align_center_with_border("Copyright (C) 2006-2018") << '\n'
+            << align_center_with_border("Daniel Kroening, Michael Tautschnig,") << '\n' // NOLINT(*)
+            << align_center_with_border("Christoph Wintersteiger") << '\n'
             <<
-  "* *               Copyright (C) 2006-2018                   * *\n"
-  "* *          Daniel Kroening, Michael Tautschnig,           * *\n"
-  "* *               Christoph Wintersteiger                   * *\n"
   "\n";
 
   help_mode();
@@ -106,6 +107,11 @@ int goto_cc_modet::main(int argc, const char **argv)
     error() << "Out of memory" << eom;
     return EX_SOFTWARE;
   }
+  catch(const cprover_exception_baset &e)
+  {
+    error() << e.what() << eom;
+    return EX_SOFTWARE;
+  }
 }
 
 /// prints a message informing the user about incorrect options
