@@ -14,8 +14,7 @@ Date: June 2006
 #ifndef CPROVER_GOTO_CC_MS_CL_MODE_H
 #define CPROVER_GOTO_CC_MS_CL_MODE_H
 
-#include <util/cout_message.h>
-
+#include "cl_message_handler.h"
 #include "goto_cc_mode.h"
 #include "ms_cl_cmdline.h"
 
@@ -35,7 +34,7 @@ public:
 
 protected:
   ms_cl_cmdlinet &cmdline;
-  console_message_handlert message_handler;
+  cl_message_handlert message_handler;
 };
 
 #endif // CPROVER_GOTO_CC_MS_CL_MODE_H
