@@ -9,14 +9,14 @@ Author: Daniel Kroening, kroening@kroening.com
 /// \file
 /// Language Registration
 
-#include "goto_diff_languages.h"
+#include "goto_diff_parse_options.h"
 
 #include <langapi/mode.h>
 
 #include <ansi-c/ansi_c_language.h>
 #include <cpp/cpp_language.h>
 
-void goto_diff_languagest::register_languages()
+void goto_diff_parse_optionst::register_languages()
 {
   register_language(new_ansi_c_language);
   register_language(new_cpp_language);
