@@ -110,10 +110,10 @@ void acceleration_utilst::find_modified(
   }
 }
 
-
 bool acceleration_utilst::check_inductive(
   std::map<exprt, polynomialt> polynomials,
-  patht &path)
+  patht &path,
+  guard_managert &guard_manager)
 {
   // Checking that our polynomial is inductive with respect to the loop body is
   // equivalent to checking safety of the following program:
@@ -126,7 +126,7 @@ bool acceleration_utilst::check_inductive(
   // assert (target1==polynomial1);
   // assert (target2==polynomial2);
   // ...
-  scratch_programt program(symbol_table, message_handler);
+  scratch_programt program(symbol_table, message_handler, guard_manager);
   std::vector<exprt> polynomials_hold;
   substitutiont substitution;
 
@@ -137,36 +137,32 @@ bool acceleration_utilst::check_inductive(
       ++it)
   {
     const equal_exprt holds(it->first, it->second.to_expr());
-    program.add_instruction(ASSUME)->guard=holds;
+    program.add(goto_programt::make_assumption(holds));
 
     polynomials_hold.push_back(holds);
   }
 
   program.append_path(path);
 
-  codet inc_loop_counter=
-    code_assignt(
-      loop_counter,
-      plus_exprt(loop_counter, from_integer(1, loop_counter.type())));
-  program.add_instruction(ASSIGN)->code=inc_loop_counter;
+  auto inc_loop_counter = code_assignt(
+    loop_counter,
+    plus_exprt(loop_counter, from_integer(1, loop_counter.type())));
+
+  program.add(goto_programt::make_assignment(inc_loop_counter));
 
   ensure_no_overflows(program);
 
-  for(std::vector<exprt>::iterator it=polynomials_hold.begin();
-      it!=polynomials_hold.end();
-      ++it)
-  {
-    program.add_instruction(ASSERT)->guard=*it;
-  }
+  for(const auto &p : polynomials_hold)
+    program.add(goto_programt::make_assertion(p));
 
 #ifdef DEBUG
   std::cout << "Checking following program for inductiveness:\n";
-  program.output(ns, "", std::cout);
+  program.output(ns, irep_idt(), std::cout);
 #endif
 
   try
   {
-    if(program.check_sat())
+    if(program.check_sat(guard_manager))
     {
       // We found a counterexample to inductiveness... :-(
   #ifdef DEBUG
@@ -215,23 +211,15 @@ void acceleration_utilst::stash_variables(
   expr_sett modified,
   substitutiont &substitution)
 {
-  find_symbols_sett vars;
-
-  for(expr_sett::iterator it=modified.begin();
-      it!=modified.end();
-      ++it)
-  {
-    find_symbols(*it, vars);
-  }
-
-  irep_idt loop_counter_name=to_symbol_expr(loop_counter).get_identifier();
+  find_symbols_sett vars =
+    find_symbols_or_nexts(modified.begin(), modified.end());
+  const irep_idt &loop_counter_name =
+    to_symbol_expr(loop_counter).get_identifier();
   vars.erase(loop_counter_name);
 
-  for(find_symbols_sett::iterator it=vars.begin();
-      it!=vars.end();
-      ++it)
+  for(const irep_idt &symbol : vars)
   {
-    symbolt orig=*symbol_table.lookup(*it);
+    const symbolt &orig = symbol_table.lookup_ref(symbol);
     symbolt stashed_sym=fresh_symbol("polynomial::stash", orig.type);
     substitution[orig.symbol_expr()]=stashed_sym.symbol_expr();
     program.assign(stashed_sym.symbol_expr(), orig.symbol_expr());
@@ -276,7 +264,7 @@ exprt acceleration_utilst::precondition(patht &path)
     }
     else if(t->is_assume() || t->is_assert())
     {
-      ret=implies_exprt(t->guard, ret);
+      ret = implies_exprt(t->get_condition(), ret);
     }
     else
     {
@@ -335,8 +323,7 @@ void acceleration_utilst::push_nondet(exprt &expr)
     push_nondet(*it);
   }
 
-  if(expr.id()==ID_not &&
-     expr.op0().id()==ID_nondet)
+  if(expr.id() == ID_not && to_not_expr(expr).op().id() == ID_nondet)
   {
     expr = side_effect_expr_nondett(expr.type(), expr.source_location());
   }
@@ -346,8 +333,8 @@ void acceleration_utilst::push_nondet(exprt &expr)
           expr.id()==ID_le ||
           expr.id()==ID_ge)
   {
-    if(expr.op0().id()==ID_nondet ||
-       expr.op1().id()==ID_nondet)
+    const auto &rel_expr = to_binary_relation_expr(expr);
+    if(rel_expr.lhs().id() == ID_nondet || rel_expr.rhs().id() == ID_nondet)
     {
       expr = side_effect_expr_nondett(expr.type(), expr.source_location());
     }
@@ -357,7 +344,8 @@ void acceleration_utilst::push_nondet(exprt &expr)
 bool acceleration_utilst::do_assumptions(
   std::map<exprt, polynomialt> polynomials,
   patht &path,
-  exprt &guard)
+  exprt &guard,
+  guard_managert &guard_manager)
 {
   // We want to check that if an assumption fails, the next iteration can't be
   // feasible again.  To do this we check the following program for safety:
@@ -384,7 +372,7 @@ bool acceleration_utilst::do_assumptions(
   // assert(!precondition);
 
   exprt condition=precondition(path);
-  scratch_programt program(symbol_table, message_handler);
+  scratch_programt program(symbol_table, message_handler, guard_manager);
 
   substitutiont substitution;
   stash_polynomials(program, polynomials, substitution, path);
@@ -437,19 +425,19 @@ bool acceleration_utilst::do_assumptions(
 
   ensure_no_overflows(program);
 
-  program.add_instruction(ASSERT)->guard=condition;
+  program.add(goto_programt::make_assertion(condition));
 
   guard=not_exprt(condition);
   simplify(guard, ns);
 
 #ifdef DEBUG
   std::cout << "Checking following program for monotonicity:\n";
-  program.output(ns, "", std::cout);
+  program.output(ns, irep_idt(), std::cout);
 #endif
 
   try
   {
-    if(program.check_sat())
+    if(program.check_sat(guard_manager))
     {
   #ifdef DEBUG
       std::cout << "Path is not monotone\n";
@@ -497,7 +485,7 @@ void acceleration_utilst::ensure_no_overflows(scratch_programt &program)
 #endif
 
   instrumenter.add_overflow_checks();
-  program.add_instruction(ASSUME)->guard=not_exprt(overflow_var);
+  program.add(goto_programt::make_assumption(not_exprt(overflow_var)));
 
   // goto_functionst::goto_functiont fn;
   // fn.body.instructions.swap(program.instructions);
@@ -556,7 +544,6 @@ acceleration_utilst::expr_pairst acceleration_utilst::gather_array_assignments(
 bool acceleration_utilst::do_arrays(
   goto_programt::instructionst &loop_body,
   std::map<exprt, polynomialt> &polynomials,
-  exprt &loop_counter,
   substitutiont &substitution,
   scratch_programt &program)
 {
@@ -881,7 +868,6 @@ bool acceleration_utilst::expr2poly(
 bool acceleration_utilst::do_nonrecursive(
   goto_programt::instructionst &body,
   std::map<exprt, polynomialt> &polynomials,
-  exprt &loop_counter,
   substitutiont &substitution,
   expr_sett &nonrecursive,
   scratch_programt &program)
@@ -892,7 +878,7 @@ bool acceleration_utilst::do_nonrecursive(
   // for these variables by just forward simulating the path and
   // taking the expressions we get at the end.
   replace_mapt state;
-  expr_sett array_writes;
+  std::unordered_set<index_exprt, irep_hash> array_writes;
   expr_sett arrays_written;
   expr_sett arrays_read;
 
@@ -929,8 +915,8 @@ bool acceleration_utilst::do_nonrecursive(
   {
     if(it->is_assign())
     {
-      exprt lhs=it->code.op0();
-      exprt rhs=it->code.op1();
+      exprt lhs = it->get_assign().lhs();
+      exprt rhs = it->get_assign().rhs();
 
       if(lhs.id()==ID_dereference)
       {
@@ -943,21 +929,22 @@ bool acceleration_utilst::do_nonrecursive(
 
       if(lhs.id()==ID_index)
       {
-        replace_expr(state, lhs.op1());
-        array_writes.insert(lhs);
+        auto &lhs_index_expr = to_index_expr(lhs);
+        replace_expr(state, lhs_index_expr.index());
+        array_writes.insert(lhs_index_expr);
 
-        if(arrays_written.find(lhs.op0())!=arrays_written.end())
+        if(arrays_written.find(lhs_index_expr.array()) != arrays_written.end())
         {
           // We've written to this array before -- be conservative and bail
           // out now.
 #ifdef DEBUG
-          std::cout << "Bailing out on array written to twice in loop: " <<
-            expr2c(lhs.op0(), ns) << '\n';
+          std::cout << "Bailing out on array written to twice in loop: "
+                    << expr2c(lhs_index_expr.array(), ns) << '\n';
 #endif
           return false;
         }
 
-        arrays_written.insert(lhs.op0());
+        arrays_written.insert(lhs_index_expr.array());
       }
 
       replace_expr(state, rhs);
@@ -997,14 +984,12 @@ bool acceleration_utilst::do_nonrecursive(
     }
   }
 
-  for(expr_sett::iterator it=array_writes.begin();
-      it!=array_writes.end();
-      ++it)
+  for(const auto &write : array_writes)
   {
-    const exprt &lhs=*it;
-    const exprt &rhs=state[*it];
+    const auto &lhs = write;
+    const auto &rhs = state[write];
 
-    if(!assign_array(lhs, rhs, loop_counter, program))
+    if(!assign_array(lhs, rhs, program))
     {
 #ifdef DEBUG
       std::cout << "Failed to assign a nonrecursive array: " <<
@@ -1018,9 +1003,8 @@ bool acceleration_utilst::do_nonrecursive(
 }
 
 bool acceleration_utilst::assign_array(
-  const exprt &lhs,
+  const index_exprt &lhs,
   const exprt &rhs,
-  const exprt &loop_counter,
   scratch_programt &program)
 {
 #ifdef DEBUG
@@ -1087,7 +1071,7 @@ bool acceleration_utilst::assign_array(
   {
     if(idx.id()==ID_pointer_offset)
     {
-      poly.from_expr(idx.op0());
+      poly.from_expr(to_unary_expr(idx).op());
     }
     else
     {
@@ -1127,13 +1111,13 @@ bool acceleration_utilst::assign_array(
   {
     replace_expr(
       loop_counter, from_integer(0, loop_counter.type()), lower_bound);
-    simplify_expr(lower_bound, ns);
+    lower_bound = simplify_expr(std::move(lower_bound), ns);
   }
   else
   {
     replace_expr(
       loop_counter, from_integer(0, loop_counter.type()), upper_bound);
-    simplify_expr(upper_bound, ns);
+    upper_bound = simplify_expr(std::move(upper_bound), ns);
   }
 
   if(stride==0)
@@ -1210,10 +1194,13 @@ void acceleration_utilst::gather_array_accesses(
   const exprt &e,
   expr_sett &arrays)
 {
-  if(e.id()==ID_index ||
-     e.id()==ID_dereference)
+  if(e.id() == ID_index)
   {
-    arrays.insert(e.op0());
+    arrays.insert(to_index_expr(e).array());
+  }
+  else if(e.id() == ID_dereference)
+  {
+    arrays.insert(to_dereference_expr(e).pointer());
   }
 
   forall_operands(it, e)
@@ -1238,19 +1225,15 @@ void acceleration_utilst::extract_polynomial(
     std::map<exprt, int> degrees;
 
     mp_integer mp=binary2integer(concrete_term.get_value().c_str(), true);
-    monomial.coeff=mp.to_long();
+    monomial.coeff = numeric_cast_v<int>(mp);
 
     if(monomial.coeff==0)
     {
       continue;
     }
 
-    for(expr_listt::iterator it=terms.begin();
-        it!=terms.end();
-        ++it)
+    for(const auto &term : terms)
     {
-      exprt term=*it;
-
       if(degrees.find(term)!=degrees.end())
       {
         degrees[term]++;
