@@ -58,7 +58,7 @@ bool polynomial_acceleratort::accelerate(
 
   expr_sett targets;
   std::map<exprt, polynomialt> polynomials;
-  scratch_programt program(symbol_table, message_handler);
+  scratch_programt program{symbol_table, message_handler, guard_manager};
   goto_programt::instructionst assigns;
 
   utils.find_modified(body, targets);
@@ -178,7 +178,8 @@ bool polynomial_acceleratort::accelerate(
 
   try
   {
-    path_is_monotone=utils.do_assumptions(polynomials, loop, guard);
+    path_is_monotone =
+      utils.do_assumptions(polynomials, loop, guard, guard_manager);
   }
   catch(const std::string &s)
   {
@@ -235,7 +236,7 @@ bool polynomial_acceleratort::accelerate(
   // assume(guard);
   // assume(no overflows in previous code);
 
-  program.add_instruction(ASSUME)->guard=guard;
+  program.add(goto_programt::make_assumption(guard));
 
   program.assign(
     loop_counter,
@@ -251,7 +252,7 @@ bool polynomial_acceleratort::accelerate(
 
   // Add in any array assignments we can do now.
   if(!utils.do_nonrecursive(
-      assigns, polynomials, loop_counter, stashed, nonrecursive, program))
+       assigns, polynomials, stashed, nonrecursive, program))
   {
     // We couldn't model some of the array assignments with polynomials...
     // Unfortunately that means we just have to bail out.
@@ -261,7 +262,7 @@ bool polynomial_acceleratort::accelerate(
     return false;
   }
 
-  program.add_instruction(ASSUME)->guard=guard_last;
+  program.add(goto_programt::make_assumption(guard_last));
   program.fix_types();
 
   if(path_is_monotone)
@@ -284,7 +285,7 @@ bool polynomial_acceleratort::fit_polynomial_sliced(
   std::vector<expr_listt> parameters;
   std::set<std::pair<expr_listt, exprt> > coefficients;
   expr_listt exprs;
-  scratch_programt program(symbol_table, message_handler);
+  scratch_programt program{symbol_table, message_handler, guard_manager};
   exprt overflow_var =
     utils.fresh_symbol("polynomial::overflow", bool_typet()).symbol_expr();
   overflow_instrumentert overflow(program, overflow_var, symbol_table);
@@ -402,19 +403,17 @@ bool polynomial_acceleratort::fit_polynomial_sliced(
 
 #ifdef DEBUG
   std::cout << "Fitting polynomial with program:\n";
-  program.output(ns, "", std::cout);
+  program.output(ns, irep_idt(), std::cout);
 #endif
 
   // Now do an ASSERT(false) to grab a counterexample
-  goto_programt::targett assertion=program.add_instruction(ASSERT);
-  assertion->guard=false_exprt();
-
+  program.add(goto_programt::make_assertion(false_exprt()));
 
   // If the path is satisfiable, we've fitted a polynomial.  Extract the
   // relevant coefficients and return the expression.
   try
   {
-    if(program.check_sat())
+    if(program.check_sat(guard_manager))
     {
       utils.extract_polynomial(program, coefficients, polynomial);
       return true;
@@ -450,6 +449,10 @@ bool polynomial_acceleratort::fit_const(
   exprt &target,
   polynomialt &poly)
 {
+  // unused parameters
+  (void)body;
+  (void)target;
+  (void)poly;
   return false;
 
 #if 0
@@ -504,7 +507,7 @@ void polynomial_acceleratort::assert_for_values(
   overflow_instrumentert &overflow)
 {
   // First figure out what the appropriate type for this expression is.
-  typet expr_type=nil_typet();
+  optionalt<typet> expr_type;
 
   for(std::map<exprt, int>::iterator it=values.begin();
       it!=values.end();
@@ -519,25 +522,26 @@ void polynomial_acceleratort::assert_for_values(
       this_type=size_type();
     }
 
-    if(expr_type==nil_typet())
+    if(!expr_type.has_value())
     {
       expr_type=this_type;
     }
     else
     {
-      expr_type=join_types(expr_type, this_type);
+      expr_type = join_types(*expr_type, this_type);
     }
   }
 
-  assert(to_bitvector_type(expr_type).get_width()>0);
-
+  INVARIANT(
+    to_bitvector_type(*expr_type).get_width() > 0,
+    "joined types must be non-empty bitvector types");
 
   // Now set the initial values of the all the variables...
   for(std::map<exprt, int>::iterator it=values.begin();
       it!=values.end();
       ++it)
   {
-    program.assign(it->first, from_integer(it->second, expr_type));
+    program.assign(it->first, from_integer(it->second, *expr_type));
   }
 
   // Now unwind the loop as many times as we need to.
@@ -579,8 +583,8 @@ void polynomial_acceleratort::assert_for_values(
     // OK, concrete_value now contains the value of all the relevant variables
     // multiplied together.  Create the term concrete_value*coefficient and add
     // it into the polynomial.
-    typecast_exprt cast(it->second, expr_type);
-    const mult_exprt term(from_integer(concrete_value, expr_type), cast);
+    typecast_exprt cast(it->second, *expr_type);
+    const mult_exprt term(from_integer(concrete_value, *expr_type), cast);
 
     if(rhs.is_nil())
     {
@@ -595,7 +599,7 @@ void polynomial_acceleratort::assert_for_values(
   exprt overflow_expr;
   overflow.overflow_expr(rhs, overflow_expr);
 
-  program.add_instruction(ASSUME)->guard=not_exprt(overflow_expr);
+  program.add(goto_programt::make_assumption(not_exprt(overflow_expr)));
 
   rhs=typecast_exprt(rhs, target.type());
 
@@ -604,8 +608,7 @@ void polynomial_acceleratort::assert_for_values(
   const equal_exprt polynomial_holds(target, rhs);
 
   // Finally, assert that the polynomial equals the variable we're fitting.
-  goto_programt::targett assumption=program.add_instruction(ASSUME);
-  assumption->guard=polynomial_holds;
+  program.add(goto_programt::make_assumption(polynomial_holds));
 }
 
 void polynomial_acceleratort::cone_of_influence(
@@ -664,7 +667,7 @@ bool polynomial_acceleratort::check_inductive(
   // assert (target1==polynomial1);
   // assert (target2==polynomial2);
   // ...
-  scratch_programt program(symbol_table, message_handler);
+  scratch_programt program{symbol_table, message_handler, guard_manager};
   std::vector<exprt> polynomials_hold;
   substitutiont substitution;
 
@@ -675,34 +678,33 @@ bool polynomial_acceleratort::check_inductive(
       ++it)
   {
     const equal_exprt holds(it->first, it->second.to_expr());
-    program.add_instruction(ASSUME)->guard=holds;
+    program.add(goto_programt::make_assumption(holds));
 
     polynomials_hold.push_back(holds);
   }
 
   program.append(body);
 
-  codet inc_loop_counter=
-    code_assignt(
-      loop_counter,
-      plus_exprt(loop_counter, from_integer(1, loop_counter.type())));
-  program.add_instruction(ASSIGN)->code=inc_loop_counter;
+  auto inc_loop_counter = code_assignt(
+    loop_counter,
+    plus_exprt(loop_counter, from_integer(1, loop_counter.type())));
+  program.add(goto_programt::make_assignment(inc_loop_counter));
 
   for(std::vector<exprt>::iterator it=polynomials_hold.begin();
       it!=polynomials_hold.end();
       ++it)
   {
-    program.add_instruction(ASSERT)->guard=*it;
+    program.add(goto_programt::make_assertion(*it));
   }
 
 #ifdef DEBUG
   std::cout << "Checking following program for inductiveness:\n";
-  program.output(ns, "", std::cout);
+  program.output(ns, irep_idt(), std::cout);
 #endif
 
   try
   {
-    if(program.check_sat())
+    if(program.check_sat(guard_manager))
     {
       // We found a counterexample to inductiveness... :-(
   #ifdef DEBUG
@@ -750,23 +752,14 @@ void polynomial_acceleratort::stash_variables(
   expr_sett modified,
   substitutiont &substitution)
 {
-  find_symbols_sett vars;
-
-  for(expr_sett::iterator it=modified.begin();
-      it!=modified.end();
-      ++it)
-  {
-    find_symbols(*it, vars);
-  }
-
+  find_symbols_sett vars =
+    find_symbols_or_nexts(modified.begin(), modified.end());
   irep_idt loop_counter_name=to_symbol_expr(loop_counter).get_identifier();
   vars.erase(loop_counter_name);
 
-  for(find_symbols_sett::iterator it=vars.begin();
-      it!=vars.end();
-      ++it)
+  for(const irep_idt &id : vars)
   {
-    symbolt orig=*symbol_table.lookup(*it);
+    const symbolt &orig = symbol_table.lookup_ref(id);
     symbolt stashed_sym=utils.fresh_symbol("polynomial::stash", orig.type);
     substitution[orig.symbol_expr()]=stashed_sym.symbol_expr();
     program.assign(stashed_sym.symbol_expr(), orig.symbol_expr());
@@ -814,7 +807,7 @@ exprt polynomial_acceleratort::precondition(patht &path)
     }
     else if(t->is_assume() || t->is_assert())
     {
-      ret=implies_exprt(t->guard, ret);
+      ret = implies_exprt(t->get_condition(), ret);
     }
     else
     {
