@@ -12,7 +12,8 @@ Author: Matt Lewis
 #include "scratch_program.h"
 
 #include <util/fixedbv.h>
-#include <util/decision_procedure.h>
+
+#include <solvers/decision_procedure.h>
 
 #include <goto-symex/slice.h>
 
@@ -22,11 +23,11 @@ Author: Matt Lewis
 #include <iostream>
 #endif
 
-bool scratch_programt::check_sat(bool do_slice)
+bool scratch_programt::check_sat(bool do_slice, guard_managert &guard_manager)
 {
   fix_types();
 
-  add_instruction(END_FUNCTION);
+  add(goto_programt::make_end_function());
 
   remove_skip(*this);
 
@@ -35,10 +36,20 @@ bool scratch_programt::check_sat(bool do_slice)
   output(ns, "scratch", std::cout);
 #endif
 
-  symex.constant_propagation=constant_propagation;
-  goto_symex_statet::propagationt::valuest constants;
+  symex_state = util_make_unique<goto_symex_statet>(
+    symex_targett::sourcet(goto_functionst::entry_point(), *this),
+    DEFAULT_MAX_FIELD_SENSITIVITY_ARRAY_SIZE,
+    guard_manager,
+    [this](const irep_idt &id) {
+      return path_storage.get_unique_l2_index(id);
+    });
 
-  symex.symex_with_state(symex_state, functions, symex_symbol_table);
+  symex.symex_with_state(
+    *symex_state,
+    [this](const irep_idt &key) -> const goto_functionst::goto_functiont & {
+      return functions.function_map.at(key);
+    },
+    symex_symbol_table);
 
   if(do_slice)
   {
@@ -60,16 +71,12 @@ bool scratch_programt::check_sat(bool do_slice)
   std::cout << "Finished symex, invoking decision procedure.\n";
 #endif
 
-  return (checker->dec_solve()==decision_proceduret::resultt::D_SATISFIABLE);
+  return ((*checker)() == decision_proceduret::resultt::D_SATISFIABLE);
 }
 
 exprt scratch_programt::eval(const exprt &e)
 {
-  exprt ssa=e;
-
-  symex_state.rename(ssa, ns);
-
-  return checker->get(ssa);
+  return checker->get(symex_state->rename<L2>(e, ns).get());
 }
 
 void scratch_programt::append(goto_programt::instructionst &new_instructions)
@@ -84,19 +91,12 @@ goto_programt::targett scratch_programt::assign(
   const exprt &lhs,
   const exprt &rhs)
 {
-  code_assignt assignment(lhs, rhs);
-  targett instruction=add_instruction(ASSIGN);
-  instruction->code=assignment;
-
-  return instruction;
+  return add(goto_programt::make_assignment(lhs, rhs));
 }
 
 goto_programt::targett scratch_programt::assume(const exprt &guard)
 {
-  targett instruction=add_instruction(ASSUME);
-  instruction->guard=guard;
-
-  return instruction;
+  return add(goto_programt::make_assumption(guard));
 }
 
 static void fix_types(exprt &expr)
@@ -113,13 +113,14 @@ static void fix_types(exprt &expr)
      expr.id()==ID_ge ||
      expr.id()==ID_le)
   {
-    exprt &lhs=expr.op0();
-    exprt &rhs=expr.op1();
+    auto &rel_expr = to_binary_relation_expr(expr);
+    exprt &lhs = rel_expr.lhs();
+    exprt &rhs = rel_expr.rhs();
 
     if(lhs.type()!=rhs.type())
     {
       typecast_exprt typecast(rhs, lhs.type());
-      expr.op1().swap(typecast);
+      rel_expr.rhs().swap(typecast);
     }
   }
 }
@@ -142,7 +143,9 @@ void scratch_programt::fix_types()
     }
     else if(it->is_assume() || it->is_assert())
     {
-      ::fix_types(it->guard);
+      exprt cond = it->get_condition();
+      ::fix_types(cond);
+      it->set_condition(cond);
     }
   }
 }
@@ -161,12 +164,12 @@ void scratch_programt::append_path(patht &path)
     {
       if(it->guard.id()!=ID_nil)
       {
-        add_instruction(ASSUME)->guard=it->guard;
+        add(goto_programt::make_assumption(it->guard));
       }
     }
     else if(it->loc->is_assert())
     {
-      add_instruction(ASSUME)->guard=it->loc->guard;
+      add(goto_programt::make_assumption(it->loc->get_condition()));
     }
   }
 }
@@ -186,9 +189,10 @@ void scratch_programt::append_loop(
   append(program);
 
   // Update any back jumps to the loop header.
+  (void)loop_header; // unused parameter
   assume(false_exprt());
 
-  goto_programt::targett end=add_instruction(SKIP);
+  goto_programt::targett end = add(goto_programt::make_skip());
 
   update();
 
@@ -203,3 +207,10 @@ void scratch_programt::append_loop(
     }
   }
 }
+
+optionst scratch_programt::get_default_options()
+{
+  optionst ret;
+  ret.set_option("simplify", true);
+  return ret;
+}
