@@ -36,23 +36,23 @@ Author: Matt Lewis
 class scratch_programt:public goto_programt
 {
 public:
-  scratch_programt(symbol_tablet &_symbol_table, message_handlert &mh)
+  scratch_programt(
+    symbol_tablet &_symbol_table,
+    message_handlert &mh,
+    guard_managert &guard_manager)
     : constant_propagation(true),
       symbol_table(_symbol_table),
       symex_symbol_table(),
       ns(symbol_table, symex_symbol_table),
-      equation(),
+      equation(mh),
       path_storage(),
-      options(),
-      symex(mh, symbol_table, equation, options, path_storage),
-      satcheck(util_make_unique<satcheckt>()),
-      satchecker(ns, *satcheck),
+      options(get_default_options()),
+      symex(mh, symbol_table, equation, options, path_storage, guard_manager),
+      satcheck(util_make_unique<satcheckt>(mh)),
+      satchecker(ns, *satcheck, mh),
       z3(ns, "accelerate", "", "", smt2_dect::solvert::Z3),
       checker(&z3) // checker(&satchecker)
   {
-    // Unconditionally set for performance reasons. This option setting applies
-    // only to this program.
-    options.set_option("simplify", true);
   }
 
   void append(goto_programt::instructionst &instructions);
@@ -63,11 +63,11 @@ public:
   targett assign(const exprt &lhs, const exprt &rhs);
   targett assume(const exprt &guard);
 
-  bool check_sat(bool do_slice);
+  bool check_sat(bool do_slice, guard_managert &guard_manager);
 
-  bool check_sat()
+  bool check_sat(guard_managert &guard_manager)
   {
-    return check_sat(true);
+    return check_sat(true, guard_manager);
   }
 
   exprt eval(const exprt &e);
@@ -77,7 +77,7 @@ public:
   bool constant_propagation;
 
 protected:
-  goto_symex_statet symex_state;
+  std::unique_ptr<goto_symex_statet> symex_state;
   goto_functionst functions;
   symbol_tablet &symbol_table;
   symbol_tablet symex_symbol_table;
@@ -90,7 +90,8 @@ protected:
   std::unique_ptr<propt> satcheck;
   bv_pointerst satchecker;
   smt2_dect z3;
-  prop_convt *checker;
+  decision_proceduret *checker;
+  static optionst get_default_options();
 };
 
 #endif // CPROVER_GOTO_INSTRUMENT_ACCELERATE_SCRATCH_PROGRAM_H
