@@ -6,15 +6,18 @@ Author:
 
 \*******************************************************************/
 
-#ifndef CPROVER_ALIGNMENT_CHECKS_H
-#define CPROVER_ALIGNMENT_CHECKS_H
+/// \file
+/// Alignment Checks
+
+#ifndef CPROVER_GOTO_INSTRUMENT_ALIGNMENT_CHECKS_H
+#define CPROVER_GOTO_INSTRUMENT_ALIGNMENT_CHECKS_H
 
 #include <iosfwd>
 
-#include <goto-programs/goto_functions.h>
+#include <util/symbol_table.h>
 
 void print_struct_alignment_problems(
   const symbol_tablet &symbol_table,
   std::ostream &out);
 
-#endif
+#endif // CPROVER_GOTO_INSTRUMENT_ALIGNMENT_CHECKS_H
