@@ -17,12 +17,13 @@ Author: Daniel Kroening
 #include "cover_basic_blocks.h"
 
 void cover_condition_instrumentert::instrument(
+  const irep_idt &function_id,
   goto_programt &goto_program,
   goto_programt::targett &i_it,
   const cover_blocks_baset &) const
 {
   if(is_non_cover_assertion(i_it))
-    i_it->make_skip();
+    i_it->turn_into_skip();
 
   // Conditions are all atomic predicates in the programs.
   if(!i_it->source_location.is_built_in())
@@ -33,20 +34,17 @@ void cover_condition_instrumentert::instrument(
 
     for(const auto &c : conditions)
     {
-      const std::string c_string = from_expr(ns, i_it->function, c);
+      const std::string c_string = from_expr(ns, function_id, c);
 
-      const std::string comment_t = "condition `" + c_string + "' true";
-      const irep_idt function = i_it->function;
+      const std::string comment_t = "condition '" + c_string + "' true";
       goto_program.insert_before_swap(i_it);
-      i_it->make_assertion(c);
-      i_it->source_location = source_location;
-      initialize_source_location(i_it, comment_t, function);
+      *i_it = goto_programt::make_assertion(c, source_location);
+      initialize_source_location(i_it, comment_t, function_id);
 
-      const std::string comment_f = "condition `" + c_string + "' false";
+      const std::string comment_f = "condition '" + c_string + "' false";
       goto_program.insert_before_swap(i_it);
-      i_it->make_assertion(not_exprt(c));
-      i_it->source_location = source_location;
-      initialize_source_location(i_it, comment_f, function);
+      *i_it = goto_programt::make_assertion(not_exprt(c), source_location);
+      initialize_source_location(i_it, comment_f, function_id);
     }
 
     for(std::size_t i = 0; i < conditions.size() * 2; i++)
