@@ -22,6 +22,15 @@ void dump_c(
   const namespacet &ns,
   std::ostream &out);
 
+void dump_c_type_header(
+  const goto_functionst &src,
+  const bool use_system_headers,
+  const bool use_all_headers,
+  const bool include_harness,
+  const namespacet &ns,
+  const std::string module,
+  std::ostream &out);
+
 void dump_cpp(
   const goto_functionst &src,
   const bool use_system_headers,
