@@ -8,18 +8,19 @@ Date: September 2011
 
 \*******************************************************************/
 
+/// \file
+/// Interrupt Instrumentation for Goto Programs
+
 #ifndef CPROVER_GOTO_INSTRUMENT_INTERRUPT_H
 #define CPROVER_GOTO_INSTRUMENT_INTERRUPT_H
 
-class symbol_tablet;
-class goto_functionst;
+class goto_modelt;
 
 #include "rw_set.h"
- 
+
 void interrupt(
-  value_setst &value_sets,
-  const class symbol_tablet &symbol_table,
-  goto_functionst &goto_functions,
+  value_setst &,
+  goto_modelt &,
   const irep_idt &interrupt_handler);
 
-#endif
+#endif // CPROVER_GOTO_INSTRUMENT_INTERRUPT_H
