@@ -24,21 +24,25 @@ Author: Daniel Kroening, kroening@kroening.com
 class k_inductiont
 {
 public:
-  typedef goto_functionst::goto_functiont goto_functiont;
-
   k_inductiont(
+    const irep_idt &_function_id,
     goto_functiont &_goto_function,
-    bool _base_case, bool _step_case,
-    unsigned _k):
-    goto_function(_goto_function),
-    local_may_alias(_goto_function),
-    natural_loops(_goto_function.body),
-    base_case(_base_case), step_case(_step_case), k(_k)
+    bool _base_case,
+    bool _step_case,
+    unsigned _k)
+    : function_id(_function_id),
+      goto_function(_goto_function),
+      local_may_alias(_goto_function),
+      natural_loops(_goto_function.body),
+      base_case(_base_case),
+      step_case(_step_case),
+      k(_k)
   {
     k_induction();
   }
 
 protected:
+  const irep_idt &function_id;
   goto_functiont &goto_function;
   local_may_aliast local_may_alias;
   natural_loops_mutablet natural_loops;
@@ -70,12 +74,17 @@ void k_inductiont::process_loop(
   {
     // now unwind k times
     goto_unwindt goto_unwind;
-    goto_unwind.unwind(goto_function.body, loop_head, loop_exit, k,
-                       goto_unwindt::unwind_strategyt::PARTIAL);
+    goto_unwind.unwind(
+      function_id,
+      goto_function.body,
+      loop_head,
+      loop_exit,
+      k,
+      goto_unwindt::unwind_strategyt::PARTIAL);
 
     // assume the loop condition has become false
-    goto_programt::instructiont assume(ASSUME);
-    assume.guard=loop_guard;
+    goto_programt::instructiont assume =
+      goto_programt::make_assumption(loop_guard);
     goto_function.body.insert_before_swap(loop_exit, assume);
   }
 
@@ -96,10 +105,11 @@ void k_inductiont::process_loop(
 
     goto_unwindt goto_unwind;
     goto_unwind.unwind(
+      function_id,
       goto_function.body,
       loop_head,
       loop_exit,
-      k+1,
+      k + 1,
       goto_unwindt::unwind_strategyt::PARTIAL,
       iteration_points);
 
@@ -108,7 +118,7 @@ void k_inductiont::process_loop(
     {
       if(t->is_assert())
         break;
-      t->make_skip();
+      t->turn_into_skip();
     }
 
     // now turn any assertions in iterations 0..k-1 into assumptions
@@ -125,8 +135,8 @@ void k_inductiont::process_loop(
     }
 
     // assume the loop condition has become false
-    goto_programt::instructiont assume(ASSUME);
-    assume.guard=loop_guard;
+    goto_programt::instructiont assume =
+      goto_programt::make_assumption(loop_guard);
     goto_function.body.insert_before_swap(loop_exit, assume);
 
     // Now havoc at the loop head. Use insert_swap to
@@ -155,5 +165,5 @@ void k_induction(
   unsigned k)
 {
   Forall_goto_functions(it, goto_model.goto_functions)
-    k_inductiont(it->second, base_case, step_case, k);
+    k_inductiont(it->first, it->second, base_case, step_case, k);
 }
