@@ -17,13 +17,19 @@ Author: Daniel Kroening, kroening@kroening.com
 class local_may_aliast;
 
 typedef std::set<exprt> modifiest;
-typedef const natural_loops_mutablet::natural_loopt loopt;
+typedef natural_loops_mutablet::natural_loopt loopt;
 
 void get_modifies(
   const local_may_aliast &local_may_alias,
   const loopt &loop,
   modifiest &modifies);
 
+void get_modifies_lhs(
+  const local_may_aliast &local_may_alias,
+  goto_programt::const_targett t,
+  const exprt &lhs,
+  modifiest &modifies);
+
 void build_havoc_code(
   const goto_programt::targett loop_head,
   const modifiest &modifies,
