@@ -18,7 +18,6 @@ Date: February 2006
 #include <vector>
 #include <set>
 
-#include <util/guard.h>
 #include <util/std_expr.h>
 
 #include <goto-programs/goto_model.h>
@@ -48,7 +47,11 @@ public:
     irep_idt object;
     exprt guard;
 
-    entryt():guard(true_exprt())
+    entryt(
+      const symbol_exprt &_symbol_expr,
+      const irep_idt &_object,
+      const exprt &_guard)
+      : symbol_expr(_symbol_expr), object(_object), guard(_guard)
     {
     }
   };
@@ -87,7 +90,10 @@ public:
   void output(std::ostream &out) const;
 
 protected:
-  virtual void track_deref(const entryt &, bool read) {}
+  virtual void track_deref(const entryt &, bool read)
+  {
+    (void)read; // unused parameter
+  }
   virtual void set_track_deref() {}
   virtual void reset_track_deref() {}
 
@@ -118,26 +124,31 @@ public:
   _rw_set_loct(
     const namespacet &_ns,
     value_setst &_value_sets,
+    const irep_idt &_function_id,
     goto_programt::const_targett _target,
-    local_may_aliast &may):
-    rw_set_baset(_ns),
-    value_sets(_value_sets),
-    target(_target),
-    local_may(may)
+    local_may_aliast &may)
+    : rw_set_baset(_ns),
+      value_sets(_value_sets),
+      function_id(_function_id),
+      target(_target),
+      local_may(may)
 #else
   _rw_set_loct(
     const namespacet &_ns,
     value_setst &_value_sets,
-    goto_programt::const_targett _target):
-    rw_set_baset(_ns),
-    value_sets(_value_sets),
-    target(_target)
+    const irep_idt &_function_id,
+    goto_programt::const_targett _target)
+    : rw_set_baset(_ns),
+      value_sets(_value_sets),
+      function_id(_function_id),
+      target(_target)
 #endif
   {
   }
 
 protected:
   value_setst &value_sets;
+  const irep_idt function_id;
   const goto_programt::const_targett target;
 
 #ifdef LOCAL_MAY
@@ -146,17 +157,17 @@ protected:
 
   void read(const exprt &expr)
   {
-    read_write_rec(expr, true, false, "", guardt());
+    read_write_rec(expr, true, false, "", exprt::operandst());
   }
 
-  void read(const exprt &expr, const guardt &guard)
+  void read(const exprt &expr, const exprt::operandst &guard_conjuncts)
   {
-    read_write_rec(expr, true, false, "", guard);
+    read_write_rec(expr, true, false, "", guard_conjuncts);
   }
 
   void write(const exprt &expr)
   {
-    read_write_rec(expr, false, true, "", guardt());
+    read_write_rec(expr, false, true, "", exprt::operandst());
   }
 
   void compute();
@@ -165,9 +176,10 @@ protected:
 
   void read_write_rec(
     const exprt &expr,
-    bool r, bool w,
+    bool r,
+    bool w,
     const std::string &suffix,
-    const guardt &guard);
+    const exprt::operandst &guard_conjuncts);
 };
 
 class rw_set_loct:public _rw_set_loct
@@ -177,15 +189,17 @@ public:
   rw_set_loct(
     const namespacet &_ns,
     value_setst &_value_sets,
+    const irep_idt &_function_id,
     goto_programt::const_targett _target,
-    local_may_aliast &may):
-    _rw_set_loct(_ns, _value_sets, _target, may)
+    local_may_aliast &may)
+    : _rw_set_loct(_ns, _value_sets, _function_id, _target, may)
 #else
   rw_set_loct(
     const namespacet &_ns,
     value_setst &_value_sets,
-    goto_programt::const_targett _target):
-    _rw_set_loct(_ns, _value_sets, _target)
+    const irep_idt &_function_id,
+    goto_programt::const_targett _target)
+    : _rw_set_loct(_ns, _value_sets, _function_id, _target)
 #endif
   {
     compute();
@@ -236,17 +250,19 @@ public:
   rw_set_with_trackt(
     const namespacet &_ns,
     value_setst &_value_sets,
+    const irep_idt &_function_id,
     goto_programt::const_targett _target,
-    local_may_aliast &may):
-    _rw_set_loct(_ns, _value_sets, _target, may),
-    dereferencing(false)
+    local_may_aliast &may)
+    : _rw_set_loct(_ns, _value_sets, _function_id, _target, may),
+      dereferencing(false)
 #else
   rw_set_with_trackt(
     const namespacet &_ns,
     value_setst &_value_sets,
-    goto_programt::const_targett _target):
-    _rw_set_loct(_ns, _value_sets, _target),
-    dereferencing(false)
+    const irep_idt &_function_id,
+    goto_programt::const_targett _target)
+    : _rw_set_loct(_ns, _value_sets, _function_id, _target),
+      dereferencing(false)
 #endif
   {
     compute();
