@@ -22,7 +22,7 @@ Date: November 2011
 symbol_exprt add_stack_depth_symbol(symbol_tablet &symbol_table)
 {
   const irep_idt identifier="$stack_depth";
-  signedbv_typet type(sizeof(int)*8);
+  unsignedbv_typet type(sizeof(std::size_t)*8);
 
   symbolt new_symbol;
   new_symbol.name=identifier;
@@ -43,7 +43,7 @@ symbol_exprt add_stack_depth_symbol(symbol_tablet &symbol_table)
 void stack_depth(
   goto_programt &goto_program,
   const symbol_exprt &symbol,
-  const int i_depth,
+  const std::size_t i_depth,
   const exprt &max_depth)
 {
   assert(!goto_program.instructions.empty());
@@ -51,38 +51,32 @@ void stack_depth(
   goto_programt::targett first=goto_program.instructions.begin();
 
   binary_relation_exprt guard(symbol, ID_le, max_depth);
-  goto_programt::targett assert_ins=goto_program.insert_before(first);
-  assert_ins->make_assertion(guard);
-  assert_ins->source_location=first->source_location;
-  assert_ins->function=first->function;
+  goto_programt::targett assert_ins = goto_program.insert_before(
+    first, goto_programt::make_assertion(guard, first->source_location));
 
   assert_ins->source_location.set_comment(
     "Stack depth exceeds "+std::to_string(i_depth));
   assert_ins->source_location.set_property_class("stack-depth");
 
-  goto_programt::targett plus_ins=goto_program.insert_before(first);
-  plus_ins->make_assignment();
-  plus_ins->code=code_assignt(symbol,
-      plus_exprt(symbol, from_integer(1, symbol.type())));
-  plus_ins->source_location=first->source_location;
-  plus_ins->function=first->function;
+  goto_program.insert_before(
+    first,
+    goto_programt::make_assignment(
+      code_assignt(symbol, plus_exprt(symbol, from_integer(1, symbol.type()))),
+      first->source_location));
 
   goto_programt::targett last=--goto_program.instructions.end();
   assert(last->is_end_function());
 
-  goto_programt::instructiont minus_ins;
-  minus_ins.make_assignment();
-  minus_ins.code=code_assignt(symbol,
-      minus_exprt(symbol, from_integer(1, symbol.type())));
-  minus_ins.source_location=last->source_location;
-  minus_ins.function=last->function;
+  goto_programt::instructiont minus_ins = goto_programt::make_assignment(
+    code_assignt(symbol, minus_exprt(symbol, from_integer(1, symbol.type()))),
+    last->source_location);
 
   goto_program.insert_before_swap(last, minus_ins);
 }
 
 void stack_depth(
   goto_modelt &goto_model,
-  const int depth)
+  const std::size_t depth)
 {
   const symbol_exprt sym=
     add_stack_depth_symbol(goto_model.symbol_table);
@@ -104,11 +98,11 @@ void stack_depth(
 
   goto_programt &init=i_it->second.body;
   goto_programt::targett first=init.instructions.begin();
-  goto_programt::targett it=init.insert_before(first);
-  it->make_assignment();
-  it->code=code_assignt(sym, from_integer(0, sym.type()));
+  init.insert_before(
+    first,
+    goto_programt::make_assignment(
+      code_assignt(sym, from_integer(0, sym.type()))));
   // no suitable value for source location -- omitted
-  it->function = INITIALIZE_FUNCTION;
 
   // update counters etc.
   goto_model.goto_functions.update();
