@@ -28,7 +28,9 @@ public:
   {
   }
 
-  void add_assertions(goto_programt &goto_program);
+  void add_assertions(
+    const irep_idt &function_identifer,
+    goto_programt &goto_program);
 
 protected:
   symbol_tablet &symbol_table;
@@ -63,9 +65,11 @@ void uninitializedt::get_tracking(goto_programt::const_targett i_it)
   }
 }
 
-void uninitializedt::add_assertions(goto_programt &goto_program)
+void uninitializedt::add_assertions(
+  const irep_idt &function_identifier,
+  goto_programt &goto_program)
 {
-  uninitialized_analysis(goto_program, ns);
+  uninitialized_analysis(function_identifier, goto_program, ns);
 
   // find out which variables need tracking
 
@@ -117,9 +121,7 @@ void uninitializedt::add_assertions(goto_programt &goto_program)
         const irep_idt new_identifier=
           id2string(identifier)+"#initialized";
 
-        symbol_exprt symbol_expr;
-        symbol_expr.set_identifier(new_identifier);
-        symbol_expr.type()=bool_typet();
+        symbol_exprt symbol_expr(new_identifier, bool_typet());
         i1->type=DECL;
         i1->source_location=instruction.source_location;
         i1->code=code_declt(symbol_expr);
@@ -154,10 +156,10 @@ void uninitializedt::add_assertions(goto_programt &goto_program)
             const irep_idt new_identifier=id2string(identifier)+"#initialized";
 
             // insert assertion
-            goto_programt::instructiont assertion;
-            assertion.type=ASSERT;
-            assertion.guard=symbol_exprt(new_identifier, bool_typet());
-            assertion.source_location=instruction.source_location;
+            goto_programt::instructiont assertion =
+              goto_programt::make_assertion(
+                symbol_exprt(new_identifier, bool_typet()),
+                instruction.source_location);
             assertion.source_location.set_comment(
               "use of uninitialized local variable");
             assertion.source_location.set_property_class("uninitialized local");
@@ -200,7 +202,7 @@ void add_uninitialized_locals_assertions(goto_modelt &goto_model)
   {
     uninitializedt uninitialized(goto_model.symbol_table);
 
-    uninitialized.add_assertions(f_it->second.body);
+    uninitialized.add_assertions(f_it->first, f_it->second.body);
   }
 }
 
@@ -218,8 +220,8 @@ void show_uninitialized(
       out << "//// Function: " << f_it->first << '\n';
       out << "////\n\n";
       uninitialized_analysist uninitialized_analysis;
-      uninitialized_analysis(f_it->second.body, ns);
-      uninitialized_analysis.output(ns, f_it->second.body, out);
+      uninitialized_analysis(f_it->first, f_it->second.body, ns);
+      uninitialized_analysis.output(ns, f_it->first, f_it->second.body, out);
     }
   }
 }
