@@ -8,20 +8,20 @@ Date: January 2010
 
 \*******************************************************************/
 
-#ifndef CPROVER_UNINITALIZED_H
-#define CPROVER_UNINITALIZED_H
+/// \file
+/// Detection for Uninitialized Local Variables
+
+#ifndef CPROVER_GOTO_INSTRUMENT_UNINITIALIZED_H
+#define CPROVER_GOTO_INSTRUMENT_UNINITIALIZED_H
 
 #include <iosfwd>
 
-#include <goto-programs/goto_functions.h>
+#include <goto-programs/goto_model.h>
 
-void add_uninitialized_locals_assertions(
-  class symbol_tablet &symbol_table,
-  goto_functionst &goto_functions);
+void add_uninitialized_locals_assertions(goto_modelt &);
 
 void show_uninitialized(
-  const class symbol_tablet &symbol_table,
-  const goto_functionst &goto_functions,
+  const goto_modelt &,
   std::ostream &out);
 
-#endif
+#endif // CPROVER_GOTO_INSTRUMENT_UNINITIALIZED_H
