@@ -89,8 +89,17 @@ void event_grapht::graph_explorert::collect_cycles(
   {
     event_idt source=*st_it;
     egraph.message.debug() << "explore " << egraph[source].id << messaget::eom;
-    backtrack(set_of_cycles, source, source,
-      false, max_po_trans, false, false, false, "", model);
+    backtrack(
+      set_of_cycles,
+      source,
+      source,
+      false,
+      max_po_trans,
+      false,
+      false,
+      false,
+      irep_idt(),
+      model);
 
     while(!marked_stack.empty())
     {
@@ -147,7 +156,6 @@ event_grapht::critical_cyclet event_grapht::graph_explorert::extract_cycle(
 }
 
 /// see event_grapht::collect_cycles
-/// \param get_po_only: used for po-transitivity
 bool event_grapht::graph_explorert::backtrack(
   std::set<critical_cyclet> &set_of_cycles,
   event_idt source,
@@ -212,7 +220,7 @@ bool event_grapht::graph_explorert::backtrack(
      cycle cannot be with x */
   irep_idt avoid_at_the_end=var_to_avoid;
   bool no_comm=false;
-  if(avoid_at_the_end!="" && avoid_at_the_end==this_vertex.variable)
+  if(!avoid_at_the_end.empty() && avoid_at_the_end == this_vertex.variable)
     no_comm=true;
 
   /* if specified, maximum number of events reached */
@@ -453,18 +461,17 @@ bool event_grapht::graph_explorert::backtrack(
           f=true;
         }
         else if(!mark[w])
-          f|=
-            backtrack(
-              set_of_cycles,
-              source,
-              w,
-              unsafe_met_updated,
-              po_trans,
-              false,
-              false,
-              false,
-              "",
-              model);
+          f |= backtrack(
+            set_of_cycles,
+            source,
+            w,
+            unsafe_met_updated,
+            po_trans,
+            false,
+            false,
+            false,
+            irep_idt(),
+            model);
       }
     }
 
