@@ -9,6 +9,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "shared_buffers.h"
 
 #include <util/c_types.h>
+#include <util/fresh_symbol.h>
 
 #include <linking/static_lifetime_init.h>
 
@@ -88,21 +89,22 @@ irep_idt shared_bufferst::add(
   const typet &type,
   bool instrument)
 {
-  const irep_idt identifier=id2string(object)+suffix;
+  const namespacet ns(symbol_table);
 
-  symbolt new_symbol;
-  new_symbol.name=identifier;
-  new_symbol.base_name=id2string(base_name)+suffix;
-  new_symbol.type=type;
+  symbolt &new_symbol = get_fresh_aux_symbol(
+    type,
+    id2string(object) + suffix,
+    id2string(base_name) + suffix,
+    ns.lookup(object).location,
+    ns.lookup(object).mode,
+    symbol_table);
   new_symbol.is_static_lifetime=true;
   new_symbol.value.make_nil();
 
   if(instrument)
-    instrumentations.insert(identifier);
+    instrumentations.insert(new_symbol.name);
 
-  symbolt *symbol_ptr;
-  symbol_table.move(new_symbol, symbol_ptr);
-  return identifier;
+  return new_symbol.name;
 }
 
 void shared_bufferst::add_initialization(goto_programt &goto_program)
@@ -243,9 +245,8 @@ void shared_bufferst::flush_read(
   const source_locationt &source_location,
   const irep_idt &write_object)
 {
-/* option 1 */
-
 #if 0
+  // option 1
   const varst &vars=(*this)(write_object);
 
   const symbol_exprt fresh_var_expr=symbol_exprt(vars.read_new_var, vars.type);
@@ -266,10 +267,15 @@ void shared_bufferst::flush_read(
 
   assignment(goto_program, target, source_location, vars.read_delayed,
              false_exprt());
+#else
+  // option 2: do nothing
+  // unused parameters
+  (void)goto_program;
+  (void)target;
+  (void)target;
+  (void)source_location;
+  (void)write_object;
 #endif
-
-/* option 2 */
-/* do nothing */
 }
 
 /// instruments write
@@ -313,12 +319,8 @@ void shared_bufferst::write(
   const exprt cond_expr=
     not_exprt(and_exprt(buff1_used_expr, buff0_used_expr));
 
-  target=goto_program.insert_before(target);
-  target->guard=cond_expr;
-  target->type=ASSERT;
-  target->code=code_assertt();
-  target->code.add_source_location()=source_location;
-  target->source_location=source_location;
+  target = goto_program.insert_before(
+    target, goto_programt::make_assertion(cond_expr, source_location));
   target++;
 
   // We update writers ownership of the values in the buffer
@@ -443,6 +445,7 @@ void shared_bufferst::det_flush(
 
 /// instruments read
 void shared_bufferst::nondet_flush(
+  const irep_idt &function_id,
   goto_programt &goto_program,
   goto_programt::targett &target,
   const source_locationt &source_location,
@@ -459,8 +462,8 @@ void shared_bufferst::nondet_flush(
   const varst &vars=(*this)(object);
 
   // Non deterministic choice
-  irep_idt choice0=choice(target->function, "0");
-  irep_idt choice2=choice(target->function, "2"); // delays the write flush
+  irep_idt choice0 = choice(function_id, "0");
+  irep_idt choice2 = choice(function_id, "2"); // delays the write flush
 
   const symbol_exprt choice0_expr=symbol_exprt(choice0, bool_typet());
   const symbol_exprt delay_expr=symbol_exprt(choice2, bool_typet());
@@ -686,7 +689,7 @@ void shared_bufferst::nondet_flush(
     // a thread can read the other threads' buffers
 
     // One extra non-deterministic choice needed
-    irep_idt choice1=choice(target->function, "1");
+    irep_idt choice1 = choice(function_id, "1");
     const symbol_exprt choice1_expr=symbol_exprt(choice1, bool_typet());
 
     // throw Boolean dice
@@ -1017,7 +1020,6 @@ bool shared_bufferst::is_buffered_in_general(
 /// variables (non necessarily shared themselves) whose value could be changed
 /// as effect of a read delay
 void shared_bufferst::affected_by_delay(
-  symbol_tablet &symbol_table,
   value_setst &value_sets,
   goto_functionst &goto_functions)
 {
@@ -1031,11 +1033,16 @@ void shared_bufferst::affected_by_delay(
 
     Forall_goto_program_instructions(i_it, f_it->second.body)
     {
-        rw_set_loct rw_set(ns, value_sets, i_it
+      rw_set_loct rw_set(
+        ns,
+        value_sets,
+        f_it->first,
+        i_it
 #ifdef LOCAL_MAY
-        , local_may
+        ,
+        local_may
 #endif
-        ); // NOLINT(whitespace/parens)
+      ); // NOLINT(whitespace/parens)
         forall_rw_set_w_entries(w_it, rw_set)
           forall_rw_set_r_entries(r_it, rw_set)
           {
@@ -1053,19 +1060,19 @@ void shared_bufferst::affected_by_delay(
 /// instruments the program for the pairs detected through the CFG
 void shared_bufferst::cfg_visitort::weak_memory(
   value_setst &value_sets,
-  const irep_idt &function,
+  const irep_idt &function_id,
   memory_modelt model)
 {
-  shared_buffers.message.debug() << "visit function "<< function
-                                 << messaget::eom;
-  if(function == INITIALIZE_FUNCTION)
+  shared_buffers.message.debug()
+    << "visit function " << function_id << messaget::eom;
+  if(function_id == INITIALIZE_FUNCTION)
     return;
 
   namespacet ns(symbol_table);
-  goto_programt &goto_program=goto_functions.function_map[function].body;
+  goto_programt &goto_program = goto_functions.function_map[function_id].body;
 
 #ifdef LOCAL_MAY
-  local_may_aliast local_may(goto_functions.function_map[function]);
+  local_may_aliast local_may(goto_functions.function_map[function_id]);
 #endif
 
   Forall_goto_program_instructions(i_it, goto_program)
@@ -1089,9 +1096,14 @@ void shared_bufferst::cfg_visitort::weak_memory(
     {
       try
       {
-        rw_set_loct rw_set(ns, value_sets, i_it
+        rw_set_loct rw_set(
+          ns,
+          value_sets,
+          function_id,
+          i_it
 #ifdef LOCAL_MAY
-        , local_may
+          ,
+          local_may
 #endif
         ); // NOLINT(whitespace/parens)
 
@@ -1110,8 +1122,7 @@ void shared_bufferst::cfg_visitort::weak_memory(
           original_instruction.source_location;
 
         // ATOMIC_BEGIN: we make the whole thing atomic
-        instruction.make_atomic_begin();
-        instruction.source_location=source_location;
+        instruction = goto_programt::make_atomic_begin(source_location);
         i_it++;
 
         // we first perform (non-deterministically) up to 2 writes for
@@ -1124,9 +1135,13 @@ void shared_bufferst::cfg_visitort::weak_memory(
 
           if(shared_buffers.is_buffered(ns, e_it->second.symbol_expr, false))
             shared_buffers.nondet_flush(
-              goto_program, i_it, source_location, e_it->second.object,
+              function_id,
+              goto_program,
+              i_it,
+              source_location,
+              e_it->second.object,
               current_thread,
-              (model==TSO || model==PSO || model==RMO));
+              (model == TSO || model == PSO || model == RMO));
         }
 
         // Now perform the write(s).
@@ -1181,8 +1196,7 @@ void shared_bufferst::cfg_visitort::weak_memory(
                     vars.read_delayed, bool_typet());
 
                   // One extra non-deterministic choice needed
-                  irep_idt choice1=shared_buffers.choice(
-                    instruction.function, "1");
+                  irep_idt choice1 = shared_buffers.choice(function_id, "1");
                   const symbol_exprt choice1_expr=symbol_exprt(choice1,
                     bool_typet());
                   const exprt nondet_bool_expr =
@@ -1200,7 +1214,7 @@ void shared_bufferst::cfg_visitort::weak_memory(
                     read_delayed_expr,
                     if_exprt(
                       choice1_expr,
-                      dereference_exprt(new_read_expr, vars.type),
+                      dereference_exprt{new_read_expr},
                       to_replace_expr),
                     to_replace_expr); // original_instruction.code.op1());
 
@@ -1242,9 +1256,8 @@ void shared_bufferst::cfg_visitort::weak_memory(
           }
 
         // ATOMIC_END
-        i_it=goto_program.insert_before(i_it);
-        i_it->make_atomic_end();
-        i_it->source_location=source_location;
+        i_it = goto_program.insert_before(
+          i_it, goto_programt::make_atomic_end(source_location));
         i_it++;
 
         i_it--; // the for loop already counts us up
@@ -1269,8 +1282,7 @@ void shared_bufferst::cfg_visitort::weak_memory(
         original_instruction.source_location;
 
       // ATOMIC_BEGIN
-      instruction.make_atomic_begin();
-      instruction.source_location=source_location;
+      instruction = goto_programt::make_atomic_begin(source_location);
       i_it++;
 
       // does it for all the previous statements
@@ -1283,9 +1295,8 @@ void shared_bufferst::cfg_visitort::weak_memory(
       }
 
       // ATOMIC_END
-      i_it=goto_program.insert_before(i_it);
-      i_it->make_atomic_end();
-      i_it->source_location=source_location;
+      i_it = goto_program.insert_before(
+        i_it, goto_programt::make_atomic_end(source_location));
       i_it++;
 
       i_it--; // the for loop already counts us up
@@ -1293,7 +1304,7 @@ void shared_bufferst::cfg_visitort::weak_memory(
     else if(is_lwfence(instruction, ns))
     {
       // po -- remove the lwfence
-      i_it->make_skip();
+      *i_it = goto_programt::make_skip(i_it->source_location);
     }
     else if(instruction.is_function_call())
     {
