@@ -30,7 +30,7 @@ static exprt build_class_identifier(
     const typet &type=ns.follow(e.type());
     const struct_typet &struct_type=to_struct_type(type);
     const struct_typet::componentst &components=struct_type.components();
-    assert(!components.empty());
+    INVARIANT(!components.empty(), "class structs cannot be empty");
 
     const auto &first_member_name=components.front().get_name();
     member_exprt member_expr(
@@ -38,10 +38,10 @@ static exprt build_class_identifier(
       first_member_name,
       components.front().type());
 
-    if(first_member_name=="@class_identifier")
+    if(first_member_name == JAVA_CLASS_IDENTIFIER_FIELD_NAME)
     {
       // found it
-      return member_expr;
+      return std::move(member_expr);
     }
     else
     {
@@ -55,35 +55,34 @@ static exprt build_class_identifier(
 /// \return Member expression to access a class identifier, as above.
 exprt get_class_identifier_field(
   const exprt &this_expr_in,
-  const symbol_typet &suggested_type,
+  const struct_tag_typet &suggested_type,
   const namespacet &ns)
 {
   // Get a pointer from which we can extract a clsid.
   // If it's already a pointer to an object of some sort, just use it;
   // if it's void* then use the suggested type.
+  PRECONDITION(this_expr_in.type().id() == ID_pointer);
 
   exprt this_expr=this_expr_in;
-  assert(this_expr.type().id()==ID_pointer &&
-         "Non-pointer this-arg in remove-virtuals?");
   const auto &points_to=this_expr.type().subtype();
   if(points_to==empty_typet())
     this_expr=typecast_exprt(this_expr, pointer_type(suggested_type));
-  const dereference_exprt deref(this_expr, this_expr.type().subtype());
+  const dereference_exprt deref{this_expr};
   return build_class_identifier(deref, ns);
 }
 
 /// If expr has its components filled in then sets the `@class_identifier`
 /// member of the struct
 /// \remarks Follows through base class members until it gets to the object
-/// type that contains the `@class_identifier` member
+///   type that contains the `@class_identifier` member
 /// \param expr: An expression that represents a struct
 /// \param ns: The namespace used to resolve symbol references in the type of
-/// the struct
+///   the struct
 /// \param class_type: A symbol whose identifier is the name of the class
 void set_class_identifier(
   struct_exprt &expr,
   const namespacet &ns,
-  const symbol_typet &class_type)
+  const struct_tag_typet &class_type)
 {
   const struct_typet &struct_type=to_struct_type(ns.follow(expr.type()));
   const struct_typet::componentst &components=struct_type.components();
@@ -93,7 +92,7 @@ void set_class_identifier(
     return;
   PRECONDITION(!expr.operands().empty());
 
-  if(components.front().get_name()=="@class_identifier")
+  if(components.front().get_name() == JAVA_CLASS_IDENTIFIER_FIELD_NAME)
   {
     INVARIANT(
       expr.op0().id()==ID_constant, "@class_identifier must be a constant");
