@@ -6,6 +6,9 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
+/// \file
+/// Program Transformation
+
 #ifndef CPROVER_GOTO_PROGRAMS_GOTO_CONVERT_H
 #define CPROVER_GOTO_PROGRAMS_GOTO_CONVERT_H
 
@@ -17,14 +20,15 @@ Author: Daniel Kroening, kroening@kroening.com
 // start from given code
 void goto_convert(
   const codet &code,
-  symbol_tablet &symbol_table,
+  symbol_table_baset &symbol_table,
   goto_programt &dest,
-  message_handlert &message_handler);
+  message_handlert &message_handler,
+  const irep_idt &mode);
 
 // start from "main"
 void goto_convert(
-  symbol_tablet &symbol_table,
+  symbol_table_baset &symbol_table,
   goto_programt &dest,
   message_handlert &message_handler);
 
-#endif
+#endif // CPROVER_GOTO_PROGRAMS_GOTO_CONVERT_H
