@@ -15,7 +15,6 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <util/prefix.h>
 #include <util/cprover_prefix.h>
-#include <util/base_type.h>
 #include <util/std_code.h>
 #include <util/std_expr.h>
 
@@ -55,9 +54,9 @@ void goto_inline(
   if(it==goto_functions.function_map.end())
     return;
 
-  goto_functiont &goto_function=it->second;
+  goto_functiont &entry_function = it->second;
   DATA_INVARIANT(
-    goto_function.body_available(),
+    entry_function.body_available(),
     "body of entry point function must be available");
 
   // gather all calls
@@ -86,7 +85,7 @@ void goto_inline(
   }
 
   goto_inline.goto_inline(
-    goto_functionst::entry_point(), goto_function, inline_map, true);
+    goto_functionst::entry_point(), entry_function, inline_map, true);
 
   // clean up
   Forall_goto_functions(f_it, goto_functions)
@@ -180,26 +179,26 @@ void goto_partial_inline(
       const symbol_exprt &symbol_expr=to_symbol_expr(function_expr);
       const irep_idt id=symbol_expr.get_identifier();
 
-      goto_functionst::function_mapt::const_iterator f_it=
+      goto_functionst::function_mapt::const_iterator called_it =
         goto_functions.function_map.find(id);
 
-      if(f_it==goto_functions.function_map.end())
+      if(called_it == goto_functions.function_map.end())
         // Function not loaded, can't check size
         continue;
 
       // called function
-      const goto_functiont &goto_function=f_it->second;
+      const goto_functiont &called_function = called_it->second;
 
-      if(!goto_function.body_available())
+      if(!called_function.body_available())
         // The bodies of functions that don't have bodies can't be inlined.
         continue;
 
-      const goto_programt &goto_program=goto_function.body;
-
-      if(goto_function.is_inlined() ||
-         goto_program.instructions.size()<=smallfunc_limit)
+      if(
+        to_code_type(ns.lookup(id).type).get_inlined() ||
+        called_function.body.instructions.size() <= smallfunc_limit)
       {
-        INVARIANT(i_it->is_function_call(), "is a call");
+        PRECONDITION(i_it->is_function_call());
+
         call_list.push_back(goto_inlinet::callt(i_it, false));
       }
     }
