@@ -15,9 +15,6 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <iostream>
 #endif
 
-#include <cassert>
-
-#include <util/base_type.h>
 #include <util/cprover_prefix.h>
 #include <util/expr_util.h>
 #include <util/invariant.h>
@@ -33,7 +30,7 @@ Author: Daniel Kroening, kroening@kroening.com
 void goto_inlinet::parameter_assignments(
   const goto_programt::targett target,
   const irep_idt &function_name, // name of called function
-  const code_typet &code_type, // type of called function
+  const goto_functiont::parameter_identifierst &parameter_identifiers,
   const exprt::operandst &arguments, // arguments of call
   goto_programt &dest)
 {
@@ -45,34 +42,21 @@ void goto_inlinet::parameter_assignments(
   // iterates over the operands
   exprt::operandst::const_iterator it1=arguments.begin();
 
-  const code_typet::parameterst &parameter_types=
-    code_type.parameters();
-
-  // iterates over the types of the parameters
-  for(const auto &parameter : parameter_types)
+  // iterates over the parameters
+  for(const auto &identifier : parameter_identifiers)
   {
-    // this is the type the n-th argument should be
-    const typet &par_type=ns.follow(parameter.type());
+    DATA_INVARIANT(
+      !identifier.empty(),
+      source_location.as_string() + ": no identifier for function parameter");
 
-    const irep_idt &identifier=parameter.get_identifier();
+    const symbolt &symbol = ns.lookup(identifier);
 
-    if(identifier.empty())
-    {
-      error().source_location=source_location;
-      error() << "no identifier for function parameter" << eom;
-      throw 0;
-    }
+    // this is the type the n-th argument should have
+    const typet &parameter_type = symbol.type;
 
-    {
-      const symbolt &symbol=ns.lookup(identifier);
-
-      goto_programt::targett decl=dest.add_instruction();
-      decl->make_decl();
-      decl->code=code_declt(symbol.symbol_expr());
-      decl->code.add_source_location()=source_location;
-      decl->source_location=source_location;
-      decl->function=adjust_function?target->function:function_name;
-    }
+    goto_programt::targett decl =
+      dest.add(goto_programt::make_decl(symbol.symbol_expr(), source_location));
+    decl->code.add_source_location() = source_location;
 
     // this is the actual parameter
     exprt actual;
@@ -81,11 +65,11 @@ void goto_inlinet::parameter_assignments(
     if(it1==arguments.end())
     {
       warning().source_location=source_location;
-      warning() << "call to `" << function_name << "': "
+      warning() << "call to '" << function_name << "': "
                 << "not enough arguments, "
                 << "inserting non-deterministic value" << eom;
 
-      actual = side_effect_expr_nondett(par_type, source_location);
+      actual = side_effect_expr_nondett(parameter_type, source_location);
     }
     else
       actual=*it1;
@@ -98,10 +82,10 @@ void goto_inlinet::parameter_assignments(
     {
       // it should be the same exact type as the parameter,
       // subject to some exceptions
-      if(!base_type_eq(par_type, actual.type(), ns))
+      if(parameter_type != actual.type())
       {
-        const typet &f_partype=ns.follow(par_type);
-        const typet &f_acttype=ns.follow(actual.type());
+        const typet &f_partype = parameter_type;
+        const typet &f_acttype = actual.type();
 
         // we are willing to do some conversion
         if((f_partype.id()==ID_pointer &&
@@ -110,7 +94,7 @@ void goto_inlinet::parameter_assignments(
             f_acttype.id()==ID_array &&
             f_partype.subtype()==f_acttype.subtype()))
         {
-          actual.make_typecast(par_type);
+          actual = typecast_exprt(actual, f_partype);
         }
         else if((f_partype.id()==ID_signedbv ||
                  f_partype.id()==ID_unsignedbv ||
@@ -119,32 +103,19 @@ void goto_inlinet::parameter_assignments(
                  f_acttype.id()==ID_unsignedbv ||
                  f_acttype.id()==ID_bool))
         {
-          actual.make_typecast(par_type);
+          actual = typecast_exprt(actual, f_partype);
         }
         else
         {
-          error().source_location=actual.find_source_location();
-
-          error() << "function call: argument `" << identifier
-                  << "' type mismatch: argument is `"
-                  // << from_type(ns, identifier, actual.type())
-                  << actual.type().pretty()
-                  << "', parameter is `"
-                  << from_type(ns, identifier, par_type)
-                  << "'" << eom;
-          throw 0;
+          UNREACHABLE;
         }
       }
 
       // adds an assignment of the actual parameter to the formal parameter
-      code_assignt assignment(symbol_exprt(identifier, par_type), actual);
+      code_assignt assignment(symbol_exprt(identifier, parameter_type), actual);
       assignment.add_source_location()=source_location;
 
-      dest.add_instruction(ASSIGN);
-      dest.instructions.back().source_location=source_location;
-      dest.instructions.back().code.swap(assignment);
-      dest.instructions.back().function=
-        adjust_function?target->function:function_name;
+      dest.add(goto_programt::make_assignment(assignment, source_location));
     }
 
     if(it1!=arguments.end())
@@ -159,8 +130,7 @@ void goto_inlinet::parameter_assignments(
 
 void goto_inlinet::parameter_destruction(
   const goto_programt::targett target,
-  const irep_idt &function_name, // name of called function
-  const code_typet &code_type, // type of called function
+  const goto_functiont::parameter_identifierst &parameter_identifiers,
   goto_programt &dest)
 {
   PRECONDITION(target->is_function_call());
@@ -168,30 +138,18 @@ void goto_inlinet::parameter_destruction(
 
   const source_locationt &source_location=target->source_location;
 
-  const code_typet::parameterst &parameter_types=
-    code_type.parameters();
-
-  // iterates over the types of the parameters
-  for(const auto &parameter : parameter_types)
+  for(const auto &identifier : parameter_identifiers)
   {
-    const irep_idt &identifier=parameter.get_identifier();
-
-    if(identifier.empty())
-    {
-      error().source_location=source_location;
-      error() << "no identifier for function parameter" << eom;
-      throw 0;
-    }
+    INVARIANT(
+      !identifier.empty(),
+      source_location.as_string() + ": no identifier for function parameter");
 
     {
       const symbolt &symbol=ns.lookup(identifier);
 
-      goto_programt::targett dead=dest.add_instruction();
-      dead->make_dead();
-      dead->code=code_deadt(symbol.symbol_expr());
+      goto_programt::targett dead = dest.add(
+        goto_programt::make_dead(symbol.symbol_expr(), source_location));
       dead->code.add_source_location()=source_location;
-      dead->source_location=source_location;
-      dead->function=adjust_function?target->function:function_name;
     }
   }
 }
@@ -207,9 +165,11 @@ void goto_inlinet::replace_return(
   {
     if(it->is_return())
     {
+      const auto &code_return = it->get_return();
+
       if(lhs.is_not_nil())
       {
-        if(it->code.operands().size()!=1)
+        if(!code_return.has_return_value())
         {
           warning().source_location=it->code.find_source_location();
           warning() << "return expects one operand!\n"
@@ -217,22 +177,19 @@ void goto_inlinet::replace_return(
           continue;
         }
 
-        code_assignt code_assign(lhs, it->code.op0());
-
-        // this may happen if the declared return type at the call site
-        // differs from the defined return type
-        if(code_assign.lhs().type()!=
-           code_assign.rhs().type())
-          code_assign.rhs().make_typecast(code_assign.lhs().type());
-
-        it->code=code_assign;
+        // a typecast may be necessary if the declared return type at the call
+        // site differs from the defined return type
+        it->code = code_assignt(
+          lhs,
+          typecast_exprt::conditional_cast(
+            code_return.return_value(), lhs.type()));
         it->type=ASSIGN;
 
         it++;
       }
-      else if(!it->code.operands().empty())
+      else if(code_return.has_return_value())
       {
-        it->code=code_expressiont(it->code.op0());
+        it->code = code_expressiont(code_return.return_value());
         it->type=OTHER;
         it++;
       }
@@ -298,22 +255,21 @@ void goto_inlinet::insert_function_body(
     "final instruction of a function must be an END_FUNCTION");
   end.type=LOCATION;
 
-  if(adjust_function)
+  // make sure the inlined function does not introduce hiding
+  if(ns.lookup(identifier).is_hidden())
+  {
     for(auto &instruction : body.instructions)
-      instruction.function=target->function;
+      instruction.labels.remove(CPROVER_PREFIX "HIDE");
+  }
 
   replace_return(body, lhs);
 
   goto_programt tmp1;
   parameter_assignments(
-    target,
-    identifier,
-    goto_function.type,
-    arguments,
-    tmp1);
+    target, identifier, goto_function.parameter_identifiers, arguments, tmp1);
 
   goto_programt tmp2;
-  parameter_destruction(target, identifier, goto_function.type, tmp2);
+  parameter_destruction(target, goto_function.parameter_identifiers, tmp2);
 
   goto_programt tmp;
   tmp.destructive_append(tmp1); // par assignment
@@ -338,37 +294,21 @@ void goto_inlinet::insert_function_body(
     call_location_number,
     identifier);
 
-#if 0
-  if(goto_function.is_hidden())
+  if(adjust_function)
   {
-    source_locationt new_source_location=
-      function.find_source_location();
-
-    if(new_source_location.is_not_nil())
+    Forall_goto_program_instructions(it, tmp)
     {
-      new_source_location.set_hide();
+      replace_location(it->source_location, target->source_location);
+      replace_location(it->code, target->source_location);
 
-      Forall_goto_program_instructions(it, tmp)
+      if(it->has_condition())
       {
-        if(it->function==identifier)
-        {
-          // don't hide assignment to lhs
-          if(it->is_assign() && to_code_assign(it->code).lhs()==lhs)
-          {
-          }
-          else
-          {
-            replace_location(it->source_location, new_source_location);
-            replace_location(it->guard, new_source_location);
-            replace_location(it->code, new_source_location);
-          }
-
-          it->function=target->function;
-        }
+        exprt c = it->get_condition();
+        replace_location(c, target->source_location);
+        it->set_condition(c);
       }
     }
   }
-#endif
 
   // kill call
   target->type=LOCATION;
@@ -391,7 +331,7 @@ void goto_inlinet::expand_function_call(
 
 #ifdef DEBUG
   std::cout << "Expanding call:\n";
-  dest.output_instruction(ns, "", std::cout, *target);
+  dest.output_instruction(ns, irep_idt(), std::cout, *target);
 #endif
 
   exprt lhs;
@@ -416,11 +356,11 @@ void goto_inlinet::expand_function_call(
     // it's recursive.
     // Uh. Buh. Give up.
     warning().source_location=function.find_source_location();
-    warning() << "recursion is ignored on call to `" << identifier << "'"
+    warning() << "recursion is ignored on call to '" << identifier << "'"
               << eom;
 
     if(force_full)
-      target->make_skip();
+      target->turn_into_skip();
 
     return;
   }
@@ -431,10 +371,10 @@ void goto_inlinet::expand_function_call(
   if(f_it==goto_functions.function_map.end())
   {
     warning().source_location=function.find_source_location();
-    warning() << "missing function `" << identifier << "' is ignored" << eom;
+    warning() << "missing function '" << identifier << "' is ignored" << eom;
 
     if(force_full)
-      target->make_skip();
+      target->turn_into_skip();
 
     return;
   }
@@ -496,12 +436,10 @@ void goto_inlinet::expand_function_call(
   }
   else // no body available
   {
-    const irep_idt identifier = function.get_identifier();
-
     if(no_body_set.insert(identifier).second) // newly inserted
     {
       warning().source_location = function.find_source_location();
-      warning() << "no body for function `" << identifier << "'" << eom;
+      warning() << "no body for function '" << identifier << "'" << eom;
     }
   }
 }
@@ -514,7 +452,7 @@ void goto_inlinet::get_call(
 {
   PRECONDITION(it->is_function_call());
 
-  const code_function_callt &call=to_code_function_call(it->code);
+  const code_function_callt &call = it->get_function_call();
 
   lhs=call.lhs();
   function=call.function();
@@ -626,9 +564,8 @@ const goto_inlinet::goto_functiont &goto_inlinet::goto_inline_transitive(
   }
 
   goto_functiont &cached=cache[identifier];
-  INVARIANT(
-    cached.body.empty(),
-    "body of new function in cache must be empty");
+  DATA_INVARIANT(
+    cached.body.empty(), "body of new function in cache must be empty");
 
   progress() << "Creating copy of " << identifier << eom;
   progress() << "Number of instructions: "
@@ -672,13 +609,9 @@ const goto_inlinet::goto_functiont &goto_inlinet::goto_inline_transitive(
 
 bool goto_inlinet::is_ignored(const irep_idt id) const
 {
-  return
-    id=="__CPROVER_cleanup" ||
-    id=="__CPROVER_set_must" ||
-    id=="__CPROVER_set_may" ||
-    id=="__CPROVER_clear_must" ||
-    id=="__CPROVER_clear_may" ||
-    id=="__CPROVER_cover";
+  return id == CPROVER_PREFIX "cleanup" || id == CPROVER_PREFIX "set_must" ||
+         id == CPROVER_PREFIX "set_may" || id == CPROVER_PREFIX "clear_must" ||
+         id == CPROVER_PREFIX "clear_may" || id == CPROVER_PREFIX "cover";
 }
 
 bool goto_inlinet::check_inline_map(
@@ -688,7 +621,7 @@ bool goto_inlinet::check_inline_map(
   goto_functionst::function_mapt::const_iterator f_it=
     goto_functions.function_map.find(identifier);
 
-  PRECONDITION(f_it!=goto_functions.function_map.end());
+  PRECONDITION(f_it != goto_functions.function_map.end());
 
   inline_mapt::const_iterator im_it=inline_map.find(identifier);
 
@@ -756,13 +689,13 @@ void goto_inlinet::output_inline_map(
     goto_functionst::function_mapt::const_iterator f_it=
       goto_functions.function_map.find(id);
 
-    std::string call="-";
-
     if(f_it!=goto_functions.function_map.end() &&
        !call_list.empty())
     {
       const goto_functiont &goto_function=f_it->second;
-      PRECONDITION(goto_function.body_available());
+      DATA_INVARIANT(
+        goto_function.body_available(),
+        "cannot inline function with empty body");
 
       const goto_programt &goto_program=goto_function.body;
 
@@ -772,7 +705,7 @@ void goto_inlinet::output_inline_map(
         bool transitive=call.second;
 
         out << "  Call:\n";
-        goto_program.output_instruction(ns, "", out, *target);
+        goto_program.output_instruction(ns, id, out, *target);
         out << "  Transitive: " << transitive << "\n";
       }
     }
@@ -825,10 +758,12 @@ void goto_inlinet::goto_inline_logt::add_segment(
 {
   PRECONDITION(!goto_program.empty());
   PRECONDITION(!function.empty());
-  PRECONDITION(end_location_number>=begin_location_number);
+  PRECONDITION(end_location_number >= begin_location_number);
 
   goto_programt::const_targett start=goto_program.instructions.begin();
-  PRECONDITION(log_map.find(start)==log_map.end());
+  INVARIANT(
+    log_map.find(start) == log_map.end(),
+    "inline function should be registered once in map of inline functions");
 
   goto_programt::const_targett end=goto_program.instructions.end();
   end--;
@@ -847,21 +782,27 @@ void goto_inlinet::goto_inline_logt::copy_from(
   const goto_programt &from,
   const goto_programt &to)
 {
-  PRECONDITION(from.instructions.size()==to.instructions.size());
+  PRECONDITION(from.instructions.size() == to.instructions.size());
 
   goto_programt::const_targett it1=from.instructions.begin();
   goto_programt::const_targett it2=to.instructions.begin();
 
   for(; it1!=from.instructions.end(); it1++, it2++)
   {
-    assert(it2!=to.instructions.end());
-    assert(it1->location_number==it2->location_number);
+    DATA_INVARIANT(
+      it2 != to.instructions.end(),
+      "'to' target function is not allowed to be empty");
+    DATA_INVARIANT(
+      it1->location_number == it2->location_number,
+      "both functions' instruction should point to the same source");
 
     log_mapt::const_iterator l_it=log_map.find(it1);
     if(l_it!=log_map.end()) // a segment starts here
     {
       // as 'to' is a fresh copy
-      assert(log_map.find(it2)==log_map.end());
+      DATA_INVARIANT(
+        log_map.find(it2) == log_map.end(),
+        "'to' target is not expected to be in the log_map");
 
       goto_inline_log_infot info=l_it->second;
       goto_programt::const_targett end=info.end;
@@ -890,27 +831,26 @@ jsont goto_inlinet::goto_inline_logt::output_inline_log_json() const
 
   for(const auto &it : log_map)
   {
-    json_objectt &object=json_inlined.push_back().make_object();
-
     goto_programt::const_targett start=it.first;
     const goto_inline_log_infot &info=it.second;
     goto_programt::const_targett end=info.end;
 
-    assert(start->location_number<=end->location_number);
+    PRECONDITION(start->location_number <= end->location_number);
 
-    object["call"]=json_numbert(std::to_string(info.call_location_number));
-    object["function"]=json_stringt(info.function.c_str());
+    json_arrayt json_orig{
+      json_numbert(std::to_string(info.begin_location_number)),
+      json_numbert(std::to_string(info.end_location_number))};
+    json_arrayt json_new{json_numbert(std::to_string(start->location_number)),
+                         json_numbert(std::to_string(end->location_number))};
 
-    json_arrayt &json_orig=object["originalSegment"].make_array();
-    json_orig.push_back()=json_numbert(std::to_string(
-      info.begin_location_number));
-    json_orig.push_back()=json_numbert(std::to_string(
-      info.end_location_number));
+    json_objectt object{
+      {"call", json_numbert(std::to_string(info.call_location_number))},
+      {"function", json_stringt(info.function.c_str())},
+      {"originalSegment", std::move(json_orig)},
+      {"inlinedSegment", std::move(json_new)}};
 
-    json_arrayt &json_new=object["inlinedSegment"].make_array();
-    json_new.push_back()=json_numbert(std::to_string(start->location_number));
-    json_new.push_back()=json_numbert(std::to_string(end->location_number));
+    json_inlined.push_back(std::move(object));
   }
 
-  return json_result;
+  return std::move(json_result);
 }
