@@ -12,13 +12,14 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_GOTO_PROGRAMS_INITIALIZE_GOTO_MODEL_H
 #define CPROVER_GOTO_PROGRAMS_INITIALIZE_GOTO_MODEL_H
 
-#include <util/message.h>
-#include <util/cmdline.h>
-
 #include "goto_model.h"
 
+class message_handlert;
+class optionst;
+
 goto_modelt initialize_goto_model(
-  const cmdlinet &cmdline,
-  message_handlert &message_handler);
+  const std::vector<std::string> &files,
+  message_handlert &message_handler,
+  const optionst &options);
 
 #endif // CPROVER_GOTO_PROGRAMS_INITIALIZE_GOTO_MODEL_H
