@@ -21,6 +21,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <util/fixedbv.h>
 #include <util/ieee_float.h>
 #include <util/invariant.h>
+#include <util/mathematical_types.h>
 #include <util/message.h>
 #include <util/std_expr.h>
 #include <util/std_types.h>
@@ -111,7 +112,7 @@ void interpretert::show_state()
 
   if(pc==function->second.body.instructions.end())
   {
-    status() << "End of function `" << function->first << "'\n";
+    status() << "End of function '" << function->first << "'\n";
   }
   else
     function->second.body.output_instruction(
@@ -358,7 +359,8 @@ void interpretert::step()
     break;
   case CATCH:
     break;
-  default:
+  case INCOMPLETE_GOTO:
+  case NO_INSTRUCTION_TYPE:
     throw "encountered instruction with undefined instruction type";
   }
   pc=next_pc;
@@ -367,7 +369,7 @@ void interpretert::step()
 /// executes a goto instruction
 void interpretert::execute_goto()
 {
-  if(evaluate_boolean(pc->guard))
+  if(evaluate_boolean(pc->get_condition()))
   {
     if(pc->targets.empty())
       throw "taken goto without target";
@@ -382,7 +384,7 @@ void interpretert::execute_goto()
 /// executes side effects of 'other' instructions
 void interpretert::execute_other()
 {
-  const irep_idt &statement=pc->code.get_statement();
+  const irep_idt &statement = pc->get_other().get_statement();
   if(statement==ID_expression)
   {
     DATA_INVARIANT(
@@ -406,7 +408,7 @@ void interpretert::execute_other()
       assign(address, rhs);
     }
   }
-  else if(statement==ID_output)
+  else if(can_cast_expr<code_outputt>(pc->get_other()))
   {
     return;
   }
@@ -425,8 +427,15 @@ struct_typet::componentt interpretert::get_component(
   const irep_idt &object,
   const mp_integer &offset)
 {
-  const symbolt &symbol=ns.lookup(object);
-  const typet real_type=ns.follow(symbol.type);
+  const symbolt &symbol = ns.lookup(object);
+  return get_component(symbol.type, offset);
+}
+
+/// Retrieves the member at \p offset of an object of type \p object_type.
+struct_typet::componentt
+interpretert::get_component(const typet &object_type, const mp_integer &offset)
+{
+  const typet real_type = ns.follow(object_type);
   if(real_type.id()!=ID_struct)
     throw "request for member of non-struct";
 
@@ -465,7 +474,7 @@ exprt interpretert::get_value(
   const typet real_type=ns.follow(type);
   if(real_type.id()==ID_struct)
   {
-    struct_exprt result(real_type);
+    struct_exprt result({}, real_type);
     const struct_typet &struct_type=to_struct_type(real_type);
     const struct_typet::componentst &components=struct_type.components();
 
@@ -482,12 +491,12 @@ exprt interpretert::get_value(
       result.copy_to_operands(operand);
     }
 
-    return result;
+    return std::move(result);
   }
   else if(real_type.id()==ID_array)
   {
     // Get size of array
-    array_exprt result(to_array_type(real_type));
+    array_exprt result({}, to_array_type(real_type));
     const exprt &size_expr=static_cast<const exprt &>(type.find(ID_size));
     mp_integer subtype_size=get_size(type.subtype());
     mp_integer count;
@@ -497,11 +506,11 @@ exprt interpretert::get_value(
     }
     else
     {
-      to_integer(size_expr, count);
+      count = numeric_cast_v<mp_integer>(to_constant_expr(size_expr));
     }
 
     // Retrieve the value for each member in the array
-    result.reserve_operands(integer2size_t(count));
+    result.reserve_operands(numeric_cast_v<std::size_t>(count));
     for(mp_integer i=0; i<count; ++i)
     {
       const exprt operand=get_value(
@@ -509,14 +518,16 @@ exprt interpretert::get_value(
         offset+i*subtype_size);
       result.copy_to_operands(operand);
     }
-    return result;
+    return std::move(result);
   }
-  if(use_non_det &&
-     memory[integer2ulong(offset)].initialized!=
-     memory_cellt::initializedt::WRITTEN_BEFORE_READ)
+  if(
+    use_non_det && memory[numeric_cast_v<std::size_t>(offset)].initialized !=
+                     memory_cellt::initializedt::WRITTEN_BEFORE_READ)
+  {
     return side_effect_expr_nondett(type, source_locationt());
+  }
   mp_vectort rhs;
-  rhs.push_back(memory[integer2ulong(offset)].value);
+  rhs.push_back(memory[numeric_cast_v<std::size_t>(offset)].value);
   return get_value(type, rhs);
 }
 
@@ -532,7 +543,7 @@ exprt interpretert::get_value(
 
   if(real_type.id()==ID_struct)
   {
-    struct_exprt result(real_type);
+    struct_exprt result({}, real_type);
     const struct_typet &struct_type=to_struct_type(real_type);
     const struct_typet::componentst &components=struct_type.components();
 
@@ -547,11 +558,11 @@ exprt interpretert::get_value(
       tmp_offset+=size;
       result.copy_to_operands(operand);
     }
-    return result;
+    return std::move(result);
   }
   else if(real_type.id()==ID_array)
   {
-    constant_exprt result(type);
+    array_exprt result({}, to_array_type(real_type));
     const exprt &size_expr=static_cast<const exprt &>(type.find(ID_size));
 
     // Get size of array
@@ -564,80 +575,74 @@ exprt interpretert::get_value(
     }
     else
     {
-      to_integer(size_expr, count);
+      count = numeric_cast_v<mp_integer>(to_constant_expr(size_expr));
     }
 
     // Retrieve the value for each member in the array
-    result.reserve_operands(integer2size_t(count));
+    result.reserve_operands(numeric_cast_v<std::size_t>(count));
     for(mp_integer i=0; i<count; ++i)
     {
       const exprt operand=get_value(type.subtype(), rhs,
           offset+i*subtype_size);
       result.copy_to_operands(operand);
     }
-    return result;
+    return std::move(result);
   }
   else if(real_type.id()==ID_floatbv)
   {
     ieee_floatt f(to_floatbv_type(type));
-    f.unpack(rhs[integer2size_t(offset)]);
+    f.unpack(rhs[numeric_cast_v<std::size_t>(offset)]);
     return f.to_expr();
   }
   else if(real_type.id()==ID_fixedbv)
   {
     fixedbvt f;
-    f.from_integer(rhs[integer2size_t(offset)]);
+    f.from_integer(rhs[numeric_cast_v<std::size_t>(offset)]);
     return f.to_expr();
   }
   else if(real_type.id()==ID_bool)
   {
-    if(rhs[integer2size_t(offset)]!=0)
+    if(rhs[numeric_cast_v<std::size_t>(offset)] != 0)
       return true_exprt();
     else
       false_exprt();
   }
   else if(real_type.id()==ID_c_bool)
   {
-    return from_integer(rhs[integer2size_t(offset)]!=0?1:0, type);
+    return from_integer(
+      rhs[numeric_cast_v<std::size_t>(offset)] != 0 ? 1 : 0, type);
   }
-  else if((real_type.id()==ID_pointer) || (real_type.id()==ID_address_of))
+  else if(real_type.id() == ID_pointer)
   {
-    if(rhs[integer2size_t(offset)]==0)
-    {
-      // NULL pointer
-      constant_exprt result(type);
-      result.set_value(ID_NULL);
-      return result;
-    }
+    if(rhs[numeric_cast_v<std::size_t>(offset)] == 0)
+      return null_pointer_exprt(to_pointer_type(real_type)); // NULL pointer
 
-    if(rhs[integer2size_t(offset)]<memory.size())
+    if(rhs[numeric_cast_v<std::size_t>(offset)] < memory.size())
     {
       // We want the symbol pointed to
-      mp_integer address=rhs[integer2size_t(offset)];
-      irep_idt identifier=address_to_identifier(address);
-      mp_integer offset=address_to_offset(address);
-      const typet type=get_type(identifier);
-      const symbol_exprt symbol_expr(identifier, type);
+      mp_integer address = rhs[numeric_cast_v<std::size_t>(offset)];
+      const symbol_exprt &symbol_expr = address_to_symbol(address);
+      mp_integer offset_from_address = address_to_offset(address);
 
-      if(offset==0)
+      if(offset_from_address == 0)
         return address_of_exprt(symbol_expr);
 
-      if(ns.follow(type).id()==ID_struct)
+      if(
+        symbol_expr.type().id() == ID_struct ||
+        symbol_expr.type().id() == ID_struct_tag)
       {
-        const auto c=get_component(identifier, offset);
+        const auto c = get_component(symbol_expr.type(), offset_from_address);
         member_exprt member_expr(symbol_expr, c);
         return address_of_exprt(member_expr);
       }
 
-      index_exprt index_expr(
-        symbol_expr,
-        from_integer(offset, integer_typet()));
-
-      return index_expr;
+      return index_exprt(
+        symbol_expr, from_integer(offset_from_address, integer_typet()));
     }
 
-    error() << "interpreter: invalid pointer " << rhs[integer2size_t(offset)]
-            << " > object count " << memory.size() << eom;
+    error() << "interpreter: invalid pointer "
+            << rhs[numeric_cast_v<std::size_t>(offset)] << " > object count "
+            << memory.size() << eom;
 
     throw "interpreter: reading from invalid pointer";
   }
@@ -645,12 +650,13 @@ exprt interpretert::get_value(
   {
     // Strings are currently encoded by their irep_idt ID.
     return constant_exprt(
-      get_string_container().get_string(rhs[integer2size_t(offset)].to_long()),
+      get_string_container().get_string(
+        numeric_cast_v<std::size_t>(rhs[numeric_cast_v<std::size_t>(offset)])),
       type);
   }
 
   // Retrieve value of basic data type
-  return from_integer(rhs[integer2ulong(offset)], type);
+  return from_integer(rhs[numeric_cast_v<std::size_t>(offset)], type);
 }
 
 /// executes the assign statement at the current pc value
@@ -677,15 +683,7 @@ void interpretert::execute_assign()
       goto_trace_stept &trace_step=steps.get_last_step();
       assign(address, rhs);
       trace_step.full_lhs=code_assign.lhs();
-
-      // TODO: need to look at other cases on ssa_exprt
-      // (dereference should be handled on ssa)
-      if(ssa_exprt::can_build_identifier(trace_step.full_lhs))
-      {
-        trace_step.lhs_object=ssa_exprt(trace_step.full_lhs);
-      }
       trace_step.full_lhs_value=get_value(trace_step.full_lhs.type(), rhs);
-      trace_step.lhs_object_value=trace_step.full_lhs_value;
     }
   }
   else if(code_assign.rhs().id()==ID_side_effect)
@@ -693,15 +691,15 @@ void interpretert::execute_assign()
     side_effect_exprt side_effect=to_side_effect_expr(code_assign.rhs());
     if(side_effect.get_statement()==ID_nondet)
     {
-      mp_integer address=
-        integer2size_t(evaluate_address(code_assign.lhs()));
+      mp_integer address =
+        numeric_cast_v<std::size_t>(evaluate_address(code_assign.lhs()));
 
       mp_integer size=
         get_size(code_assign.lhs().type());
 
       for(mp_integer i=0; i<size; ++i)
       {
-        memory[integer2ulong(address+i)].initialized=
+        memory[numeric_cast_v<std::size_t>(address + i)].initialized =
           memory_cellt::initializedt::READ_BEFORE_WRITTEN;
       }
     }
@@ -718,16 +716,16 @@ void interpretert::assign(
     if((address+i)<memory.size())
     {
       mp_integer address_val=address+i;
-      memory_cellt &cell=memory[integer2ulong(address_val)];
+      memory_cellt &cell = memory[numeric_cast_v<std::size_t>(address_val)];
       if(show)
       {
         status() << total_steps << " ** assigning "
-                 << address_to_identifier(address_val) << "["
-                 << address_to_offset(address_val) << "]:="
-                 << rhs[integer2size_t(i)]
-                 << "\n" << eom;
+                 << address_to_symbol(address_val).get_identifier() << "["
+                 << address_to_offset(address_val)
+                 << "]:=" << rhs[numeric_cast_v<std::size_t>(i)] << "\n"
+                 << eom;
       }
-      cell.value=rhs[integer2size_t(i)];
+      cell.value = rhs[numeric_cast_v<std::size_t>(i)];
       if(cell.initialized==memory_cellt::initializedt::UNKNOWN)
         cell.initialized=memory_cellt::initializedt::WRITTEN_BEFORE_READ;
     }
@@ -736,13 +734,13 @@ void interpretert::assign(
 
 void interpretert::execute_assume()
 {
-  if(!evaluate_boolean(pc->guard))
+  if(!evaluate_boolean(pc->get_condition()))
     throw "assumption failed";
 }
 
 void interpretert::execute_assert()
 {
-  if(!evaluate_boolean(pc->guard))
+  if(!evaluate_boolean(pc->get_condition()))
   {
     if((target_assert==pc) || stop_on_assertion)
       throw "program assertion reached";
@@ -754,8 +752,7 @@ void interpretert::execute_assert()
 
 void interpretert::execute_function_call()
 {
-  const code_function_callt &function_call=
-    to_code_function_call(pc->code);
+  const code_function_callt &function_call = pc->get_function_call();
 
   // function to be called
   mp_integer address=evaluate_address(function_call.function());
@@ -771,8 +768,8 @@ void interpretert::execute_function_call()
 #if 0
   const memory_cellt &cell=memory[address];
 #endif
-  const irep_idt &identifier=address_to_identifier(address);
-  trace_step.identifier=identifier;
+  const irep_idt &identifier = address_to_symbol(address).get_identifier();
+  trace_step.called_function = identifier;
 
   const goto_functionst::function_mapt::const_iterator f_it=
     goto_functions.function_map.find(identifier);
@@ -816,20 +813,18 @@ void interpretert::execute_function_call()
     for(const auto &id : locals)
     {
       const symbolt &symbol=ns.lookup(id);
-      frame.local_map[id]=build_memory_map(id, symbol.type);
+      frame.local_map[id] = build_memory_map(symbol.symbol_expr());
     }
 
     // assign the arguments
-    const code_typet::parameterst &parameters=
-      to_code_type(f_it->second.type).parameters();
-
-    if(argument_values.size()<parameters.size())
+    const auto &parameter_identifiers = f_it->second.parameter_identifiers;
+    if(argument_values.size() < parameter_identifiers.size())
       throw "not enough arguments";
 
-    for(std::size_t i=0; i<parameters.size(); i++)
+    for(std::size_t i = 0; i < parameter_identifiers.size(); i++)
     {
-      const code_typet::parametert &a=parameters[i];
-      const symbol_exprt symbol_expr(a.get_identifier(), a.type());
+      const symbol_exprt symbol_expr =
+        ns.lookup(parameter_identifiers[i]).symbol_expr();
       assign(evaluate_address(symbol_expr), argument_values[i]);
     }
 
@@ -866,7 +861,7 @@ void interpretert::build_memory_map()
 {
   // put in a dummy for NULL
   memory.resize(1);
-  inverse_memory_map[0] = ID_null_object;
+  inverse_memory_map[0] = {};
 
   num_dynamic_objects=0;
   dynamic_types.clear();
@@ -895,9 +890,9 @@ void interpretert::build_memory_map(const symbolt &symbol)
   if(size!=0)
   {
     mp_integer address=memory.size();
-    memory.resize(integer2ulong(address+size));
+    memory.resize(numeric_cast_v<std::size_t>(address + size));
     memory_map[symbol.name]=address;
-    inverse_memory_map[address]=symbol.name;
+    inverse_memory_map[address] = symbol.symbol_expr();
   }
 }
 
@@ -929,21 +924,19 @@ typet interpretert::concretize_type(const typet &type)
 
 /// Populates dynamic entries of the memory map
 /// \return Updates the memory map to include variable id if it does not exist
-mp_integer interpretert::build_memory_map(
-  const irep_idt &id,
-  const typet &type)
+mp_integer interpretert::build_memory_map(const symbol_exprt &symbol_expr)
 {
-  typet alloc_type=concretize_type(type);
+  typet alloc_type = concretize_type(symbol_expr.type());
   mp_integer size=get_size(alloc_type);
-  auto it=dynamic_types.find(id);
+  auto it = dynamic_types.find(symbol_expr.get_identifier());
 
   if(it!=dynamic_types.end())
   {
-    mp_integer address=memory_map[id];
+    mp_integer address = memory_map[symbol_expr.get_identifier()];
     mp_integer current_size=base_address_to_alloc_size(address);
     // current size <= size already recorded
     if(size<=current_size)
-      return memory_map[id];
+      return memory_map[symbol_expr.get_identifier()];
   }
 
   // The current size is bigger then the one previously recorded
@@ -953,10 +946,11 @@ mp_integer interpretert::build_memory_map(
     size=1; // This is a hack to create existence
 
   mp_integer address=memory.size();
-  memory.resize(integer2ulong(address+size));
-  memory_map[id]=address;
-  inverse_memory_map[address]=id;
-  dynamic_types.insert(std::pair<const irep_idt, typet>(id, alloc_type));
+  memory.resize(numeric_cast_v<std::size_t>(address + size));
+  memory_map[symbol_expr.get_identifier()] = address;
+  inverse_memory_map[address] = symbol_expr;
+  dynamic_types.insert(
+    std::pair<const irep_idt, typet>(symbol_expr.get_identifier(), alloc_type));
 
   return address;
 }
@@ -1035,18 +1029,17 @@ mp_integer interpretert::get_size(const typet &type)
     {
       // Go via the binary representation to reproduce any
       // overflow behaviour.
-      exprt size_const=from_integer(i[0], size_expr.type());
-      mp_integer size_mp;
-      bool ret=to_integer(size_const, size_mp);
-      CHECK_RETURN(!ret);
+      const constant_exprt size_const = from_integer(i[0], size_expr.type());
+      const mp_integer size_mp = numeric_cast_v<mp_integer>(size_const);
       return subtype_size*size_mp;
     }
     return subtype_size;
   }
-  else if(type.id()==ID_symbol)
+  else if(type.id() == ID_struct_tag)
   {
-    return get_size(ns.follow(type));
+    return get_size(ns.follow_tag(to_struct_tag_type(type)));
   }
+
   return 1;
 }
 
@@ -1076,7 +1069,7 @@ void interpretert::print_memory(bool input_flags)
   {
     mp_integer i=cell_address.first;
     const memory_cellt &cell=cell_address.second;
-    const auto identifier=address_to_identifier(i);
+    const auto identifier = address_to_symbol(i).get_identifier();
     const auto offset=address_to_offset(i);
     debug() << identifier << "[" << offset << "]"
             << "=" << cell.value << eom;
@@ -1097,4 +1090,3 @@ void interpreter(
     message_handler);
   interpreter();
 }
-
