@@ -13,10 +13,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <iostream>
 
-#include <util/xml.h>
-#include <util/xml_expr.h>
-#include <util/json.h>
-#include <util/json_expr.h>
+#include <util/json_irep.h>
+#include <util/xml_irep.h>
 
 void show_loop_ids(
   ui_message_handlert::uit ui,
@@ -27,6 +25,7 @@ void show_loop_ids(
 
 void show_loop_ids(
   ui_message_handlert::uit ui,
+  const irep_idt &function_id,
   const goto_programt &goto_program)
 {
   switch(ui)
@@ -37,10 +36,9 @@ void show_loop_ids(
       {
         if(it->is_backwards_goto())
         {
-          unsigned loop_id=it->loop_number;
-
-          std::cout << "Loop "
-                    << it->function << "." << loop_id << ":" << "\n";
+          std::cout << "Loop " << goto_programt::loop_id(function_id, *it)
+                    << ":"
+                    << "\n";
 
           std::cout << "  " << it->source_location << "\n";
           std::cout << "\n";
@@ -54,11 +52,9 @@ void show_loop_ids(
       {
         if(it->is_backwards_goto())
         {
-          unsigned loop_id=it->loop_number;
-          std::string id=id2string(it->function)+"."+std::to_string(loop_id);
+          std::string id = id2string(goto_programt::loop_id(function_id, *it));
 
-          xmlt xml_loop("loop");
-          xml_loop.set_attribute("name", id);
+          xmlt xml_loop("loop", {{"name", id}}, {});
           xml_loop.new_element("loop-id").data=id;
           xml_loop.new_element()=xml(it->source_location);
           std::cout << xml_loop << "\n";
@@ -73,21 +69,21 @@ void show_loop_ids(
 
 void show_loop_ids_json(
   ui_message_handlert::uit ui,
+  const irep_idt &function_id,
   const goto_programt &goto_program,
   json_arrayt &loops)
 {
-  assert(ui==ui_message_handlert::uit::JSON_UI); // use function above
+  PRECONDITION(ui == ui_message_handlert::uit::JSON_UI); // use function above
 
   forall_goto_program_instructions(it, goto_program)
   {
     if(it->is_backwards_goto())
     {
-      unsigned loop_id=it->loop_number;
-      std::string id=id2string(it->function)+"."+std::to_string(loop_id);
+      std::string id = id2string(goto_programt::loop_id(function_id, *it));
 
-      json_objectt &loop=loops.push_back().make_object();
-      loop["name"]=json_stringt(id);
-      loop["sourceLocation"]=json(it->source_location);
+      loops.push_back(
+        json_objectt({{"name", json_stringt(id)},
+                      {"sourceLocation", json(it->source_location)}}));
     }
   }
 }
@@ -100,15 +96,16 @@ void show_loop_ids(
   {
     case ui_message_handlert::uit::PLAIN:
     case ui_message_handlert::uit::XML_UI:
-      forall_goto_functions(it, goto_functions)
-        show_loop_ids(ui, it->second.body);
+      for(const auto &f: goto_functions.function_map)
+        show_loop_ids(ui, f.first, f.second.body);
       break;
+
     case ui_message_handlert::uit::JSON_UI:
       json_objectt json_result;
       json_arrayt &loops=json_result["loops"].make_array();
 
-      forall_goto_functions(it, goto_functions)
-        show_loop_ids_json(ui, it->second.body, loops);
+      for(const auto &f : goto_functions.function_map)
+        show_loop_ids_json(ui, f.first, f.second.body, loops);
 
       std::cout << ",\n" << json_result;
       break;
