@@ -8,8 +8,11 @@ Date:   September 2015
 
 \*******************************************************************/
 
-#ifndef CPROVER_PARAMETER_ASSIGNMENTS_H
-#define CPROVER_PARAMETER_ASSIGNMENTS_H
+/// \file
+/// Add parameter assignments
+
+#ifndef CPROVER_GOTO_PROGRAMS_PARAMETER_ASSIGNMENTS_H
+#define CPROVER_GOTO_PROGRAMS_PARAMETER_ASSIGNMENTS_H
 
 #include <goto-programs/goto_model.h>
 
@@ -17,4 +20,4 @@ void parameter_assignments(symbol_tablet &, goto_functionst &);
 
 void parameter_assignments(goto_modelt &);
 
-#endif
+#endif // CPROVER_GOTO_PROGRAMS_PARAMETER_ASSIGNMENTS_H
