@@ -14,23 +14,18 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <string>
 
+#include <util/deprecate.h>
+#include <util/optional.h>
+
 class goto_functionst;
 class goto_modelt;
 class message_handlert;
 class symbol_tablet;
 
-bool read_goto_binary(
-  const std::string &filename,
-  symbol_tablet &,
-  goto_functionst &,
-  message_handlert &);
+optionalt<goto_modelt>
+read_goto_binary(const std::string &filename, message_handlert &);
 
-bool read_goto_binary(
-  const std::string &filename,
-  goto_modelt &dest,
-  message_handlert &);
-
-bool is_goto_binary(const std::string &filename);
+bool is_goto_binary(const std::string &filename, message_handlert &);
 
 bool read_object_and_link(
   const std::string &file_name,
