@@ -13,7 +13,6 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <cassert>
 
-#include <util/base_type.h>
 #include <util/c_types.h>
 #include <util/fresh_symbol.h>
 #include <util/invariant.h>
@@ -22,7 +21,6 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <util/replace_expr.h>
 #include <util/source_location.h>
 #include <util/std_expr.h>
-#include <util/type_eq.h>
 
 #include <analyses/does_remove_const.h>
 
@@ -30,7 +28,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "compute_called_functions.h"
 #include "remove_const_function_pointers.h"
 
-class remove_function_pointerst:public messaget
+class remove_function_pointerst
 {
 public:
   remove_function_pointerst(
@@ -42,9 +40,28 @@ public:
 
   void operator()(goto_functionst &goto_functions);
 
-  bool remove_function_pointers(goto_programt &goto_program);
+  bool remove_function_pointers(
+    goto_programt &goto_program,
+    const irep_idt &function_id);
+
+  // a set of function symbols
+  using functionst = remove_const_function_pointerst::functionst;
+
+  /// Replace a call to a dynamic function at location
+  /// target in the given goto-program by a case-split
+  /// over a given set of functions
+  /// \param goto_program: The goto program that contains target
+  /// \param function_id: Name of function containing the target
+  /// \param target: location with function call with function pointer
+  /// \param functions: The set of functions to consider
+  void remove_function_pointer(
+    goto_programt &goto_program,
+    const irep_idt &function_id,
+    goto_programt::targett target,
+    const functionst &functions);
 
 protected:
+  messaget log;
   const namespacet ns;
   symbol_tablet &symbol_table;
   bool add_safety_assertion;
@@ -57,8 +74,15 @@ protected:
   // --remove-const-function-pointers instead of --remove-function-pointers
   bool only_resolve_const_fps;
 
+  /// Replace a call to a dynamic function at location
+  /// target in the given goto-program by determining
+  /// functions that have a compatible signature
+  /// \param goto_program: The goto program that contains target
+  /// \param function_id: Name of function containing the target
+  /// \param target: location with function call with function pointer
   void remove_function_pointer(
     goto_programt &goto_program,
+    const irep_idt &function_id,
     goto_programt::targett target);
 
   std::unordered_set<irep_idt> address_taken;
@@ -77,68 +101,57 @@ protected:
 
   void fix_argument_types(code_function_callt &function_call);
   void fix_return_type(
+    const irep_idt &in_function_id,
     code_function_callt &function_call,
     goto_programt &dest);
-
-  void
-  compute_address_taken_in_symbols(std::unordered_set<irep_idt> &address_taken)
-  {
-    const symbol_tablet &symbol_table=ns.get_symbol_table();
-
-    for(const auto &s : symbol_table.symbols)
-      compute_address_taken_functions(s.second.value, address_taken);
-  }
 };
 
 remove_function_pointerst::remove_function_pointerst(
   message_handlert &_message_handler,
   symbol_tablet &_symbol_table,
-  bool _add_safety_assertion, bool only_resolve_const_fps,
-  const goto_functionst &goto_functions):
-  messaget(_message_handler),
-  ns(_symbol_table),
-  symbol_table(_symbol_table),
-  add_safety_assertion(_add_safety_assertion),
-  only_resolve_const_fps(only_resolve_const_fps)
+  bool _add_safety_assertion,
+  bool only_resolve_const_fps,
+  const goto_functionst &goto_functions)
+  : log(_message_handler),
+    ns(_symbol_table),
+    symbol_table(_symbol_table),
+    add_safety_assertion(_add_safety_assertion),
+    only_resolve_const_fps(only_resolve_const_fps)
 {
-  compute_address_taken_in_symbols(address_taken);
+  for(const auto &s : symbol_table.symbols)
+    compute_address_taken_functions(s.second.value, address_taken);
+
   compute_address_taken_functions(goto_functions, address_taken);
 
   // build type map
   forall_goto_functions(f_it, goto_functions)
-    type_map[f_it->first]=f_it->second.type;
+    type_map.emplace(f_it->first, f_it->second.type);
 }
 
 bool remove_function_pointerst::arg_is_type_compatible(
   const typet &call_type,
   const typet &function_type)
 {
-  if(type_eq(call_type, function_type, ns))
+  if(call_type == function_type)
     return true;
 
   // any integer-vs-enum-vs-pointer is ok
-  if(call_type.id()==ID_signedbv ||
-     call_type.id()==ID_unsigned ||
-     call_type.id()==ID_bool ||
-     call_type.id()==ID_pointer ||
-     call_type.id()==ID_c_enum ||
-     call_type.id()==ID_c_enum_tag)
+  if(
+    call_type.id() == ID_signedbv || call_type.id() == ID_unsigned ||
+    call_type.id() == ID_bool || call_type.id() == ID_c_bool ||
+    call_type.id() == ID_c_enum_tag || call_type.id() == ID_c_enum ||
+    call_type.id() == ID_pointer)
   {
-    if(function_type.id()==ID_signedbv ||
-       function_type.id()==ID_unsigned ||
-       function_type.id()==ID_bool ||
-       function_type.id()==ID_pointer ||
-       function_type.id()==ID_c_enum ||
-       function_type.id()==ID_c_enum_tag)
-      return true;
-
-    return false;
+    return function_type.id() == ID_signedbv ||
+           function_type.id() == ID_unsigned || function_type.id() == ID_bool ||
+           function_type.id() == ID_c_bool ||
+           function_type.id() == ID_pointer ||
+           function_type.id() == ID_c_enum ||
+           function_type.id() == ID_c_enum_tag;
   }
 
   return pointer_offset_bits(call_type, ns) ==
          pointer_offset_bits(function_type, ns);
-
-  return false;
 }
 
 bool remove_function_pointerst::is_type_compatible(
@@ -151,7 +164,7 @@ bool remove_function_pointerst::is_type_compatible(
   if(!return_value_used)
   {
   }
-  else if(type_eq(call_type.return_type(), empty_typet(), ns))
+  else if(call_type.return_type() == empty_typet())
   {
     // ok
   }
@@ -194,8 +207,7 @@ bool remove_function_pointerst::is_type_compatible(
 void remove_function_pointerst::fix_argument_types(
   code_function_callt &function_call)
 {
-  const code_typet &code_type=
-    to_code_type(ns.follow(function_call.function().type()));
+  const code_typet &code_type = to_code_type(function_call.function().type());
 
   const code_typet::parameterst &function_parameters=
     code_type.parameters();
@@ -207,16 +219,17 @@ void remove_function_pointerst::fix_argument_types(
   {
     if(i<call_arguments.size())
     {
-      if(!type_eq(call_arguments[i].type(),
-                  function_parameters[i].type(), ns))
+      if(call_arguments[i].type() != function_parameters[i].type())
       {
-        call_arguments[i].make_typecast(function_parameters[i].type());
+        call_arguments[i] =
+          typecast_exprt(call_arguments[i], function_parameters[i].type());
       }
     }
   }
 }
 
 void remove_function_pointerst::fix_return_type(
+  const irep_idt &in_function_id,
   code_function_callt &function_call,
   goto_programt &dest)
 {
@@ -224,13 +237,10 @@ void remove_function_pointerst::fix_return_type(
   if(function_call.lhs().is_nil())
     return;
 
-  const code_typet &code_type=
-    to_code_type(ns.follow(function_call.function().type()));
+  const code_typet &code_type = to_code_type(function_call.function().type());
 
   // type already ok?
-  if(type_eq(
-       function_call.lhs().type(),
-       code_type.return_type(), ns))
+  if(function_call.lhs().type() == code_type.return_type())
     return;
 
   const symbolt &function_symbol =
@@ -238,7 +248,7 @@ void remove_function_pointerst::fix_return_type(
 
   symbolt &tmp_symbol = get_fresh_aux_symbol(
     code_type.return_type(),
-    id2string(function_call.source_location().get_function()),
+    id2string(in_function_id),
     "tmp_return_val_" + id2string(function_symbol.base_name),
     function_call.source_location(),
     function_symbol.mode,
@@ -249,20 +259,18 @@ void remove_function_pointerst::fix_return_type(
   exprt old_lhs=function_call.lhs();
   function_call.lhs()=tmp_symbol_expr;
 
-  goto_programt::targett t_assign=dest.add_instruction();
-  t_assign->make_assignment();
-  t_assign->code=code_assignt(
-    old_lhs, typecast_exprt(tmp_symbol_expr, old_lhs.type()));
+  dest.add(goto_programt::make_assignment(
+    code_assignt(old_lhs, typecast_exprt(tmp_symbol_expr, old_lhs.type()))));
 }
 
 void remove_function_pointerst::remove_function_pointer(
   goto_programt &goto_program,
+  const irep_idt &function_id,
   goto_programt::targett target)
 {
-  const code_function_callt &code=
-    to_code_function_call(target->code);
+  const code_function_callt &code = target->get_function_call();
 
-  const exprt &function=code.function();
+  const auto &function = to_dereference_expr(code.function());
 
   // this better have the right type
   code_typet call_type=to_code_type(function.type());
@@ -277,26 +285,24 @@ void remove_function_pointerst::remove_function_pointer(
         code_typet::parametert(it->type()));
   }
 
-  assert(function.id()==ID_dereference);
-  assert(function.operands().size()==1);
-
   bool found_functions;
 
-  const exprt &pointer=function.op0();
+  const exprt &pointer = function.pointer();
   remove_const_function_pointerst::functionst functions;
   does_remove_constt const_removal_check(goto_program, ns);
   const auto does_remove_const = const_removal_check();
   if(does_remove_const.first)
   {
-    warning().source_location = does_remove_const.second;
-    warning() << "cast from const to non-const pointer found, only worst case"
-              << " function pointer removal will be done." << eom;
+    log.warning().source_location = does_remove_const.second;
+    log.warning() << "cast from const to non-const pointer found, "
+                  << "only worst case function pointer removal will be done."
+                  << messaget::eom;
     found_functions=false;
   }
   else
   {
     remove_const_function_pointerst fpr(
-    get_message_handler(), ns, symbol_table);
+      log.get_message_handler(), ns, symbol_table);
 
     found_functions=fpr(pointer, functions);
 
@@ -308,7 +314,9 @@ void remove_function_pointerst::remove_function_pointer(
 
     if(functions.size()==1)
     {
-      to_code_function_call(target->code).function()=*functions.cbegin();
+      auto call = target->get_function_call();
+      call.function() = *functions.cbegin();
+      target->set_function_call(call);
       return;
     }
   }
@@ -348,11 +356,24 @@ void remove_function_pointerst::remove_function_pointer(
     }
   }
 
+  remove_function_pointer(goto_program, function_id, target, functions);
+}
+
+void remove_function_pointerst::remove_function_pointer(
+  goto_programt &goto_program,
+  const irep_idt &function_id,
+  goto_programt::targett target,
+  const functionst &functions)
+{
+  const code_function_callt &code = target->get_function_call();
+
+  const exprt &function = code.function();
+  const exprt &pointer = to_dereference_expr(function).pointer();
+
   // the final target is a skip
   goto_programt final_skip;
 
-  goto_programt::targett t_final=final_skip.add_instruction();
-  t_final->make_skip();
+  goto_programt::targett t_final = final_skip.add(goto_programt::make_skip());
 
   // build the calls and gotos
 
@@ -362,36 +383,40 @@ void remove_function_pointerst::remove_function_pointer(
   for(const auto &fun : functions)
   {
     // call function
-    goto_programt::targett t1=new_code_calls.add_instruction();
-    t1->make_function_call(code);
-    to_code_function_call(t1->code).function()=fun;
+    auto new_call = code;
+    new_call.function() = fun;
 
     // the signature of the function might not match precisely
-    fix_argument_types(to_code_function_call(t1->code));
+    fix_argument_types(new_call);
+
+    goto_programt tmp;
+    fix_return_type(function_id, new_call, tmp);
+
+    auto call = new_code_calls.add(goto_programt::make_function_call(new_call));
+    new_code_calls.destructive_append(tmp);
 
-    fix_return_type(to_code_function_call(t1->code), new_code_calls);
     // goto final
-    goto_programt::targett t3=new_code_calls.add_instruction();
-    t3->make_goto(t_final, true_exprt());
+    new_code_calls.add(goto_programt::make_goto(t_final, true_exprt()));
 
     // goto to call
-    address_of_exprt address_of(fun, pointer_type(fun.type()));
+    const address_of_exprt address_of(fun, pointer_type(fun.type()));
 
-    if(address_of.type()!=pointer.type())
-      address_of.make_typecast(pointer.type());
+    const auto casted_address =
+      typecast_exprt::conditional_cast(address_of, pointer.type());
 
-    goto_programt::targett t4=new_code_gotos.add_instruction();
-    t4->make_goto(t1, equal_exprt(pointer, address_of));
+    new_code_gotos.add(
+      goto_programt::make_goto(call, equal_exprt(pointer, casted_address)));
   }
 
   // fall-through
   if(add_safety_assertion)
   {
-    goto_programt::targett t=new_code_gotos.add_instruction();
-    t->make_assertion(false_exprt());
+    goto_programt::targett t =
+      new_code_gotos.add(goto_programt::make_assertion(false_exprt()));
     t->source_location.set_property_class("pointer dereference");
     t->source_location.set_comment("invalid function pointer");
   }
+  new_code_gotos.add(goto_programt::make_assumption(false_exprt()));
 
   goto_programt new_code;
 
@@ -406,7 +431,6 @@ void remove_function_pointerst::remove_function_pointer(
     irep_idt property_class=it->source_location.get_property_class();
     irep_idt comment=it->source_location.get_comment();
     it->source_location=target->source_location;
-    it->function=target->function;
     if(!property_class.empty())
       it->source_location.set_property_class(property_class);
     if(!comment.empty())
@@ -426,14 +450,13 @@ void remove_function_pointerst::remove_function_pointer(
   target->type=OTHER;
 
   // report statistics
-  statistics().source_location=target->source_location;
-  statistics() << "replacing function pointer by "
-               << functions.size() << " possible targets" << eom;
+  log.statistics().source_location = target->source_location;
+  log.statistics() << "replacing function pointer by " << functions.size()
+                   << " possible targets" << messaget::eom;
 
   // list the names of functions when verbosity is at debug level
-  conditional_output(
-    debug(),
-    [&functions](mstreamt &mstream) {
+  log.conditional_output(
+    log.debug(), [&functions](messaget::mstreamt &mstream) {
       mstream << "targets: ";
 
       bool first = true;
@@ -446,24 +469,24 @@ void remove_function_pointerst::remove_function_pointer(
         first = false;
       }
 
-      mstream << eom;
+      mstream << messaget::eom;
     });
 }
 
 bool remove_function_pointerst::remove_function_pointers(
-  goto_programt &goto_program)
+  goto_programt &goto_program,
+  const irep_idt &function_id)
 {
   bool did_something=false;
 
   Forall_goto_program_instructions(target, goto_program)
     if(target->is_function_call())
     {
-      const code_function_callt &code=
-        to_code_function_call(target->code);
+      const code_function_callt &code = target->get_function_call();
 
       if(code.function().id()==ID_dereference)
       {
-        remove_function_pointer(goto_program, target);
+        remove_function_pointer(goto_program, function_id, target);
         did_something=true;
       }
     }
@@ -485,7 +508,7 @@ void remove_function_pointerst::operator()(goto_functionst &functions)
   {
     goto_programt &goto_program=f_it->second.body;
 
-    if(remove_function_pointers(goto_program))
+    if(remove_function_pointers(goto_program, f_it->first))
       did_something=true;
   }
 
@@ -493,10 +516,12 @@ void remove_function_pointerst::operator()(goto_functionst &functions)
     functions.compute_location_numbers();
 }
 
-bool remove_function_pointers(message_handlert &_message_handler,
+bool remove_function_pointers(
+  message_handlert &_message_handler,
   symbol_tablet &symbol_table,
   const goto_functionst &goto_functions,
   goto_programt &goto_program,
+  const irep_idt &function_id,
   bool add_safety_assertion,
   bool only_remove_const_fps)
 {
@@ -508,7 +533,7 @@ bool remove_function_pointers(message_handlert &_message_handler,
       only_remove_const_fps,
       goto_functions);
 
-  return rfp.remove_function_pointers(goto_program);
+  return rfp.remove_function_pointers(goto_program, function_id);
 }
 
 void remove_function_pointers(
