@@ -6,6 +6,9 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
+/// \file
+/// Set the properties to check
+
 #ifndef CPROVER_GOTO_PROGRAMS_SET_PROPERTIES_H
 #define CPROVER_GOTO_PROGRAMS_SET_PROPERTIES_H
 
@@ -19,11 +22,11 @@ void set_properties(
   goto_modelt &goto_model,
   const std::list<std::string> &properties);
 
-void make_assertions_false(goto_functionst &);  
-void make_assertions_false(goto_modelt &);  
+void make_assertions_false(goto_functionst &);
+void make_assertions_false(goto_modelt &);
 
 void label_properties(goto_functionst &);
 void label_properties(goto_programt &);
 void label_properties(goto_modelt &);
 
-#endif
+#endif // CPROVER_GOTO_PROGRAMS_SET_PROPERTIES_H
