@@ -11,12 +11,8 @@ Author: Peter Schrammel
 
 #include "show_goto_functions.h"
 
-#include <iostream>
-
 #include <util/xml.h>
 #include <util/json.h>
-#include <util/json_expr.h>
-#include <util/xml_expr.h>
 #include <util/cprover_prefix.h>
 #include <util/prefix.h>
 
@@ -29,12 +25,12 @@ Author: Peter Schrammel
 
 void show_goto_functions(
   const namespacet &ns,
-  message_handlert &message_handler,
-  ui_message_handlert::uit ui,
+  ui_message_handlert &ui_message_handler,
   const goto_functionst &goto_functions,
   bool list_only)
 {
-  messaget msg(message_handler);
+  ui_message_handlert::uit ui = ui_message_handler.get_ui();
+  messaget msg(ui_message_handler);
   switch(ui)
   {
   case ui_message_handlert::uit::XML_UI:
@@ -52,24 +48,32 @@ void show_goto_functions(
   break;
 
   case ui_message_handlert::uit::PLAIN:
-    if(list_only)
     {
-      for(const auto &fun : goto_functions.function_map)
-      {
-        const symbolt &symbol = ns.lookup(fun.first);
-        msg.status() << '\n'
-                     << symbol.display_name() << " /* " << symbol.name
-                     << (fun.second.body_available() ? ""
-                                                     : ", body not available")
-                     << " */";
-      }
+      // sort alphabetically
+      const auto sorted = goto_functions.sorted();
 
-      msg.status() << messaget::eom;
-    }
-    else
-    {
-      goto_functions.output(ns, msg.status());
-      msg.status() << messaget::eom;
+      for(const auto &fun : sorted)
+      {
+        const symbolt &symbol = ns.lookup(fun->first);
+        const bool has_body = fun->second.body_available();
+
+        if(list_only)
+        {
+          msg.status() << '\n'
+                       << symbol.display_name() << " /* " << symbol.name
+                       << (has_body ? "" : ", body not available") << " */";
+          msg.status() << messaget::eom;
+        }
+        else if(has_body)
+        {
+          msg.status() << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n\n";
+
+          msg.status() << messaget::bold << symbol.display_name()
+                       << messaget::reset << " /* " << symbol.name << " */\n";
+          fun->second.body.output(ns, symbol.name, msg.status());
+          msg.status() << messaget::eom;
+        }
+      }
     }
 
     break;
@@ -78,11 +82,10 @@ void show_goto_functions(
 
 void show_goto_functions(
   const goto_modelt &goto_model,
-  message_handlert &message_handler,
-  ui_message_handlert::uit ui,
+  ui_message_handlert &ui_message_handler,
   bool list_only)
 {
   const namespacet ns(goto_model.symbol_table);
   show_goto_functions(
-    ns, message_handler, ui, goto_model.goto_functions, list_only);
+    ns, ui_message_handler, goto_model.goto_functions, list_only);
 }
