@@ -13,10 +13,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <iostream>
 
-#include <util/xml.h>
-#include <util/xml_expr.h>
-#include <util/json.h>
-#include <util/json_expr.h>
+#include <util/json_irep.h>
+#include <util/xml_irep.h>
 
 #include <langapi/language_util.h>
 
@@ -63,8 +61,7 @@ void show_properties(
 
     const irep_idt &comment=source_location.get_comment();
     const irep_idt &property_class=source_location.get_property_class();
-    const irep_idt description=
-      (comment==""?"assertion":comment);
+    const irep_idt description = (comment.empty() ? "assertion" : comment);
 
     irep_idt property_id=source_location.get_property_id();
 
@@ -73,16 +70,18 @@ void show_properties(
     case ui_message_handlert::uit::XML_UI:
       {
         // use me instead
-        xmlt xml_property("property");
-        xml_property.set_attribute("name", id2string(property_id));
-        xml_property.set_attribute("class", id2string(property_class));
+        xmlt xml_property(
+          "property",
+          {{"name", id2string(property_id)},
+           {"class", id2string(property_class)}},
+          {});
 
         xmlt &property_l=xml_property.new_element();
         property_l=xml(source_location);
 
         xml_property.new_element("description").data=id2string(description);
-        xml_property.new_element("expression").data=
-          from_expr(ns, identifier, ins.guard);
+        xml_property.new_element("expression").data =
+          from_expr(ns, identifier, ins.get_condition());
 
         msg.result() << xml_property;
       }
@@ -97,7 +96,8 @@ void show_properties(
 
       msg.result() << "  " << ins.source_location << '\n'
                    << "  " << description << '\n'
-                   << "  " << from_expr(ns, identifier, ins.guard) << '\n';
+                   << "  " << from_expr(ns, identifier, ins.get_condition())
+                   << '\n';
 
       msg.result() << messaget::eom;
       break;
@@ -124,22 +124,23 @@ void convert_properties_json(
     const irep_idt &comment=source_location.get_comment();
     // const irep_idt &function=location.get_function();
     const irep_idt &property_class=source_location.get_property_class();
-    const irep_idt description=
-      (comment==""?"assertion":comment);
+    const irep_idt description = (comment.empty() ? "assertion" : comment);
 
     irep_idt property_id=source_location.get_property_id();
 
-    json_objectt &json_property=
-      json_properties.push_back(jsont()).make_object();
-    json_property["name"] = json_stringt(property_id);
-    json_property["class"] = json_stringt(property_class);
+    json_objectt json_property{
+      {"name", json_stringt(property_id)},
+      {"class", json_stringt(property_class)},
+      {"sourceLocation", json(source_location)},
+      {"description", json_stringt(description)},
+      {"expression",
+       json_stringt(from_expr(ns, identifier, ins.get_condition()))}};
+
     if(!source_location.get_basic_block_covered_lines().empty())
       json_property["coveredLines"] =
         json_stringt(source_location.get_basic_block_covered_lines());
-    json_property["sourceLocation"]=json(source_location);
-    json_property["description"] = json_stringt(description);
-    json_property["expression"]=
-      json_stringt(from_expr(ns, identifier, ins.guard));
+
+    json_properties.push_back(std::move(json_property));
   }
 }
 
@@ -152,36 +153,33 @@ void show_properties_json(
   json_arrayt json_properties;
 
   for(const auto &fct : goto_functions.function_map)
-    if(!fct.second.is_inlined())
-      convert_properties_json(json_properties, ns, fct.first, fct.second.body);
+    convert_properties_json(json_properties, ns, fct.first, fct.second.body);
 
-  json_objectt json_result;
-  json_result["properties"] = json_properties;
+  json_objectt json_result{{"properties", json_properties}};
   msg.result() << json_result;
 }
 
 void show_properties(
   const namespacet &ns,
-  message_handlert &message_handler,
-  ui_message_handlert::uit ui,
+  ui_message_handlert &ui_message_handler,
   const goto_functionst &goto_functions)
 {
+  ui_message_handlert::uit ui = ui_message_handler.get_ui();
   if(ui == ui_message_handlert::uit::JSON_UI)
-    show_properties_json(ns, message_handler, goto_functions);
+    show_properties_json(ns, ui_message_handler, goto_functions);
   else
     for(const auto &fct : goto_functions.function_map)
-      if(!fct.second.is_inlined())
-        show_properties(ns, fct.first, message_handler, ui, fct.second.body);
+      show_properties(ns, fct.first, ui_message_handler, ui, fct.second.body);
 }
 
 void show_properties(
   const goto_modelt &goto_model,
-  message_handlert &message_handler,
-  ui_message_handlert::uit ui)
+  ui_message_handlert &ui_message_handler)
 {
+  ui_message_handlert::uit ui = ui_message_handler.get_ui();
   const namespacet ns(goto_model.symbol_table);
   if(ui == ui_message_handlert::uit::JSON_UI)
-    show_properties_json(ns, message_handler, goto_model.goto_functions);
+    show_properties_json(ns, ui_message_handler, goto_model.goto_functions);
   else
-    show_properties(ns, message_handler, ui, goto_model.goto_functions);
+    show_properties(ns, ui_message_handler, goto_model.goto_functions);
 }
