@@ -8,6 +8,9 @@ Date: June 2011
 
 \*******************************************************************/
 
+/// \file
+/// Traces of GOTO Programs in VCD (Value Change Dump) Format
+
 #ifndef CPROVER_GOTO_PROGRAMS_VCD_GOTO_TRACE_H
 #define CPROVER_GOTO_PROGRAMS_VCD_GOTO_TRACE_H
 
@@ -22,4 +25,4 @@ void output_vcd(
   const goto_tracet &goto_trace,
   std::ostream &out);
 
-#endif
+#endif // CPROVER_GOTO_PROGRAMS_VCD_GOTO_TRACE_H
