@@ -8,6 +8,9 @@ Date: November 2005
 
 \*******************************************************************/
 
+/// \file
+/// Traces of GOTO Programs
+
 #ifndef CPROVER_GOTO_PROGRAMS_XML_GOTO_TRACE_H
 #define CPROVER_GOTO_PROGRAMS_XML_GOTO_TRACE_H
 
@@ -20,4 +23,4 @@ void convert(
   const goto_tracet &goto_trace,
   xmlt &xml);
 
-#endif
+#endif // CPROVER_GOTO_PROGRAMS_XML_GOTO_TRACE_H
