@@ -11,37 +11,41 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include "goto_symex.h"
 
+#include <util/exception_utils.h>
 #include <util/expr_initializer.h>
 
+#include "expr_skeleton.h"
+#include "symex_assign.h"
+
 void goto_symext::symex_start_thread(statet &state)
 {
-  if(state.guard.is_false())
+  if(!state.reachable)
     return;
 
-  // we don't allow spawning threads out of atomic sections
-  // this would require amendments to ordering constraints
-  if(state.atomic_section_id!=0)
-    throw "start_thread in atomic section detected";
+  if(state.atomic_section_id != 0)
+    throw incorrect_goto_program_exceptiont(
+      "spawning threads out of atomic sections is not allowed; "
+      "this would require amendments to ordering constraints",
+      state.source.pc->source_location);
 
   // record this
   target.spawn(state.guard.as_expr(), state.source);
 
   const goto_programt::instructiont &instruction=*state.source.pc;
 
-  if(instruction.targets.size()!=1)
-    throw "start_thread expects one target";
+  INVARIANT(instruction.targets.size() == 1, "start_thread expects one target");
 
   goto_programt::const_targett thread_target=
     instruction.get_target();
 
   // put into thread vector
   std::size_t t=state.threads.size();
-  state.threads.push_back(statet::threadt());
+  state.threads.push_back(statet::threadt(guard_manager));
   // statet::threadt &cur_thread=state.threads[state.source.thread_nr];
   statet::threadt &new_thread=state.threads.back();
   new_thread.pc=thread_target;
   new_thread.guard=state.guard;
-  new_thread.call_stack.push_back(state.top());
+  new_thread.call_stack.push_back(state.call_stack().top());
   new_thread.call_stack.back().local_objects.clear();
   new_thread.call_stack.back().goto_state_map.clear();
   #if 0
@@ -49,49 +53,45 @@ void goto_symext::symex_start_thread(statet &state)
   #endif
 
   // create a copy of the local variables for the new thread
-  statet::framet &frame=state.top();
+  framet &frame = state.call_stack().top();
 
-  for(goto_symex_statet::renaming_levelt::current_namest::const_iterator
-      c_it=state.level2.current_names.begin();
-      c_it!=state.level2.current_names.end();
-      ++c_it)
+  symex_renaming_levelt::viewt view;
+  state.get_level2().current_names.get_view(view);
+
+  for(const auto &pair : view)
   {
-    const irep_idt l1_o_id=c_it->second.first.get_l1_object_identifier();
+    const irep_idt l1_o_id = pair.second.first.get_l1_object_identifier();
+
     // could use iteration over local_objects as l1_o_id is prefix
     if(frame.local_objects.find(l1_o_id)==frame.local_objects.end())
       continue;
 
     // get original name
-    ssa_exprt lhs(c_it->second.first.get_original_expr());
+    ssa_exprt lhs(pair.second.first.get_original_expr());
 
     // get L0 name for current thread
-    lhs.set_level_0(t);
+    const renamedt<ssa_exprt, L0> l0_lhs = symex_level0(std::move(lhs), ns, t);
+    const irep_idt &l0_name = l0_lhs.get().get_identifier();
+    std::size_t l1_index = path_storage.get_unique_l1_index(l0_name, 0);
+    CHECK_RETURN(l1_index == 0);
 
     // set up L1 name
-    if(!state.level1.current_names.insert(
-        std::make_pair(lhs.get_l1_object_identifier(),
-                       std::make_pair(lhs, 0))).second)
-      UNREACHABLE;
-    state.rename(lhs, ns, goto_symex_statet::L1);
-    const irep_idt l1_name=lhs.get_l1_object_identifier();
+    state.level1.insert(l0_lhs, 0);
+
+    const ssa_exprt lhs_l1 = state.rename_ssa<L1>(l0_lhs.get(), ns).get();
+    const irep_idt l1_name = lhs_l1.get_l1_object_identifier();
     // store it
-    state.l1_history.insert(l1_name);
     new_thread.call_stack.back().local_objects.insert(l1_name);
 
     // make copy
-    ssa_exprt rhs=c_it->second.first;
+    ssa_exprt rhs = pair.second.first;
 
-    guardt guard;
-    const bool record_events=state.record_events;
-    state.record_events=false;
-    symex_assign_symbol(
-      state,
-      lhs,
-      nil_exprt(),
-      rhs,
-      guard,
-      symex_targett::assignment_typet::HIDDEN);
-    state.record_events=record_events;
+    exprt::operandst lhs_conditions;
+    state.record_events.push(false);
+    symex_assignt{
+      state, symex_targett::assignment_typet::HIDDEN, ns, symex_config, target}
+      .assign_symbol(lhs_l1, expr_skeletont{}, rhs, lhs_conditions);
+    state.record_events.pop();
   }
 
   // initialize all variables marked thread-local
@@ -114,15 +114,15 @@ void goto_symext::symex_start_thread(statet &state)
 
     exprt rhs=symbol.value;
     if(rhs.is_nil())
-      rhs=zero_initializer(symbol.type, symbol.location, ns);
+    {
+      const auto zero = zero_initializer(symbol.type, symbol.location, ns);
+      CHECK_RETURN(zero.has_value());
+      rhs = *zero;
+    }
 
-    guardt guard;
-    symex_assign_symbol(
-      state,
-      lhs,
-      nil_exprt(),
-      rhs,
-      guard,
-      symex_targett::assignment_typet::HIDDEN);
+    exprt::operandst lhs_conditions;
+    symex_assignt{
+      state, symex_targett::assignment_typet::HIDDEN, ns, symex_config, target}
+      .assign_symbol(lhs, expr_skeletont{}, rhs, lhs_conditions);
   }
 }
