@@ -80,19 +80,18 @@ bool jsil_convertt::convert_code(const symbolt &symbol, codet &code)
 
     if(a.rhs().id()==ID_with)
     {
-      exprt to_try=a.rhs().op0();
+      exprt to_try = to_with_expr(a.rhs()).old();
       codet t(code_assignt(a.lhs(), to_try));
       if(convert_code(symbol, t))
         return true;
 
-      irep_idt c_target=
-        to_symbol_expr(a.rhs().op1()).get_identifier();
+      irep_idt c_target =
+        to_symbol_expr(to_with_expr(a.rhs()).where()).get_identifier();
       code_gotot g(c_target);
 
-      code_try_catcht t_c;
-      t_c.try_code().swap(t);
+      code_try_catcht t_c(std::move(t));
       // Adding empty symbol to catch decl
-      code_declt d(symbol_exprt("decl_symbol"));
+      code_declt d(symbol_exprt::typeless("decl_symbol"));
       t_c.add_catch(d, g);
       t_c.add_source_location()=code.source_location();
 
@@ -104,10 +103,7 @@ bool jsil_convertt::convert_code(const symbolt &symbol, codet &code)
       side_effect_expr_function_callt f_expr=
         to_side_effect_expr_function_call(a.rhs());
 
-      code_function_callt f;
-      f.lhs().swap(a.lhs());
-      f.function().swap(f_expr.function());
-      f.arguments().swap(f_expr.arguments());
+      code_function_callt f(a.lhs(), f_expr.function(), f_expr.arguments());
       f.add_source_location()=code.source_location();
 
       code.swap(f);
