@@ -17,6 +17,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <util/base_type.h>
 #include <util/find_symbols.h>
+#include <util/mathematical_types.h>
 #include <util/pointer_offset_size.h>
 #include <util/simplify_expr.h>
 
@@ -24,8 +25,22 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include "linking_class.h"
 
+bool casting_replace_symbolt::replace_symbol_expr(symbol_exprt &s) const
+{
+  expr_mapt::const_iterator it = expr_map.find(s.get_identifier());
+
+  if(it == expr_map.end())
+    return true;
+
+  const exprt &e = it->second;
+
+  typet type = s.type();
+  static_cast<exprt &>(s) = typecast_exprt::conditional_cast(e, type);
+
+  return false;
+}
+
 std::string linkingt::expr_to_string(
-  const namespacet &ns,
   const irep_idt &identifier,
   const exprt &expr) const
 {
@@ -33,7 +48,6 @@ std::string linkingt::expr_to_string(
 }
 
 std::string linkingt::type_to_string(
-  const namespacet &ns,
   const irep_idt &identifier,
   const typet &type) const
 {
@@ -44,9 +58,7 @@ static const typet &follow_tags_symbols(
   const namespacet &ns,
   const typet &type)
 {
-  if(type.id()==ID_symbol)
-    return ns.follow(type);
-  else if(type.id()==ID_c_enum_tag)
+  if(type.id() == ID_c_enum_tag)
     return ns.follow_tag(to_c_enum_tag_type(type));
   else if(type.id()==ID_struct_tag)
     return ns.follow_tag(to_struct_tag_type(type));
@@ -57,7 +69,6 @@ static const typet &follow_tags_symbols(
 }
 
 std::string linkingt::type_to_string_verbose(
-  const namespacet &ns,
   const symbolt &symbol,
   const typet &type) const
 {
@@ -68,46 +79,43 @@ std::string linkingt::type_to_string_verbose(
     std::string result=followed.id_string();
 
     const std::string &tag=followed.get_string(ID_tag);
-    if(tag!="")
+    if(!tag.empty())
       result+=" "+tag;
-    result+=" {\n";
 
-    const struct_union_typet::componentst &components=
-      to_struct_union_type(followed).components();
-
-    for(struct_union_typet::componentst::const_iterator
-        it=components.begin();
-        it!=components.end();
-        it++)
+    if(to_struct_union_type(followed).is_incomplete())
     {
-      const typet &subtype=it->type();
-      result+="  ";
-      result+=type_to_string(ns, symbol.name, subtype);
-      result+=' ';
-
-      if(it->get_base_name()!="")
-        result+=id2string(it->get_base_name());
-      else
-        result+=id2string(it->get_name());
-
-      result+=";\n";
+      result += "   (incomplete)";
     }
+    else
+    {
+      result += " {\n";
 
-    result+='}';
+      for(const auto &c : to_struct_union_type(followed).components())
+      {
+        const typet &subtype = c.type();
+        result += "  ";
+        result += type_to_string(symbol.name, subtype);
+        result += ' ';
+
+        if(!c.get_base_name().empty())
+          result += id2string(c.get_base_name());
+        else
+          result += id2string(c.get_name());
+
+        result += ";\n";
+      }
+
+      result += '}';
+    }
 
     return result;
   }
   else if(followed.id()==ID_pointer)
   {
-    return type_to_string_verbose(ns, symbol, followed.subtype())+" *";
-  }
-  else if(followed.id()==ID_incomplete_struct ||
-          followed.id()==ID_incomplete_union)
-  {
-    return type_to_string(ns, symbol.name, type)+"   (incomplete)";
+    return type_to_string_verbose(symbol, followed.subtype()) + " *";
   }
 
-  return type_to_string(ns, symbol.name, type);
+  return type_to_string(symbol.name, type);
 }
 
 void linkingt::detailed_conflict_report_rec(
@@ -135,7 +143,8 @@ void linkingt::detailed_conflict_report_rec(
     if(depth>0 &&
        !base_type_eq(t1.subtype(), t2.subtype(), ns))
     {
-      conflict_path=dereference_exprt(conflict_path);
+      if(conflict_path.type().id() == ID_pointer)
+        conflict_path = dereference_exprt(conflict_path);
 
       detailed_conflict_report_rec(
         old_symbol,
@@ -218,7 +227,7 @@ void linkingt::detailed_conflict_report_rec(
             {
               std::string msg_bak;
               msg_bak.swap(msg);
-              symbol_exprt c(ID_C_this);
+              symbol_exprt c = symbol_exprt::typeless(ID_C_this);
               detailed_conflict_report_rec(
                 old_symbol,
                 new_symbol,
@@ -247,8 +256,8 @@ void linkingt::detailed_conflict_report_rec(
     if(t1.subtype()!=t2.subtype())
     {
       msg="enum value types are different (";
-      msg+=type_to_string(ns, old_symbol.name, t1.subtype())+'/';
-      msg+=type_to_string(ns, new_symbol.name, t2.subtype())+')';
+      msg += type_to_string(old_symbol.name, t1.subtype()) + '/';
+      msg += type_to_string(new_symbol.name, t2.subtype()) + ')';
     }
     else if(members1.size()!=members2.size())
     {
@@ -351,12 +360,11 @@ void linkingt::detailed_conflict_report_rec(
   {
     error() << '\n';
     error() << "reason for conflict at "
-            << expr_to_string(ns, "", conflict_path)
-            << ": " << msg << '\n';
+            << expr_to_string(irep_idt(), conflict_path) << ": " << msg << '\n';
 
     error() << '\n';
-    error() << type_to_string_verbose(ns, old_symbol, t1) << '\n';
-    error() << type_to_string_verbose(ns, new_symbol, t2) << '\n';
+    error() << type_to_string_verbose(old_symbol, t1) << '\n';
+    error() << type_to_string_verbose(new_symbol, t2) << '\n';
   }
 
   #ifdef DEBUG
@@ -371,15 +379,14 @@ void linkingt::link_error(
 {
   error().source_location=new_symbol.location;
 
-  error() << "error: " << msg << " `"
-          << old_symbol.display_name()
-          << "'" << '\n';
-  error() << "old definition in module `" << old_symbol.module
-          << "' " << old_symbol.location << '\n'
-          << type_to_string_verbose(ns, old_symbol) << '\n';
-  error() << "new definition in module `" << new_symbol.module
-          << "' " << new_symbol.location << '\n'
-          << type_to_string_verbose(ns, new_symbol) << eom;
+  error() << "error: " << msg << " '" << old_symbol.display_name() << "'"
+          << '\n';
+  error() << "old definition in module '" << old_symbol.module << "' "
+          << old_symbol.location << '\n'
+          << type_to_string_verbose(old_symbol) << '\n';
+  error() << "new definition in module '" << new_symbol.module << "' "
+          << new_symbol.location << '\n'
+          << type_to_string_verbose(new_symbol) << eom;
 }
 
 void linkingt::link_warning(
@@ -392,12 +399,12 @@ void linkingt::link_warning(
   warning() << "warning: " << msg << " \""
             << old_symbol.display_name()
             << "\"" << '\n';
-  warning() << "old definition in module " << old_symbol.module
-            << " " << old_symbol.location << '\n'
-            << type_to_string_verbose(ns, old_symbol) << '\n';
-  warning() << "new definition in module " << new_symbol.module
-            << " " << new_symbol.location << '\n'
-            << type_to_string_verbose(ns, new_symbol) << eom;
+  warning() << "old definition in module " << old_symbol.module << " "
+            << old_symbol.location << '\n'
+            << type_to_string_verbose(old_symbol) << '\n';
+  warning() << "new definition in module " << new_symbol.module << " "
+            << new_symbol.location << '\n'
+            << type_to_string_verbose(new_symbol) << eom;
 }
 
 irep_idt linkingt::rename(const irep_idt id)
@@ -452,8 +459,7 @@ void linkingt::duplicate_code_symbol(
     // return type are an error as we would end up with assignments with
     // mismatching types; as we currently do not patch these by inserting type
     // casts we need to fail hard
-    if(!old_symbol.location.get_function().empty() &&
-       old_symbol.value.is_nil())
+    if(old_symbol.type.get_bool(ID_C_incomplete) && old_symbol.value.is_nil())
     {
       if(base_type_eq(old_t.return_type(), new_t.return_type(), ns))
          link_warning(
@@ -470,8 +476,8 @@ void linkingt::duplicate_code_symbol(
       old_symbol.location=new_symbol.location;
       old_symbol.is_weak=new_symbol.is_weak;
     }
-    else if(!new_symbol.location.get_function().empty() &&
-            new_symbol.value.is_nil())
+    else if(
+      new_symbol.type.get_bool(ID_C_incomplete) && new_symbol.value.is_nil())
     {
       if(base_type_eq(old_t.return_type(), new_t.return_type(), ns))
         link_warning(
@@ -686,11 +692,8 @@ void linkingt::duplicate_code_symbol(
           const typet &src_type=t1.id()==ID_union?t2:t1;
 
           bool found=false;
-          for(union_typet::componentst::const_iterator
-              it=union_type.components().begin();
-              !found && it!=union_type.components().end();
-              it++)
-            if(base_type_eq(it->type(), src_type, ns))
+          for(const auto &c : union_type.components())
+            if(base_type_eq(c.type(), src_type, ns))
             {
               found=true;
               if(warn_msg.empty())
@@ -770,9 +773,10 @@ void linkingt::duplicate_code_symbol(
       // keep the one in old_symbol -- libraries come last!
       warning().source_location=new_symbol.location;
 
-      warning() << "function `" << old_symbol.name << "' in module `"
-        << new_symbol.module << "' is shadowed by a definition in module `"
-        << old_symbol.module << "'" << eom;
+      warning() << "function '" << old_symbol.name << "' in module '"
+                << new_symbol.module
+                << "' is shadowed by a definition in module '"
+                << old_symbol.module << "'" << eom;
     }
     else
       link_error(
@@ -790,12 +794,11 @@ bool linkingt::adjust_object_type_rec(
   if(base_type_eq(t1, t2, ns))
     return false;
 
-  if(t1.id()==ID_symbol ||
-     t1.id()==ID_struct_tag ||
-     t1.id()==ID_union_tag ||
-     t1.id()==ID_c_enum_tag)
+  if(
+    t1.id() == ID_struct_tag || t1.id() == ID_union_tag ||
+    t1.id() == ID_c_enum_tag)
   {
-    const irep_idt &identifier=t1.get(ID_identifier);
+    const irep_idt &identifier = to_tag_type(t1).get_identifier();
 
     if(info.o_symbols.insert(identifier).second)
     {
@@ -808,12 +811,11 @@ bool linkingt::adjust_object_type_rec(
 
     return false;
   }
-  else if(t2.id()==ID_symbol ||
-          t2.id()==ID_struct_tag ||
-          t2.id()==ID_union_tag ||
-          t2.id()==ID_c_enum_tag)
+  else if(
+    t2.id() == ID_struct_tag || t2.id() == ID_union_tag ||
+    t2.id() == ID_c_enum_tag)
   {
-    const irep_idt &identifier=t2.get(ID_identifier);
+    const irep_idt &identifier = to_tag_type(t2).get_identifier();
 
     if(info.n_symbols.insert(identifier).second)
     {
@@ -837,15 +839,32 @@ bool linkingt::adjust_object_type_rec(
     // ignore
     return false;
   }
-  else if((t1.id()==ID_incomplete_struct && t2.id()==ID_struct) ||
-          (t1.id()==ID_incomplete_union && t2.id()==ID_union))
+  else if(
+    t1.id() == ID_struct && to_struct_type(t1).is_incomplete() &&
+    t2.id() == ID_struct && !to_struct_type(t2).is_incomplete())
   {
     info.set_to_new=true; // store new type
 
     return false;
   }
-  else if((t1.id()==ID_struct && t2.id()==ID_incomplete_struct) ||
-          (t1.id()==ID_union && t2.id()==ID_incomplete_union))
+  else if(
+    t1.id() == ID_union && to_union_type(t1).is_incomplete() &&
+    t2.id() == ID_union && !to_union_type(t2).is_incomplete())
+  {
+    info.set_to_new = true; // store new type
+
+    return false;
+  }
+  else if(
+    t1.id() == ID_struct && !to_struct_type(t1).is_incomplete() &&
+    t2.id() == ID_struct && to_struct_type(t2).is_incomplete())
+  {
+    // ignore
+    return false;
+  }
+  else if(
+    t1.id() == ID_union && !to_union_type(t1).is_incomplete() &&
+    t2.id() == ID_union && to_union_type(t2).is_incomplete())
   {
     // ignore
     return false;
@@ -875,6 +894,11 @@ bool linkingt::adjust_object_type_rec(
       "conflicting pointer types for variable");
     #endif
 
+    if(info.old_symbol.is_extern && !info.new_symbol.is_extern)
+    {
+      info.set_to_new = true; // store new type
+    }
+
     return false;
   }
   else if(t1.id()==ID_array &&
@@ -961,10 +985,10 @@ void linkingt::duplicate_object_symbol(
   symbolt &new_symbol)
 {
   // both are variables
+  bool set_to_new = false;
 
   if(!base_type_eq(old_symbol.type, new_symbol.type, ns))
   {
-    bool set_to_new=false;
     bool failed=
       adjust_object_type(old_symbol, new_symbol, set_to_new);
 
@@ -995,7 +1019,8 @@ void linkingt::duplicate_object_symbol(
     else if(set_to_new)
       old_symbol.type=new_symbol.type;
 
-    object_type_updates.insert(old_symbol.name, old_symbol.symbol_expr());
+    object_type_updates.insert(
+      old_symbol.symbol_expr(), old_symbol.symbol_expr());
   }
 
   // care about initializers
@@ -1030,19 +1055,23 @@ void linkingt::duplicate_object_symbol(
 
         warning() << "warning: conflicting initializers for"
                   << " variable \"" << old_symbol.name << "\"\n";
-        warning() << "using old value in module "
-                  << old_symbol.module << " "
+        warning() << "using old value in module " << old_symbol.module << " "
                   << old_symbol.value.find_source_location() << '\n'
-                  << expr_to_string(ns, old_symbol.name, tmp_old)
-                  << '\n';
-        warning() << "ignoring new value in module "
-                  << new_symbol.module << " "
+                  << expr_to_string(old_symbol.name, tmp_old) << '\n';
+        warning() << "ignoring new value in module " << new_symbol.module << " "
                   << new_symbol.value.find_source_location() << '\n'
-                  << expr_to_string(ns, new_symbol.name, tmp_new)
-                  << eom;
+                  << expr_to_string(new_symbol.name, tmp_new) << eom;
       }
     }
   }
+  else if(
+    set_to_new && !old_symbol.value.is_nil() &&
+    !old_symbol.value.get_bool(ID_C_zero_initializer))
+  {
+    // the type has been updated, now make sure that the initialising assignment
+    // will have matching types
+    old_symbol.value = typecast_exprt(old_symbol.value, old_symbol.type);
+  }
 }
 
 void linkingt::duplicate_non_type_symbol(
@@ -1099,31 +1128,43 @@ void linkingt::duplicate_type_symbol(
   if(old_symbol.type==new_symbol.type)
     return;
 
-  if(old_symbol.type.id()==ID_incomplete_struct &&
-     new_symbol.type.id()==ID_struct)
+  if(
+    old_symbol.type.id() == ID_struct &&
+    to_struct_type(old_symbol.type).is_incomplete() &&
+    new_symbol.type.id() == ID_struct &&
+    !to_struct_type(new_symbol.type).is_incomplete())
   {
     old_symbol.type=new_symbol.type;
     old_symbol.location=new_symbol.location;
     return;
   }
 
-  if(old_symbol.type.id()==ID_struct &&
-     new_symbol.type.id()==ID_incomplete_struct)
+  if(
+    old_symbol.type.id() == ID_struct &&
+    !to_struct_type(old_symbol.type).is_incomplete() &&
+    new_symbol.type.id() == ID_struct &&
+    to_struct_type(new_symbol.type).is_incomplete())
   {
     // ok, keep old
     return;
   }
 
-  if(old_symbol.type.id()==ID_incomplete_union &&
-     new_symbol.type.id()==ID_union)
+  if(
+    old_symbol.type.id() == ID_union &&
+    to_union_type(old_symbol.type).is_incomplete() &&
+    new_symbol.type.id() == ID_union &&
+    !to_union_type(new_symbol.type).is_incomplete())
   {
     old_symbol.type=new_symbol.type;
     old_symbol.location=new_symbol.location;
     return;
   }
 
-  if(old_symbol.type.id()==ID_union &&
-     new_symbol.type.id()==ID_incomplete_union)
+  if(
+    old_symbol.type.id() == ID_union &&
+    !to_union_type(old_symbol.type).is_incomplete() &&
+    new_symbol.type.id() == ID_union &&
+    to_union_type(new_symbol.type).is_incomplete())
   {
     // ok, keep old
     return;
@@ -1173,20 +1214,32 @@ bool linkingt::needs_renaming_type(
   if(old_symbol.type==new_symbol.type)
     return false;
 
-  if(old_symbol.type.id()==ID_incomplete_struct &&
-     new_symbol.type.id()==ID_struct)
+  if(
+    old_symbol.type.id() == ID_struct &&
+    to_struct_type(old_symbol.type).is_incomplete() &&
+    new_symbol.type.id() == ID_struct &&
+    !to_struct_type(new_symbol.type).is_incomplete())
     return false; // not different
 
-  if(old_symbol.type.id()==ID_struct &&
-     new_symbol.type.id()==ID_incomplete_struct)
+  if(
+    old_symbol.type.id() == ID_struct &&
+    !to_struct_type(old_symbol.type).is_incomplete() &&
+    new_symbol.type.id() == ID_struct &&
+    to_struct_type(new_symbol.type).is_incomplete())
     return false; // not different
 
-  if(old_symbol.type.id()==ID_incomplete_union &&
-     new_symbol.type.id()==ID_union)
+  if(
+    old_symbol.type.id() == ID_union &&
+    to_union_type(old_symbol.type).is_incomplete() &&
+    new_symbol.type.id() == ID_union &&
+    !to_union_type(new_symbol.type).is_incomplete())
     return false; // not different
 
-  if(old_symbol.type.id()==ID_union &&
-     new_symbol.type.id()==ID_incomplete_union)
+  if(
+    old_symbol.type.id() == ID_union &&
+    !to_union_type(old_symbol.type).is_incomplete() &&
+    new_symbol.type.id() == ID_union &&
+    to_union_type(new_symbol.type).is_incomplete())
     return false; // not different
 
   if(old_symbol.type.id()==ID_array &&
@@ -1257,7 +1310,7 @@ void linkingt::rename_symbols(
 
   for(const irep_idt &id : needs_to_be_renamed)
   {
-    symbolt &new_symbol=*src_symbol_table.get_writeable(id);
+    symbolt &new_symbol = src_symbol_table.get_writeable_ref(id);
 
     irep_idt new_identifier;
 
@@ -1320,7 +1373,7 @@ void linkingt::copy_symbols()
   // Now do the collisions
   for(const irep_idt &collision : collisions)
   {
-    symbolt &old_symbol=*main_symbol_table.get_writeable(collision);
+    symbolt &old_symbol = main_symbol_table.get_writeable_ref(collision);
     symbolt &new_symbol=src_symbols.at(collision);
 
     if(new_symbol.is_type)
