@@ -19,45 +19,44 @@ Author: Daniel Kroening
 
 #include "static_lifetime_init.h"
 
-void get_symbols_rec(
+static void get_symbols(
   const namespacet &ns,
-  const symbolt &symbol,
+  const symbolt &in_symbol,
   find_symbols_sett &dest)
 {
-  dest.insert(symbol.name);
+  std::vector<const symbolt *> working_set;
 
-  find_symbols_sett new_symbols;
+  working_set.push_back(&in_symbol);
 
-  find_type_and_expr_symbols(symbol.type, new_symbols);
-  find_type_and_expr_symbols(symbol.value, new_symbols);
-
-  if(symbol.type.id()==ID_code)
+  while(!working_set.empty())
   {
-    const code_typet &code_type=to_code_type(symbol.type);
-    const code_typet::parameterst &parameters=code_type.parameters();
+    const symbolt *current_symbol_ptr = working_set.back();
+    working_set.pop_back();
+    const symbolt &symbol = *current_symbol_ptr;
 
-    for(code_typet::parameterst::const_iterator
-        it=parameters.begin();
-        it!=parameters.end();
-        it++)
-    {
-      irep_idt id=it->get_identifier();
-      const symbolt *s;
-      // identifiers for prototypes need not exist
-      if(!ns.lookup(id, s))
-        new_symbols.insert(id);
-    }
-  }
+    if(!dest.insert(symbol.name).second)
+      continue;
 
-  for(find_symbols_sett::const_iterator
-      it=new_symbols.begin();
-      it!=new_symbols.end();
-      it++)
-  {
-    if(dest.find(*it)==dest.end())
+    find_symbols_sett new_symbols;
+
+    find_type_and_expr_symbols(symbol.type, new_symbols);
+    find_type_and_expr_symbols(symbol.value, new_symbols);
+
+    for(const auto &s : new_symbols)
+      working_set.push_back(&ns.lookup(s));
+
+    if(symbol.type.id() == ID_code)
     {
-      dest.insert(*it);
-      get_symbols_rec(ns, ns.lookup(*it), dest); // recursive call
+      const code_typet &code_type = to_code_type(symbol.type);
+      const code_typet::parameterst &parameters = code_type.parameters();
+
+      for(const auto &p : parameters)
+      {
+        const symbolt *s;
+        // identifiers for prototypes need not exist
+        if(!ns.lookup(p.get_identifier(), s))
+          working_set.push_back(s);
+      }
     }
   }
 }
@@ -72,11 +71,17 @@ void get_symbols_rec(
 ///          http://gcc.gnu.org/ml/gcc/2006-11/msg00006.html
 ///          on "extern inline"
 /// \param symbol_table: symbol table to clean up
+/// \param mh: log handler
+/// \param keep_file_local: keep file-local functions with bodies even if we
+///                         would otherwise remove them
 void remove_internal_symbols(
-  symbol_tablet &symbol_table)
+  symbol_tablet &symbol_table,
+  message_handlert &mh,
+  const bool keep_file_local)
 {
   namespacet ns(symbol_table);
   find_symbols_sett exported;
+  messaget log(mh);
 
   // we retain certain special ones
   find_symbols_sett special;
@@ -90,6 +95,12 @@ void remove_internal_symbols(
   special.insert(CPROVER_PREFIX "deallocated");
   special.insert(CPROVER_PREFIX "dead_object");
   special.insert(CPROVER_PREFIX "rounding_mode");
+  special.insert("__new");
+  special.insert("__new_array");
+  special.insert("__placement_new");
+  special.insert("__placement_new_array");
+  special.insert("__delete");
+  special.insert("__delete_array");
 
   for(symbol_tablet::symbolst::const_iterator
       it=symbol_table.symbols.begin();
@@ -105,7 +116,7 @@ void remove_internal_symbols(
 
     if(special.find(symbol.name)!=special.end())
     {
-      get_symbols_rec(ns, symbol, exported);
+      get_symbols(ns, symbol, exported);
       continue;
     }
 
@@ -133,9 +144,17 @@ void remove_internal_symbols(
     else if(is_function)
     {
       // body? not local (i.e., "static")?
-      if(has_body &&
-         (!is_file_local || (config.main==symbol.name.c_str())))
-        get_symbols_rec(ns, symbol, exported);
+      if(
+        has_body &&
+        (!is_file_local ||
+         (config.main.has_value() && symbol.base_name == config.main.value())))
+      {
+        get_symbols(ns, symbol, exported);
+      }
+      else if(has_body && is_file_local && keep_file_local)
+      {
+        get_symbols(ns, symbol, exported);
+      }
     }
     else
     {
@@ -144,7 +163,7 @@ void remove_internal_symbols(
       if((has_initializer || !symbol.is_extern) &&
          !is_file_local)
       {
-        get_symbols_rec(ns, symbol, exported);
+        get_symbols(ns, symbol, exported);
       }
     }
   }
