@@ -13,9 +13,12 @@ Author: Daniel Kroening, kroening@kroening.com
 #define CPROVER_POINTER_ANALYSIS_VALUE_SET_H
 
 #include <set>
+#include <type_traits>
+#include <unordered_set>
 
 #include <util/mp_arith.h>
 #include <util/reference_counting.h>
+#include <util/sharing_map.h>
 
 #include "object_numbering.h"
 #include "value_sets.h"
@@ -45,12 +48,26 @@ public:
   {
   }
 
+  value_sett(value_sett &&other)
+    : location_number(other.location_number), values(std::move(other.values))
+  {
+  }
+
   virtual ~value_sett() = default;
 
-  static bool field_sensitive(
-    const irep_idt &id,
-    const typet &type,
-    const namespacet &);
+  value_sett(const value_sett &other) = default;
+
+  value_sett &operator=(const value_sett &other) = delete;
+
+  value_sett &operator=(value_sett &&other)
+  {
+    values = std::move(other.values);
+    return *this;
+  }
+
+  /// Determines whether an identifier of a given type should have its fields
+  /// distinguished. Virtual so that subclasses can override this behaviour.
+  virtual bool field_sensitive(const irep_idt &id, const typet &type);
 
   /// Matches the location_number field of the instruction that corresponds
   /// to this value_sett instance in value_set_domaint's state map
@@ -60,11 +77,10 @@ public:
   /// in value sets.
   static object_numberingt object_numbering;
 
-  typedef irep_idt idt;
-
   /// Represents the offset into an object: either a unique integer offset,
   /// or an unknown value, represented by `!offset`.
   typedef optionalt<mp_integer> offsett;
+  DEPRECATED(SINCE(2019, 05, 22, "Unused"))
   bool offset_is_zero(const offsett &offset) const
   {
     return offset && offset->is_zero();
@@ -74,75 +90,12 @@ public:
   /// offsets (`offsett` instances). This is the RHS set of a single row of
   /// the enclosing `value_sett`, such as `{ null, dynamic_object1 }`.
   /// The set is represented as a map from numbered `exprt`s to `offsett`
-  /// instead of a set of pairs to make lookup by `exprt` easier. All
-  /// methods matching the interface of `std::map` forward those methods
-  /// to the internal map.
-  class object_map_dt
-  {
-    typedef std::map<object_numberingt::number_type, offsett> data_typet;
-    data_typet data;
+  /// instead of a set of pairs to make lookup by `exprt` easier.
+  using object_map_dt = std::map<object_numberingt::number_type, offsett>;
 
-  public:
-    // NOLINTNEXTLINE(readability/identifiers)
-    typedef data_typet::iterator iterator;
-    // NOLINTNEXTLINE(readability/identifiers)
-    typedef data_typet::const_iterator const_iterator;
-    // NOLINTNEXTLINE(readability/identifiers)
-    typedef data_typet::value_type value_type;
-    // NOLINTNEXTLINE(readability/identifiers)
-    typedef data_typet::key_type key_type;
+  static const object_map_dt empty_object_map;
 
-    iterator begin() { return data.begin(); }
-    const_iterator begin() const { return data.begin(); }
-    const_iterator cbegin() const { return data.cbegin(); }
-
-    iterator end() { return data.end(); }
-    const_iterator end() const { return data.end(); }
-    const_iterator cend() const { return data.cend(); }
-
-    size_t size() const { return data.size(); }
-    bool empty() const { return data.empty(); }
-
-    void erase(key_type i) { data.erase(i); }
-    void erase(const_iterator it) { data.erase(it); }
-
-    offsett &operator[](key_type i)
-    {
-      return data[i];
-    }
-    offsett &at(key_type i)
-    {
-      return data.at(i);
-    }
-    const offsett &at(key_type i) const
-    {
-      return data.at(i);
-    }
-
-    template <typename It>
-    void insert(It b, It e) { data.insert(b, e); }
-
-    template <typename T>
-    const_iterator find(T &&t) const { return data.find(std::forward<T>(t)); }
-
-    static const object_map_dt blank;
-
-    object_map_dt()=default;
-
-    bool operator==(const object_map_dt &other) const
-    {
-      return data==other.data;
-    }
-    bool operator!=(const object_map_dt &other) const
-    {
-      return !(*this==other);
-    }
-
-  protected:
-    ~object_map_dt()=default;
-  };
-
-  /// Converts an `object_map_dt` entry `object_number -> offset` into an
+  /// Converts an `object_map_dt` element `object_number -> offset` into an
   /// `object_descriptor_exprt` with
   /// `.object() == object_numbering.at(object_number)` and
   /// `.offset() == offset`.
@@ -160,31 +113,31 @@ public:
   ///
   /// Then the set { dynamic_object1 }, represented by an `object_map_dt`, can
   /// be shared between the two `value_sett` instances.
-  typedef reference_counting<object_map_dt> object_mapt;
+  using object_mapt = reference_counting<object_map_dt, empty_object_map>;
 
-  /// Sets an entry in object map `dest` to match an existing entry `it`
-  /// from a different map. Any existing entry for the same `exprt` is
+  /// Sets an element in object map `dest` to match an existing element `it`
+  /// from a different map. Any existing element for the same `exprt` is
   /// overwritten.
   /// \param dest: object map to update.
-  /// \param it: iterator pointing to new entry
+  /// \param it: iterator pointing to new element
   void set(object_mapt &dest, const object_map_dt::value_type &it) const
   {
     dest.write()[it.first]=it.second;
   }
 
-  /// Merges an existing entry into an object map. If the destination map
-  /// has an existing entry for the same expression with a different offset
+  /// Merges an existing element into an object map. If the destination map
+  /// has an existing element for the same expression with a different offset
   /// its offset is marked unknown (so e.g. `x -> 0` and `x -> 1` merge into
   /// `x -> ?`).
   /// \param dest: object map to update.
-  /// \param it: iterator pointing to new entry
+  /// \param it: iterator pointing to new element
   bool insert(object_mapt &dest, const object_map_dt::value_type &it) const
   {
     return insert(dest, it.first, it.second);
   }
 
   /// Adds an expression to an object map, with unknown offset. If the
-  /// destination map has an existing entry for the same expression
+  /// destination map has an existing element for the same expression
   /// its offset is marked unknown.
   /// \param dest: object map to update
   /// \param src: expression to add
@@ -194,7 +147,7 @@ public:
   }
 
   /// Adds an expression to an object map, with known offset. If the
-  /// destination map has an existing entry for the same expression
+  /// destination map has an existing element for the same expression
   /// with a differing offset its offset is marked unknown.
   /// \param dest: object map to update
   /// \param src: expression to add
@@ -208,23 +161,41 @@ public:
   }
 
   /// Adds a numbered expression and offset to an object map. If the
-  /// destination map has an existing entry for the same expression
+  /// destination map has an existing element for the same expression
   /// with a differing offset its offset is marked unknown.
   /// \param dest: object map to update
   /// \param n: object number to add; must be mapped to the corresponding
   ///   expression by `object_numbering`.
-  /// \param object: offset into object `n` (may be unknown).
+  /// \param offset: offset into object `n` (may be unknown).
   bool insert(
     object_mapt &dest,
     object_numberingt::number_type n,
     const offsett &offset) const;
 
+  enum class insert_actiont
+  {
+    INSERT,
+    RESET_OFFSET,
+    NONE
+  };
+
+  /// Determines what would happen if object number \p n was inserted into map
+  /// \p dest with offset \p offset -- the possibilities being, nothing (the
+  /// entry is already present), a new entry would be inserted (no existing
+  /// entry with number \p n was found), or an existing entry's offset would be
+  /// reset (indicating there is already an entry with number \p n, but with
+  /// differing offset).
+  insert_actiont get_insert_action(
+    const object_mapt &dest,
+    object_numberingt::number_type n,
+    const offsett &offset) const;
+
   /// Adds an expression and offset to an object map. If the
-  /// destination map has an existing entry for the same expression
+  /// destination map has an existing element for the same expression
   /// with a differing offset its offset is marked unknown.
   /// \param dest: object map to update
   /// \param expr: expression to add
-  /// \param object: offset into `expr` (may be unknown).
+  /// \param offset: offset into `expr` (may be unknown).
   bool insert(object_mapt &dest, const exprt &expr, const offsett &offset) const
   {
     return insert(dest, object_numbering.number(expr), offset);
@@ -233,29 +204,30 @@ public:
   /// Represents a row of a `value_sett`. For example, this might represent
   /// `dynamic_object1.field1 -> { <dynamic_object2, 0> }`, with
   /// `identifier == dynamic_object1`, `suffix` == `.field1` and
-  /// a single-entry `object_map` representing `{ <dynamic_object2, 0> }`.
+  /// a single-element `object_map` representing `{ <dynamic_object2, 0> }`.
   struct entryt
   {
     object_mapt object_map;
-    idt identifier;
+    irep_idt identifier;
     std::string suffix;
 
     entryt()
     {
     }
 
-    entryt(const idt &_identifier, const std::string &_suffix):
-      identifier(_identifier),
-      suffix(_suffix)
+    entryt(const irep_idt &_identifier, const std::string &_suffix)
+      : identifier(_identifier), suffix(_suffix)
     {
     }
 
     bool operator==(const entryt &other) const
     {
-      return
-        identifier==other.identifier &&
-        suffix==other.suffix &&
-        object_map==other.object_map;
+      // Note that the object_map comparison below is duplicating the code of
+      // operator== defined in reference_counting.h because old versions of
+      // clang (3.7 and 3.8) do not resolve the template instantiation correctly
+      return identifier == other.identifier && suffix == other.suffix &&
+             (object_map.get_d() == other.object_map.get_d() ||
+              object_map.read() == other.object_map.read());
     }
     bool operator!=(const entryt &other) const
     {
@@ -265,11 +237,13 @@ public:
 
   /// Set of expressions; only used for the `get` API, not for internal
   /// data representation.
+  DEPRECATED(SINCE(2019, 05, 22, "Only used in deprecated function"))
   typedef std::set<exprt> expr_sett;
 
   /// Set of dynamic object numbers, equivalent to a set of
   /// `dynamic_object_exprt`s with corresponding IDs. Used only in internal
   /// implementation details.
+  DEPRECATED(SINCE(2019, 05, 22, "Unused"))
   typedef std::set<unsigned int> dynamic_object_id_sett;
 
   /// Map representing the entire value set, mapping from string LHS IDs
@@ -285,58 +259,70 @@ public:
   ///
   /// The components of the ID are thus duplicated in the `valuest` key and in
   /// `entryt` fields.
-  #ifdef USE_DSTRING
-  typedef std::map<idt, entryt> valuest;
-  #else
-  typedef std::unordered_map<idt, entryt, string_hash> valuest;
-  #endif
+  typedef sharing_mapt<irep_idt, entryt> valuest;
+
+  /// Gets values pointed to by \p expr, including following dereference
+  /// operators (i.e. this is not a simple lookup in `valuest`).
+  DEPRECATED(
+    SINCE(2019, 05, 22, "Use get_value_set(exprt, const namespacet &) instead"))
+  void get_value_set(
+    exprt expr,
+    value_setst::valuest &dest,
+    const namespacet &ns) const;
 
   /// Gets values pointed to by `expr`, including following dereference
   /// operators (i.e. this is not a simple lookup in `valuest`).
   /// \param expr: query expression
-  /// \param [out] dest: assigned a set of expressions that `expr` may point to
   /// \param ns: global namespace
-  void get_value_set(
-    const exprt &expr,
-    value_setst::valuest &dest,
-    const namespacet &ns) const;
+  /// \return list of expressions that `expr` may point to. These expressions
+  ///   are object_descriptor_exprt or have id ID_invalid or ID_unknown.
+  std::vector<exprt> get_value_set(exprt expr, const namespacet &ns) const;
 
   /// Appears to be unimplemented.
-  expr_sett &get(
-    const idt &identifier,
-    const std::string &suffix);
-
-  /// Clears value set (not used in the CBMC repository)
-  void make_any()
-  {
-    values.clear();
-  }
+  DEPRECATED(SINCE(2019, 05, 22, "Unimplemented"))
+  expr_sett &get(const irep_idt &identifier, const std::string &suffix);
 
   void clear()
   {
     values.clear();
   }
 
-  /// Gets or inserts an entry in this value-set.
+  /// Finds an entry in this value-set. The interface differs from
+  /// \ref update_entry because get_value_set_rec wants to check for a struct's
+  /// first component before stripping the suffix as is done in
+  /// \ref update_entry.
+  /// \param id: identifier to find.
+  /// \return a constant pointer to an entry if found, or null otherwise.
+  const entryt *find_entry(const irep_idt &id) const;
+
+  /// Adds or replaces an entry in this value-set.
   /// \param e: entry to find. Its `id` and `suffix` fields will be used
   ///   to find a corresponding entry; if a fresh entry is created its
-  ///   `object_map` (RHS value set) will be copied; otherwise it will be
-  ///   ignored. Therefore it should probably be left blank and any RHS updates
-  ///   conducted against the returned reference.
+  ///   `object_map` (RHS value set) will be merged with or replaced by \p
+  ///   new_values, depending on the value of \p add_to_sets. If an entry
+  ///   already exists, the object map of \p e is ignored.
   /// \param type: type of `e.identifier`, used to determine whether `e`'s
   ///   suffix should be used to find a field-sensitive value-set or whether
   ///   a single entry should be shared by all of symbol `e.identifier`.
-  /// \param ns: global namespace
-  entryt &get_entry(
-    const entryt &e, const typet &type,
-    const namespacet &ns);
+  /// \param new_values: values to be stored for the entry.
+  /// \param add_to_sets: if true, merge in \p new_values instead of replacing
+  ///   the existing values.
+  void update_entry(
+    const entryt &e,
+    const typet &type,
+    const object_mapt &new_values,
+    bool add_to_sets);
 
   /// Pretty-print this value-set
-  /// \param ns: global namespace
   /// \param [out] out: stream to write to
+  /// \param indent: string to use for indentation of the output
+  void output(std::ostream &out, const std::string &indent = "") const;
+
+  DEPRECATED(SINCE(2019, 06, 11, "Use the version without ns argument"))
   void output(
     const namespacet &ns,
-    std::ostream &out) const;
+    std::ostream &out,
+    const std::string &indent = "") const;
 
   /// Stores the LHS ID -> RHS expression set map. See `valuest` documentation
   /// for more detail.
@@ -348,6 +334,13 @@ public:
   /// \return true if anything changed.
   bool make_union(object_mapt &dest, const object_mapt &src) const;
 
+  /// Determines whether merging two RHS expression sets would cause any change
+  /// \param dest: set that would be merged into
+  /// \param src: set that would be merged in
+  /// \return true if anything changed.
+  bool make_union_would_change(const object_mapt &dest, const object_mapt &src)
+    const;
+
   /// Merges an entire existing value_sett's data into this one
   /// \param new_values: new values to merge in
   /// \return true if anything changed.
@@ -450,19 +443,49 @@ public:
     exprt &expr,
     const namespacet &ns) const;
 
+  /// Get the index of the symbol and suffix
+  /// \param identifier: The identifier for the symbol
+  /// \param type: The type of the symbol
+  /// \param suffix: The suffix for the entry
+  /// \param ns: The global namespace, for following \p type if it is a
+  ///   struct tag type or a union tag type
+  /// \return The index if the symbol is known, else `nullopt`.
+  optionalt<irep_idt> get_index_of_symbol(
+    irep_idt identifier,
+    const typet &type,
+    const std::string &suffix,
+    const namespacet &ns) const;
+
+  /// Update the entry stored at \p index by erasing any values listed in
+  /// \p values_to_erase.
+  /// \param index: index in the value set
+  /// \param values_to_erase: set of values to remove from the entry
+  void erase_values_from_entry(
+    const irep_idt &index,
+    const std::unordered_set<exprt, irep_hash> &values_to_erase);
+
+  void erase_symbol(const symbol_exprt &symbol_expr, const namespacet &ns);
+
 protected:
-  /// Reads the set of objects pointed to by `expr`, including making
+  /// Reads the set of objects pointed to by \p expr, including making
   /// recursive lookups for dereference operations etc.
-  /// \param expr: query expression
-  /// \param dest [out]: overwritten by the set of object numbers pointed to
-  /// \param ns; global namespace
-  /// \param is_simplified: if false, simplify `expr` before reading.
+  DEPRECATED(
+    SINCE(2019, 05, 22, "Use the version returning object_mapt instead"))
   void get_value_set(
-    const exprt &expr,
+    exprt expr,
     object_mapt &dest,
     const namespacet &ns,
     bool is_simplified) const;
 
+  /// Reads the set of objects pointed to by `expr`, including making
+  /// recursive lookups for dereference operations etc.
+  /// \param expr: query expression
+  /// \param ns: global namespace
+  /// \param is_simplified: if false, simplify `expr` before reading.
+  /// \return the set of object numbers pointed to
+  object_mapt
+  get_value_set(exprt expr, const namespacet &ns, bool is_simplified) const;
+
   /// See the other overload of `get_reference_set`. This one returns object
   /// numbers and offsets instead of expressions.
   void get_reference_set(
@@ -484,22 +507,14 @@ protected:
     const exprt &src,
     exprt &dest) const;
 
-  /// Marks objects that may be pointed to by `op` as maybe-invalid
-  /// \param op: pointer to invalidate
-  /// \param ns: global namespace
-  void do_free(
-    const exprt &op,
+  void erase_symbol_rec(
+    const typet &type,
+    const std::string &erase_prefix,
     const namespacet &ns);
 
-  /// Extracts a member from a struct- or union-typed expression.
-  /// Usually that means making a `member_exprt`, but this can shortcut
-  /// extracting members from constants or with-expressions.
-  /// \param src: base struct-or-union-typed expression
-  /// \param component_name: member to extract
-  /// \param ns: global namespace
-  exprt make_member(
-    const exprt &src,
-    const irep_idt &component_name,
+  void erase_struct_union_symbol(
+    const struct_union_typet &struct_union_type,
+    const std::string &erase_prefix,
     const namespacet &ns);
 
   // Subclass customisation points:
@@ -536,6 +551,9 @@ protected:
     const namespacet &,
     object_mapt &rhs_values) const
   {
+    // unused parameters
+    (void)rhs;
+    (void)rhs_values;
   }
 
   /// Subclass customisation point to apply global side-effects to this domain,
@@ -547,6 +565,9 @@ protected:
     const exprt &rhs,
     const namespacet &)
   {
+    // unused parameters
+    (void)lhs;
+    (void)rhs;
   }
 };
 
