@@ -11,10 +11,9 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include "value_set_analysis.h"
 
-#include <util/prefix.h>
 #include <util/cprover_prefix.h>
-#include <util/xml_expr.h>
-#include <util/xml.h>
+#include <util/prefix.h>
+#include <util/xml_irep.h>
 
 #include <langapi/language_util.h>
 
@@ -42,16 +41,15 @@ void value_sets_to_xml(
     xmlt &i=dest.new_element("instruction");
     i.new_element()=::xml(location);
 
-    for(value_sett::valuest::const_iterator
-        v_it=value_set.values.begin();
-        v_it!=value_set.values.end();
-        v_it++)
+    value_sett::valuest::viewt view;
+    value_set.values.get_view(view);
+
+    for(const auto &values_entry : view)
     {
       xmlt &var=i.new_element("variable");
-      var.new_element("identifier").data=
-        id2string(v_it->first);
+      var.new_element("identifier").data = id2string(values_entry.first);
 
-      #if 0
+#if 0
       const value_sett::expr_sett &expr_set=
         v_it->second.expr_set();
 
@@ -66,7 +64,7 @@ void value_sets_to_xml(
         var.new_element("value").data=
           xmlt::escape(value_str);
       }
-      #endif
+#endif
     }
   }
 }
