@@ -16,17 +16,19 @@ Author: Daniel Kroening, kroening@kroening.com
 
 bool value_set_domain_fit::transform(
   const namespacet &ns,
+  const irep_idt &function_from,
   locationt from_l,
+  const irep_idt &function_to,
   locationt to_l)
 {
   value_set.changed = false;
 
-  value_set.set_from(from_l->function, from_l->location_number);
-  value_set.set_to(to_l->function, to_l->location_number);
+  value_set.set_from(function_from, from_l->location_number);
+  value_set.set_to(function_to, to_l->location_number);
 
-//  std::cout << "transforming: " <<
-//      from_l->function << " " << from_l->location_number << " to " <<
-//      to_l->function << " " << to_l->location_number << '\n';
+  //  std::cout << "transforming: " <<
+  //      from_l->function << " " << from_l->location_number << " to " <<
+  //      to_l->function << " " << to_l->location_number << '\n';
 
   switch(from_l->type)
   {
@@ -45,18 +47,29 @@ bool value_set_domain_fit::transform(
     break;
 
   case FUNCTION_CALL:
-    {
-      const code_function_callt &code=
-        to_code_function_call(from_l->code);
+  {
+    const code_function_callt &code = from_l->get_function_call();
 
-      value_set.do_function_call(to_l->function, code.arguments(), ns);
-    }
+    value_set.do_function_call(function_to, code.arguments(), ns);
     break;
+  }
 
-  default:
-    {
-      // do nothing
-    }
+  case CATCH:
+  case THROW:
+  case DECL:
+  case DEAD:
+  case ATOMIC_BEGIN:
+  case ATOMIC_END:
+  case START_THREAD:
+  case END_THREAD:
+  case LOCATION:
+  case SKIP:
+  case ASSERT:
+  case ASSUME:
+  case INCOMPLETE_GOTO:
+  case NO_INSTRUCTION_TYPE:
+    // do nothing
+    break;
   }
 
   return (value_set.changed);
