@@ -28,10 +28,18 @@ public:
   typedef std::list<exprt> valuest;
 
   // this is not const to allow a lazy evaluation
+  DEPRECATED(SINCE(2019, 05, 22, "use vector returning version instead"))
   virtual void get_values(
+    const irep_idt &function_id,
     goto_programt::const_targett l,
     const exprt &expr,
-    valuest &dest)=0;
+    valuest &dest) = 0;
+
+  // this is not const to allow a lazy evaluation
+  virtual std::vector<exprt> get_values(
+    const irep_idt &function_id,
+    goto_programt::const_targett l,
+    const exprt &expr) = 0;
 
   virtual ~value_setst()
   {
