@@ -14,18 +14,21 @@ Author: Daniel Kroening, kroening@kroening.com
 // convert expression to boolean formula
 //
 
-#include <util/mp_arith.h>
-#include <util/expr.h>
 #include <util/byte_operators.h>
+#include <util/expr.h>
+#include <util/mp_arith.h>
+#include <util/optional.h>
+
+#include <solvers/lowering/functions.h>
 
 #include "bv_utils.h"
 #include "boolbv_width.h"
 #include "boolbv_map.h"
 #include "arrays.h"
-#include "functions.h"
 
 class extractbit_exprt;
 class extractbits_exprt;
+class array_comprehension_exprt;
 class member_exprt;
 
 class boolbvt:public arrayst
@@ -33,17 +36,21 @@ class boolbvt:public arrayst
 public:
   boolbvt(
     const namespacet &_ns,
-    propt &_prop):
-    arrayst(_ns, _prop),
-    unbounded_array(unbounded_arrayt::U_NONE),
-    boolbv_width(_ns),
-    bv_utils(_prop),
-    functions(*this),
-    map(_prop, _ns, boolbv_width)
+    propt &_prop,
+    message_handlert &message_handler)
+    : arrayst(_ns, _prop, message_handler),
+      unbounded_array(unbounded_arrayt::U_NONE),
+      boolbv_width(_ns),
+      bv_utils(_prop),
+      functions(*this),
+      map(_prop, boolbv_width)
   {
   }
 
-  virtual const bvt &convert_bv(const exprt &expr); // check cache
+  virtual const bvt &convert_bv( // check cache
+    const exprt &expr,
+    const optionalt<std::size_t> expected_width = nullopt);
+
   virtual bvt convert_bitvector(const exprt &expr); // no cache
 
   // overloading
@@ -70,8 +77,6 @@ public:
     std::size_t bit,
     literalt &literal) const;
 
-  using arrayst::literal;
-
   enum class unbounded_arrayt { U_NONE, U_ALL, U_AUTO };
   unbounded_arrayt unbounded_array;
 
@@ -119,7 +124,7 @@ protected:
     const typet &src_type, const bvt &src,
     const typet &dest_type, bvt &dest);
 
-  virtual literalt convert_bv_rel(const exprt &expr);
+  virtual literalt convert_bv_rel(const binary_relation_exprt &);
   virtual literalt convert_typecast(const typecast_exprt &expr);
   virtual literalt convert_reduction(const unary_exprt &expr);
   virtual literalt convert_onehot(const unary_exprt &expr);
@@ -128,8 +133,8 @@ protected:
   virtual literalt convert_equality(const equal_exprt &expr);
   virtual literalt convert_verilog_case_equality(
     const binary_relation_exprt &expr);
-  virtual literalt convert_ieee_float_rel(const exprt &expr);
-  virtual literalt convert_quantifier(const exprt &expr);
+  virtual literalt convert_ieee_float_rel(const binary_relation_exprt &);
+  virtual literalt convert_quantifier(const quantifier_exprt &expr);
 
   virtual bvt convert_index(const exprt &array, const mp_integer &index);
   virtual bvt convert_index(const index_exprt &expr);
@@ -140,31 +145,31 @@ protected:
   virtual bvt convert_if(const if_exprt &expr);
   virtual bvt convert_struct(const struct_exprt &expr);
   virtual bvt convert_array(const exprt &expr);
-  virtual bvt convert_vector(const exprt &expr);
-  virtual bvt convert_complex(const exprt &expr);
-  virtual bvt convert_complex_real(const exprt &expr);
-  virtual bvt convert_complex_imag(const exprt &expr);
-  virtual bvt convert_lambda(const exprt &expr);
+  virtual bvt convert_vector(const vector_exprt &expr);
+  virtual bvt convert_complex(const complex_exprt &expr);
+  virtual bvt convert_complex_real(const complex_real_exprt &expr);
+  virtual bvt convert_complex_imag(const complex_imag_exprt &expr);
+  virtual bvt convert_array_comprehension(const array_comprehension_exprt &);
   virtual bvt convert_let(const let_exprt &);
   virtual bvt convert_array_of(const array_of_exprt &expr);
   virtual bvt convert_union(const union_exprt &expr);
   virtual bvt convert_bv_typecast(const typecast_exprt &expr);
   virtual bvt convert_add_sub(const exprt &expr);
-  virtual bvt convert_mult(const exprt &expr);
+  virtual bvt convert_mult(const mult_exprt &expr);
   virtual bvt convert_div(const div_exprt &expr);
   virtual bvt convert_mod(const mod_exprt &expr);
-  virtual bvt convert_floatbv_op(const exprt &expr);
+  virtual bvt convert_floatbv_op(const ieee_float_op_exprt &);
   virtual bvt convert_floatbv_typecast(const floatbv_typecast_exprt &expr);
   virtual bvt convert_member(const member_exprt &expr);
-  virtual bvt convert_with(const exprt &expr);
-  virtual bvt convert_update(const exprt &expr);
+  virtual bvt convert_with(const with_exprt &expr);
+  virtual bvt convert_update(const update_exprt &);
   virtual bvt convert_case(const exprt &expr);
-  virtual bvt convert_cond(const exprt &expr);
+  virtual bvt convert_cond(const cond_exprt &);
   virtual bvt convert_shift(const binary_exprt &expr);
   virtual bvt convert_bitwise(const exprt &expr);
-  virtual bvt convert_unary_minus(const unary_exprt &expr);
+  virtual bvt convert_unary_minus(const unary_minus_exprt &expr);
   virtual bvt convert_abs(const abs_exprt &expr);
-  virtual bvt convert_concatenation(const exprt &expr);
+  virtual bvt convert_concatenation(const concatenation_exprt &expr);
   virtual bvt convert_replication(const replication_exprt &expr);
   virtual bvt convert_bv_literals(const exprt &expr);
   virtual bvt convert_constant(const constant_exprt &expr);
@@ -223,6 +228,7 @@ protected:
   virtual exprt bv_get_unbounded_array(const exprt &) const;
 
   virtual exprt bv_get_rec(
+    const exprt &expr,
     const bvt &bv,
     const std::vector<bool> &unknown,
     std::size_t offset,
@@ -238,6 +244,11 @@ protected:
   class quantifiert
   {
   public:
+    quantifiert(exprt _expr, literalt _l)
+      : expr(std::move(_expr)), l(std::move(_l))
+    {
+    }
+
     exprt expr;
     literalt l;
   };
