@@ -6,6 +6,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
+#include <util/arith_tools.h>
 
 #include "boolbv.h"
 
@@ -30,12 +31,10 @@ bvt boolbvt::convert_constant(const constant_exprt &expr)
     {
       const bvt &tmp=convert_bv(*it);
 
-      if(tmp.size()!=op_width)
-      {
-        error().source_location=expr.find_source_location();
-        error() << "convert_constant: unexpected operand width" << eom;
-        throw 0;
-      }
+      DATA_INVARIANT_WITH_DIAGNOSTICS(
+        tmp.size() == op_width,
+        "convert_constant: unexpected operand width",
+        irep_pretty_diagnosticst{expr});
 
       for(std::size_t j=0; j<op_width; j++)
         bv[offset+j]=tmp[j];
@@ -67,30 +66,18 @@ bvt boolbvt::convert_constant(const constant_exprt &expr)
 
     return bv;
   }
-  else if(expr_type.id()==ID_unsignedbv ||
-          expr_type.id()==ID_signedbv ||
-          expr_type.id()==ID_bv ||
-          expr_type.id()==ID_fixedbv ||
-          expr_type.id()==ID_floatbv ||
-          expr_type.id()==ID_c_enum ||
-          expr_type.id()==ID_c_enum_tag ||
-          expr_type.id()==ID_c_bool ||
-          expr_type.id()==ID_c_bit_field ||
-          expr_type.id()==ID_incomplete_c_enum)
+  else if(
+    expr_type.id() == ID_unsignedbv || expr_type.id() == ID_signedbv ||
+    expr_type.id() == ID_bv || expr_type.id() == ID_fixedbv ||
+    expr_type.id() == ID_floatbv || expr_type.id() == ID_c_enum ||
+    expr_type.id() == ID_c_enum_tag || expr_type.id() == ID_c_bool ||
+    expr_type.id() == ID_c_bit_field)
   {
-    const std::string &binary=id2string(expr.get_value());
-
-    if(binary.size()!=width)
-    {
-      error().source_location=expr.find_source_location();
-      error() << "wrong value length in constant: "
-              << expr.pretty() << eom;
-      throw 0;
-    }
+    const auto &value = expr.get_value();
 
     for(std::size_t i=0; i<width; i++)
     {
-      bool bit=(binary[binary.size()-i-1]=='1');
+      const bool bit = get_bvrep_bit(value, width, i);
       bv[i]=const_literal(bit);
     }
 
@@ -110,13 +97,10 @@ bvt boolbvt::convert_constant(const constant_exprt &expr)
   {
     const std::string &binary=id2string(expr.get_value());
 
-    if(binary.size()*2!=width)
-    {
-      error().source_location=expr.find_source_location();
-      error() << "wrong value length in constant: "
-              << expr.pretty() << eom;
-      throw 0;
-    }
+    DATA_INVARIANT_WITH_DIAGNOSTICS(
+      binary.size() * 2 == width,
+      "wrong value length in constant",
+      irep_pretty_diagnosticst{expr});
 
     for(std::size_t i=0; i<binary.size(); i++)
     {
@@ -146,10 +130,10 @@ bvt boolbvt::convert_constant(const constant_exprt &expr)
         break;
 
       default:
-        error().source_location=expr.find_source_location();
-        error() << "unknown character in Verilog constant:"
-                << expr.pretty() << eom;
-        throw 0;
+        DATA_INVARIANT_WITH_DIAGNOSTICS(
+          false,
+          "unknown character in Verilog constant",
+          irep_pretty_diagnosticst{expr});
       }
     }
 
