@@ -8,19 +8,22 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include "boolbv.h"
 
-#include <cassert>
+#include <algorithm>
 
 #include <util/arith_tools.h>
+#include <util/cprover_prefix.h>
 #include <util/pointer_offset_size.h>
 #include <util/simplify_expr.h>
 #include <util/std_expr.h>
 
+#include <solvers/lowering/expr_lowering.h>
+
 bvt boolbvt::convert_index(const index_exprt &expr)
 {
   const exprt &array=expr.array();
   const exprt &index=expr.index();
 
-  const typet &array_op_type=ns.follow(array.type());
+  const typet &array_op_type = array.type();
 
   bvt bv;
 
@@ -40,37 +43,58 @@ bvt boolbvt::convert_index(const index_exprt &expr)
     {
       // use array decision procedure
 
-      // free variables
+      if(has_byte_operator(expr))
+      {
+        const index_exprt final_expr =
+          to_index_expr(lower_byte_operators(expr, ns));
+        CHECK_RETURN(final_expr != expr);
+        bv = convert_bv(final_expr);
 
-      bv.resize(width);
-      for(std::size_t i=0; i<width; i++)
-        bv[i]=prop.new_variable();
+        // record type if array is a symbol
+        const exprt &final_array = final_expr.array();
+        if(
+          final_array.id() == ID_symbol || final_array.id() == ID_nondet_symbol)
+        {
+          map.get_map_entry(final_array.get(ID_identifier), array_type);
+        }
 
-      record_array_index(expr);
+        // make sure we have the index in the cache
+        convert_bv(final_expr.index());
+      }
+      else
+      {
+        // free variables
+        bv.reserve(width);
+        for(std::size_t i = 0; i < width; i++)
+          bv.push_back(prop.new_variable());
 
-      // record type if array is a symbol
+        record_array_index(expr);
 
-      if(array.id()==ID_symbol)
-        map.get_map_entry(
-          to_symbol_expr(array).get_identifier(), array_type);
+        // record type if array is a symbol
+        if(array.id() == ID_symbol || array.id() == ID_nondet_symbol)
+          map.get_map_entry(array.get(ID_identifier), array_type);
 
-      // make sure we have the index in the cache
-      convert_bv(index);
+        // make sure we have the index in the cache
+        convert_bv(index);
+      }
 
       return bv;
     }
 
     // Must have a finite size
-    mp_integer array_size;
-    if(to_integer(array_type.size(), array_size))
-      throw "failed to convert array size";
+    mp_integer array_size =
+      numeric_cast_v<mp_integer>(to_constant_expr(array_type.size()));
 
-    // see if the index address is constant
-    // many of these are compacted by simplify_expr
-    // but variable location writes will block this
-    mp_integer index_value;
-    if(!to_integer(index, index_value))
-      return convert_index(array, index_value);
+    {
+      // see if the index address is constant
+      // many of these are compacted by simplify_expr
+      // but variable location writes will block this
+      auto maybe_index_value = numeric_cast<mp_integer>(index);
+      if(maybe_index_value.has_value())
+      {
+        return convert_index(array, maybe_index_value.value());
+      }
+    }
 
     // Special case : arrays of one thing (useful for constants)
     // TODO : merge with ACTUAL_ARRAY_HACK so that ranges of the same
@@ -78,52 +102,44 @@ bvt boolbvt::convert_index(const index_exprt &expr)
     // this rather than as a series of individual options.
     #define UNIFORM_ARRAY_HACK
     #ifdef UNIFORM_ARRAY_HACK
-    bool is_uniform = false;
+    bool is_uniform = array.id() == ID_array_of;
 
-    if(array.id()==ID_array_of)
+    if(array.id() == ID_constant || array.id() == ID_array)
     {
-      is_uniform = true;
-    }
-    else if(array.id()==ID_constant || array.id()==ID_array)
-    {
-      bool found_exception = false;
-      forall_expr(it, array.operands())
-      {
-        if(*it != array.op0())
-        {
-          found_exception = true;
-          break;
-        }
-      }
-
-      if(!found_exception)
-        is_uniform = true;
+      is_uniform = array.operands().size() <= 1 ||
+                   std::all_of(
+                     ++array.operands().begin(),
+                     array.operands().end(),
+                     [&array](const exprt &expr) {
+                       return expr == to_multi_ary_expr(array).op0();
+                     });
     }
 
     if(is_uniform && prop.has_set_to())
     {
       static int uniform_array_counter;  // Temporary hack
 
-      std::string identifier=
-        "__CPROVER_internal_uniform_array_"+
-        std::to_string(uniform_array_counter++);
+      const std::string identifier = CPROVER_PREFIX "internal_uniform_array_" +
+                                     std::to_string(uniform_array_counter++);
 
       symbol_exprt result(identifier, expr.type());
       bv = convert_bv(result);
 
-      equal_exprt value_equality(result, array.op0());
+      // return an unconstrained value in case of an empty array (the access is
+      // necessarily out-of-bounds)
+      if(!array.has_operands())
+        return bv;
+
+      equal_exprt value_equality(result, to_multi_ary_expr(array).op0());
 
       binary_relation_exprt lower_bound(
         from_integer(0, index.type()), ID_le, index);
       binary_relation_exprt upper_bound(
         index, ID_lt, from_integer(array_size, index.type()));
 
-      if(lower_bound.lhs().is_nil() ||
-         upper_bound.rhs().is_nil())
-        throw "number conversion failed (2)";
-
-      and_exprt range_condition(lower_bound, upper_bound);
-      implies_exprt implication(range_condition, value_equality);
+      and_exprt range_condition(std::move(lower_bound), std::move(upper_bound));
+      implies_exprt implication(
+        std::move(range_condition), std::move(value_equality));
 
       // Simplify may remove the lower bound if the type
       // is correct.
@@ -147,21 +163,14 @@ bvt boolbvt::convert_index(const index_exprt &expr)
       // Symbol for output
       static int actual_array_counter;  // Temporary hack
 
-      std::string identifier=
-        "__CPROVER_internal_actual_array_"+
-        std::to_string(actual_array_counter++);
+      const std::string identifier = CPROVER_PREFIX "internal_actual_array_" +
+                                     std::to_string(actual_array_counter++);
 
       symbol_exprt result(identifier, expr.type());
       bv = convert_bv(result);
 
       // add implications
 
-      equal_exprt index_equality;
-      index_equality.lhs()=index; // index operand
-
-      equal_exprt value_equality;
-      value_equality.lhs()=result;
-
 #ifdef COMPACT_EQUAL_CONST
       bv_utils.equal_const_register(convert_bv(index));  // Definitely
       bv_utils.equal_const_register(convert_bv(result)); // Maybe
@@ -171,18 +180,15 @@ bvt boolbvt::convert_index(const index_exprt &expr)
 
       for(mp_integer i=0; i<array_size; i=i+1)
       {
-        index_equality.rhs()=from_integer(i, index_equality.lhs().type());
-
-        if(index_equality.rhs().is_nil())
-          throw "number conversion failed (1)";
-
-        assert(it != array.operands().end());
-
-        value_equality.rhs()=*it++;
+        INVARIANT(
+          it != array.operands().end(),
+          "this loop iterates over the array, so `it` shouldn't be increased "
+          "past the array's end");
 
         // Cache comparisons and equalities
-        prop.l_set_to_true(convert(implies_exprt(index_equality,
-                                                 value_equality)));
+        prop.l_set_to_true(convert(implies_exprt(
+          equal_exprt(index, from_integer(i, index.type())),
+          equal_exprt(result, *it++))));
       }
 
       return bv;
@@ -198,10 +204,8 @@ bvt boolbvt::convert_index(const index_exprt &expr)
 
     // get literals for the whole array
 
-    const bvt &array_bv=convert_bv(array);
-
-    if(array_size*width!=array_bv.size())
-      throw "unexpected array size";
+    const bvt &array_bv =
+      convert_bv(array, numeric_cast_v<std::size_t>(array_size * width));
 
     // TODO: maybe a shifter-like construction would be better
     // Would be a lot more compact but propagate worse
@@ -216,9 +220,6 @@ bvt boolbvt::convert_index(const index_exprt &expr)
 
       // add implications
 
-      equal_exprt index_equality;
-      index_equality.lhs()=index; // index operand
-
 #ifdef COMPACT_EQUAL_CONST
       bv_utils.equal_const_register(convert_bv(index));  // Definitely
 #endif
@@ -228,47 +229,41 @@ bvt boolbvt::convert_index(const index_exprt &expr)
 
       for(mp_integer i=0; i<array_size; i=i+1)
       {
-        index_equality.rhs()=from_integer(i, index_equality.lhs().type());
-
-        if(index_equality.rhs().is_nil())
-          throw "number conversion failed (1)";
-
         mp_integer offset=i*width;
 
         for(std::size_t j=0; j<width; j++)
-          equal_bv[j]=prop.lequal(bv[j],
-                             array_bv[integer2size_t(offset+j)]);
+          equal_bv[j] = prop.lequal(
+            bv[j], array_bv[numeric_cast_v<std::size_t>(offset + j)]);
 
-        prop.l_set_to_true(
-          prop.limplies(convert(index_equality), prop.land(equal_bv)));
+        prop.l_set_to_true(prop.limplies(
+          convert(equal_exprt(index, from_integer(i, index.type()))),
+          prop.land(equal_bv)));
       }
     }
     else
     {
       bv.resize(width);
 
-      equal_exprt equality;
-      equality.lhs()=index; // index operand
-
 #ifdef COMPACT_EQUAL_CONST
       bv_utils.equal_const_register(convert_bv(index));  // Definitely
 #endif
 
       typet constant_type=index.type(); // type of index operand
 
-      assert(array_size>0);
+      DATA_INVARIANT(
+        array_size > 0,
+        "non-positive array sizes are forbidden in goto programs");
 
       for(mp_integer i=0; i<array_size; i=i+1)
       {
-        equality.op1()=from_integer(i, constant_type);
-
-        literalt e=convert(equality);
+        literalt e =
+          convert(equal_exprt(index, from_integer(i, constant_type)));
 
         mp_integer offset=i*width;
 
         for(std::size_t j=0; j<width; j++)
         {
-          literalt l=array_bv[integer2size_t(offset+j)];
+          literalt l = array_bv[numeric_cast_v<std::size_t>(offset + j)];
 
           if(i==0) // this initializes bv
             bv[j]=l;
@@ -289,8 +284,7 @@ bvt boolbvt::convert_index(
   const exprt &array,
   const mp_integer &index)
 {
-  const array_typet &array_type=
-    to_array_type(ns.follow(array.type()));
+  const array_typet &array_type = to_array_type(array.type());
 
   std::size_t width=boolbv_width(array_type.subtype());
 
@@ -326,7 +320,7 @@ bvt boolbvt::convert_index(
     // If not there are large improvements possible as above
 
     for(std::size_t i=0; i<width; i++)
-      bv[i]=tmp[integer2size_t(offset+i)];
+      bv[i] = tmp[numeric_cast_v<std::size_t>(offset + i)];
   }
   else if(
     array.id() == ID_member || array.id() == ID_index ||
@@ -337,11 +331,13 @@ bvt boolbvt::convert_index(
     o.build(array, ns);
     CHECK_RETURN(o.offset().id() != ID_unknown);
 
-    const mp_integer subtype_bytes =
+    const auto subtype_bytes_opt =
       pointer_offset_size(array_type.subtype(), ns);
+    CHECK_RETURN(subtype_bytes_opt.has_value());
+
     exprt new_offset = simplify_expr(
       plus_exprt(
-        o.offset(), from_integer(index * subtype_bytes, o.offset().type())),
+        o.offset(), from_integer(index * (*subtype_bytes_opt), o.offset().type())),
       ns);
 
     byte_extract_exprt be(
