@@ -14,23 +14,21 @@ Author: Daniel Kroening, kroening@kroening.com
 
 literalt boolbvt::convert_overflow(const exprt &expr)
 {
-  const exprt::operandst &operands=expr.operands();
-
   if(expr.id()==ID_overflow_plus ||
      expr.id()==ID_overflow_minus)
   {
-    if(operands.size()!=2)
-      throw "operator "+expr.id_string()+" takes two operands";
+    const auto &overflow_expr = to_binary_expr(expr);
 
-    const bvt &bv0=convert_bv(operands[0]);
-    const bvt &bv1=convert_bv(operands[1]);
+    const bvt &bv0 = convert_bv(overflow_expr.lhs());
+    const bvt &bv1 = convert_bv(overflow_expr.rhs());
 
     if(bv0.size()!=bv1.size())
       return SUB::convert_rest(expr);
 
-    bv_utilst::representationt rep=
-      expr.op0().type().id()==ID_signedbv?bv_utilst::representationt::SIGNED:
-                                          bv_utilst::representationt::UNSIGNED;
+    bv_utilst::representationt rep =
+      overflow_expr.lhs().type().id() == ID_signedbv
+        ? bv_utilst::representationt::SIGNED
+        : bv_utilst::representationt::UNSIGNED;
 
     return expr.id()==ID_overflow_minus?
       bv_utils.overflow_sub(bv0, bv1, rep):
@@ -38,27 +36,25 @@ literalt boolbvt::convert_overflow(const exprt &expr)
   }
   else if(expr.id()==ID_overflow_mult)
   {
-    if(operands.size()!=2)
-      throw "operator "+expr.id_string()+" takes two operands";
+    const auto &overflow_expr = to_binary_expr(expr);
 
-    if(operands[0].type().id()!=ID_unsignedbv &&
-       operands[0].type().id()!=ID_signedbv)
+    if(
+      overflow_expr.lhs().type().id() != ID_unsignedbv &&
+      overflow_expr.lhs().type().id() != ID_signedbv)
       return SUB::convert_rest(expr);
 
-    bvt bv0=convert_bv(operands[0]);
-    bvt bv1=convert_bv(operands[1]);
+    bvt bv0 = convert_bv(overflow_expr.lhs());
+    bvt bv1 = convert_bv(overflow_expr.rhs(), bv0.size());
 
-    if(bv0.size()!=bv1.size())
-      throw "operand size mismatch on overflow-*";
+    bv_utilst::representationt rep =
+      overflow_expr.lhs().type().id() == ID_signedbv
+        ? bv_utilst::representationt::SIGNED
+        : bv_utilst::representationt::UNSIGNED;
 
-    bv_utilst::representationt rep=
-      operands[0].type().id()==ID_signedbv?bv_utilst::representationt::SIGNED:
-                                           bv_utilst::representationt::UNSIGNED;
+    DATA_INVARIANT(
+      overflow_expr.lhs().type() == overflow_expr.rhs().type(),
+      "operands of overflow_mult expression shall have same type");
 
-    if(operands[0].type()!=operands[1].type())
-      throw "operand type mismatch on overflow-*";
-
-    DATA_INVARIANT(bv0.size()==bv1.size(), "operands size mismatch");
     std::size_t old_size=bv0.size();
     std::size_t new_size=old_size*2;
 
@@ -97,28 +93,27 @@ literalt boolbvt::convert_overflow(const exprt &expr)
   }
   else if(expr.id() == ID_overflow_shl)
   {
-    if(operands.size() != 2)
-      throw "operator " + expr.id_string() + " takes two operands";
+    const auto &overflow_expr = to_binary_expr(expr);
 
-    const bvt &bv0=convert_bv(operands[0]);
-    const bvt &bv1=convert_bv(operands[1]);
+    const bvt &bv0 = convert_bv(overflow_expr.lhs());
+    const bvt &bv1 = convert_bv(overflow_expr.rhs());
 
     std::size_t old_size = bv0.size();
     std::size_t new_size = old_size * 2;
 
-    bv_utilst::representationt rep=
-      operands[0].type().id()==ID_signedbv?bv_utilst::representationt::SIGNED:
-                                           bv_utilst::representationt::UNSIGNED;
+    bv_utilst::representationt rep =
+      overflow_expr.lhs().type().id() == ID_signedbv
+        ? bv_utilst::representationt::SIGNED
+        : bv_utilst::representationt::UNSIGNED;
 
     bvt bv_ext=bv_utils.extension(bv0, new_size, rep);
 
     bvt result=bv_utils.shift(bv_ext, bv_utilst::shiftt::SHIFT_LEFT, bv1);
 
     // a negative shift is undefined; yet this isn't an overflow
-    literalt neg_shift =
-      operands[1].type().id() == ID_unsignedbv ?
-        const_literal(false) :
-        bv1.back(); // sign bit
+    literalt neg_shift = overflow_expr.lhs().type().id() == ID_unsignedbv
+                           ? const_literal(false)
+                           : bv1.back(); // sign bit
 
     // an undefined shift of a non-zero value always results in overflow; the
     // use of unsigned comparision is safe here as we cover the signed negative
@@ -163,47 +158,12 @@ literalt boolbvt::convert_overflow(const exprt &expr)
   }
   else if(expr.id()==ID_overflow_unary_minus)
   {
-    if(operands.size()!=1)
-      throw "operator "+expr.id_string()+" takes one operand";
+    const auto &overflow_expr = to_unary_expr(expr);
 
-    const bvt &bv=convert_bv(operands[0]);
+    const bvt &bv = convert_bv(overflow_expr.op());
 
     return bv_utils.overflow_negate(bv);
   }
-  else if(has_prefix(expr.id_string(), "overflow-typecast-"))
-  {
-    std::size_t bits=unsafe_string2unsigned(id2string(expr.id()).substr(18));
-
-    const exprt::operandst &operands=expr.operands();
-
-    if(operands.size()!=1)
-      throw "operator "+expr.id_string()+" takes one operand";
-
-    const exprt &op=operands[0];
-
-    const bvt &bv=convert_bv(op);
-
-    if(bits>=bv.size() || bits==0)
-      throw "overflow-typecast got wrong number of bits";
-
-    // signed or unsigned?
-    if(op.type().id()==ID_signedbv)
-    {
-      bvt tmp_bv;
-      for(std::size_t i=bits; i<bv.size(); i++)
-        tmp_bv.push_back(prop.lxor(bv[bits-1], bv[i]));
-
-      return prop.lor(tmp_bv);
-    }
-    else
-    {
-      bvt tmp_bv;
-      for(std::size_t i=bits; i<bv.size(); i++)
-        tmp_bv.push_back(bv[i]);
-
-      return prop.lor(tmp_bv);
-    }
-  }
 
   return SUB::convert_rest(expr);
 }
