@@ -23,8 +23,7 @@ class pointer_logict
 {
 public:
   // this numbers the objects
-  typedef hash_numbering<exprt, irep_hash> objectst;
-  objectst objects;
+  hash_numbering<exprt, irep_hash> objects;
 
   struct pointert
   {
@@ -40,12 +39,12 @@ public:
     }
   };
 
-  // converts an (object,offset) pair to an expression
+  /// Convert an (object,offset) pair to an expression
   exprt pointer_expr(
     const pointert &pointer,
     const pointer_typet &type) const;
 
-  // converts an (object,0) pair to an expression
+  /// Convert an (object,0) pair to an expression
   exprt pointer_expr(
     std::size_t object,
     const pointer_typet &type) const;
@@ -55,13 +54,13 @@ public:
 
   std::size_t add_object(const exprt &expr);
 
-  // number of NULL object
+  /// \return number of NULL object
   std::size_t get_null_object() const
   {
     return null_object;
   }
 
-  // number of INVALID object
+  /// \return number of INVALID object
   std::size_t get_invalid_object() const
   {
     return invalid_object;
@@ -74,15 +73,6 @@ public:
 protected:
   const namespacet &ns;
   std::size_t null_object, invalid_object;
-
-  exprt pointer_expr(
-    const mp_integer &offset,
-    const exprt &object) const;
-
-  exprt object_rec(
-    const mp_integer &offset,
-    const typet &pointer_type,
-    const exprt &src) const;
 };
 
 #endif // CPROVER_SOLVERS_FLATTENING_POINTER_LOGIC_H
