@@ -57,7 +57,9 @@ public:
         round_to_zero=const_literal(true);
         break;
 
-      default: assert(false);
+      case ieee_floatt::NONDETERMINISTIC:
+      case ieee_floatt::UNKNOWN:
+        UNREACHABLE;
       }
     }
   };
