@@ -11,6 +11,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include "cover_goals.h"
 
+#include <util/message.h>
 #include <util/threeval.h>
 
 #include "literal_expr.h"
@@ -27,8 +28,9 @@ void cover_goalst::mark()
     o->satisfying_assignment();
 
   for(auto &g : goals)
-    if(g.status==goalt::statust::UNKNOWN &&
-       prop_conv.l_get(g.condition).is_true())
+    if(
+      g.status == goalt::statust::UNKNOWN &&
+      decision_procedure.get(g.condition).is_true())
     {
       g.status=goalt::statust::COVERED;
       _number_covered++;
@@ -46,47 +48,29 @@ void cover_goalst::constraint()
 
   // cover at least one unknown goal
 
-  for(std::list<goalt>::const_iterator
-      g_it=goals.begin();
-      g_it!=goals.end();
-      g_it++)
-    if(g_it->status==goalt::statust::UNKNOWN &&
-       !g_it->condition.is_false())
-      disjuncts.push_back(literal_exprt(g_it->condition));
+  for(const auto &g : goals)
+    if(g.status == goalt::statust::UNKNOWN && !g.condition.is_false())
+      disjuncts.push_back(g.condition);
 
   // this is 'false' if there are no disjuncts
-  prop_conv.set_to_true(disjunction(disjuncts));
-}
-
-/// Build clause
-void cover_goalst::freeze_goal_variables()
-{
-  for(std::list<goalt>::const_iterator
-      g_it=goals.begin();
-      g_it!=goals.end();
-      g_it++)
-    if(!g_it->condition.is_constant())
-      prop_conv.set_frozen(g_it->condition);
+  decision_procedure.set_to_true(disjunction(disjuncts));
 }
 
 /// Try to cover all goals
-decision_proceduret::resultt cover_goalst::operator()()
+decision_proceduret::resultt cover_goalst::
+operator()(message_handlert &message_handler)
 {
   _iterations=_number_covered=0;
 
   decision_proceduret::resultt dec_result;
 
-  // We use incremental solving, so need to freeze some variables
-  // to prevent them from being eliminated.
-  freeze_goal_variables();
-
   do
   {
     // We want (at least) one of the remaining goals, please!
     _iterations++;
 
     constraint();
-    dec_result=prop_conv.dec_solve();
+    dec_result = decision_procedure();
 
     switch(dec_result)
     {
@@ -98,10 +82,13 @@ decision_proceduret::resultt cover_goalst::operator()()
       mark();
       break;
 
-    default:
-      error() << "decision procedure has failed" << eom;
+    case decision_proceduret::resultt::D_ERROR:
+    {
+      messaget log(message_handler);
+      log.error() << "decision procedure has failed" << messaget::eom;
       return dec_result;
     }
+    }
   }
   while(dec_result==decision_proceduret::resultt::D_SATISFIABLE &&
         number_covered()<size());
