@@ -36,28 +36,26 @@ public:
   }
 };
 
-/*! \brief Cast a generic exprt to a \ref literal_exprt
- *
- * This is an unchecked conversion. \a expr must be known to be \ref
- * literal_exprt.
- *
- * \param expr Source expression
- * \return Object of type \ref literal_exprt
- *
- * \ingroup gr_std_expr
-*/
+/// Cast a generic exprt to a \ref literal_exprt. This is an unchecked
+/// conversion. \a expr must be known to be \ref literal_exprt.
+/// \param expr: Source expression
+/// \return Object of type \ref literal_exprt
+/// \ingroup gr_std_expr
 inline const literal_exprt &to_literal_expr(const exprt &expr)
 {
-  assert(expr.id()==ID_literal && !expr.has_operands());
+  PRECONDITION(expr.id() == ID_literal);
+  DATA_INVARIANT(
+    !expr.has_operands(), "literal expression should not have operands");
   return static_cast<const literal_exprt &>(expr);
 }
 
-/*! \copydoc to_literal_expr(const exprt &)
- * \ingroup gr_std_expr
-*/
+/// \copydoc to_literal_expr(const exprt &)
+/// \ingroup gr_std_expr
 inline literal_exprt &to_literal_expr(exprt &expr)
 {
-  assert(expr.id()==ID_literal && !expr.has_operands());
+  PRECONDITION(expr.id() == ID_literal);
+  DATA_INVARIANT(
+    !expr.has_operands(), "literal expression should not have operands");
   return static_cast<literal_exprt &>(expr);
 }
 
