@@ -8,14 +8,15 @@ Author: CM Wintersteiger
 
 #include "qbf_qube_core.h"
 
-#include <cassert>
 #include <cstdlib>
-#include <fstream>
 #include <cstring>
+#include <fstream>
 
-#include <util/mp_arith.h>
+#include <util/arith_tools.h>
+#include <util/invariant.h>
 
-qbf_qube_coret::qbf_qube_coret() : qdimacs_coret()
+qbf_qube_coret::qbf_qube_coret(message_handlert &message_handler)
+  : qdimacs_coret(message_handler)
 {
   break_lines=false;
   qbf_tmp_file="qube.qdimacs";
@@ -36,9 +37,8 @@ propt::resultt qbf_qube_coret::prop_solve()
     return resultt::P_SATISFIABLE;
 
   {
-    messaget::status() << "QuBE: "
-      << no_variables() << " variables, "
-      << no_clauses() << " clauses" << eom;
+    log.status() << "QuBE: " << no_variables() << " variables, " << no_clauses()
+                 << " clauses" << messaget::eom;
   }
 
   std::string result_tmp_file="qube.out";
@@ -51,7 +51,7 @@ propt::resultt qbf_qube_coret::prop_solve()
     write_qdimacs_cnf(out);
   }
 
-  std::string options="";
+  std::string options;
 
   // solve it
   int res=system((
@@ -71,16 +71,16 @@ propt::resultt qbf_qube_coret::prop_solve()
 
       std::getline(in, line);
 
-      if(line!="" && line[line.size()-1]=='\r')
+      if(!line.empty() && line[line.size() - 1] == '\r')
         line.resize(line.size()-1);
 
       if(line[0]=='V')
       {
         mp_integer b(line.substr(2).c_str());
         if(b<0)
-          assignment[integer2unsigned(b.negate())]=false;
+          assignment[numeric_cast_v<std::size_t>(b.negate())] = false;
         else
-          assignment[integer2unsigned(b)]=true;
+          assignment[numeric_cast_v<std::size_t>(b)] = true;
       }
       else if(line=="s cnf 1")
       {
@@ -98,7 +98,7 @@ propt::resultt qbf_qube_coret::prop_solve()
 
     if(!result_found)
     {
-      messaget::error() << "QuBE failed: unknown result" << eom;
+      log.error() << "QuBE failed: unknown result" << messaget::eom;
       return resultt::P_ERROR;
     }
   }
@@ -106,35 +106,35 @@ propt::resultt qbf_qube_coret::prop_solve()
   int remove_result=remove(result_tmp_file.c_str());
   if(remove_result!=0)
   {
-    messaget::error() << "Remove failed: " << std::strerror(errno) << eom;
+    log.error() << "Remove failed: " << std::strerror(errno) << messaget::eom;
     return resultt::P_ERROR;
   }
 
   remove_result=remove(qbf_tmp_file.c_str());
   if(remove_result!=0)
   {
-    messaget::error() << "Remove failed: " << std::strerror(errno) << eom;
+    log.error() << "Remove failed: " << std::strerror(errno) << messaget::eom;
     return resultt::P_ERROR;
   }
 
   if(result)
   {
-    messaget::status() << "QuBE: TRUE" << eom;
+    log.status() << "QuBE: TRUE" << messaget::eom;
     return resultt::P_SATISFIABLE;
   }
   else
   {
-    messaget::status() << "QuBE: FALSE" << eom;
+    log.status() << "QuBE: FALSE" << messaget::eom;
     return resultt::P_UNSATISFIABLE;
   }
 }
 
 bool qbf_qube_coret::is_in_core(literalt) const
 {
-  throw "not supported";
+  UNIMPLEMENTED;
 }
 
 qdimacs_coret::modeltypet qbf_qube_coret::m_get(literalt) const
 {
-  throw "not supported";
+  UNIMPLEMENTED;
 }
