@@ -78,7 +78,7 @@ void satcheck_minisat2_baset<T>::set_polarity(literalt a, bool value)
   }
   catch(Minisat::OutOfMemoryException)
   {
-    messaget::error() << "SAT checker ran out of memory" << eom;
+    log.error() << "SAT checker ran out of memory" << messaget::eom;
     status = statust::ERROR;
     throw std::bad_alloc();
   }
@@ -142,9 +142,9 @@ void satcheck_minisat2_baset<T>::lcnf(const bvt &bv)
 
     clause_counter++;
   }
-  catch(Minisat::OutOfMemoryException)
+  catch(const Minisat::OutOfMemoryException &)
   {
-    messaget::error() << "SAT checker ran out of memory" << eom;
+    log.error() << "SAT checker ran out of memory" << messaget::eom;
     status = statust::ERROR;
     throw std::bad_alloc();
   }
@@ -156,21 +156,19 @@ static Minisat::Solver *solver_to_interrupt=nullptr;
 
 static void interrupt_solver(int signum)
 {
+  (void)signum; // unused parameter -- just removing the name trips up cpplint
   solver_to_interrupt->interrupt();
 }
 
 #endif
 
-template<typename T>
-propt::resultt satcheck_minisat2_baset<T>::prop_solve()
+template <typename T>
+propt::resultt satcheck_minisat2_baset<T>::do_prop_solve()
 {
   PRECONDITION(status != statust::ERROR);
 
-  {
-    messaget::status() <<
-      (no_variables()-1) << " variables, " <<
-      solver->nClauses() << " clauses" << eom;
-  }
+  log.statistics() << (no_variables() - 1) << " variables, "
+                   << solver->nClauses() << " clauses" << messaget::eom;
 
   try
   {
@@ -178,96 +176,86 @@ propt::resultt satcheck_minisat2_baset<T>::prop_solve()
 
     if(!solver->okay())
     {
-      messaget::status() <<
-        "SAT checker inconsistent: instance is UNSATISFIABLE" << eom;
+      log.status() << "SAT checker inconsistent: instance is UNSATISFIABLE"
+                   << messaget::eom;
+      status = statust::UNSAT;
+      return resultt::P_UNSATISFIABLE;
     }
-    else
+
+    // if assumptions contains false, we need this to be UNSAT
+    for(const auto &assumption : assumptions)
     {
-      // if assumptions contains false, we need this to be UNSAT
-      bool has_false=false;
-
-      forall_literals(it, assumptions)
-        if(it->is_false())
-          has_false=true;
-
-      if(has_false)
+      if(assumption.is_false())
       {
-        messaget::status() <<
-          "got FALSE as assumption: instance is UNSATISFIABLE" << eom;
+        log.status() << "got FALSE as assumption: instance is UNSATISFIABLE"
+                     << messaget::eom;
+        status = statust::UNSAT;
+        return resultt::P_UNSATISFIABLE;
       }
-      else
-      {
-        Minisat::vec<Minisat::Lit> solver_assumptions;
-        convert(assumptions, solver_assumptions);
+    }
 
-        using Minisat::lbool;
+    Minisat::vec<Minisat::Lit> solver_assumptions;
+    convert(assumptions, solver_assumptions);
+
+    using Minisat::lbool;
 
 #ifndef _WIN32
 
-        void (*old_handler)(int)=SIG_ERR;
+    void (*old_handler)(int) = SIG_ERR;
 
-        if(time_limit_seconds!=0)
-        {
-          solver_to_interrupt=solver;
-          old_handler=signal(SIGALRM, interrupt_solver);
-          if(old_handler==SIG_ERR)
-            warning() << "Failed to set solver time limit" << eom;
-          else
-            alarm(time_limit_seconds);
-        }
+    if(time_limit_seconds != 0)
+    {
+      solver_to_interrupt = solver;
+      old_handler = signal(SIGALRM, interrupt_solver);
+      if(old_handler == SIG_ERR)
+        log.warning() << "Failed to set solver time limit" << messaget::eom;
+      else
+        alarm(time_limit_seconds);
+    }
 
-        lbool solver_result=solver->solveLimited(solver_assumptions);
+    lbool solver_result = solver->solveLimited(solver_assumptions);
 
-        if(old_handler!=SIG_ERR)
-        {
-          alarm(0);
-          signal(SIGALRM, old_handler);
-          solver_to_interrupt=solver;
-        }
+    if(old_handler != SIG_ERR)
+    {
+      alarm(0);
+      signal(SIGALRM, old_handler);
+      solver_to_interrupt = solver;
+    }
 
 #else // _WIN32
 
-        if(time_limit_seconds!=0)
-        {
-          messaget::warning() <<
-            "Time limit ignored (not supported on Win32 yet)" << messaget::eom;
-        }
+    if(time_limit_seconds != 0)
+    {
+      log.warning() << "Time limit ignored (not supported on Win32 yet)"
+                    << messaget::eom;
+    }
 
-        lbool solver_result=
-          solver->solve(solver_assumptions) ? l_True : l_False;
+    lbool solver_result = solver->solve(solver_assumptions) ? l_True : l_False;
 
 #endif
 
-        if(solver_result==l_True)
-        {
-          messaget::status() <<
-            "SAT checker: instance is SATISFIABLE" << eom;
-          CHECK_RETURN(solver->model.size()>0);
-          status=statust::SAT;
-          return resultt::P_SATISFIABLE;
-        }
-        else if(solver_result==l_False)
-        {
-          messaget::status() <<
-            "SAT checker: instance is UNSATISFIABLE" << eom;
-        }
-        else
-        {
-          messaget::status() <<
-            "SAT checker: timed out or other error" << eom;
-          status=statust::ERROR;
-          return resultt::P_ERROR;
-        }
-      }
+    if(solver_result == l_True)
+    {
+      log.status() << "SAT checker: instance is SATISFIABLE" << messaget::eom;
+      CHECK_RETURN(solver->model.size() > 0);
+      status = statust::SAT;
+      return resultt::P_SATISFIABLE;
     }
 
-    status=statust::UNSAT;
-    return resultt::P_UNSATISFIABLE;
+    if(solver_result == l_False)
+    {
+      log.status() << "SAT checker: instance is UNSATISFIABLE" << messaget::eom;
+      status = statust::UNSAT;
+      return resultt::P_UNSATISFIABLE;
+    }
+
+    log.status() << "SAT checker: timed out or other error" << messaget::eom;
+    status = statust::ERROR;
+    return resultt::P_ERROR;
   }
-  catch(Minisat::OutOfMemoryException)
+  catch(const Minisat::OutOfMemoryException &)
   {
-    messaget::error() <<
-      "SAT checker ran out of memory" << eom;
+    log.error() << "SAT checker ran out of memory" << messaget::eom;
     status=statust::ERROR;
     return resultt::P_ERROR;
   }
@@ -288,17 +276,19 @@ void satcheck_minisat2_baset<T>::set_assignment(literalt a, bool value)
     value ^= sign;
     solver->model[v] = Minisat::lbool(value);
   }
-  catch(Minisat::OutOfMemoryException)
+  catch(const Minisat::OutOfMemoryException &)
   {
-    messaget::error() << "SAT checker ran out of memory" << eom;
+    log.error() << "SAT checker ran out of memory" << messaget::eom;
     status = statust::ERROR;
     throw std::bad_alloc();
   }
 }
 
-template<typename T>
-satcheck_minisat2_baset<T>::satcheck_minisat2_baset(T *_solver):
-  solver(_solver), time_limit_seconds(0)
+template <typename T>
+satcheck_minisat2_baset<T>::satcheck_minisat2_baset(
+  T *_solver,
+  message_handlert &message_handler)
+  : cnf_solvert(message_handler), solver(_solver), time_limit_seconds(0)
 {
 }
 
@@ -329,23 +319,31 @@ bool satcheck_minisat2_baset<T>::is_in_conflict(literalt a) const
 template<typename T>
 void satcheck_minisat2_baset<T>::set_assumptions(const bvt &bv)
 {
-  assumptions=bv;
-
-  forall_literals(it, assumptions)
-    if(it->is_true())
+  // We filter out 'true' assumptions which cause an assertion violation
+  // in Minisat2.
+  assumptions.clear();
+  for(const auto &assumption : bv)
+  {
+    if(!assumption.is_true())
     {
-      assumptions.clear();
-      break;
+      assumptions.push_back(assumption);
     }
+  }
 }
 
-satcheck_minisat_no_simplifiert::satcheck_minisat_no_simplifiert():
-  satcheck_minisat2_baset<Minisat::Solver>(new Minisat::Solver)
+satcheck_minisat_no_simplifiert::satcheck_minisat_no_simplifiert(
+  message_handlert &message_handler)
+  : satcheck_minisat2_baset<Minisat::Solver>(
+      new Minisat::Solver,
+      message_handler)
 {
 }
 
-satcheck_minisat_simplifiert::satcheck_minisat_simplifiert():
-  satcheck_minisat2_baset<Minisat::SimpSolver>(new Minisat::SimpSolver)
+satcheck_minisat_simplifiert::satcheck_minisat_simplifiert(
+  message_handlert &message_handler)
+  : satcheck_minisat2_baset<Minisat::SimpSolver>(
+      new Minisat::SimpSolver,
+      message_handler)
 {
 }
 
@@ -359,9 +357,9 @@ void satcheck_minisat_simplifiert::set_frozen(literalt a)
       solver->setFrozen(a.var_no(), true);
     }
   }
-  catch(Minisat::OutOfMemoryException)
+  catch(const Minisat::OutOfMemoryException &)
   {
-    messaget::error() << "SAT checker ran out of memory" << eom;
+    log.error() << "SAT checker ran out of memory" << messaget::eom;
     status = statust::ERROR;
     throw std::bad_alloc();
   }
