@@ -27,17 +27,16 @@ template<typename T>
 class satcheck_minisat2_baset:public cnf_solvert
 {
 public:
-  explicit satcheck_minisat2_baset(T *);
+  satcheck_minisat2_baset(T *, message_handlert &message_handler);
   virtual ~satcheck_minisat2_baset();
 
-  virtual resultt prop_solve() override;
-  virtual tvt l_get(literalt a) const final;
+  tvt l_get(literalt a) const override final;
 
-  virtual void lcnf(const bvt &bv) final;
-  virtual void set_assignment(literalt a, bool value) override;
+  void lcnf(const bvt &bv) override final;
+  void set_assignment(literalt a, bool value) override;
 
   // extra MiniSat feature: solve with assumptions
-  virtual void set_assumptions(const bvt &_assumptions) override;
+  void set_assumptions(const bvt &_assumptions) override;
 
   // extra MiniSat feature: default branching decision
   void set_polarity(literalt a, bool value);
@@ -48,9 +47,15 @@ public:
   // extra MiniSat feature: permit previously interrupted SAT query to continue
   void clear_interrupt();
 
-  virtual bool is_in_conflict(literalt a) const override;
-  virtual bool has_set_assumptions() const final { return true; }
-  virtual bool has_is_in_conflict() const final { return true; }
+  bool is_in_conflict(literalt a) const override;
+  bool has_set_assumptions() const override final
+  {
+    return true;
+  }
+  bool has_is_in_conflict() const override final
+  {
+    return true;
+  }
 
   void set_time_limit_seconds(uint32_t lim) override
   {
@@ -58,6 +63,8 @@ public:
   }
 
 protected:
+  resultt do_prop_solve() override;
+
   T *solver;
   uint32_t time_limit_seconds;
 
@@ -69,17 +76,17 @@ class satcheck_minisat_no_simplifiert:
   public satcheck_minisat2_baset<Minisat::Solver>
 {
 public:
-  satcheck_minisat_no_simplifiert();
-  virtual const std::string solver_text();
+  explicit satcheck_minisat_no_simplifiert(message_handlert &message_handler);
+  const std::string solver_text() override;
 };
 
 class satcheck_minisat_simplifiert:
   public satcheck_minisat2_baset<Minisat::SimpSolver>
 {
 public:
-  satcheck_minisat_simplifiert();
-  virtual const std::string solver_text() final;
-  virtual void set_frozen(literalt a) final;
+  explicit satcheck_minisat_simplifiert(message_handlert &message_handler);
+  const std::string solver_text() override final;
+  void set_frozen(literalt a) override final;
   bool is_eliminated(literalt a) const;
 };
 
