@@ -10,19 +10,11 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_SOLVERS_SMT2_SMT2_DEC_H
 #define CPROVER_SOLVERS_SMT2_SMT2_DEC_H
 
-#include <fstream>
-
 #include "smt2_conv.h"
 
-class smt2_temp_filet
-{
-public:
-  smt2_temp_filet();
-  ~smt2_temp_filet();
+#include <util/message.h>
 
-  std::ofstream temp_out;
-  std::string temp_out_filename, temp_result_filename;
-};
+#include <fstream>
 
 class smt2_stringstreamt
 {
@@ -32,7 +24,9 @@ protected:
 
 /*! \brief Decision procedure interface for various SMT 2.x solvers
 */
-class smt2_dect:protected smt2_stringstreamt, public smt2_convt
+class smt2_dect : protected smt2_stringstreamt,
+                  public smt2_convt,
+                  public messaget
 {
 public:
   smt2_dect(
@@ -45,11 +39,8 @@ public:
   {
   }
 
-  virtual resultt dec_solve();
-  virtual std::string decision_procedure_text() const;
-
-  // yes, we are incremental!
-  virtual bool has_set_assumptions() const { return true; }
+  resultt dec_solve() override;
+  std::string decision_procedure_text() const override;
 
 protected:
   resultt read_result(std::istream &in);
