@@ -12,9 +12,10 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "array_name.h"
 
 #include "expr.h"
+#include "invariant.h"
 #include "namespace.h"
-#include "symbol.h"
 #include "ssa_expr.h"
+#include "symbol.h"
 
 std::string array_name(
   const namespacet &ns,
@@ -22,21 +23,18 @@ std::string array_name(
 {
   if(expr.id()==ID_index)
   {
-    if(expr.operands().size()!=2)
-      throw "index takes two operands";
-
-    return array_name(ns, expr.op0())+"[]";
+    return array_name(ns, to_index_expr(expr).array()) + "[]";
   }
   else if(is_ssa_expr(expr))
   {
     const symbolt &symbol=
       ns.lookup(to_ssa_expr(expr).get_object_name());
-    return "array `"+id2string(symbol.base_name)+"'";
+    return "array '" + id2string(symbol.base_name) + "'";
   }
   else if(expr.id()==ID_symbol)
   {
     const symbolt &symbol = ns.lookup(to_symbol_expr(expr));
-    return "array `"+id2string(symbol.base_name)+"'";
+    return "array '" + id2string(symbol.base_name) + "'";
   }
   else if(expr.id()==ID_string_constant)
   {
@@ -44,9 +42,10 @@ std::string array_name(
   }
   else if(expr.id()==ID_member)
   {
-    assert(expr.operands().size()==1);
-    return array_name(ns, expr.op0())+"."+
-           expr.get_string(ID_component_name);
+    const member_exprt &member_expr = to_member_expr(expr);
+
+    return array_name(ns, member_expr.compound()) + "." +
+           id2string(member_expr.get_component_name());
   }
 
   return "array";
