@@ -11,10 +11,20 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <iostream>
 
 #ifdef _WIN32
+#include <util/pragma_push.def>
+#ifdef _MSC_VER
+#pragma warning(disable:4668)
+  // using #if/#elif on undefined macro
+#pragma warning(disable : 5039)
+// pointer or reference to potentially throwing function passed to extern C
+#endif
 #include <windows.h>
 #include <fcntl.h>
 #include <io.h>
 #include <cstdio>
+#include <util/pragma_pop.def>
+#else
+#include <unistd.h>
 #endif
 
 #include "unicode.h"
@@ -29,6 +39,40 @@ cerr_message_handlert::cerr_message_handlert():
 {
 }
 
+console_message_handlert::console_message_handlert(bool _always_flush)
+  : always_flush(_always_flush), is_a_tty(false), use_SGR(false)
+{
+#ifdef _WIN32
+  HANDLE out_handle=GetStdHandle(STD_OUTPUT_HANDLE);
+
+  DWORD consoleMode;
+  if(GetConsoleMode(out_handle, &consoleMode))
+  {
+    is_a_tty = true;
+
+#ifdef ENABLE_VIRTUAL_TERMINAL_PROCESSING
+    consoleMode |= ENABLE_VIRTUAL_TERMINAL_PROCESSING;
+    if(SetConsoleMode(out_handle, consoleMode))
+      use_SGR = true;
+#endif
+  }
+#else
+  is_a_tty = isatty(STDOUT_FILENO);
+  use_SGR = is_a_tty;
+#endif
+}
+
+/// Create an ECMA-48 SGR (Select Graphic Rendition) command with
+/// given code.
+/// \param c: ECMA-48 command code
+std::string console_message_handlert::command(unsigned c) const
+{
+  if(!use_SGR)
+    return std::string();
+
+  return "\x1b[" + std::to_string(c) + 'm';
+}
+
 void console_message_handlert::print(
   unsigned level,
   const std::string &message)
@@ -96,59 +140,3 @@ void console_message_handlert::flush(unsigned level)
   else
     std::cerr << std::flush;
 }
-
-void gcc_message_handlert::print(
-  unsigned level,
-  const std::string &message,
-  int,
-  const source_locationt &location)
-{
-  const irep_idt file=location.get_file();
-  const irep_idt line=location.get_line();
-  const irep_idt column=location.get_column();
-  const irep_idt function=location.get_function();
-
-  std::string dest;
-
-  if(!function.empty())
-  {
-    if(!file.empty())
-      dest+=id2string(file)+":";
-    if(dest!="")
-      dest+=' ';
-    dest+="In function '"+id2string(function)+"':\n";
-  }
-
-  if(!line.empty())
-  {
-    if(!file.empty())
-      dest+=id2string(file)+":";
-
-    dest+=id2string(line)+":";
-
-    if(column.empty())
-      dest+="1: ";
-    else
-      dest+=id2string(column)+": ";
-
-    if(level==messaget::M_ERROR)
-      dest+="error: ";
-    else if(level==messaget::M_WARNING)
-      dest+="warning: ";
-  }
-
-  dest+=message;
-
-  print(level, dest);
-}
-
-void gcc_message_handlert::print(
-  unsigned level,
-  const std::string &message)
-{
-  message_handlert::print(level, message);
-
-  // gcc appears to send everything to cerr
-  if(verbosity>=level)
-    std::cerr << message << '\n' << std::flush;
-}
