@@ -10,7 +10,6 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <ostream>
 
-#include "invariant.h"
 #include "std_types.h"
 #include "pointer_offset_size.h"
 #include "arith_tools.h"
@@ -38,11 +37,12 @@ void endianness_mapt::build(const typet &src, bool little_endian)
 
 void endianness_mapt::build_little_endian(const typet &src)
 {
-  mp_integer s=pointer_offset_bits(src, ns); // error is -1
-  if(s<=0)
+  auto s = pointer_offset_bits(src, ns);
+
+  if(!s.has_value())
     return;
 
-  std::size_t new_size=map.size()+integer2size_t(s);
+  const std::size_t new_size = map.size() + numeric_cast_v<std::size_t>(*s);
   map.reserve(new_size);
 
   for(std::size_t i=map.size(); i<new_size; ++i)
@@ -51,9 +51,7 @@ void endianness_mapt::build_little_endian(const typet &src)
 
 void endianness_mapt::build_big_endian(const typet &src)
 {
-  if(src.id()==ID_symbol)
-    build_big_endian(ns.follow(src));
-  else if(src.id()==ID_c_enum_tag)
+  if(src.id() == ID_c_enum_tag)
     build_big_endian(ns.follow_tag(to_c_enum_tag_type(src)));
   else if(src.id()==ID_unsignedbv ||
           src.id()==ID_signedbv ||
@@ -63,10 +61,11 @@ void endianness_mapt::build_big_endian(const typet &src)
           src.id()==ID_c_bit_field)
   {
     // these do get re-ordered!
-    mp_integer bits=pointer_offset_bits(src, ns); // error is -1
-    CHECK_RETURN(bits>=0);
+    auto bits = pointer_offset_bits(src, ns); // error is -1
+    CHECK_RETURN(bits.has_value());
 
-    size_t bits_int=integer2size_t(bits), base=map.size();
+    const std::size_t bits_int = numeric_cast_v<std::size_t>(*bits);
+    const std::size_t base = map.size();
 
     for(size_t bit=0; bit<bits_int; bit++)
     {
@@ -78,26 +77,27 @@ void endianness_mapt::build_big_endian(const typet &src)
     const struct_typet &struct_type=to_struct_type(src);
 
     // todo: worry about padding being in wrong order
-    for(struct_typet::componentst::const_iterator
-        it=struct_type.components().begin();
-        it!=struct_type.components().end();
-        it++)
+    for(const auto &c : struct_type.components())
     {
-      build_big_endian(it->type());
+      build_big_endian(c.type());
     }
   }
+  else if(src.id() == ID_struct_tag)
+  {
+    build_big_endian(ns.follow_tag(to_struct_tag_type(src)));
+  }
   else if(src.id()==ID_array)
   {
     const array_typet &array_type=to_array_type(src);
 
     // array size constant?
-    mp_integer s;
-    if(!to_integer(array_type.size(), s))
+    auto s = numeric_cast<mp_integer>(array_type.size());
+    if(s.has_value())
     {
-      while(s>0)
+      while(*s > 0)
       {
         build_big_endian(array_type.subtype());
-        --s;
+        --(*s);
       }
     }
   }
@@ -105,11 +105,9 @@ void endianness_mapt::build_big_endian(const typet &src)
   {
     const vector_typet &vector_type=to_vector_type(src);
 
-    mp_integer s;
-    if(to_integer(vector_type.size(), s))
-      CHECK_RETURN(false);
+    mp_integer s = numeric_cast_v<mp_integer>(vector_type.size());
 
-    while(s>0)
+    while(s > 0)
     {
       build_big_endian(vector_type.subtype());
       --s;
@@ -119,11 +117,12 @@ void endianness_mapt::build_big_endian(const typet &src)
   {
     // everything else (unions in particular)
     // is treated like a byte-array
-    mp_integer s=pointer_offset_bits(src, ns); // error is -1
-    if(s<=0)
+    auto s = pointer_offset_bits(src, ns); // error is -1
+
+    if(!s.has_value())
       return;
 
-    std::size_t new_size=map.size()+integer2size_t(s);
+    const std::size_t new_size = map.size() + numeric_cast_v<std::size_t>(*s);
     map.reserve(new_size);
 
     for(std::size_t i=map.size(); i<new_size; ++i)
