@@ -6,10 +6,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-
 #include "expr_util.h"
 
+#include <algorithm>
 #include <unordered_set>
+
 #include "expr.h"
 #include "expr_iterator.h"
 #include "fixedbv.h"
@@ -55,8 +56,8 @@ exprt make_binary(const exprt &expr)
     exprt tmp=expr;
     tmp.operands().clear();
     tmp.operands().resize(2);
-    tmp.op0().swap(previous);
-    tmp.op1()=*it;
+    to_binary_expr(tmp).op0().swap(previous);
+    to_binary_expr(tmp).op1() = *it;
     previous.swap(tmp);
   }
 
@@ -68,12 +69,12 @@ with_exprt make_with_expr(const update_exprt &src)
   const exprt::operandst &designator=src.designator();
   PRECONDITION(!designator.empty());
 
-  with_exprt result;
+  with_exprt result{exprt{}, exprt{}, exprt{}};
   exprt *dest=&result;
 
   forall_expr(it, designator)
   {
-    with_exprt tmp;
+    with_exprt tmp{exprt{}, exprt{}, exprt{}};
 
     if(it->id()==ID_index_designator)
     {
@@ -103,10 +104,9 @@ exprt is_not_zero(
   // Note that this returns a proper bool_typet(), not a C/C++ boolean.
   // To get a C/C++ boolean, add a further typecast.
 
-  const typet &src_type=
-    src.type().id()==ID_c_enum_tag?
-    ns.follow_tag(to_c_enum_tag_type(src.type())):
-    ns.follow(src.type());
+  const typet &src_type = src.type().id() == ID_c_enum_tag
+                            ? ns.follow_tag(to_c_enum_tag_type(src.type()))
+                            : src.type();
 
   if(src_type.id()==ID_bool) // already there
     return src; // do nothing
@@ -115,18 +115,19 @@ exprt is_not_zero(
     src_type.id()==ID_floatbv?ID_ieee_float_notequal:ID_notequal;
 
   exprt zero=from_integer(0, src_type);
-  CHECK_RETURN(zero.is_not_nil());
+  // Use tag type if applicable:
+  zero.type() = src.type();
 
-  binary_exprt comparison(src, id, zero, bool_typet());
+  binary_relation_exprt comparison(src, id, std::move(zero));
   comparison.add_source_location()=src.source_location();
 
-  return comparison;
+  return std::move(comparison);
 }
 
 exprt boolean_negate(const exprt &src)
 {
-  if(src.id()==ID_not && src.operands().size()==1)
-    return src.op0();
+  if(src.id() == ID_not)
+    return to_not_expr(src).op();
   else if(src.is_true())
     return false_exprt();
   else if(src.is_false())
@@ -170,10 +171,12 @@ bool has_subtype(
 
     if(pred(top))
       return true;
-    else if(top.id() == ID_symbol)
-      push_if_not_visited(ns.follow(top));
     else if(top.id() == ID_c_enum_tag)
       push_if_not_visited(ns.follow_tag(to_c_enum_tag_type(top)));
+    else if(top.id() == ID_struct_tag)
+      push_if_not_visited(ns.follow_tag(to_struct_tag_type(top)));
+    else if(top.id() == ID_union_tag)
+      push_if_not_visited(ns.follow_tag(to_union_tag_type(top)));
     else if(top.id() == ID_struct || top.id() == ID_union)
     {
       for(const auto &comp : to_struct_union_type(top).components())
@@ -181,7 +184,7 @@ bool has_subtype(
     }
     else
     {
-      for(const auto &subtype : top.subtypes())
+      for(const auto &subtype : to_type_with_subtypes(top).subtypes())
         push_if_not_visited(subtype);
     }
   }
@@ -204,12 +207,8 @@ if_exprt lift_if(const exprt &src, std::size_t operand_number)
   const exprt true_case=if_expr.true_case();
   const exprt false_case=if_expr.false_case();
 
-  if_exprt result;
-  result.cond()=if_expr.cond();
-  result.type()=src.type();
-  result.true_case()=src;
+  if_exprt result(if_expr.cond(), src, src);
   result.true_case().operands()[operand_number]=true_case;
-  result.false_case()=src;
   result.false_case().operands()[operand_number]=false_case;
 
   return result;
@@ -222,3 +221,92 @@ const exprt &skip_typecast(const exprt &expr)
 
   return skip_typecast(to_typecast_expr(expr).op());
 }
+
+/// This function determines what expressions are to be propagated as
+/// "constants"
+bool is_constantt::is_constant(const exprt &expr) const
+{
+  if(expr.is_constant())
+    return true;
+
+  if(expr.id() == ID_address_of)
+  {
+    return is_constant_address_of(to_address_of_expr(expr).object());
+  }
+  else if(
+    expr.id() == ID_typecast || expr.id() == ID_array_of ||
+    expr.id() == ID_plus || expr.id() == ID_mult || expr.id() == ID_array ||
+    expr.id() == ID_with || expr.id() == ID_struct || expr.id() == ID_union ||
+    // byte_update works, byte_extract may be out-of-bounds
+    expr.id() == ID_byte_update_big_endian ||
+    expr.id() == ID_byte_update_little_endian)
+  {
+    return std::all_of(
+      expr.operands().begin(), expr.operands().end(), [this](const exprt &e) {
+        return is_constant(e);
+      });
+  }
+
+  return false;
+}
+
+/// this function determines which reference-typed expressions are constant
+bool is_constantt::is_constant_address_of(const exprt &expr) const
+{
+  if(expr.id() == ID_symbol)
+  {
+    return true;
+  }
+  else if(expr.id() == ID_index)
+  {
+    const index_exprt &index_expr = to_index_expr(expr);
+
+    return is_constant_address_of(index_expr.array()) &&
+           is_constant(index_expr.index());
+  }
+  else if(expr.id() == ID_member)
+  {
+    return is_constant_address_of(to_member_expr(expr).compound());
+  }
+  else if(expr.id() == ID_dereference)
+  {
+    const dereference_exprt &deref = to_dereference_expr(expr);
+
+    return is_constant(deref.pointer());
+  }
+  else if(expr.id() == ID_string_constant)
+    return true;
+
+  return false;
+}
+
+constant_exprt make_boolean_expr(bool value)
+{
+  if(value)
+    return true_exprt();
+  else
+    return false_exprt();
+}
+
+exprt make_and(exprt a, exprt b)
+{
+  PRECONDITION(a.type().id() == ID_bool && b.type().id() == ID_bool);
+  if(b.is_constant())
+  {
+    if(b.get(ID_value) == ID_false)
+      return false_exprt{};
+    return a;
+  }
+  if(a.is_constant())
+  {
+    if(a.get(ID_value) == ID_false)
+      return false_exprt{};
+    return b;
+  }
+  if(b.id() == ID_and)
+  {
+    b.add_to_operands(std::move(a));
+    return b;
+  }
+  return and_exprt{std::move(a), std::move(b)};
+}
