@@ -15,6 +15,10 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <string>
 #include <vector>
 
-std::string format_number_range(std::vector<unsigned> &);
+std::string format_number_range(const std::vector<unsigned> &);
+
+/// Parse a compressed range into a vector of numbers,
+/// e.g. "2,4-6" -> [2,4,5,6]
+std::vector<unsigned> parse_number_range(const std::string &);
 
 #endif // CPROVER_UTIL_FORMAT_NUMBER_RANGE_H
