@@ -6,8 +6,9 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#ifndef CPROVER_FORMAT_SPEC_H
-#define CPROVER_FORMAT_SPEC_H
+
+#ifndef CPROVER_UTIL_FORMAT_SPEC_H
+#define CPROVER_UTIL_FORMAT_SPEC_H
 
 // this mimics the 'printf' format string for a single 'directive'
 
@@ -17,20 +18,20 @@ public:
   unsigned min_width;
   unsigned precision;
   bool zero_padding;
-  
+
   // 'printf' equivalents:
   // fF: DECIMAL
   // eE: SCIENTIFIC
   // gG: AUTOMATIC
 
-  typedef enum { DECIMAL, SCIENTIFIC, AUTOMATIC } stylet;
+  enum class stylet { DECIMAL, SCIENTIFIC, AUTOMATIC };
   stylet style;
-  
+
   format_spect():
     min_width(0),
     precision(6),
     zero_padding(false),
-    style(AUTOMATIC)
+    style(stylet::AUTOMATIC)
   {
   }
 
@@ -44,13 +45,13 @@ public:
 
   static format_spect scientific()
   {
-    return format_spect(SCIENTIFIC);
+    return format_spect(stylet::SCIENTIFIC);
   }
 
   static format_spect automatic()
   {
-    return format_spect(AUTOMATIC);
+    return format_spect(stylet::AUTOMATIC);
   }
 };
 
-#endif
+#endif // CPROVER_UTIL_FORMAT_SPEC_H
