@@ -28,7 +28,8 @@ std::size_t to_be_merged_irept::hash() const
         result, static_cast<const merged_irept &>(it->second).hash());
   }
 
-  result=hash_finalize(result, named_sub.size()+sub.size());
+  const std::size_t named_sub_size = named_sub.size();
+  result = hash_finalize(result, named_sub_size + sub.size());
 
   return result;
 }
@@ -44,9 +45,18 @@ bool to_be_merged_irept::operator == (const to_be_merged_irept &other) const
   const irept::named_subt &o_named_sub=other.get_named_sub();
 
   if(sub.size()!=o_sub.size())
-    return true;
+    return false;
+#ifdef NAMED_SUB_IS_FORWARD_LIST
+  if(
+    std::distance(named_sub.begin(), named_sub.end()) !=
+    std::distance(o_named_sub.begin(), o_named_sub.end()))
+  {
+    return false;
+  }
+#else
   if(named_sub.size()!=o_named_sub.size())
-    return true;
+    return false;
+#endif
 
   {
     irept::subt::const_iterator s_it=sub.begin();
@@ -95,13 +105,19 @@ const merged_irept &merged_irepst::merged(const irept &irep)
   const irept::named_subt &src_named_sub=irep.get_named_sub();
   irept::named_subt &dest_named_sub=new_irep.get_named_sub();
 
+#ifdef NAMED_SUB_IS_FORWARD_LIST
+  irept::named_subt::iterator before = dest_named_sub.before_begin();
+#endif
   forall_named_irep(it, src_named_sub)
-    #ifdef SUB_IS_LIST
-    dest_named_sub.push_back(
-      std::make_pair(it->first, merged(it->second))); // recursive call
-    #else
+  {
+#ifdef NAMED_SUB_IS_FORWARD_LIST
+    dest_named_sub.emplace_after(
+      before, it->first, merged(it->second)); // recursive call
+    ++before;
+#else
     dest_named_sub[it->first]=merged(it->second); // recursive call
-    #endif
+#endif
+  }
 
   std::pair<to_be_merged_irep_storet::const_iterator, bool> result=
     to_be_merged_irep_store.insert(to_be_merged_irept(new_irep));
@@ -124,42 +140,47 @@ void merge_irept::operator()(irept &irep)
 
 const irept &merge_irept::merged(const irept &irep)
 {
-  irep_storet::const_iterator entry=irep_store.find(irep);
-  if(entry!=irep_store.end())
-    return *entry;
-
-  irept new_irep(irep.id());
+  auto entry = irep_store.insert(irep);
+  if(!entry.second)
+    return *entry.first;
 
   const irept::subt &src_sub=irep.get_sub();
-  irept::subt &dest_sub=new_irep.get_sub();
-  dest_sub.reserve(src_sub.size());
+  irept::subt *dest_sub_ptr = nullptr;
 
+  std::size_t index = 0;
   forall_irep(it, src_sub)
-    dest_sub.push_back(merged(*it)); // recursive call
+  {
+    const irept &op = merged(*it); // recursive call
+    if(&op.read() != &(it->read()))
+    {
+      if(!dest_sub_ptr)
+        dest_sub_ptr = &(const_cast<irept &>(*entry.first)).get_sub();
+      (*dest_sub_ptr)[index] = op;
+    }
+    ++index;
+  }
 
   const irept::named_subt &src_named_sub=irep.get_named_sub();
-  irept::named_subt &dest_named_sub=new_irep.get_named_sub();
+  irept::named_subt *dest_named_sub_ptr = nullptr;
 
+  std::ptrdiff_t advance_by = 0;
   forall_named_irep(it, src_named_sub)
-    #ifdef SUB_IS_LIST
-    dest_named_sub.push_back(
-      std::make_pair(it->first, merged(it->second))); // recursive call
-    #else
-    dest_named_sub[it->first]=merged(it->second); // recursive call
-    #endif
+  {
+    if(!irept::is_comment(it->first))
+    {
+      const irept &op = merged(it->second); // recursive call
+      if(&op.read() != &(it->second.read()))
+      {
+        if(!dest_named_sub_ptr)
+          dest_named_sub_ptr =
+            &(const_cast<irept &>(*entry.first)).get_named_sub();
+        std::next(dest_named_sub_ptr->begin(), advance_by)->second = op;
+      }
+    }
+    ++advance_by;
+  }
 
-  const irept::named_subt &src_comments=irep.get_comments();
-  irept::named_subt &dest_comments=new_irep.get_comments();
-
-  forall_named_irep(it, src_comments)
-    #ifdef SUB_IS_LIST
-    dest_comments.push_back(
-      std::make_pair(it->first, merged(it->second))); // recursive call
-    #else
-    dest_comments[it->first]=merged(it->second); // recursive call
-    #endif
-
-  return *irep_store.insert(new_irep).first;
+  return *entry.first;
 }
 
 void merge_full_irept::operator()(irept &irep)
@@ -188,24 +209,19 @@ const irept &merge_full_irept::merged(const irept &irep)
   const irept::named_subt &src_named_sub=irep.get_named_sub();
   irept::named_subt &dest_named_sub=new_irep.get_named_sub();
 
+#ifdef NAMED_SUB_IS_FORWARD_LIST
+  irept::named_subt::iterator before = dest_named_sub.before_begin();
+#endif
   forall_named_irep(it, src_named_sub)
-    #ifdef SUB_IS_LIST
-    dest_named_sub.push_back(
-      std::make_pair(it->first, merged(it->second))); // recursive call
-    #else
+  {
+#ifdef NAMED_SUB_IS_FORWARD_LIST
+    dest_named_sub.emplace_after(
+      before, it->first, merged(it->second)); // recursive call
+    ++before;
+#else
     dest_named_sub[it->first]=merged(it->second); // recursive call
-    #endif
+#endif
+  }
 
-  const irept::named_subt &src_comments=irep.get_comments();
-  irept::named_subt &dest_comments=new_irep.get_comments();
-
-  forall_named_irep(it, src_comments)
-    #ifdef SUB_IS_LIST
-    dest_comments.push_back(
-      std::make_pair(it->first, merged(it->second))); // recursive call
-    #else
-    dest_comments[it->first]=merged(it->second); // recursive call
-    #endif
-
-  return *irep_store.insert(new_irep).first;
+  return *irep_store.insert(std::move(new_irep)).first;
 }
