@@ -8,8 +8,10 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include "replace_symbol.h"
 
-#include "std_types.h"
+#include "expr_util.h"
+#include "invariant.h"
 #include "std_expr.h"
+#include "std_types.h"
 
 replace_symbolt::replace_symbolt()
 {
@@ -23,13 +25,38 @@ void replace_symbolt::insert(
   const symbol_exprt &old_expr,
   const exprt &new_expr)
 {
+  PRECONDITION_WITH_DIAGNOSTICS(
+    old_expr.type() == new_expr.type(),
+    "types to be replaced should match. old type:\n" +
+      old_expr.type().pretty() + "\nnew.type:\n" + new_expr.type().pretty());
   expr_map.insert(std::pair<irep_idt, exprt>(
     old_expr.get_identifier(), new_expr));
 }
 
-bool replace_symbolt::replace(
-  exprt &dest,
-  const bool replace_with_const) const
+void replace_symbolt::set(const symbol_exprt &old_expr, const exprt &new_expr)
+{
+  PRECONDITION(old_expr.type() == new_expr.type());
+  expr_map[old_expr.get_identifier()] = new_expr;
+}
+
+
+bool replace_symbolt::replace_symbol_expr(symbol_exprt &s) const
+{
+  expr_mapt::const_iterator it = expr_map.find(s.get_identifier());
+
+  if(it == expr_map.end())
+    return true;
+
+  DATA_INVARIANT(
+    s.type() == it->second.type(),
+    "types to be replaced should match. s.type:\n" + s.type().pretty() +
+      "\nit->second.type:\n" + it->second.type().pretty());
+  static_cast<exprt &>(s) = it->second;
+
+  return false;
+}
+
+bool replace_symbolt::replace(exprt &dest) const
 {
   bool result=true; // unchanged
 
@@ -49,14 +76,14 @@ bool replace_symbolt::replace(
   {
     member_exprt &me=to_member_expr(dest);
 
-    if(!replace(me.struct_op(), replace_with_const)) // Could give non l-value.
+    if(!replace(me.struct_op()))
       result=false;
   }
   else if(dest.id()==ID_index)
   {
     index_exprt &ie=to_index_expr(dest);
 
-    if(!replace(ie.array(), replace_with_const)) // Could give non l-value.
+    if(!replace(ie.array()))
       result=false;
 
     if(!replace(ie.index()))
@@ -66,27 +93,13 @@ bool replace_symbolt::replace(
   {
     address_of_exprt &aoe=to_address_of_expr(dest);
 
-    if(!replace(aoe.object(), false))
+    if(!replace(aoe.object()))
       result=false;
   }
   else if(dest.id()==ID_symbol)
   {
-    const symbol_exprt &s=to_symbol_expr(dest);
-
-    expr_mapt::const_iterator it=
-      expr_map.find(s.get_identifier());
-
-    if(it!=expr_map.end())
-    {
-      const exprt &e=it->second;
-
-      if(!replace_with_const && e.is_constant())  // Would give non l-value.
-        return true;
-
-      dest=e;
-
+    if(!replace_symbol_expr(to_symbol_expr(dest)))
       return false;
-    }
   }
   else
   {
@@ -110,7 +123,7 @@ bool replace_symbolt::replace(
 
 bool replace_symbolt::have_to_replace(const exprt &dest) const
 {
-  if(expr_map.empty() && type_map.empty())
+  if(empty())
     return false;
 
   // first look at type
@@ -123,7 +136,7 @@ bool replace_symbolt::have_to_replace(const exprt &dest) const
   if(dest.id()==ID_symbol)
   {
     const irep_idt &identifier = to_symbol_expr(dest).get_identifier();
-    return expr_map.find(identifier) != expr_map.end();
+    return replaces_symbol(identifier);
   }
 
   forall_operands(it, dest)
@@ -164,14 +177,9 @@ bool replace_symbolt::replace(typet &dest) const
      dest.id()==ID_union)
   {
     struct_union_typet &struct_union_type=to_struct_union_type(dest);
-    struct_union_typet::componentst &components=
-      struct_union_type.components();
 
-    for(struct_union_typet::componentst::iterator
-        it=components.begin();
-        it!=components.end();
-        it++)
-      if(!replace(*it))
+    for(auto &c : struct_union_type.components())
+      if(!replace(c))
         result=false;
   }
   else if(dest.id()==ID_code)
@@ -179,23 +187,10 @@ bool replace_symbolt::replace(typet &dest) const
     code_typet &code_type=to_code_type(dest);
     if(!replace(code_type.return_type()))
       result = false;
-    code_typet::parameterst &parameters=code_type.parameters();
-    for(code_typet::parameterst::iterator it = parameters.begin();
-        it!=parameters.end();
-        it++)
-      if(!replace(*it))
-        result=false;
-  }
-  else if(dest.id()==ID_symbol)
-  {
-    type_mapt::const_iterator it =
-      type_map.find(to_symbol_type(dest).get_identifier());
 
-    if(it!=type_map.end())
-    {
-      dest=it->second;
-      result=false;
-    }
+    for(auto &p : code_type.parameters())
+      if(!replace(p))
+        result=false;
   }
   else if(dest.id()==ID_array)
   {
@@ -209,7 +204,7 @@ bool replace_symbolt::replace(typet &dest) const
 
 bool replace_symbolt::have_to_replace(const typet &dest) const
 {
-  if(expr_map.empty() && type_map.empty())
+  if(expr_map.empty())
     return false;
 
   if(dest.has_subtype())
@@ -226,14 +221,8 @@ bool replace_symbolt::have_to_replace(const typet &dest) const
     const struct_union_typet &struct_union_type=
       to_struct_union_type(dest);
 
-    const struct_union_typet::componentst &components=
-      struct_union_type.components();
-
-    for(struct_union_typet::componentst::const_iterator
-        it=components.begin();
-        it!=components.end();
-        it++)
-      if(have_to_replace(*it))
+    for(const auto &c : struct_union_type.components())
+      if(have_to_replace(c))
         return true;
   }
   else if(dest.id()==ID_code)
@@ -242,22 +231,112 @@ bool replace_symbolt::have_to_replace(const typet &dest) const
     if(have_to_replace(code_type.return_type()))
       return true;
 
-    const code_typet::parameterst &parameters=code_type.parameters();
-
-    for(code_typet::parameterst::const_iterator
-        it=parameters.begin();
-        it!=parameters.end();
-        it++)
-      if(have_to_replace(*it))
+    for(const auto &p : code_type.parameters())
+      if(have_to_replace(p))
         return true;
   }
-  else if(dest.id()==ID_symbol)
-  {
-    const irep_idt &identifier = to_symbol_type(dest).get_identifier();
-    return type_map.find(identifier) != type_map.end();
-  }
   else if(dest.id()==ID_array)
     return have_to_replace(to_array_type(dest).size());
 
   return false;
 }
+
+void unchecked_replace_symbolt::insert(
+  const symbol_exprt &old_expr,
+  const exprt &new_expr)
+{
+  expr_map.emplace(old_expr.get_identifier(), new_expr);
+}
+
+bool unchecked_replace_symbolt::replace_symbol_expr(symbol_exprt &s) const
+{
+  expr_mapt::const_iterator it = expr_map.find(s.get_identifier());
+
+  if(it == expr_map.end())
+    return true;
+
+  static_cast<exprt &>(s) = it->second;
+
+  return false;
+}
+
+bool address_of_aware_replace_symbolt::replace(exprt &dest) const
+{
+  const exprt &const_dest(dest);
+  if(!require_lvalue && const_dest.id() != ID_address_of)
+    return unchecked_replace_symbolt::replace(dest);
+
+  bool result = true; // unchanged
+
+  // first look at type
+  if(have_to_replace(const_dest.type()))
+  {
+    const set_require_lvalue_and_backupt backup(require_lvalue, false);
+    if(!unchecked_replace_symbolt::replace(dest.type()))
+      result = false;
+  }
+
+  // now do expression itself
+
+  if(!have_to_replace(dest))
+    return result;
+
+  if(dest.id() == ID_index)
+  {
+    index_exprt &ie = to_index_expr(dest);
+
+    // Could yield non l-value.
+    if(!replace(ie.array()))
+      result = false;
+
+    const set_require_lvalue_and_backupt backup(require_lvalue, false);
+    if(!replace(ie.index()))
+      result = false;
+  }
+  else if(dest.id() == ID_address_of)
+  {
+    address_of_exprt &aoe = to_address_of_expr(dest);
+
+    const set_require_lvalue_and_backupt backup(require_lvalue, true);
+    if(!replace(aoe.object()))
+      result = false;
+  }
+  else
+  {
+    if(!unchecked_replace_symbolt::replace(dest))
+      return false;
+  }
+
+  const set_require_lvalue_and_backupt backup(require_lvalue, false);
+
+  const typet &c_sizeof_type =
+    static_cast<const typet &>(dest.find(ID_C_c_sizeof_type));
+  if(c_sizeof_type.is_not_nil() && have_to_replace(c_sizeof_type))
+    result &= unchecked_replace_symbolt::replace(
+      static_cast<typet &>(dest.add(ID_C_c_sizeof_type)));
+
+  const typet &va_arg_type =
+    static_cast<const typet &>(dest.find(ID_C_va_arg_type));
+  if(va_arg_type.is_not_nil() && have_to_replace(va_arg_type))
+    result &= unchecked_replace_symbolt::replace(
+      static_cast<typet &>(dest.add(ID_C_va_arg_type)));
+
+  return result;
+}
+
+bool address_of_aware_replace_symbolt::replace_symbol_expr(
+  symbol_exprt &s) const
+{
+  symbol_exprt s_copy = s;
+  if(unchecked_replace_symbolt::replace_symbol_expr(s_copy))
+    return true;
+
+  if(require_lvalue && !is_lvalue(s_copy))
+    return true;
+
+  // Note s_copy is no longer a symbol_exprt due to the replace operation,
+  // and after this line `s` won't be either
+  s = s_copy;
+
+  return false;
+}
