@@ -7,6 +7,7 @@ Author: Daniel Kroening, kroening@kroening.com
 \*******************************************************************/
 
 #include "simplify_utils.h"
+#include "as_const.h"
 
 #include <algorithm>
 
@@ -121,13 +122,13 @@ static const struct saj_tablet &sort_and_join(
 
 bool sort_and_join(exprt &expr)
 {
-  bool result=true;
+  bool no_change = true;
 
   if(!expr.has_operands())
     return true;
 
-  const struct saj_tablet &saj_entry=
-    sort_and_join(expr.id(), expr.type().id());
+  const struct saj_tablet &saj_entry =
+    sort_and_join(expr.id(), as_const(expr).type().id());
   if(saj_entry.id.empty())
     return true;
 
@@ -151,7 +152,7 @@ bool sort_and_join(exprt &expr)
       forall_operands(it2, *it)
         new_ops.push_back(*it2);
 
-      result=false;
+      no_change = false;
     }
     else
       new_ops.push_back(*it);
@@ -159,8 +160,8 @@ bool sort_and_join(exprt &expr)
 
   // sort it
 
-  result=sort_operands(new_ops) && result;
+  no_change = sort_operands(new_ops) && no_change;
   expr.operands().swap(new_ops);
 
-  return result;
+  return no_change;
 }
