@@ -7,11 +7,13 @@ Author: Daniel Poetzl
 \*******************************************************************/
 
 #include "string_utils.h"
+#include "exception_utils.h"
 #include "invariant.h"
 
+#include <algorithm>
 #include <cassert>
 #include <cctype>
-#include <algorithm>
+#include <iomanip>
 
 /// Remove all whitespace characters from either end of a string. Whitespace
 /// in the middle of the string is left unchanged
@@ -35,16 +37,6 @@ std::string strip_string(const std::string &s)
   return s.substr(i, (j-i+1));
 }
 
-/// Given a string s, split into a sequence of substrings when separated by
-/// specified delimiter.
-/// \param s: The string to split up
-/// \param delim: The character to use as the delimiter
-/// \param [out] result: The sub strings. Must be empty.
-/// \param strip: If true, strip_string will be used on each element, removing
-/// whitespace from the beginning and end of each element
-/// \param remove_empty: If true, all empty-string elements will be removed.
-/// This is applied after strip so whitespace only elements will be removed if
-/// both are set to true
 void split_string(
   const std::string &s,
   char delim,
@@ -58,7 +50,8 @@ void split_string(
 
   if(s.empty())
   {
-    result.push_back("");
+    if(!remove_empty)
+      result.push_back("");
     return;
   }
 
@@ -92,7 +85,7 @@ void split_string(
   if(!remove_empty || !new_s.empty())
     result.push_back(new_s);
 
-  if(result.empty())
+  if(!remove_empty && result.empty())
     result.push_back("");
 }
 
@@ -109,17 +102,27 @@ void split_string(
   std::vector<std::string> result;
 
   split_string(s, delim, result, strip);
-  if(result.size()!=2)
-    throw "split string did not generate exactly 2 parts";
+  if(result.size() != 2)
+  {
+    throw deserialization_exceptiont{"expected string '" + s +
+                                     "' to contain two substrings "
+                                     "delimited by " +
+                                     delim + " but has " +
+                                     std::to_string(result.size())};
+  }
 
   left=result[0];
   right=result[1];
 }
 
-std::vector<std::string> split_string(const std::string &s, char delim)
+std::vector<std::string> split_string(
+  const std::string &s,
+  char delim,
+  bool strip,
+  bool remove_empty)
 {
   std::vector<std::string> result;
-  split_string(s, delim, result);
+  split_string(s, delim, result, strip, remove_empty);
   return result;
 }
 
@@ -127,7 +130,7 @@ std::string trim_from_last_delimiter(
   const std::string &s,
   const char delim)
 {
-  std::string result="";
+  std::string result;
   const size_t index=s.find_last_of(delim);
   if(index!=std::string::npos)
     result=s.substr(0, index);
@@ -149,23 +152,18 @@ std::string escape(const std::string &s)
   return result;
 }
 
-/// Replace all occurrences of a string inside a string
-/// \param [out] str: string to search
-/// \param from: string to replace
-/// \param to: string to replace with
-/// Copyright notice:
-/// Attributed to Gauthier Boaglio
-/// Source: https://stackoverflow.com/a/24315631/7501486
-/// Used under MIT license
-void replace_all(
-  std::string &str,
-  const std::string &from,
-  const std::string &to)
+std::string escape_non_alnum(const std::string &to_escape)
 {
-  size_t start_pos = 0;
-  while((start_pos = str.find(from, start_pos)) != std::string::npos)
+  std::ostringstream escaped;
+  for(auto &ch : to_escape)
   {
-    str.replace(start_pos, from.length(), to);
-    start_pos += to.length();
+    if(ch == '_')
+      escaped << "__";
+    else if(isalnum(ch))
+      escaped << ch;
+    else
+      escaped << '_' << std::hex << std::setfill('0') << std::setw(2)
+              << (unsigned int)ch;
   }
+  return escaped.str();
 }
