@@ -29,17 +29,4 @@ public:
   ~temp_dirt();
 };
 
-// Produces a temporary directory,
-// chdir()s there,
-// and deletes it upon destruction,
-// and goes back to the old working directory.
-class temp_working_dirt:public temp_dirt
-{
-public:
-  std::string old_working_directory;
-
-  explicit temp_working_dirt(const std::string &name_template);
-  ~temp_working_dirt();
-};
-
 #endif // CPROVER_UTIL_TEMPDIR_H
