@@ -9,6 +9,13 @@ Author: Daniel Kroening
 #include "tempfile.h"
 
 #ifdef _WIN32
+#include <util/pragma_push.def>
+#ifdef _MSC_VER
+#pragma warning(disable:4668)
+  // using #if/#elif on undefined macro
+#pragma warning(disable : 5039)
+// pointer or reference to potentially throwing function passed to extern C
+#endif
 #include <process.h>
 #include <sys/stat.h>
 #include <windows.h>
@@ -17,6 +24,7 @@ Author: Daniel Kroening
 #define getpid _getpid
 #define open _open
 #define close _close
+#include <util/pragma_pop.def>
 #endif
 
 #include <fcntl.h>
@@ -25,6 +33,8 @@ Author: Daniel Kroening
 #include <cstdio>
 #include <cstring>
 
+#include "exception_utils.h"
+
 #if defined(__linux__) || \
     defined(__FreeBSD_kernel__) || \
     defined(__GNU__) || \
@@ -98,7 +108,7 @@ std::string get_temporary_file(
       lpTempPathBuffer); // buffer for path
 
   if(dwRetVal>MAX_PATH || (dwRetVal==0))
-    throw "GetTempPath failed"; // NOLINT(readability/throw)
+    throw system_exceptiont("Failed to get temporary directory");
 
   // the path returned by GetTempPath ends with a backslash
   std::string t_template=
@@ -121,7 +131,7 @@ std::string get_temporary_file(
   int fd=mkstemps(t_ptr, suffix.size());
 
   if(fd<0)
-    throw "mkstemps failed";
+    throw system_exceptiont("Failed to open temporary file");
 
   close(fd);
 
