@@ -9,7 +9,7 @@ Author:
 #include <iostream>
 
 #include <testing-utils/call_graph_test_utils.h>
-#include <testing-utils/catch.hpp>
+#include <testing-utils/use_catch.h>
 
 #include <analyses/call_graph.h>
 #include <analyses/call_graph_helpers.h>
@@ -47,29 +47,19 @@ SCENARIO("call_graph",
     code_typet void_function_type({}, empty_typet());
 
     {
-      code_blockt calls;
-      code_function_callt call1;
-      call1.function()=symbol_exprt("A", void_function_type);
-      code_function_callt call2;
-      call2.function()=symbol_exprt("B", void_function_type);
-      code_function_callt call3;
-      call3.function()=symbol_exprt("B", void_function_type);
-      calls.move_to_operands(call1);
-      calls.move_to_operands(call2);
-      calls.move_to_operands(call3);
+      code_blockt calls(
+        {code_function_callt(symbol_exprt("A", void_function_type)),
+         code_function_callt(symbol_exprt("B", void_function_type)),
+         code_function_callt(symbol_exprt("B", void_function_type))});
 
       goto_model.symbol_table.add(
         create_void_function_symbol("A", calls));
     }
 
     {
-      code_blockt calls;
-      code_function_callt call1;
-      call1.function()=symbol_exprt("C", void_function_type);
-      code_function_callt call2;
-      call2.function()=symbol_exprt("D", void_function_type);
-      calls.move_to_operands(call1);
-      calls.move_to_operands(call2);
+      code_blockt calls(
+        {code_function_callt(symbol_exprt("C", void_function_type)),
+         code_function_callt(symbol_exprt("D", void_function_type))});
 
       goto_model.symbol_table.add(
         create_void_function_symbol("B", calls));
@@ -119,11 +109,12 @@ SCENARIO("call_graph",
 
     WHEN("A call graph is constructed with call-site tracking")
     {
-      call_grapht call_graph_from_goto_functions(goto_model, true);
+      call_grapht call_graph_from_goto_functions_tracking(goto_model, true);
       THEN("We expect two callsites for the A -> B edge, one for all others")
       {
-        const auto &check_callsites=call_graph_from_goto_functions.callsites;
-        for(const auto &edge : call_graph_from_goto_functions.edges)
+        const auto &check_callsites =
+          call_graph_from_goto_functions_tracking.callsites;
+        for(const auto &edge : call_graph_from_goto_functions_tracking.edges)
         {
           if(edge==call_grapht::edgest::value_type("A", "B"))
             REQUIRE(check_callsites.at(edge).size()==2);
@@ -133,7 +124,8 @@ SCENARIO("call_graph",
       }
       WHEN("Such a graph is inverted")
       {
-        call_grapht inverted=call_graph_from_goto_functions.get_inverted();
+        call_grapht inverted =
+          call_graph_from_goto_functions_tracking.get_inverted();
         THEN("The callsite data should be discarded")
         {
           REQUIRE(inverted.callsites.empty());
@@ -289,9 +281,9 @@ SCENARIO("call_graph",
 
     WHEN("The call graph, with call sites, is exported as a grapht")
     {
-      call_grapht call_graph_from_goto_functions(goto_model, true);
-      call_grapht::directed_grapht exported=
-        call_graph_from_goto_functions.get_directed_graph();
+      call_grapht call_graph_from_goto_functions_tracking(goto_model, true);
+      call_grapht::directed_grapht exported =
+        call_graph_from_goto_functions_tracking.get_directed_graph();
 
       typedef call_grapht::directed_grapht::node_indext node_indext;
       std::map<irep_idt, node_indext> nodes_by_name;
