@@ -6,18 +6,19 @@ Author: Chris Smowton, chris.smowton@diffblue.com
 
 \*******************************************************************/
 
-#include <iostream>
+#include <testing-utils/call_graph_test_utils.h>
+#include <testing-utils/message.h>
+#include <testing-utils/use_catch.h>
+
+#include <util/arith_tools.h>
+#include <util/c_types.h>
+#include <util/std_code.h>
+#include <util/symbol_table.h>
 
 #include <analyses/dependence_graph.h>
 #include <ansi-c/ansi_c_language.h>
 #include <goto-programs/goto_convert_functions.h>
 #include <langapi/mode.h>
-#include <testing-utils/call_graph_test_utils.h>
-#include <testing-utils/catch.hpp>
-#include <util/arith_tools.h>
-#include <util/c_types.h>
-#include <util/std_code.h>
-#include <util/symbol_table.h>
 
 const std::set<goto_programt::const_targett>&
     dependence_graph_test_get_control_deps(const dep_graph_domaint &domain)
@@ -64,36 +65,26 @@ SCENARIO("dependence_graph", "[core][analyses][dependence_graph]")
 
     const code_typet void_function_type({}, empty_typet());
 
-    code_blockt a_body;
-    code_declt declare_x(x_symbol.symbol_expr());
-    a_body.move_to_operands(declare_x);
+    code_blockt a_body({code_declt(x_symbol.symbol_expr())});
 
-    code_ifthenelset if_block;
-
-    if_block.cond() =
-      equal_exprt(
-        side_effect_expr_nondett(int_type),
-        from_integer(0, int_type));
-
-    code_blockt then_block;
-    code_function_callt call;
-    call.function() = symbol_exprt("b", void_function_type);
-    then_block.move_to_operands(call);
+    code_function_callt call(symbol_exprt("b", void_function_type));
     code_assignt assign_x(
       x_symbol.symbol_expr(), from_integer(1, int_type));
-    then_block.move_to_operands(assign_x);
 
-    if_block.then_case() = then_block;
+    code_ifthenelset if_block(
+      equal_exprt(
+        side_effect_expr_nondett(int_type, source_locationt{}),
+        from_integer(0, int_type)),
+      code_blockt({call, assign_x}));
 
-    a_body.move_to_operands(if_block);
+    a_body.add(std::move(if_block));
 
     goto_model.symbol_table.add(
       create_void_function_symbol(goto_functionst::entry_point(), a_body));
     goto_model.symbol_table.add(
       create_void_function_symbol("b", code_skipt()));
 
-    stream_message_handlert msg(std::cerr);
-    goto_convert(goto_model, msg);
+    goto_convert(goto_model, null_message_handler);
 
     WHEN("Constructing a dependence graph")
     {
