@@ -1,15 +1,15 @@
 /*******************************************************************\
 
- Module: Unit tests for goto_trace_stept::output
+Module: Unit tests for goto_trace_stept::output
 
- Author: Diffblue Ltd.
+Author: Diffblue Ltd.
 
 \*******************************************************************/
 
-#include <testing-utils/catch.hpp>
 #include <goto-programs/goto_program.h>
 #include <goto-programs/goto_trace.h>
 #include <sstream>
+#include <testing-utils/use_catch.h>
 
 SCENARIO(
   "Output trace with nil lhs object",
