@@ -6,7 +6,7 @@ Author: Kareem Khazem <karkhaz@karkhaz.com>, 2018
 
 \*******************************************************************/
 
-#include <testing-utils/catch.hpp>
+#include <testing-utils/use_catch.h>
 
 #include <path_strategies.h>
 
@@ -16,16 +16,16 @@ Author: Kareem Khazem <karkhaz@karkhaz.com>, 2018
 
 #include <ansi-c/ansi_c_language.h>
 
-#include <cbmc/bmc.h>
 #include <cbmc/cbmc_parse_options.h>
-#include <cbmc/cbmc_solvers.h>
 
-#include <langapi/language_ui.h>
+#include <goto-checker/bmc_util.h>
+#include <goto-checker/goto_symex_property_decider.h>
+#include <goto-checker/symex_bmc.h>
+
 #include <langapi/mode.h>
 
 #include <util/cmdline.h>
 #include <util/config.h>
-#include <util/cout_message.h>
 #include <util/tempfile.h>
 
 // The actual test suite.
@@ -43,8 +43,8 @@ Author: Kareem Khazem <karkhaz@karkhaz.com>, 2018
 // `goto` instruction.
 //
 // Whenever symbolic execution reaches the end of a path, you should expect a
-// result. Results are either SUCCESS, meaning that verification of that path
-// succeeded, or FAILURE, meaning that there was an assertion failure on that
+// result. Results are either DONE, meaning that verification of that path
+// succeeded, or FOUND_FAIL, meaning that there was an assertion failure on that
 // path.
 //
 // To figure out what the events should be, run CBMC on the test program with
@@ -65,7 +65,7 @@ SCENARIO("path strategies")
   std::string c;
   GIVEN("a simple conditional program")
   {
-    std::function<void(optionst &)> opts_callback = [](optionst &opts) {};
+    std::function<void(optionst &)> opts_callback = [](optionst &) {};
 
     c =
       "/*  1 */  int main()      \n"
@@ -81,23 +81,25 @@ SCENARIO("path strategies")
       "lifo",
       opts_callback,
       c,
-      {symex_eventt::resume(symex_eventt::enumt::JUMP, 7),
-       symex_eventt::result(symex_eventt::enumt::SUCCESS),
+      {// Entry state is line 0
+       symex_eventt::resume(symex_eventt::enumt::NEXT, 0),
+       symex_eventt::resume(symex_eventt::enumt::JUMP, 7),
        symex_eventt::resume(symex_eventt::enumt::NEXT, 5),
        symex_eventt::result(symex_eventt::enumt::SUCCESS)});
     check_with_strategy(
       "fifo",
       opts_callback,
       c,
-      {symex_eventt::resume(symex_eventt::enumt::NEXT, 5),
-       symex_eventt::result(symex_eventt::enumt::SUCCESS),
+      {// Entry state is line 0
+       symex_eventt::resume(symex_eventt::enumt::NEXT, 0),
+       symex_eventt::resume(symex_eventt::enumt::NEXT, 5),
        symex_eventt::resume(symex_eventt::enumt::JUMP, 7),
        symex_eventt::result(symex_eventt::enumt::SUCCESS)});
   }
 
   GIVEN("a program with nested conditionals")
   {
-    std::function<void(optionst &)> opts_callback = [](optionst &opts) {};
+    std::function<void(optionst &)> opts_callback = [](optionst &) {};
 
     c =
       "/*  1 */  int main()            \n"
@@ -123,17 +125,16 @@ SCENARIO("path strategies")
       "lifo",
       opts_callback,
       c,
-      {// Outer else, inner else
+      {// Entry state is line 0
+       symex_eventt::resume(symex_eventt::enumt::NEXT, 0),
+       // Outer else, inner else
        symex_eventt::resume(symex_eventt::enumt::JUMP, 13),
        symex_eventt::resume(symex_eventt::enumt::JUMP, 16),
-       symex_eventt::result(symex_eventt::enumt::SUCCESS),
        // Outer else, inner if
        symex_eventt::resume(symex_eventt::enumt::NEXT, 14),
-       symex_eventt::result(symex_eventt::enumt::SUCCESS),
        // Outer if, inner else
        symex_eventt::resume(symex_eventt::enumt::NEXT, 6),
        symex_eventt::resume(symex_eventt::enumt::JUMP, 9),
-       symex_eventt::result(symex_eventt::enumt::SUCCESS),
        // Outer if, inner if
        symex_eventt::resume(symex_eventt::enumt::NEXT, 7),
        symex_eventt::result(symex_eventt::enumt::SUCCESS)});
@@ -142,19 +143,17 @@ SCENARIO("path strategies")
       "fifo",
       opts_callback,
       c,
-      {// Expand outer if, but don't continue depth-first
+      {// Entry state is line 0
+       symex_eventt::resume(symex_eventt::enumt::NEXT, 0),
+       // Expand outer if, but don't continue depth-first
        symex_eventt::resume(symex_eventt::enumt::NEXT, 6),
        // Jump to outer else, but again don't continue depth-first
        symex_eventt::resume(symex_eventt::enumt::JUMP, 13),
        // Expand inner if of the outer if
        symex_eventt::resume(symex_eventt::enumt::NEXT, 7),
-       // No more branch points, so complete the path
-       symex_eventt::result(symex_eventt::enumt::SUCCESS),
-       // Continue BFSing
+       // No more branch points, so complete the path. Then continue BFSing
        symex_eventt::resume(symex_eventt::enumt::JUMP, 9),
-       symex_eventt::result(symex_eventt::enumt::SUCCESS),
        symex_eventt::resume(symex_eventt::enumt::NEXT, 14),
-       symex_eventt::result(symex_eventt::enumt::SUCCESS),
        symex_eventt::resume(symex_eventt::enumt::JUMP, 16),
        symex_eventt::result(symex_eventt::enumt::SUCCESS)});
   }
@@ -165,25 +164,35 @@ SCENARIO("path strategies")
       opts.set_option("unwind", 2U);
     };
 
+    // clang-format off
     c =
-      "/*  1 */   int main()                       \n"
-      "/*  2 */   {                                \n"
-      "/*  3 */     int x;                         \n"
-      "/*  4 */     __CPROVER_assume(x == 1);      \n"
-      "/*  5 */                                    \n"
-      "/*  6 */     while(x)                       \n"
-      "/*  7 */       --x;                         \n"
-      "/*  8 */                                    \n"
-      "/*  9 */     assert(x);                     \n"
-      "/* 10 */   }                                \n";
+      "/*  1 */   int main()                          \n"
+      "/*  2 */   {                                   \n"
+      "/*  3 */     int x;                            \n"
+      "/*  4 */     if(x == 1) {                      \n"
+      "/*  5 */                                       \n"
+      "/*  6 */       while(x)                        \n"
+      "/*  7 */         --x;                          \n"
+      "/*  8 */                                       \n"
+      "/*  9 */       assert(x);                      \n"
+      "/* 10 */     }                                 \n"
+      "/* 11 */   }                                   \n";
+    // clang-format on
 
     check_with_strategy(
       "lifo",
       opts_callback,
       c,
       {
+        // Entry state is line 0
+        symex_eventt::resume(symex_eventt::enumt::NEXT, 0),
+
+        // The path where x != 1 and we have nothing to check:
+        symex_eventt::resume(symex_eventt::enumt::JUMP, 11),
+
         // The path where we skip the loop body. Successful because the path is
         // implausible, we cannot have skipped the body if x == 1.
+        symex_eventt::resume(symex_eventt::enumt::NEXT, 6),
         symex_eventt::resume(symex_eventt::enumt::JUMP, 9),
         symex_eventt::result(symex_eventt::enumt::SUCCESS),
 
@@ -193,10 +202,12 @@ SCENARIO("path strategies")
         symex_eventt::resume(symex_eventt::enumt::JUMP, 9),
         symex_eventt::result(symex_eventt::enumt::FAILURE),
 
-        // The path where we enter the loop body twice. Successful because
-        // infeasible.
+        // The path where we enter the loop body twice. No result because
+        // this path hits an unwind bound.
         symex_eventt::resume(symex_eventt::enumt::NEXT, 7),
-        symex_eventt::result(symex_eventt::enumt::SUCCESS),
+
+        // Overall we fail.
+        symex_eventt::result(symex_eventt::enumt::FAILURE),
       });
 
     check_with_strategy(
@@ -204,6 +215,15 @@ SCENARIO("path strategies")
       opts_callback,
       c,
       {
+        // Entry state is line 0
+        symex_eventt::resume(symex_eventt::enumt::NEXT, 0),
+
+        // First the path where we enter the if-block at line 4 then on hitting
+        // the branch at line 6 consider skipping straight to the end, but find
+        // nothing there to investigate:
+        symex_eventt::resume(symex_eventt::enumt::NEXT, 6),
+        symex_eventt::resume(symex_eventt::enumt::JUMP, 11),
+
         // The path where we skip the loop body. Successful because the path is
         // implausible, we cannot have skipped the body if x == 1.
         //
@@ -214,14 +234,16 @@ SCENARIO("path strategies")
         symex_eventt::result(symex_eventt::enumt::SUCCESS),
 
         // Pop line 7 that we saved from above, and execute the loop a second
-        // time. Successful, since this path is infeasible.
+        // time. No result, since this path exceeds an unwind bound
         symex_eventt::resume(symex_eventt::enumt::NEXT, 7),
-        symex_eventt::result(symex_eventt::enumt::SUCCESS),
 
         // Pop line 7 that we saved from above and bail out. That corresponds to
         // executing the loop once, decrementing x to 0; assert(x) should fail.
         symex_eventt::resume(symex_eventt::enumt::JUMP, 9),
         symex_eventt::result(symex_eventt::enumt::FAILURE),
+
+        // Overall we fail.
+        symex_eventt::result(symex_eventt::enumt::FAILURE),
       });
   }
 
@@ -230,7 +252,7 @@ SCENARIO("path strategies")
     std::function<void(optionst &)> halt_callback = [](optionst &opts) {
       opts.set_option("stop-on-fail", true);
     };
-    std::function<void(optionst &)> no_halt_callback = [](optionst &opts) {};
+    std::function<void(optionst &)> no_halt_callback = [](optionst &) {};
 
     c =
       "/*  1 */  int main()      \n"
@@ -248,9 +270,13 @@ SCENARIO("path strategies")
         "lifo",
         no_halt_callback,
         c,
-        {symex_eventt::resume(symex_eventt::enumt::JUMP, 7),
+        {// Entry state is line 0
+         symex_eventt::resume(symex_eventt::enumt::NEXT, 0),
+         symex_eventt::resume(symex_eventt::enumt::JUMP, 7),
          symex_eventt::result(symex_eventt::enumt::FAILURE),
          symex_eventt::resume(symex_eventt::enumt::NEXT, 5),
+         symex_eventt::result(symex_eventt::enumt::FAILURE),
+         // Overall result
          symex_eventt::result(symex_eventt::enumt::FAILURE)});
     }
     GIVEN("stopping on failure")
@@ -259,7 +285,11 @@ SCENARIO("path strategies")
         "lifo",
         halt_callback,
         c,
-        {symex_eventt::resume(symex_eventt::enumt::JUMP, 7),
+        {// Entry state is line 0
+         symex_eventt::resume(symex_eventt::enumt::NEXT, 0),
+         symex_eventt::resume(symex_eventt::enumt::JUMP, 7),
+         symex_eventt::result(symex_eventt::enumt::FAILURE),
+         // Overall result
          symex_eventt::result(symex_eventt::enumt::FAILURE)});
     }
   }
@@ -270,22 +300,24 @@ SCENARIO("path strategies")
 
 void symex_eventt::validate_result(
   listt &events,
-  const safety_checkert::resultt result)
+  const incremental_goto_checkert::resultt::progresst result,
+  std::size_t &counter)
 {
   INFO(
     "Expecting result to be '"
-    << (result == safety_checkert::resultt::SAFE ? "success" : "failure")
-    << "'");
+    << (result == incremental_goto_checkert::resultt::progresst::DONE
+          ? "success"
+          : "failure")
+    << "' (item at index [" << counter << "] in expected results list");
+  ++counter;
 
-  REQUIRE(result != safety_checkert::resultt::ERROR);
-
-  if(result == safety_checkert::resultt::SAFE)
+  if(result == incremental_goto_checkert::resultt::progresst::DONE)
   {
     REQUIRE(!events.empty());
     REQUIRE(events.front().first == symex_eventt::enumt::SUCCESS);
     events.pop_front();
   }
-  else if(result == safety_checkert::resultt::UNSAFE)
+  else
   {
     REQUIRE(!events.empty());
     REQUIRE(events.front().first == symex_eventt::enumt::FAILURE);
@@ -295,35 +327,42 @@ void symex_eventt::validate_result(
 
 void symex_eventt::validate_resume(
   listt &events,
-  const goto_symex_statet &state)
+  const goto_symex_statet &state,
+  std::size_t &counter)
 {
   REQUIRE(!events.empty());
 
-  int dst = std::stoi(state.saved_target->source_location.get_line().c_str());
+  int dst = 0;
+  if(!state.saved_target->source_location.get_line().empty())
+    dst = std::stoi(state.saved_target->source_location.get_line().c_str());
 
-  if(state.has_saved_jump_target)
+  if(state.has_saved_next_instruction)
   {
-    INFO("Expecting resume to be 'jump' to line " << dst);
-    REQUIRE(events.front().first == symex_eventt::enumt::JUMP);
-  }
-  else if(state.has_saved_next_instruction)
-  {
-    INFO("Expecting resume to be 'next' to line " << dst);
+    INFO(
+      "Expecting resume to be 'next' to line "
+      << dst << " (item at index [" << counter
+      << "] in expected resumes list)");
     REQUIRE(events.front().first == symex_eventt::enumt::NEXT);
   }
+  else if(state.has_saved_jump_target)
+  {
+    INFO(
+      "Expecting resume to be 'jump' to line "
+      << dst << " (item at index [" << counter
+      << "] in expected resumes list)");
+    REQUIRE(events.front().first == symex_eventt::enumt::JUMP);
+  }
   else
     REQUIRE(false);
 
+  ++counter;
   REQUIRE(events.front().second == dst);
 
   events.pop_front();
 }
 
-// This is a simplified version of bmct::do_language_agnostic_bmc, without all
-// the edge cases to deal with java programs, bytecode loaded on demand, etc. We
-// need to replicate some of this stuff because the worklist is a local variable
-// of do_language_agnostic_bmc, and we need to check the worklist every time
-// symex returns.
+// This is a simplified copy of single_path_symex_checkert
+// because we have to check the worklist every time goto-symex returns.
 void _check_with_strategy(
   const std::string &strategy,
   const std::string &program,
@@ -340,77 +379,126 @@ void _check_with_strategy(
   register_language(new_ansi_c_language);
   cmdlinet cmdline;
   cmdline.args.push_back(tmp());
-  config.main = "main";
+  config.main = std::string("main");
   config.set(cmdline);
 
-  optionst opts;
-  cbmc_parse_optionst::set_default_options(opts);
-  opts.set_option("paths", true);
-  opts.set_option("exploration-strategy", strategy);
+  optionst options;
+  cbmc_parse_optionst::set_default_options(options);
+  options.set_option("paths", true);
+  options.set_option("exploration-strategy", strategy);
+  REQUIRE(is_valid_path_strategy(strategy));
+  opts_callback(options);
 
-  opts_callback(opts);
+  ui_message_handlert ui_message_handler(cmdline, "path-explore");
+  ui_message_handler.set_verbosity(0);
+  messaget log(ui_message_handler);
 
-  ui_message_handlert mh(cmdline, "path-explore");
-  mh.set_verbosity(0);
-  messaget log(mh);
-
-  path_strategy_choosert chooser;
-  REQUIRE(chooser.is_valid_strategy(strategy));
-  std::unique_ptr<path_storaget> worklist = chooser.get(strategy);
-
-  goto_modelt gm;
+  goto_modelt goto_model;
   int ret;
-  ret = cbmc_parse_optionst::get_goto_program(gm, opts, cmdline, log, mh);
+  ret = cbmc_parse_optionst::get_goto_program(
+    goto_model, options, cmdline, ui_message_handler);
   REQUIRE(ret == -1);
 
-  cbmc_solverst solvers(opts, gm.get_symbol_table(), mh);
-  solvers.set_ui(mh.get_ui());
-  std::unique_ptr<cbmc_solverst::solvert> cbmc_solver = solvers.get_solver();
-  prop_convt &pc = cbmc_solver->prop_conv();
-  std::function<bool(void)> callback = []() { return false; };
+  symbol_tablet symex_symbol_table;
+  namespacet ns(goto_model.get_symbol_table(), symex_symbol_table);
+  propertiest properties(initialize_properties(goto_model));
+  std::unique_ptr<path_storaget> worklist = get_path_strategy(strategy);
+  guard_managert guard_manager;
 
-  bmct bmc(opts, gm.get_symbol_table(), mh, pc, *worklist, callback);
-  bmc.set_ui(mh.get_ui());
-  safety_checkert::resultt result = bmc.run(gm);
-  symex_eventt::validate_result(events, result);
-
-  if(
-    result == safety_checkert::resultt::UNSAFE &&
-    opts.get_bool_option("stop-on-fail") && opts.is_set("paths"))
   {
-    worklist->clear();
+    // Put initial state into the work list
+    symex_target_equationt equation(ui_message_handler);
+    symex_bmct symex(
+      ui_message_handler,
+      goto_model.get_symbol_table(),
+      equation,
+      options,
+      *worklist,
+      guard_manager);
+    setup_symex(symex, ns, options, ui_message_handler);
+
+    symex.initialize_path_storage_from_entry_point_of(
+      goto_symext::get_goto_function(goto_model), symex_symbol_table);
   }
 
+  std::size_t expected_results_cnt = 0;
   while(!worklist->empty())
   {
-    cbmc_solverst solvers(opts, gm.get_symbol_table(), mh);
-    solvers.set_ui(mh.get_ui());
-    cbmc_solver = solvers.get_solver();
-    prop_convt &pc = cbmc_solver->prop_conv();
     path_storaget::patht &resume = worklist->peek();
 
-    symex_eventt::validate_resume(events, resume.state);
+    symex_eventt::validate_resume(events, resume.state, expected_results_cnt);
 
-    path_explorert pe(
-      opts,
-      gm.get_symbol_table(),
-      mh,
-      pc,
+    symex_bmct symex(
+      ui_message_handler,
+      goto_model.get_symbol_table(),
       resume.equation,
-      resume.state,
+      options,
       *worklist,
-      callback);
-    result = pe.run(gm);
+      guard_manager);
+    setup_symex(symex, ns, options, ui_message_handler);
+
+    symex.resume_symex_from_saved_state(
+      goto_symext::get_goto_function(goto_model),
+      resume.state,
+      &resume.equation,
+      symex_symbol_table);
+    postprocess_equation(
+      symex, resume.equation, options, ns, ui_message_handler);
+
+    incremental_goto_checkert::resultt result(
+      incremental_goto_checkert::resultt::progresst::DONE);
+
+    if(symex.get_remaining_vccs() > 0)
+    {
+      update_properties_status_from_symex_target_equation(
+        properties, result.updated_properties, resume.equation);
+
+      goto_symex_property_decidert property_decider(
+        options, ui_message_handler, resume.equation, ns);
+
+      const auto solver_runtime = prepare_property_decider(
+        properties, resume.equation, property_decider, ui_message_handler);
+
+      run_property_decider(
+        result,
+        properties,
+        property_decider,
+        ui_message_handler,
+        solver_runtime,
+        false);
+
+      symex_eventt::validate_result(
+        events, result.progress, expected_results_cnt);
+    }
 
-    symex_eventt::validate_result(events, result);
     worklist->pop();
 
     if(
-      result == safety_checkert::resultt::UNSAFE &&
-      opts.get_bool_option("stop-on-fail"))
+      result.progress ==
+        incremental_goto_checkert::resultt::progresst::FOUND_FAIL &&
+      options.get_bool_option("stop-on-fail"))
     {
       worklist->clear();
     }
+
+    if(worklist->empty())
+    {
+      update_status_of_not_checked_properties(
+        properties, result.updated_properties);
+
+      update_status_of_unknown_properties(
+        properties, result.updated_properties);
+    }
   }
+
+  const resultt overall_result = determine_result(properties);
+  symex_eventt::validate_result(
+    events,
+    overall_result == resultt::FAIL
+      ? incremental_goto_checkert::resultt::progresst::FOUND_FAIL
+      : incremental_goto_checkert::resultt::progresst::DONE,
+    expected_results_cnt);
+
+  INFO("The expected results list contains " << events.size() << " items");
   REQUIRE(events.empty());
 }
