@@ -6,7 +6,8 @@
 #define CPROVER_PATH_STRATEGIES_H
 
 #include <goto-programs/goto_model.h>
-#include <goto-programs/safety_checker.h>
+
+#include <goto-checker/incremental_goto_checker.h>
 
 #include <goto-symex/goto_symex_state.h>
 
@@ -36,10 +37,13 @@ public:
     return pairt(kind, -1);
   }
 
-  static void
-  validate_result(listt &events, const safety_checkert::resultt result);
+  static void validate_result(
+    listt &events,
+    const incremental_goto_checkert::resultt::progresst result,
+    std::size_t &);
 
-  static void validate_resume(listt &events, const goto_symex_statet &state);
+  static void
+  validate_resume(listt &events, const goto_symex_statet &state, std::size_t &);
 };
 
 void _check_with_strategy(
