@@ -1,13 +1,13 @@
 /*******************************************************************\
 
- Module: Unit test utilities
+Module: Unit test utilities
 
- Author: DiffBlue Limited. All rights reserved.
+Author: Diffblue Limited.
 
 \*******************************************************************/
 
 #include "require_symbol.h"
-#include "catch.hpp"
+#include "use_catch.h"
 
 /// Verify whether a given identifier is found in the symbol table and return it
 /// \param symbol_table: The symbol table to look in
