@@ -2,7 +2,7 @@
 
 /// \file Tests that irept memory consumption is fixed
 
-#include <testing-utils/catch.hpp>
+#include <testing-utils/use_catch.h>
 #include <util/irep.h>
 
 SCENARIO("irept_memory", "[core][utils][irept]")
@@ -35,7 +35,7 @@ SCENARIO("irept_memory", "[core][utils][irept]")
       REQUIRE(sizeof(std::vector<int>) == 3 * sizeof(void *));
 #endif
 
-#ifndef SUB_IS_LIST
+#ifndef NAMED_SUB_IS_FORWARD_LIST
       const std::size_t named_size = sizeof(std::map<int, int>);
 #  ifndef _GLIBCXX_DEBUG
 #    ifdef __APPLE__
@@ -47,13 +47,13 @@ SCENARIO("irept_memory", "[core][utils][irept]")
 #    endif
 #  endif
 #else
-      const std::size_t named_size = sizeof(std::list<int>);
+      const std::size_t named_size = sizeof(std::forward_list<int>);
 #  ifndef _GLIBCXX_DEBUG
-      REQUIRE(sizeof(std::list<int>) == 3 * sizeof(void *));
+      REQUIRE(sizeof(std::forward_list<int>) == sizeof(void *));
 #  endif
 #endif
 
-#ifdef HASH_CODE
+#if HASH_CODE
       const std::size_t hash_code_size = sizeof(std::size_t);
 #else
       const std::size_t hash_code_size = 0;
@@ -61,8 +61,7 @@ SCENARIO("irept_memory", "[core][utils][irept]")
 
       REQUIRE(
         sizeof(irept::dt) ==
-        ref_count_size + data_size + sub_size + 2 * named_size +
-          hash_code_size);
+        ref_count_size + data_size + sub_size + named_size + hash_code_size);
     }
 
     THEN("get_nil_irep yields ID_nil")
@@ -73,6 +72,19 @@ SCENARIO("irept_memory", "[core][utils][irept]")
     }
   }
 
+  GIVEN("Parts of an irep")
+  {
+    irept irep("some_id", {{"some_member", irept("other")}}, {irept("op")});
+
+    THEN("It is properly initialized")
+    {
+      REQUIRE(irep.id() == "some_id");
+      REQUIRE(irep.find("some_member") == irept("other"));
+      REQUIRE(irep.get_sub().size() == 1);
+      REQUIRE(irep.get_sub().front() == irept("op"));
+    }
+  }
+
   GIVEN("An initialized irep")
   {
     irept irep("some_id");
@@ -131,7 +143,6 @@ SCENARIO("irept_memory", "[core][utils][irept]")
     {
       REQUIRE(irep.get_sub().empty());
       REQUIRE(irep.get_named_sub().empty());
-      REQUIRE(irep.get_comments().empty());
 
       irept &e = irep.add("a_new_element");
       REQUIRE(e.id().empty());
@@ -139,13 +150,20 @@ SCENARIO("irept_memory", "[core][utils][irept]")
       REQUIRE(irep.find("a_new_element").id() == "some_id");
 
       irept irep2("second_irep");
-      irept &e2 = irep.add("a_new_element", irep2);
+      irep.add("a_new_element", irep2);
+      REQUIRE(!irept::is_comment("a_new_element"));
       REQUIRE(irep.find("a_new_element").id() == "second_irep");
-      REQUIRE(irep.get_named_sub().size() == 1);
+      std::size_t named_sub_size =
+        std::distance(irep.get_named_sub().begin(), irep.get_named_sub().end());
+      REQUIRE(named_sub_size == 1);
 
       irep.add("#a_comment", irep2);
+      REQUIRE(irept::is_comment("#a_comment"));
       REQUIRE(irep.find("#a_comment").id() == "second_irep");
-      REQUIRE(irep.get_comments().size() == 1);
+      named_sub_size =
+        std::distance(irep.get_named_sub().begin(), irep.get_named_sub().end());
+      REQUIRE(named_sub_size == 2);
+      REQUIRE(irept::number_of_non_comments(irep.get_named_sub()) == 1);
 
       irept bak(irep);
       irep.remove("no_such_id");
@@ -159,19 +177,30 @@ SCENARIO("irept_memory", "[core][utils][irept]")
 
       irep.move_to_named_sub("another_entry", irep2);
       REQUIRE(irep.get_sub().size() == 1);
-      REQUIRE(irep.get_named_sub().size() == 1);
-      REQUIRE(irep.get_comments().size() == 1);
+      named_sub_size =
+        std::distance(irep.get_named_sub().begin(), irep.get_named_sub().end());
+      REQUIRE(named_sub_size == 2);
 
       irept irep3;
       irep.move_to_named_sub("#a_comment", irep3);
       REQUIRE(irep.find("#a_comment").id().empty());
       REQUIRE(irep.get_sub().size() == 1);
-      REQUIRE(irep.get_named_sub().size() == 1);
-      REQUIRE(irep.get_comments().size() == 1);
+      named_sub_size =
+        std::distance(irep.get_named_sub().begin(), irep.get_named_sub().end());
+      REQUIRE(named_sub_size == 2);
 
       irept irep4;
       irep.move_to_named_sub("#another_comment", irep4);
-      REQUIRE(irep.get_comments().size() == 2);
+      named_sub_size =
+        std::distance(irep.get_named_sub().begin(), irep.get_named_sub().end());
+      REQUIRE(named_sub_size == 3);
+
+      irept irep5("moved_irep");
+      irep.add("a_moved_element", std::move(irep5));
+      REQUIRE(irep.find("a_moved_element").id() == "moved_irep");
+      named_sub_size =
+        std::distance(irep.get_named_sub().begin(), irep.get_named_sub().end());
+      REQUIRE(named_sub_size == 4);
     }
 
     THEN("Setting and getting works")
@@ -201,17 +230,19 @@ SCENARIO("irept_memory", "[core][utils][irept]")
       REQUIRE(irep.get_size_t("numeric_id") == 42u);
       REQUIRE(irep.get_long_long("numeric_id") == 42);
 
+      irept irep3("move me");
+      irep.set("another_moved_element", std::move(irep3));
+      REQUIRE(irep.find("another_moved_element").id() == "move me");
+
       irep.clear();
       REQUIRE(irep.id().empty());
       REQUIRE(irep.get_sub().empty());
       REQUIRE(irep.get_named_sub().empty());
-      REQUIRE(irep.get_comments().empty());
 
       irep.make_nil();
       REQUIRE(irep.id() == ID_nil);
       REQUIRE(irep.get_sub().empty());
       REQUIRE(irep.get_named_sub().empty());
-      REQUIRE(irep.get_comments().empty());
     }
 
     THEN("Pretty printing works")
