@@ -4,20 +4,70 @@
 
 #define SN_INTERNAL_CHECKS
 
-#include <testing-utils/catch.hpp>
+#include <testing-utils/use_catch.h>
 #include <util/sharing_node.h>
 
-void sharing_node_test()
+// could be an internal node or a container node
+class innert : public sharing_nodet<int, int>
+{
+public:
+  friend void sharing_node_internals_test();
+};
+
+void sharing_node_internals_test()
+{
+  SECTION("Internal node test")
+  {
+    innert internal;
+    REQUIRE(!internal.data);
+
+    internal.write_internal();
+    REQUIRE(internal.data);
+    REQUIRE(internal.data.use_count() == 1);
+
+    innert internal2(internal);
+    REQUIRE(internal.use_count() == 2);
+    REQUIRE(internal2.use_count() == 2);
+
+    internal2.write_internal();
+    REQUIRE(internal.use_count() == 1);
+    REQUIRE(internal2.use_count() == 1);
+  }
+
+  SECTION("Container node test")
+  {
+    innert container;
+    REQUIRE(!container.data);
+
+    container.write_container();
+    REQUIRE(container.data);
+    REQUIRE(container.data.use_count() == 1);
+
+    innert container2(container);
+    REQUIRE(container.use_count() == 2);
+    REQUIRE(container2.use_count() == 2);
+
+    container2.write_container();
+    REQUIRE(container.use_count() == 1);
+    REQUIRE(container2.use_count() == 1);
+  }
+}
+
+TEST_CASE("Sharing node internals", "[core][util]")
+{
+  sharing_node_internals_test();
+}
+
+TEST_CASE("Sharing node", "[core][util]")
 {
   SECTION("Leaf test")
   {
-    typedef sharing_node_leaft<int, int> leaft;
+    typedef sharing_nodet<int, int> leaft;
 
     // Basic leaf
     {
       const leaft leaf(1, 2);
 
-      REQUIRE(!leaf.empty());
       REQUIRE(leaf.shares_with(leaf));
       REQUIRE(leaf.get_key() == 1);
       REQUIRE(leaf.get_value() == 2);
@@ -40,26 +90,16 @@ void sharing_node_test()
 
       REQUIRE(leaf2.shares_with(leaf1));
 
-      auto &v = leaf2.get_value();
-      v = 3;
+      leaf2.set_value(3);
+
       REQUIRE(leaf2.get_value() == 3);
       REQUIRE(!leaf2.shares_with(leaf1));
     }
-
-    // Detaching
-    {
-      leaft leaf(1, 2);
-
-      auto p = leaf.data.get();
-      leaf.write();
-
-      REQUIRE(leaf.data.get() == p);
-    }
   }
 
   SECTION("Inner node test")
   {
-    typedef sharing_node_innert<int, int> innert;
+    typedef sharing_nodet<int, int> innert;
 
     // Empty container
     {
@@ -98,8 +138,8 @@ void sharing_node_test()
 
       innert c2(c1);
       auto leaf = c2.find_leaf(1);
-      auto &v = leaf->get_value();
-      v = 7;
+
+      leaf->set_value(7);
 
       REQUIRE(!c1.shares_with(c2));
 
@@ -133,76 +173,6 @@ void sharing_node_test()
 
       REQUIRE(i.get_to_map().empty());
       REQUIRE(ci.find_child(0) == nullptr);
-
-      innert *p;
-
-      p = i.add_child(1);
-
-      REQUIRE(p != nullptr);
     }
   }
-
-  SECTION("Combined")
-  {
-    typedef sharing_node_leaft<int, int> leaft;
-    typedef sharing_node_innert<int, int> innert;
-
-    innert map;
-
-    REQUIRE(map.empty());
-
-    innert *ip;
-    innert *cp;
-    leaft *lp;
-
-    // Add 0 -> 0 -> (0, 1)
-
-    ip = &map;
-
-    ip = ip->add_child(0);
-    REQUIRE(ip != nullptr);
-
-    cp = ip->add_child(0);
-    REQUIRE(cp != nullptr);
-
-    lp = cp->place_leaf(0, 1);
-    REQUIRE(lp != nullptr);
-
-    // Add 1 -> 2 -> (3, 4)
-
-    ip = &map;
-
-    ip = ip->add_child(1);
-    REQUIRE(ip != nullptr);
-
-    cp = ip->add_child(2);
-    REQUIRE(cp != nullptr);
-
-    lp = cp->place_leaf(3, 4);
-    REQUIRE(lp != nullptr);
-
-    // Add 1 -> 3 -> (4, 5)
-
-    ip = &map;
-
-    ip = ip->add_child(1);
-    REQUIRE(ip != nullptr);
-
-    cp = ip->add_child(3);
-    REQUIRE(cp != nullptr);
-
-    lp = cp->place_leaf(4, 5);
-    REQUIRE(lp != nullptr);
-
-    // Copy
-
-    innert map2(map);
-
-    REQUIRE(map2.shares_with(map));
-  }
-}
-
-TEST_CASE("Sharing node")
-{
-  sharing_node_test();
 }
