@@ -1,11 +1,15 @@
-/*
-  Author: Diffblue Ltd.
-*/
+/*******************************************************************\
+
+Module: Unit tests of strip_string
+
+Author: Diffblue Ltd.
+
+\*******************************************************************/
 
 /// \file
 /// strip_string Unit Tests
 
-#include <testing-utils/catch.hpp>
+#include <testing-utils/use_catch.h>
 #include <util/string_utils.h>
 
 SCENARIO("strip_string", "[core][utils][string_utils][strip_string]")
